# coding: utf-8
module DungeonMasterHelper
  # 定数用
  require_rrk 'src/app/helper/constants/dungeon/dungeon_constants'
  # 定数系は一番上
  include DungeonConstants

  require_rrk 'src/app/helper/game_define_helper'
  include GameDefineHelper
  require_rrk 'src/defines/dungeon/utils/version_helper'
  include VersionHelper

  # 1.2モンスターを追加したら定義をここに追加
  dungeon_master_summons_list =
    if TRIAL_MODE
      MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST
    else
      if DUNGEON_VERSION < 5
        # 旧作はVer5仕様に統合
        MonsterListV1::DUNGEON_MASTER_SUMMONS_LIST
      else
        case DUNGEON_VERSION.floor
          when 5
            MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST
          else
            MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST
        end
      end
    end

  dungeon_master_summons_list.each do |monster_key, monster_params|
    summon_detail_method_name = SHOW_SUMMON_DETAIL_PREFIX + monster_key.to_s
    summon_unlock_method_name = SHOW_SUMMON_UNLOCK_PREFIX + monster_key.to_s

    define_method(summon_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      enemy = monster_define_list[monster_key]
      monster_params.merge!({power: enemy[:power], life_point: enemy[:life_point],})
      monster_description = enemy[:description]
      elements = @dungeon_logic_controller.monster_detail(dungeon_master, monster_key, monster_params, monster_description)
      assign_to_lists(elements)
    end

    define_method(summon_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.monster_summon(dungeon_master, monster_key, monster_params)
      assign_to_lists(elements)
    end
  end

  TrapList::DUNGEON_MASTER_ROCKED_TRAPS_LIST.each do |trap_key, trap_params|
    trap_detail_method_name = SHOW_TRAP_DETAIL_PREFIX + trap_key.to_s
    trap_unlock_method_name = SHOW_TRAP_UNLOCK_PREFIX + trap_key.to_s

    define_method(trap_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.trap_detail(dungeon_master, trap_key, trap_params)
      assign_to_lists(elements)
    end

    define_method(trap_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.trap_development(dungeon_master, trap_key, trap_params)
      assign_to_lists(elements)
    end
  end

  ItemList::DUNGEON_MASTER_ROCKED_ITEMS_LIST.each do |item_key, item_params|
    item_detail_method_name = SHOW_ITEM_DETAIL_PREFIX + item_key.to_s
    item_unlock_method_name = SHOW_ITEM_UNLOCK_PREFIX + item_key.to_s

    define_method(item_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.item_detail(dungeon_master, item_key, item_params)
      assign_to_lists(elements)
    end

    define_method(item_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.item_development(dungeon_master, item_key, item_params)
      assign_to_lists(elements)
    end
  end

  ItemList::DUNGEON_MASTER_ROCKED_BOOKS_LIST.each do |item_key, item_params|
    book_detail_method_name = SHOW_BOOK_DETAIL_PREFIX + item_key.to_s
    book_unlock_method_name = SHOW_BOOK_UNLOCK_PREFIX + item_key.to_s

    define_method(book_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.book_detail(dungeon_master, item_key, item_params)
      assign_to_lists(elements)
    end

    define_method(book_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.book_development(dungeon_master, item_key, item_params)
      assign_to_lists(elements)
    end
  end

  CarpenterList::BROTHEL_BUILDINGS_LIST.each do |building_key, building_params|
    building_detail_method_name = SHOW_BUILDING_DETAIL_PREFIX + building_key.to_s
    building_unlock_method_name = SHOW_BUILDING_UNLOCK_PREFIX + building_key.to_s

    define_method(building_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.building_detail(dungeon_master, building_key, building_params)
      assign_to_lists(elements)
    end

    define_method(building_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.building_development(dungeon_master, building_key, building_params)
      assign_to_lists(elements)
    end
  end

  TentacleList::TENTACLE_LABS_LIST.each do |tentacle_key, tentacle_params|
    tentacle_detail_method_name = SHOW_TENTACLE_DETAIL_PREFIX + tentacle_key.to_s
    tentacle_unlock_method_name = SHOW_TENTACLE_UNLOCK_PREFIX + tentacle_key.to_s

    define_method(tentacle_detail_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.tentacle_detail(dungeon_master, tentacle_key, tentacle_params)
      assign_to_lists(elements)
    end

    define_method(tentacle_unlock_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.tentacle_development(dungeon_master, tentacle_key, tentacle_params)
      assign_to_lists(elements)
    end
  end

  require_rrk 'src/defines/dungeon/monster_list_by_type'
  MonsterListByType::elite_monster_define_list.each do |monster_key, monster_params|
    summon_elite_method_name = "summon_elite_monster_#{monster_key}"
    define_method(summon_elite_method_name) do
      dungeon_master = @game_master.dungeon_master
      elements = @dungeon_logic_controller.summon_elite_monster(dungeon_master, monster_params)
      assign_to_lists(elements)
    end
  end
end
