# coding: utf-8
# LogicControllerは本質的にControllerが担うべき役割を持つ
# 共通ロジックの管理が超煩雑になったため
# この子は奴隷の売買画面を管轄する

#view_element_logic
%w(auction_view_element_logic).each do |src_name|
  require_rrk 'src/app/view_element_logic/dungeon/' + src_name
end
require_rrk 'src/defines/script/dungeon/auction_script'
require_rrk 'src/app/helper/game_define_helper'
require_rrk 'src/app/helper/dungeon/auction_helper'

class AuctionLogicController
  include GameDefineHelper
  include AuctionHelper

  def initialize
    @auction_script = AuctionScript.new
    @view_element_logic = AuctionViewElementLogic.new
    super
  end

  def show_auction_room(game_master, script = [])
    slaves = game_master.auction_room_slaves
    if slaves.size.zero?
      script += @auction_script.show_blank_auction_room
      return {
        'script_list' => {
          'clear'    => true,
          'elements' => @view_element_logic.make_scripts_from(script),
        },
      }
    end

    script += @auction_script.show_auction_room(slaves, game_master.wallet_money)
    elements = @view_element_logic.show_auction_room(slaves, script)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def show_slave_auction(game_master, slave_id)
    slave = game_master.auction_room_slaves[slave_id]
    script = @auction_script.show_slave_auction(slave, game_master.wallet_money)

    return _show_slave_auction_common(game_master.dungeon_master, slave, script)
  end

  BUY_PREMIUM_RATE = 2.0
  def _show_slave_auction_common(dungeon_master, slave, script)
    sell_price = slave[:sell_info][:slave_price]
    buy_price = (slave[:sell_info][:slave_price] * BUY_PREMIUM_RATE).to_i

    elements = @view_element_logic.show_slave_auction(dungeon_master, slave, script, sell_price, buy_price)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def buy_slave(game_master, slave_id)
    slave = game_master.auction_room_slaves[slave_id]
    dungeon_master = game_master.dungeon_master
    return _show_trial_buy_script(dungeon_master, slave) if TRIAL_MODE

    buy_price = get_buy_price(slave[:sell_info][:slave_price])

    # お金が足りない
    unless dungeon_master.has_money_to_buy(slave_id)
      slave = game_master.auction_room_slaves[slave_id]
      script = @auction_script.lack_to_buy_slave(slave, buy_price, dungeon_master.money)
      return _show_slave_auction_common(dungeon_master, slave, script)
    end

    adventurer = game_master.bar_adventurers[slave_id]
    dungeon_master.buy_slave_to_factory(slave_id, adventurer, buy_price)
    script = @auction_script.buy_slave(slave, buy_price, dungeon_master.money)
    return show_auction_room(game_master, script)
  end

  def buy_slave_n_convert_slave(game_master, slave_id)
    slave = game_master.auction_room_slaves[slave_id]
    dungeon_master = game_master.dungeon_master
    return _show_trial_buy_script(dungeon_master, slave) if TRIAL_MODE

    buy_price = get_buy_price(slave[:sell_info][:slave_price])

    # お金が足りない
    unless dungeon_master.has_money_to_buy(slave_id)
      slave = game_master.auction_room_slaves[slave_id]
      script = @auction_script.lack_to_buy_slave(slave, buy_price, dungeon_master.money)
      return _show_slave_auction_common(dungeon_master, slave, script)
    end

    adventurer = game_master.bar_adventurers[slave_id]
    dungeon_master.buy_slave_n_convert(slave_id, adventurer, buy_price)
    script = @auction_script.buy_slave_n_convert(slave, buy_price, dungeon_master.money)
    return show_auction_room(game_master, script)
  end

  def buy_slave_n_convert_magic(game_master, slave_id)
    slave = game_master.auction_room_slaves[slave_id]
    dungeon_master = game_master.dungeon_master
    return _show_trial_buy_script(dungeon_master, slave) if TRIAL_MODE

    buy_price = get_buy_price(slave[:sell_info][:slave_price])

    # お金が足りない
    unless dungeon_master.has_money_to_buy(slave_id)
      slave = game_master.auction_room_slaves[slave_id]
      script = @auction_script.lack_to_buy_slave(slave, buy_price, dungeon_master.money)
      return _show_slave_auction_common(dungeon_master, slave, script)
    end

    adventurer = game_master.bar_adventurers[slave_id]

    # 再利用の処理
    game_master.remove_using_name(adventurer) if ORIGINAL_MODE
    game_master.add_lost_adventurer_ids(adventurer.original_id) if FGO_MODE

    dungeon_master.buy_slave_n_convert_magic(slave_id, adventurer, buy_price)
    script = @auction_script.buy_slave_n_convert_magic(slave, buy_price, dungeon_master.money, dungeon_master.magic_power)
    return show_auction_room(game_master, script)
  end

  def sell_slave(game_master, slave_id)
    dungeon_master = game_master.dungeon_master
    slave = game_master.auction_room_slaves[slave_id]
    sell_price = slave[:sell_info][:slave_price]

    adventurer = game_master.bar_adventurers[slave_id]

    # 再利用の処理
    game_master.remove_using_name(adventurer) if ORIGINAL_MODE
    game_master.add_lost_adventurer_ids(adventurer.original_id) if FGO_MODE

    dungeon_master.sell_slave!(adventurer, sell_price)
    script = @auction_script.sell_slave(adventurer, slave[:sell_info], dungeon_master.money)
    return show_auction_room(game_master, script)
  end

  def _show_trial_buy_script(dungeon_master, slave)
    script = @auction_script.show_trial_buy_script
    _show_slave_auction_common(dungeon_master, slave, script)
  end
end
