# coding: utf-8

require_rrk 'src/app/view_element_logic/dungeon/dungeon_view_element_logic'
require_rrk 'src/defines/script/dungeon/dungeon_master_script'

# ダンジョンマスターのダンジョン強化とかを扱うよ
class DungeonLogicController

  def initialize(options)
    @trial_mode = options[:trial_mode]
    @view_element_logic = DungeonViewElementLogic.new(options)
    @dungeon_master_script = DungeonMasterScript.new
  end

  def show_dungeon_top_menu(dungeon_master)
    script = @dungeon_master_script.show_dungeon_top_menu(dungeon_master.money, dungeon_master.magic_power)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_dungeon_top_menu(dungeon_master),
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_summons_power_up(dungeon_master)
    script = @dungeon_master_script.show_summons_power_up(dungeon_master.money)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_summons_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_traps_power_up(dungeon_master)
    script = @dungeon_master_script.show_traps_power_up(dungeon_master.money, dungeon_master.magic_power)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_traps_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_merchant_shop(dungeon_master)
    script = @dungeon_master_script.show_merchant_shop(dungeon_master.money)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_merchant_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_book_shop(dungeon_master)
    script = @dungeon_master_script.show_book_shop(dungeon_master.money)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_book_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_summon_magic_room(dungeon_master)
    script = @dungeon_master_script.show_summon_magic_room(
      dungeon_master.guard_unit_count,
      dungeon_master.money,
      dungeon_master.magic_power
    )

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.show_summon_magic_room(script),
      },
    }
  end

  def summon_elite_monster(dungeon_master, monster_params)
    cost = monster_params[:magic_power]
    return shortage_magic_power_in_summon(dungeon_master, cost) unless dungeon_master.have_magic_power?(cost)

    dungeon_master.decrease_magic_power!(cost)
    dungeon_master.add_elite_guards_in_deep!

    header_script = @dungeon_master_script.summon_elite_monster(
      dungeon_master.guard_unit_count,
      dungeon_master.money,
      dungeon_master.magic_power,
      monster_params,
    )
    script = @dungeon_master_script.show_summon_magic_room(
      dungeon_master.guard_unit_count,
      dungeon_master.money,
      dungeon_master.magic_power
    )

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.summon_elite_monster(header_script, script),
      },
    }
  end

  def shortage_magic_power_in_summon(dungeon_master, cost)
    header_script = @dungeon_master_script.shortage_of_magic_power(cost, dungeon_master.magic_power)
    script = @dungeon_master_script.show_summon_magic_room(
      dungeon_master.guard_unit_count,
      dungeon_master.money,
      dungeon_master.magic_power
    )

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.shortage_of_summon_elite_monster(header_script, script),
      },
    }
  end

  def show_carpenter_shop(dungeon_master)
    script = @dungeon_master_script.show_carpenter_shop(dungeon_master.money)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_carpenter_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def show_tentacle_shop(dungeon_master)
    script = @dungeon_master_script.show_tentacle_shop(dungeon_master.money, dungeon_master.magic_power)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_tentacle_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def monster_detail(dungeon_master, monster_key, monster_params, monster_description)
    monster_description_with = monster_description + [monster_params[:floor],@dungeon_master_script.get_monster_power_desc(monster_params)]
    elements =
      if dungeon_master.have_money?(monster_params[:price])
        description = @view_element_logic.make_scripts_from(monster_description_with)
        @view_element_logic.show_summon_detail_of(description, monster_key, monster_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_money(dungeon_master.money, monster_params[:price])
        @view_element_logic.make_scripts_from(monster_description_with + short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def monster_summon(dungeon_master, monster_key, monster_params)
    dungeon_master.decrease_money!(monster_params[:price])
    dungeon_master.unlock_summon!(monster_key)

    monster_name = monster_params[:name]
    scripts = @dungeon_master_script.deploy_monster(monster_name, dungeon_master.money, monster_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_summons_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def trap_detail(dungeon_master, trap_key, trap_params)
    trap_description = @dungeon_master_script.trap_description(trap_params)
    magic_power_desc = @dungeon_master_script.magic_power_description(dungeon_master)
    description = @view_element_logic.make_scripts_from(trap_description + magic_power_desc)
    elements =
      if dungeon_master.have_magic_power?(trap_params[:price])
        @view_element_logic.show_trap_detail_of(description, trap_key, trap_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_magic_power(dungeon_master.magic_power, trap_params[:price])
        description + @view_element_logic.make_scripts_from(short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def trap_development(dungeon_master, trap_key, trap_params)
    dungeon_master.decrease_magic_power!(trap_params[:price])
    dungeon_master.unlock_trap!(trap_key)

    trap_name = trap_params[:name]
    scripts = @dungeon_master_script.deploy_trap(trap_name, dungeon_master.magic_power, trap_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_traps_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def item_detail(dungeon_master, item_key, item_params)
    item_description = @dungeon_master_script.item_description(item_params)
    money_desc = @dungeon_master_script.money_description(dungeon_master)
    description = @view_element_logic.make_scripts_from(item_description + money_desc)
    elements =
      if dungeon_master.have_money?(item_params[:price])
        @view_element_logic.show_item_detail_of(description, item_key, item_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_money(dungeon_master.money, item_params[:price])
        description + @view_element_logic.make_scripts_from(short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def item_development(dungeon_master, item_key, item_params)
    dungeon_master.decrease_money!(item_params[:price])
    dungeon_master.unlock_item!(item_key)
    dungeon_master.develop_item!(item_key)

    item_name = item_params[:name]
    scripts = @dungeon_master_script.deploy_item(item_name, dungeon_master.money, item_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_items_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def book_detail(dungeon_master, item_key, item_params)
    item_description = @dungeon_master_script.item_description(item_params)
    money_desc = @dungeon_master_script.money_description(dungeon_master)
    description = @view_element_logic.make_scripts_from(item_description + money_desc)
    elements =
      if dungeon_master.have_money?(item_params[:price])
        @view_element_logic.show_book_detail_of(description, item_key, item_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_money(dungeon_master.money, item_params[:price])
        description + @view_element_logic.make_scripts_from(short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def book_development(dungeon_master, item_key, item_params)
    dungeon_master.decrease_money!(item_params[:price])
    dungeon_master.unlock_book!(item_params[:key])

    item_name = item_params[:name]
    scripts = @dungeon_master_script.deploy_book(item_name, dungeon_master.money, item_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_books_power_up(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def building_detail(dungeon_master, building_key, building_params)
    building_description = @dungeon_master_script.building_description(building_params)
    money_desc = @dungeon_master_script.money_description(dungeon_master)
    description = @view_element_logic.make_scripts_from(building_description + money_desc)
    elements =
      if dungeon_master.have_money?(building_params[:price])
        @view_element_logic.show_building_detail_of(description, building_key, building_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_money(dungeon_master.money, building_params[:price])
        description + @view_element_logic.make_scripts_from(short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def building_development(dungeon_master, building_key, building_params)
    dungeon_master.decrease_money!(building_params[:price])
    dungeon_master.unlock_building_or_buy!(building_key)

    building_name = building_params[:name]
    scripts = @dungeon_master_script.deploy_building(building_name, dungeon_master.money, building_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_carpenter_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def tentacle_detail(dungeon_master, tentacle_key, tentacle_params)
    tentacle_description = @dungeon_master_script.tentacle_description(tentacle_params)
    magic_power_desc = @dungeon_master_script.magic_power_description(dungeon_master)
    description = @view_element_logic.make_scripts_from(tentacle_description + magic_power_desc)
    elements =
      if dungeon_master.have_magic_power?(tentacle_params[:price])
        @view_element_logic.show_tentacle_detail_of(description, tentacle_key, tentacle_params[:name])
      else
        short_cost_script = @dungeon_master_script.short_of_magic_power(dungeon_master.magic_power, tentacle_params[:price])
        description + @view_element_logic.make_scripts_from(short_cost_script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def tentacle_development(dungeon_master, tentacle_key, tentacle_params)
    dungeon_master.decrease_magic_power!(tentacle_params[:price])
    dungeon_master.unlock_tentacle!(tentacle_key)

    tentacle_name = tentacle_params[:name]
    scripts = @dungeon_master_script.deploy_tentacle(tentacle_name, dungeon_master.magic_power, tentacle_params[:price])

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_tentacle_shop(dungeon_master)
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end
end

