# coding: utf-8

require_rrk 'src/defines/script/dungeon/adventurer_script'
require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/model/dungeon/utils/secure_random' # ランダム文字列生成
require_rrk 'src/app/util/debug_util'
require_rrk 'src/app/model/dungeon/adventurer/sex_status'
require_rrk 'src/app/model/dungeon/adventurer/script_status'
require_rrk 'src/defines/dungeon/skill_list'
require_rrk 'src/app/helper/game_define_helper'
require_rrk 'src/app/model/dungeon/adventurer/betrayed_servant'
require_rrk 'src/app/model/dungeon/adventurer/helper/personality_helper'

%w(forwardable).each do |src_name|
  require_rrk 'src/app/helper/' + src_name
end

class Adventurer
  extend Forwardable
  include DungeonCommon
  include DungeonSecureRandom
  include DebugUtil
  include DungeonSkillList
  include GameDefineHelper
  include PersonalityHelper

  attr_accessor :id, :original_id, :friendly_list, :class_type, :class_name, :short_class_type,
                :name, :short_name, :level,
                :life_point_limit, :life_point, :life_point_in_turn_start,
                :friendly_damage_point_in_turn,
                :dexterity, :cunning, :wisdom, :magic, :weapon_technique, :lightness,
                :battle_bonus, :betrayed_battle_bonus,
                :lawful_level, :good_level,
                :leader_aptitude_bonus, :friendly_list, :positiveness,
                :kill_score, :kill_score_in_this_adventure,
                :total_exp_point,
                :status_text,
                # FGOの何日目に来たのテキスト
                :coming_day_text
  attr_reader :skills, :rest_turns, :floor_in_lonely, :captured_initial_count, :captured_monster,
              :bad_status, :bad_sex_status,
              :talents, # talentsはskillsの後継
              :equip_belt_in_dive,
              :ero_equips, # エロ装備。ひん剥かれると初期化されるよ
              :bad_status_in_dive,
              :bad_status_in_body,
              :skill_personality,
              :skill_holic,
              :tattoos,
              :hooter_level,
              :jail_mode,
              :jail_detail_script,
              # オリジナル用
              :race_type,
              :hierarchy,
              :motivation,
              :origin_from,
              :virgin_text,
              :personality_type,

              :skill_params_for_betrayed,
              :captured_betrayed_servant,
              :bounty_target,
              :max_level, :max_honor, :honor, :honor_plus_in_this_adventure, :party_identify_code,
              :equip_point, :ero_equip_point,
              :resist_point, :resist_point_max, :resist_point_before_turn,
              :san_level, :san_level_max, :san_level_before_turn, :base_san_decrease_point, :result_san_decrease_point, :san_decrease_point_before_bonus_rate,
              :generated_magic_power, # 抵抗や正気が削れると生成される。あとトラップ絶頂
              :self_call, :trap_count, :buyer_type,
              :this_turn_damage_point, :this_turn_trap_damage_point, :this_turn_ero_trap_damage_point, :this_turn_ero_trap_orgasm_count,
              :inventory, :teleport_pass_inventory,
              :sold_identify_code,
              :friendly_adv_list, :best_friendly_adv_list

  def_delegators :@script_status, :flags, :jail_flash_flags, :script,
                 :clear_jail_flash_flags!,
                 :get_teleport_pass, :get_teleport_pass_shorten, :teleport_script, :teleport_for_rescue_script,
                 :ero_trap_effect_script,
                 :ero_trap_sperm_n_orgasms_script, :ero_trap_sperm_shot_script, :ero_trap_orgasms_by_params_script,
                 :status_script_in_jail, :resist_status_script_in_jail,
                 :daily_status_script_in_jail, :daily_resist_status_script_in_jail,
                 :set_daily_status_script_in_jail, :set_daily_resist_status_script_in_jail,
                 :down_to_slave_script, :jail_training_script, :jail_training_short_script, :betrayed_servant_battle_opening_script,
                 :battle_prelude_only_n_washed_script, :battle_prelude_all_washed_script, :battle_prelude_script,
                 :jail_summary_text, :set_jail_summary_text!,
                 :get_treasure_script
  def_delegators :@adventurer_script,
                 :show_bounty_script, :set_jail_mode_script, :pay_bounty_script, :short_of_bounty_script

  # ここを更新したらslave.rbも忘れないこと
  def_delegators :@sex_status, :virgin?, :lost_virgin!, :lost_virgin_script, :set_lost_virgin_script, :add_masturbation_exp!,
                 :masturbation!, :get_ero_tunnel_sex!, :get_ero_tunnel_lesbian_sex!,
                 :men_exp, :lesbian_exp, :force_sex_exp, :tentacle_exp, :spring_count,
                 :spring_group_count, :spring_v_count, :spring_a_count,
                 :custom_body_exp, :custom_mind_exp,
                 :add_men_exp!, :add_lesbian_exp!, :add_force_sex_exp!, :add_tentacle_exp!, :add_spring_count!,
                 :add_sex_stared_exp!,
                 :add_spring_group_count!, :add_spring_v_count!, :add_spring_a_count!,
                 :add_custom_body_exp!, :add_custom_mind_exp!,
                 :add_c_exp!, :add_v_exp!, :add_a_exp!, :add_b_exp!,
                 :add_v_sex_exp!, :add_a_sex_exp!, :add_oral_exp!,
                 :add_ero_develop_rate!, :is_max_ero_develop_rate?,
                 :orgasm_count, :sperm_shot,
                 :add_c_orgasm_by_spring!, :add_v_orgasm_by_spring!, :add_a_orgasm_by_spring!, :add_b_orgasm_by_spring!,
                 :_add_v_orgasm!, :_add_a_orgasm!,
                 :add_skin_sperm_shot!,
                 :decrease_by_orgasm!,
                 :decrease_resist, :clear_decrease_resist!, :set_decrease_resist,
                 :decrease_san, :clear_decrease_san!, :set_decrease_san,
                 :masturbation_exp, :v_sex_exp, :a_sex_exp,
                 :c_exp, :v_exp, :a_exp, :b_exp, :oral_exp,
                 :c_ero_develop_rate, :v_ero_develop_rate, :a_ero_develop_rate, :b_ero_develop_rate,
                 :oral_sperm_shot, :v_sperm_shot, :a_sperm_shot, :skin_sperm_shot,
                 :c_orgasm_count, :v_orgasm_count, :a_orgasm_count, :b_orgasm_count,
                 :this_turn_v_sperm_shot_count, :this_turn_a_sperm_shot_count, :this_turn_oral_sperm_shot_count, :this_turn_skin_sperm_shot_count,
                 :this_turn_c_orgasm_count, :this_turn_v_orgasm_count, :this_turn_a_orgasm_count, :this_turn_b_orgasm_count,
                 :get_this_turn_sperm_shot_points, :get_this_turn_sperm_shot_max_count, :get_this_turn_sperm_shot_total_count, :get_this_turn_orgasms_sum,
                 :get_this_turn_orgasms, :get_this_turn_orgasms_max_count, :clear_this_turn_orgasms_and_sperm_shot!

  def_delegators :@betrayed_servant, # 洗脳済みサーヴァントとしてのスキルはこちら
                 :betrayed_status_text,
                 :has_betrayed_penis?,
                 :toggle_betrayed_penis!,
                 :set_betrayed_status,
                 :set_betrayed_jail_text,
                 :betrayed_status,
                 :betrayed_status_text!,
                 # セリフ はじめ
                 :show_betray_servant_detail_script, # セリフ
                 :level_up_betray_servant_script, # セリフ
                 :level_up_betray_servant_in_lack_script, # セリフ
                 :level_up_betray_servant_in_over_limit_script, # セリフ
                 :toggle_betray_servant_penis_scirpt,
                 # セリフ おわり
                 :this_battle_damage_point_sum,
                 :empty_skill_for_betrayed?,
                 :skill_for_betrayed, :training_in_jail_as_betrayed,
                 :add_skill_for_betrayed!, :has_skill_for_betrayed?,
                 :in_training_in_jail_as_betrayed?,
                 :set_training_in_jail_as_betrayed!, :unset_training_in_jail_as_betrayed!,
                 :magic_power_betrayed_level_up, :gain_betrayed_damage_point!,
                 :start_battle_of_betray_servant!,
                 :set_battle_status!, :set_battle_status_with_down!,
                 :set_stalking_target_status!, :set_battle_with_all_down_status!,
                 :add_betrayed_capture_status!,
                 :set_betrayed_fire_status!,
                 :result_down_text, :result_escape_text, :result_victory_text

  HOLIC_BAD_STATUS_NAMES = {
    semen_holic: '精液中毒',
    oral_semen_holic: '飲精中毒',
    v_semen_holic: '膣内射精中毒',
    beast_sex_holic: '獣姦中毒',
    tentacle_sex_holic: '触手姦中毒',
    drug_sex_holic: '媚薬姦中毒',
    hypno_sex_holic: '催眠姦中毒',
    dirty_sex_holic: '汚辱中毒',
    gang_rape_holic: '輪姦被虐中毒',
    masochism_ecstasy_holic: '被虐快楽中毒',
    onany_holic: '自慰中毒',
    # 常習で中毒
    aphrodisiac_semen_holic: '催淫精液中毒',
    aphrodisiac_fever_holic: '媚薬発情中毒',
  }
  TEMP_BAD_STATUS_NAMES = {
    fever: '疫熱',
    paralysis: '麻痺',
  }
  PERMANENT_ERO_BAD_STATUS_NAMES = {
    c_sensitive: '陰核敏感',
    v_sensitive: '蜜壺敏感',
    a_sensitive: 'アナル弱い',
    b_sensitive: 'Ｂ敏感',
    c_sensitive_high: 'クリ過敏',
    v_sensitive_high: '蜜壺過敏',
    a_sensitive_high: '尻穴狂い',
    b_sensitive_high: '淫乳中毒',
    c_sensitive_fleshy: '肥大クリ',
    b_sensitive_fleshy: '乳頭肥大',
    uterus_sensitive: 'ポルチオ快楽開発済',
    fertilization_ecstasy: '受精快楽感覚',
    induction: 'モンスター引き寄せ',
    exhibition_desire: '露出癖',
    humiliation_desire: '恥辱願望',
    androphobia: '男性恐怖症',
    v_expanded: '蜜壺拡張済',
    a_expanded: 'アナル拡張済',
    uterus_expanded: '子宮開発済',
    customed_ovary: '卵巣改造済',
    # ここからは戦闘に支障がある系
    lewdness: '淫乱',
  }
  TEMP_ERO_BAD_STATUS_NAMES = {
    aphrodisiac: '催淫',
    aphrodisiac_fever: '媚薬発情',
    hypno: '催眠',
  }
  ERO_BAD_STATUS_NAMES = PERMANENT_ERO_BAD_STATUS_NAMES.merge(TEMP_ERO_BAD_STATUS_NAMES)
  BAD_STATUS_NAMES = HOLIC_BAD_STATUS_NAMES.merge(ERO_BAD_STATUS_NAMES).merge(TEMP_BAD_STATUS_NAMES)

  def initialize(id, params)
    @id = id
    @original_id = params[:original_id] || id
    @name = params[:name]
    @short_name = params[:short_name]
    @self_call = params[:self_call] || '私'

    @hooter_level = params[:hooter_level] || 2
    @base_hooter_level = @hooter_level

    @level = 1
    @inventory = [] # アイテム
    @teleport_pass_inventory = [] # アイテム
    @max_level = 1
    @exp_point = 0
    @treasure_money = 0 # 潜った時に手に入れたお宝
    default_life_point_limit =
      if FGO_MODE
        50
      else
        30
      end
    @life_point_limit = default_life_point_limit + (params[:life_point_bonus] || 0)

    # 属性「善」なら監禁されたら助けに行くリスト
    @friendly_adv_list = params[:friendly_adv_list] || []
    # 監禁されたら助けに行くリスト
    @best_friendly_adv_list = params[:best_friendly_adv_list] || []

    @life_point_in_turn_start = 0
    @friendly_damage_point_in_turn = 0 # 戦闘中に味方から受けたダメージ
    @this_turn_damage_point = 0
    @this_turn_trap_damage_point = 0
    @this_turn_ero_trap_damage_point = 0
    @this_turn_ero_trap_orgasm_count = 0

    @dark_power = 0 # 洗脳従属サーヴァントになったら使う
    @equip_point = 20 + (params[:equip_point_bonus] || 0) # 装備レベル
    @ero_equip_point = 0 # エロ装備レベル
    raise 'Lawful Level is invalid!!!!' if params[:lawful_level] && !(%w(lawful neutral chaotic).include?(params[:lawful_level]))
    @positiveness = params[:positiveness] || 0
    @lawful_level = params[:lawful_level] || 'neutral'
    raise 'Good Level is invalid!!!!' if params[:good_level] && !(%w(good neutral evil).include?(params[:good_level]))
    @good_level = params[:good_level] || 'neutral'
    @honor = params[:honor] || 0 # 名声
    @max_honor = @honor
    @leader_aptitude_bonus = params[:leader_aptitude_bonus] || 0
    @friendly_list = params[:friendly_list] || [] # リーダーになった時に優先的に加える仲間リスト

    raise 'Class Name is invalid!!!!' unless %w(scout fighter wizard).include?(params[:class_type])
    @class_type = params[:class_type]
    @class_name = params[:class_name]
    @race_type = params[:race_type] || ''
    @hierarchy = params[:hierarchy]
    @short_class_type = params[:short_class_type] || ''

    @dexterity = params[:dexterity] || 0 # 器用さ
    @cunning = params[:cunning] || 0 # 狡猾さ
    @wisdom = params[:wisdom] || 0 # 賢さ
    @magic = params[:magic] || 0 # 魔術知識
    @battle_bonus = params[:battle_bonus] || 0 # 戦闘ボーナス
    @betrayed_battle_bonus = 0 # 洗脳サーヴァント化されたあとのボーナス
    @lightness = params[:lightness] || 0 # 身軽さ、敏捷
    @weapon_technique = params[:weapon_technique] || 0 # 武器さばき
    @total_exp_point = 0
    @kill_score = 0
    @trap_count = 0 # 罠に引っかかった回数
    @skills = params[:skills] || [] # スキル
    @talents = params[:talents] || [] # タレント
    @skill_params_for_betrayed = params[:skill_for_betrayed] || []

    @equip_belt_in_dive = [] # 地上に帰還したらクリアされるエロ装備
    @bad_status_in_dive = [] # 地上に帰還したらクリアされる奴
    @ero_equips = [] # ひん剥かれない限り、地上に帰還してもクリアされないエロ装備
    @bad_status = []
    @bad_sex_status = []
    @bad_status_in_body = []
    @skill_personality = []
    @skill_holic = []
    @tattoos = [] # 刻印
    @captured_betrayed_servant = nil # 地下牢に捕獲されたときの洗脳サーヴァント

    @status_text = '' # 今何している？っていうテキスト
    @jail_detail_script = [] # 地下牢監禁中のセリフ
    # FGO用 何日目に来たの？ってテキスト
    @coming_day_text = ''

    @rest_turns = 0

    @is_now_slave_call = false # 奴隷宣言。trueになったらダンジョンから回収される。

    # idle 稼働可能
    # rest 休憩中
    # in_prison 捕獲中
    # out 退場済み
    @status = 'idle'
    @generated_magic_power = 0

    @jail_mode = 'default'
    @bookmarked = false

    @in_jail = false
    @adventurer_script = AdventurerScript.new

    set_personality_for_original

    @script_status = AdventurerScriptStatus.new(original_id, self)
    @sex_status = AdventurerSexStatus.new(self, @script_status.script)
    @buyer_type = nil

    @sold_identify_code = nil

    if ORIGINAL_MODE
      @talents << :gifted       if random_roll(3, 100)
      @talents << :hard_worker  if random_roll(3, 100)
      @talents << :tough_mind   if random_roll(5, 100)
      @talents << :good_growth  if random_roll(3, 100)
      @talents << :good_looking if random_roll(2, 100)

      if noble_or_more?
        success_rate = (hierarchy == 'royal') ? 30 : 7
        @talents << :gifted_of_load if random_roll(success_rate, 100)
      end

      if in_devout?
        success_rate =
          if noble_or_more?
            40
          else
            20
          end
        @talents << :grace_of_god if random_roll(success_rate, 100)
      end

      if params[:hierarchy]
        @motivation =  @script_status.generate_motivation_by(params[:hierarchy])
        @origin_from = @script_status.generate_origin_from(params[:hierarchy])
      end

      @virgin_text = @script_status.generate_virgin_text
    end

    @bounty_target = false
    initialize_in_start_adventure!

    if params[:level]
      (params[:level] - 1).times { _level_up_common }
    end

    if has_gifted_of_load?
      @battle_bonus += 50
    end
  end

  def set_skill_params_for_betrayed(skill_for_betrayed)
    @skill_params_for_betrayed = skill_for_betrayed || []
  end

  def initialize_in_start_adventure!
    @life_point = life_point_limit
    @kill_score_in_this_adventure = 0
    @honor_plus_in_this_adventure = 0
    @treasure_money = 0

    initialize_in_join_party!
  end

  # ダンジョン内で合流した場合もこれに含まれる
  def initialize_in_join_party!
    @floor_in_lonely = 0 # 捕まった、はぐれたときのフロア階層

    @now_down = false # このターンで倒れた
    @is_now_captured = false # たったいま捕まった
    @is_now_stray = false # たったいまはぐれた
    @in_lonely_adventure = false # 単独行動中
    @in_lonely_adventure_by_trap = false # 罠による単独行動中
    @is_to_merge = false # 捕まりかけたけど合流予定
    @party_identify_code = nil

    restart_adventurer_in_dungeon!
  end

  # 捕獲されたけど脱出。捕獲中の値クリア
  def restart_adventurer_in_dungeon!
    @captured_initial_count = 0 # 玄室に連れ込んだ人数
    @captured_monster = nil
    @san_level = 0 # 正気度。これがゼロになると奴隷宣言
    @san_level_max = 0
    @san_level_before_turn = 0
    @base_san_decrease_point = 0
    @result_san_decrease_point = 0
    @san_decrease_point_before_bonus_rate = 0
  end

  def set_bookmarked!
    @bookmarked = true
  end

  def reset_bookmarked!
    @bookmarked = false
  end

  def bookmarked?
    @bookmarked
  end

  BRAINWASH_HP_BONUS = 50
  BETRAYED_BATTLE_BONUS = 30
  def brainwash_slave!
    @life_point_limit += BRAINWASH_HP_BONUS
    @life_point = @life_point_limit
    @betrayed_battle_bonus = BETRAYED_BATTLE_BONUS
    @bad_status_in_dive = [] # バステ全解除
    @betrayed_servant = BetrayedServant.new(@skill_params_for_betrayed)

    (@level - 1).times do
      _level_up_betray_params!
    end
  end

  def short_status_bar
    bad_xxx_status_names = []
    DungeonSkillList::SHORT_BAD_STATUS_IN_DIVES_NAMES.each do |status, name|
      bad_xxx_status_names << name if has_bad_status_in_dive?(status)
    end

    return '' if bad_xxx_status_names.empty?

    "[#{bad_xxx_status_names.join('、')}]"
  end

  def positiveness_name_with_suffix
    "性格：#{positiveness_name}"
  end

  def positiveness_name
    case @positiveness
      when -2
        '臆病'
      when -1
        '慎重'
      when 0
        '普通'
      when 1
        '積極的'
      when 2
        '無謀'
      else
        '（不明）'
    end
  end

  def good_name
    case @good_level
      when 'good'
        '善良'
      when 'neutral'
        '中庸'
      when 'evil'
        '無責任'
      else
        '（不明）'
    end
  end

  def good_level_rescue_bonus
    case @good_level
      when 'good'
        1.3
      when 'neutral'
        1.0
      when 'evil'
        0.7
      else
        1.0
    end
  end

  def type
    raise 'なぜここを読んだし'
  end

  def leader_aptitude
    (@level + (@leader_aptitude_bonus * 8) * (@honor.to_f * 0.5)).to_i
  end

  def scout_level
    level + dexterity + cunning
  end

  def wizard_level
    level + wisdom + magic
  end

  def fighter_level
    level + weapon_technique
  end

  def fire_power
    return 0 if is_down?

    power = battle_level + (equip_point * 0.07)
    unless bad_status_in_dive.empty?
      power *= 0.6 if has_bad_status_in_dive?(:paralysis) && random_roll(50, 100)
      power *= 0.6 if has_bad_status_in_dive?(:fever) && random_roll(50, 100)
      power *= 0.6 if has_bad_status_in_dive?(:hypno) && random_roll(50, 100)
    end
    power.to_i
  end

  BORDER_LEVEL_LV05 = 05
  BORDER_LEVEL_LV10 = 10
  BORDER_LEVEL_LV20 = 20
  BORDER_LEVEL_LV30 = 30
  def battle_level
    base_value =
      case @class_type
        when 'fighter'
          battle_bonus + fighter_level + lightness + betrayed_battle_bonus
        when 'wizard'
          battle_bonus + wizard_level + lightness + betrayed_battle_bonus
        when 'scout'
          battle_bonus + scout_level + lightness + betrayed_battle_bonus
        else
          raise "class_type of id_#{@id} is invalid"
      end
    bonus_rate_by_level =
      if level < BORDER_LEVEL_LV05
        1.0
      elsif level < BORDER_LEVEL_LV10
        2.0
      elsif level < BORDER_LEVEL_LV20
        2.5
      elsif level < BORDER_LEVEL_LV30
        3.0
      else
        3.5
      end
    ((base_value + @dark_power)* bonus_rate_by_level)
  end

  def monster_adjust_by_skill(base_count)
    base_count = 1.0
    base_count -= 0.2 if talents.include?(:half_assed)
    base_count += 0.2 if talents.include?(:good_responsibility)
    base_count
  end

  def add_experience!(source_exp_point)
    fixed_exp_point =
      if wizard?
        (source_exp_point * 0.7).to_i
      else
        source_exp_point
      end

    @exp_point += fixed_exp_point
    @total_exp_point += fixed_exp_point
  end

  def add_trap_count!
    @trap_count += 1
  end

  # ================= in酒場 start =================
  def in_rest?
    rest_turns > 0
  end

  # 友好的冒険者なら何をおいても動員
  def in_rest_for_rescue?(rescue_target_id)
    in_rest? || is_best_friendly_adventurer?(rescue_target_id)
  end

  # rest_turnsが0になっても扱い的には休養中だから
  def in_rest_or_ready?
    rest_turns > 0
  end

  def decrease_rest_turns!
    @rest_turns -= 1
  end

  def become_rest!(long_rest=false)
    rest_count =
      if has_talent?(:weak) || has_talent?(:lazy)
        2
      else
        1
      end

    @rest_turns =
      if long_rest
        rest_count + 1
      else
        rest_count
      end
  end

  def has_any_equip_belt_in_dive?
    !@equip_belt_in_dive.empty?
  end

  def has_any_bad_status_in_dive?
    !@bad_status_in_dive.empty?
  end

  def clear_equip_belt_in_dive!
    @equip_belt_in_dive = []
  end

  def get_ero_equip!(ero_equip_key)
    @ero_equips = [ero_equip_key]
    @ero_equip_point = 30
  end

  def has_ero_equip?
    !@ero_equips.empty?
  end

  def clear_ero_equips!
    @ero_equips = []
    @ero_equip_point = 0
  end

  def clear_status_in_dive!
    clear_equip_belt_in_dive!
    @bad_status_in_dive = []
    add_bad_status_in_dive!(:rat) if has_tattoo?(:rat)
  end

  def get_treasure!(money)
    @treasure_money += money
  end

  MAX_LEVEL_LIMIT = 80
  LIFE_POINT_GROWTH = 5
  def level_up_if(level_up_diff)
    return nil unless (_exp_for_level_up < @exp_point)
    return nil if max_level_limit_over? # 一定値でキャップ

    @exp_point -= _exp_for_level_up
    level_up_diff += 1
    _level_up_common

    level_up_if(level_up_diff)
  end

  # event化：置き換え予定
  def level_up_script(level_up_diff)
    @adventurer_script.level_up(self, level_up_diff)
  end

  HONOR_LIMIT = 500
  def _honor_up!(honor, honor_plus_in_this_adventure, treasure_money)
    return HONOR_LIMIT if honor > HONOR_LIMIT
    honor_plus_in_this_adventure ||= 0
    honor_plus_in_this_adventure += (treasure_money / 50)

    honor +=
      if honor >= 500
        (honor_plus_in_this_adventure % 10)
      elsif honor < 100
        honor_plus_in_this_adventure
      elsif honor < 200
        if honor_plus_in_this_adventure < 50
          honor_plus_in_this_adventure
        else
          [honor_plus_in_this_adventure * 0.3, rand(44..56)].max
        end
      elsif honor < 300
        if honor_plus_in_this_adventure < 30
          honor_plus_in_this_adventure
        else
          [honor_plus_in_this_adventure * 0.2, rand(26..34)].min
        end
      elsif honor < 400
        if honor_plus_in_this_adventure < 20
          honor_plus_in_this_adventure
        else
          [honor_plus_in_this_adventure * 0.1, rand(18..22)].min
        end
      else # 400 ~ 500
        if honor_plus_in_this_adventure < 10
          honor_plus_in_this_adventure
        else
          [honor_plus_in_this_adventure * 0.05, rand(9..11)].min
        end
      end

    return HONOR_LIMIT if honor > HONOR_LIMIT
    honor.to_i
  end

  def status_up_if!
    before_honor = @honor
    @honor = _honor_up!(@honor, @honor_plus_in_this_adventure, @treasure_money)
    @max_honor = @honor if @max_honor < @honor

    honor_up_params = {
      adventurer: self,
      honor_diff: (honor - before_honor),
      now_honor: honor,
    }
    @honor_plus_in_this_adventure = 0

    score = (@kill_score_in_this_adventure * 1.3) + (@treasure_money / 100)
    score_by_class =
      if fighter?
        score * 1.2
      else
        score
      end
    up_rate = _equip_up_rate(score_by_class + @equip_point)
    equip_point_diff = (score_by_class * up_rate).to_i
    honor_bonus = ((@honor - @equip_point) / 3)

    temp_equip_point = @equip_point + equip_point_diff + honor_bonus
    if (temp_equip_point > _equip_point_cap)
      if (@equip_point <= _equip_point_cap)
        @equip_point = _equip_point_cap.to_i
      else
        # 何らかの理由で @equip_point は規定値を超えているが放置
        ;
      end
    else
      @equip_point = temp_equip_point.to_i
    end
    equip_up_params = {
      adventurer: self,
      equip_point_diff: equip_point_diff,
      now_equip_point: equip_point,
    }

    @kill_score_in_this_adventure = 0
    @treasure_money = 0
    status_up_params = {
      honor_up: honor_up_params,
      equip_up: equip_up_params,
    }
    status_up_params
  end

  EQUIP_POINT_LEVEL_CAP_1 = 10
  EQUIP_POINT_LEVEL_CAP_2 = 30
  EQUIP_POINT_CAP_RATE_1 = 15.0
  EQUIP_POINT_CAP_RATE_2 = 7
  EQUIP_POINT_CAP_RATE_3 = 3
  def _equip_point_cap
    cap =
      if level < EQUIP_POINT_LEVEL_CAP_1
        (level * EQUIP_POINT_CAP_RATE_1)
      elsif level < EQUIP_POINT_LEVEL_CAP_2
        (EQUIP_POINT_LEVEL_CAP_1 * EQUIP_POINT_CAP_RATE_1) +
          ((level - EQUIP_POINT_LEVEL_CAP_1) * EQUIP_POINT_CAP_RATE_2)
      else
        (EQUIP_POINT_LEVEL_CAP_1 * EQUIP_POINT_CAP_RATE_1) +
          ((EQUIP_POINT_LEVEL_CAP_2 - EQUIP_POINT_LEVEL_CAP_1) * EQUIP_POINT_CAP_RATE_2) +
          ((level - EQUIP_POINT_LEVEL_CAP_2) * EQUIP_POINT_CAP_RATE_3)
      end
    cap.to_i
  end

  def _equip_up_rate(after_equip_point)
    if after_equip_point > 250
      0.02
    elsif after_equip_point > 200
      0.03
    elsif after_equip_point > 160
      0.1
    elsif after_equip_point > 120
      0.3
    elsif after_equip_point > 80
      0.5
    elsif after_equip_point > 40
      0.7
    else
      0.7
    end
  end

  MAX_HONOR_PLUS_IN_THIS_TURN = 200
  def honor_plus_by_bonus!(honor_bonus)
    if (@honor_plus_in_this_adventure + honor_bonus) > MAX_HONOR_PLUS_IN_THIS_TURN
      @honor_plus_in_this_adventure = MAX_HONOR_PLUS_IN_THIS_TURN
      return
    end

    @honor_plus_in_this_adventure += honor_bonus
  end

  LEVEL_UP_RATE_LV10 = 10.0
  LEVEL_UP_RATE_LV30 = 15.0
  LEVEL_UP_RATE_LV50 = 30.0
  LEVEL_UP_RATE_LV70 = 50.0
  LEVEL_UP_RATE_MAX  = 70.0
  def _exp_for_level_up
    rate = 1.0
    rate *= 0.90 if has_talent?(:good_growth)
    rate *= 0.90 if has_talent?(:grace_of_god)
    rate *= 0.85 if has_talent?(:hard_worker)
    rate *= 0.80 if has_talent?(:gifted)

    lv_up_rate =
      if level < 10
        LEVEL_UP_RATE_LV10
      elsif level < 30
        LEVEL_UP_RATE_LV30
      elsif level < 50
        LEVEL_UP_RATE_LV50
      elsif level < 70
        LEVEL_UP_RATE_LV70
      else
        LEVEL_UP_RATE_MAX
      end
    (@level * lv_up_rate * rate).to_i
  end

  def status_short_script
    @adventurer_script.status(self)
  end
  # ================= in酒場 end =================

  # ================= 探索中 start =================
  def funatic? # 遭遇時の回避率が低下。
    talents.include?(:funatic)
  end

  # 単独行？
  def in_lonely_adventure?
    @in_lonely_adventure
  end

  # 単独行？
  def in_lonely_adventure_by_trap?
    @in_lonely_adventure_by_trap
  end

  def set_lonely_adventure!
    @in_lonely_adventure = true
  end

  def set_lonely_adventure_by_trap!
    @in_lonely_adventure = true
    @in_lonely_adventure_by_trap = true
  end
  # ================= 探索中 end =================

  # ================= 戦闘中 start =================
  def enemy_gravity_in_defense
    level + (honor * 0.5).to_i
  end

  def enemy_gravity_in_attack
    level + (honor * 0.2).to_i
  end

  def start_battle!
    @life_point_in_turn_start = life_point
    @friendly_damage_point_in_turn = 0
  end

  def fighter?
    @class_type == 'fighter'
  end

  def wizard?
    @class_type == 'wizard'
  end

  def scout?
    @class_type == 'scout'
  end
  # ================= 戦闘中 end =================

  # ================= 監禁中 start ================
  def set_jail_detail_script(script)
    @jail_detail_script = script
  end

  def clear_jail_detail_script
    @jail_detail_script = []
  end

  DEFAULT_RESIST = 50
  def capture_in_jail!
    set_in_jail!
    clear_jail_detail_script
    set_san_level_in_max

    @resist_point_max = _max_resist_point
    if has_tattoo?(:brainwashing)
      @resist_point = 0
      down_to_submit_if!
    else
      @resist_point = _max_resist_point
    end

    set_daily_resist_status_script_in_jail # 抵抗中のセリフを更新
    @resist_point_before_turn = resist_point
  end

  def _max_resist_point
    DEFAULT_RESIST + equip_point + ero_equip_point
  end

  def resist_rate
    (resist_point * 100) / _max_resist_point
  end

  MAGIC_POWER_BY_RESIST_RATE = 1.5
  def decrease_resist_in_jail!(decrease)
    max_level_bonus = (max_level > 40) ? ((max_level - 40) * 0.2) : 0
    level_bonus = (level > 20) ? ((level - 20) * 0.03) : 0
    @generated_magic_power += (
      (
        (decrease + max_level_bonus + level_bonus) *
        MAGIC_POWER_BY_RESIST_RATE
      ) + 50
    ).to_i

    if @equip_point > 0
      @equip_point -= decrease
      _clear_equip_point! if @equip_point < 0
    end

    @resist_point_before_turn = resist_point
    @resist_point -= decrease

    set_daily_resist_status_script_in_jail # 抵抗中のセリフを更新

    return unless give_up_to_resist?

    # マイナスの値にならないように
    @resist_point = 0
    # 身ぐるみ剥がされた
    _clear_equip_point!
    clear_ero_equips! # エロ装備も解除
    set_daily_status_script_in_jail # 陵辱中のセリフを更新
    @inventory = []
    @teleport_pass_inventory = []
  end

  def _clear_equip_point!
    @equip_point = 0
  end

  def is_no_equip_point?
    @equip_point.zero? && @ero_equip_point.zero?
  end

  def give_up_to_resist?
    @resist_point <= 0
  end

  def set_san_level_in_max
    @san_level = _san_level_max
    @san_level_max = @san_level
  end

  def down_to_submit_if!
    if give_up_to_resist?
      set_san_level_in_max
      @san_level_before_turn = san_level
      @base_san_decrease_point = 0
      @result_san_decrease_point = 0
      @san_decrease_point_before_bonus_rate = 0
    end
  end

  DEFAULT_SAN_LEVEL = 100
  def _san_level_max
    level = DEFAULT_SAN_LEVEL + (@level * 15) - 1
    level
  end

  # 正気値の充足度。比率
  def san_rate
    ((100 * san_level) / _san_level_max).to_i
  end

  MONSTER_COUNT_BASE_LINE = 30
  MONSTER_COUNT_LINE2 = 50
  def base_decrease_san_level(monster_count)
    value =
      if monster_count < MONSTER_COUNT_BASE_LINE
        monster_count * 0.4
      elsif monster_count < MONSTER_COUNT_LINE2
        (MONSTER_COUNT_BASE_LINE * 0.4) +
        ((monster_count - MONSTER_COUNT_BASE_LINE) * 0.2)
      else
        (MONSTER_COUNT_BASE_LINE * 0.4) +
          ((MONSTER_COUNT_LINE2 - MONSTER_COUNT_BASE_LINE) * 0.2) +
          ((monster_count - MONSTER_COUNT_LINE2) * 0.3)
      end

    @base_san_decrease_point = value
  end

  FORCE_SEX_FULL_DAY_COUNT_BORDER = 3
  FORCE_SEX_FULL_DAY_COUNT_HIGH_BORDER = 6
  FORCE_SEX_FULL_DAY_COUNT_PENALTY = 5
  FORCE_SEX_FULL_DAY_COUNT_PENALTY_RATE = 0.2
  MAGIC_POWER_BY_SAN_RATE = 4.4
  DECREASE_SAN_BY_ORGASM = 0.12
  def decrease_san_level!(monster_count, is_beast, is_tentacle, is_dirty, force_sex_full_day_count)
    clear_decrease_san!

    base_decrease_san_level(monster_count)
    decrease = base_san_decrease_point
    @base_san_decrease_point = @base_san_decrease_point.to_i

    # 1.定数プラス型が先
    if has_bad_sex_status?(:oral_semen_holic) # 飲精中毒
      decrease += _decrease_san_penalty_value('飲精中毒', (this_turn_oral_sperm_shot_count * 0.1))
    end
    if has_bad_sex_status?(:v_semen_holic) # 膣内射精中毒
      decrease += _decrease_san_penalty_value('膣内射精中毒', (this_turn_v_sperm_shot_count * 0.15))
    end
    if has_bad_sex_status?(:semen_holic) # 精液中毒
      decrease += _decrease_san_penalty_value('精液中毒', (get_this_turn_sperm_shot_total_count * 0.03))
    end
    if has_bad_sex_status?(:gang_rape_holic) # 輪姦被虐中毒
      decrease += _decrease_san_penalty_value('輪姦被虐中毒', (monster_count * 0.04))
    end

    @san_decrease_point_before_bonus_rate = decrease.to_i

    # 2.rateをかける型が後
    if has_talent?(:tough_mind) # 精神強い
      # 正気度の消耗が下がる
      decrease += _decrease_san_penalty_rate('精神強い(-50%)', decrease, -0.5)
    end
    if has_bad_sex_status?(:lewdness) # 淫乱
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('淫乱(+30%)', decrease, 0.3)
    end
    if is_beast && has_bad_sex_status?(:beast_sex_holic) # 獣姦中毒
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('獣姦中毒(+30%)', decrease, 0.3)
    end
    if is_tentacle && has_bad_sex_status?(:tentacle_sex_holic) # 触手姦中毒
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('触手姦中毒(+20%)', decrease, 0.2)
    end
    if is_dirty && has_bad_sex_status?(:dirty_sex_holic) # 汚辱中毒
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('汚辱中毒(+30%)', decrease, 0.3)
    end

    if has_bad_status_in_body?(:broken_reason) # 壊れた理性
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('壊れた理性(+50%)', decrease, 0.5)
    end
    if has_bad_status_in_body?(:breast_milk) # 噴乳体質
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('噴乳体質(+20%)', decrease, 0.2)
    end
    if has_bad_status_in_body?(:breast_milk_ecstasy) # 噴乳快楽体質
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('噴乳快楽体質(+30%)', decrease, 0.3)
    end
    if has_talent?(:saucy)
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('生意気(-20%)', decrease, -0.2)
    end
    if has_talent?(:gallant)
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('凛々しい(-20%)', decrease, -0.2)
    end
    if has_talent?(:tenacious)
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('しぶとい(-30%)', decrease, -0.3)
    end
    if has_talent?(:broken_rebellion)
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('へし折られた抵抗(+20%)', decrease, 0.2)
    end
    if has_talent?(:marked_submit)
      # 正気度の消耗が上がる
      decrease += _decrease_san_penalty_rate('隷従の烙印(+50%)', decrease, 0.5)
    end

    # 満員陵辱３日以上だと陥落が加速
    if force_sex_full_day_count >= FORCE_SEX_FULL_DAY_COUNT_BORDER
      penalty_day_count = (force_sex_full_day_count - FORCE_SEX_FULL_DAY_COUNT_BORDER + 1)
      full_day_penalty = penalty_day_count * FORCE_SEX_FULL_DAY_COUNT_PENALTY

      # 満員陵辱６日以上だと陥落がさらに加速
      if force_sex_full_day_count >= FORCE_SEX_FULL_DAY_COUNT_HIGH_BORDER
        high_penalty_day_count = (force_sex_full_day_count - FORCE_SEX_FULL_DAY_COUNT_HIGH_BORDER + 1)
        high_penalty_day_rate = FORCE_SEX_FULL_DAY_COUNT_PENALTY_RATE * high_penalty_day_count
        high_penalty_day_rate = [high_penalty_day_rate, 5.0].min
        full_day_penalty += (high_penalty_day_rate * decrease).to_i
      end

      decrease +=
        _decrease_san_penalty_value(
          "連日の大量輪姦ＬＶ#{force_sex_full_day_count}", full_day_penalty
          )
    end

    dec_by_orgasm = get_this_turn_orgasms_sum * DECREASE_SAN_BY_ORGASM
    if dec_by_orgasm.to_i > 0
      decrease +=
        _decrease_san_penalty_value(
          "輪姦陵辱されながら絶頂（#{get_this_turn_orgasms_sum}回）", dec_by_orgasm.to_i
        )
    end

    max_level_bonus = (max_level > 40) ? ((max_level - 40) * 0.3) : 0
    level_bonus = (level > 20) ? ((level - 20) * 0.05) : 0
    @generated_magic_power += (
    ((decrease + max_level_bonus + level_bonus) *
    MAGIC_POWER_BY_SAN_RATE)
    ).to_i

    _cure_life_point_by_sex!(decrease)

    set_daily_status_script_in_jail # 陵辱中のセリフを更新

    decrease_i = decrease.to_i
    @result_san_decrease_point = decrease_i
    @san_level_before_turn = san_level
    @san_level -= decrease_i

    # 一定値を下回るとへし折られた抵抗を取得
    if san_rate < 30
      if has_talent?(:saucy)
        add_talent!(:broken_saucy)
        add_talent!(:broken_rebellion)
        remove_talent!(:saucy)
      end
      if has_talent?(:gallant)
        add_talent!(:broken_gallant)
        add_talent!(:broken_rebellion)
        remove_talent!(:gallant)
      end
    end
  end

  # 洗脳されていると、陵辱されると体力が回復する
  CURE_RATE_BY_SEX = 3.0
  def _cure_life_point_by_sex!(decrease)
    if in_brain_washing?
      _cure_life_point_common(decrease * CURE_RATE_BY_SEX)
    end
  end

  def _cure_life_point_common(cure)
    @life_point += cure.to_i
    if @life_point_limit < @life_point
      @life_point = @life_point_limit
    end
  end

  # 神の恩寵があると体力回復
  def cure_by_grace_if
    return unless has_gifted_of_load?

    if has_talent?(:gifted_of_load)
      _cure_life_point_common(10)
    elsif has_talent?(:high_gifted_of_load)
      _cure_life_point_common(20)
    end
  end

  def has_gifted_of_load?
    has_talent?(:gifted_of_load) || has_talent?(:high_gifted_of_load)
  end

  # 神の恩寵があると体力回復
  def cure_by_grace_in_jail_break_if
    _cure_life_point_common(@life_point_limit / 2) if has_gifted_of_load?
  end

  # slave_servant用
  def has_more_half_of_life_point_limit
    (life_point * 2) >= life_point_limit
  end

  # slave_servant用
  BETRAYED_SERVANT_REST_RECOVER_RATE = 0.27
  def rest_in_slave_cage!
    @betrayed_servant.set_rest_status!
    _cure_life_point_common((life_point_limit * BETRAYED_SERVANT_REST_RECOVER_RATE).to_i)
  end

  def rest_with_work!
    rest_point = (life_point_limit * BETRAYED_SERVANT_REST_RECOVER_RATE * 0.5).to_i
    _cure_life_point_common(rest_point)
  end

  # slave_servant用
  def rest_by_idle_in_slave_cage!
    @betrayed_servant.set_rest_status_by_idle!
    _cure_life_point_common((life_point_limit * BETRAYED_SERVANT_REST_RECOVER_RATE).to_i)
  end

  def _decrease_san_penalty_value(status_name, value)
    value = value.to_i
    set_decrease_san(status_name, value)
    value
  end

  def _decrease_san_penalty_rate(status_name, base_value, rate)
    value = (base_value * rate).to_i
    set_decrease_san(status_name, value)
    value
  end

  def break_san_level?
    @san_level <= 0
  end

  def is_now_slave_call?
    @is_now_slave_call
  end

  MAGIC_POWER_BY_SLAVE_RATE = 23.0
  def is_now_slave_call!
    # 奴隷宣言したら魔力が回収される
    max_level_bonus = (max_level > 50) ? ((max_level - 50) * (max_level - 50) * 100) : 0
    level_bonus = (level > 30) ? ((level - 30) * (level - 30) * 3) : 0
    @generated_magic_power += (
    (_san_level_max * MAGIC_POWER_BY_SLAVE_RATE) +
      (max_level_bonus + level_bonus)
    ).to_i

    # 調教中の洗脳奴隷がいたらそいつも解放
    @captured_betrayed_servant.unset_training_in_jail_as_betrayed! if @captured_betrayed_servant

    @is_now_slave_call = true
  end

  def clear_slave_call!
    @is_now_slave_call = false
  end

  # 陵辱されたので今回の冒険の成果がパーになる
  def lost_honor!
    @equip_point = 0
    clear_ero_equips!
    @inventory = []
    @teleport_pass_inventory = []
    @kill_score_in_this_adventure =  0
    @treasure_money = 0
    @honor = 0
    @honor_plus_in_this_adventure = 0
  end

  def release_jail_mode?
    jail_mode == 'release' # 即解放
  end

  def left_jail_mode?
    jail_mode == 'left' # 手出しせず放置
  end

  def set_jail_mode(mode)
    @jail_mode = mode
  end
  # ================= 監禁中 end ================

  # ================= 罠 start ================
  def lost_equip!
    @inventory = []
    @teleport_pass_inventory = []
    @equip_point = 0
    clear_ero_equips!
  end

  def has_bad_status?(status)
    bad_status.include?(status)
  end

  def remove_bad_status!(status)
    @bad_status.delete((status)) if has_bad_status?(status)
  end

  def has_bad_sex_status?(status)
    bad_sex_status.include?(status)
  end

  # 付与できたかもtrueかfalseで返すよ
  def add_bad_sex_status!(status)
    if has_bad_sex_status?(status)
      false
    else
      @bad_sex_status << status
      true
    end
  end

  def get_talents_name(talent)
    TALENT_NAMES[talent] || ''
  end

  def get_bad_status_in_dive_name(bad_sex_status)
    BAD_STATUS_IN_DIVE_NAMES[bad_sex_status] || ''
  end

  def get_bad_sex_status_name(bad_sex_status)
    BAD_STATUS_NAMES[bad_sex_status] || ''
  end

  def get_skill_personality_name(skill_personality)
    SKILL_PERSONALITY_NAMES[skill_personality] || ''
  end

  def get_skill_holic_name(skill_holic)
    SKILL_HOLIC_NAMES[skill_holic] || ''
  end

  def get_ero_equip_name(ero_equip)
    ERO_EQUIPS_NAMES[ero_equip] || ''
  end

  def get_bad_status_in_body_name(status)
    BAD_STATUS_IN_BODY_NAMES[status] || ''
  end

  # 基本的に前にあった装備を剥がして、新たに置き換える。
  # 構造的に後から来たトラップの方が上位互換のはず。
  def add_equip_belt_in_dive!(equip)
    @equip_belt_in_dive = [equip]
  end

  # 付与できたかもtrueかfalseで返すよ
  def add_bad_status_in_dive!(status)
    if has_bad_status_in_dive?(status)
      false
    else
      @bad_status_in_dive << status
      true
    end
  end

  def has_bad_status_in_dive?(status)
    bad_status_in_dive.include?(status)
  end

  # 付与できたかもtrueかfalseで返すよ
  def add_bad_status_in_body!(status)
    if has_bad_status_in_body?(status)
      false
    else
      @bad_status_in_body << status
      true
    end
  end

  def has_bad_status_in_body?(status)
    bad_status_in_body.include?(status)
  end

  def add_skill_personality!(status)
    if has_skill_personality?(status)
      false
    else
      @skill_personality << status
      true
    end
  end

  def has_skill_personality?(status)
    skill_personality.include?(status)
  end

  def add_skill_holic!(status)
    if has_skill_holic?(status)
      false
    else
      @skill_holic << status
      true
    end
  end

  def has_skill_holic?(status)
    skill_holic.include?(status)
  end

  def add_talent!(talent)
    if has_talent?(talent)
      false
    else
      @talents << talent
      true
    end
  end

  def remove_talent!(talent)
    @talents.delete(talent)
  end

  def has_talent?(talent)
    talents.include?(talent)
  end
  # ================= 罠 end ================

  def has_skill?(skill_name)
    skills.include?(skill_name)
  end

  MAX_KILL_SCORE = 99_999
  def add_kill_score!(kill_score)
    if (@kill_score + kill_score) > MAX_KILL_SCORE
      @kill_score = MAX_KILL_SCORE
      return
    end

    @kill_score += kill_score
    @kill_score_in_this_adventure += kill_score
    @honor_plus_in_this_adventure += kill_score
  end

  def add_inventory!(item)
    return if has_item?(item)
    @inventory << item
  end

  def has_item?(item)
    @inventory.include?(item)
  end

  def add_teleport_pass_inventory!(item)
    return if has_item?(item)
    @teleport_pass_inventory << item
  end

  def has_any_teleport_pass?
    !(@teleport_pass_inventory.empty?)
  end

  def last_teleport_pass
    @teleport_pass_inventory.last
  end

  def print_status_for_debug
    puts "#{short_name}(HP: #{life_point}/#{life_point_limit}, lv: #{@level}, 名声：#{honor}, 装備：#{equip_point}, 総経験値：#{total_exp_point},  討伐数：#{kill_score})"
  end

  def get_trap_damage!(damage_point)
    before_life_point = @life_point

    @life_point -= damage_point
    @life_point = 1 if @life_point < 1
    @this_turn_trap_damage_point = @life_point - before_life_point
  end

  def get_friendly_damage!(base_damage)
    damage_point = get_damage!(base_damage)
    @friendly_damage_point_in_turn += damage_point
    damage_point
  end

  def get_damage!(base_damage)
    miss_attack_rate =
      if has_bad_status_in_dive?(:paralysis) # 麻痺
        5
      elsif has_bad_status_in_dive?(:fever) # 疫熱
        5
      elsif has_bad_status_in_dive?(:rat) # 発情
        10
      elsif has_bad_status_in_dive?(:aphrodisiac_fever) # 媚薬発情
        10
      elsif has_bad_status_in_dive?(:aphrodisiac) # 催淫
        10
      else
        30
      end
    return 0 if random_roll(miss_attack_rate, 100)

    base_damage_point = base_damage

    # 敏捷が高ければ回避
    return 0 if base_damage == 0

    if base_damage < 1
      return 0 if random_roll(75, 100) # 回避
    end

    # サイコロで数値調整
    base_damage_point *= rand(0.5 .. 1.5)

    if lightness > base_damage_point
      # 回避 or 半減
      return 0 if random_roll(50, 100) # 回避
      base_damage_point *= 0.5
    else
      missed_rate = [(lightness.to_f / base_damage_point), 0.8].min
      base_damage_point *= 0.5 if random_roll(missed_rate, 1.0) # 回避
    end

    # 装備値が高ければダメージ軽減
    damage_on_equip =
      # 装備レベルによってダメージ軽減, 最大80%までカット
      [[(base_damage_point * 0.2), 1].max, (base_damage_point - ((equip_point + ero_equip_point) * 0.3).to_f)].max

    # 直撃！
    damage_point = [damage_on_equip.to_i, 1].max

    if DEBUG_MODE && PRINT_DAMAGE_FOR_DEBUG
      puts "#{short_name}は#{damage_point}ポイントのダメージを受けた(base:#{base_damage})"
    end

    @this_turn_damage_point = damage_point
    @life_point -= damage_point
    @life_point = 0 if @life_point < 0
    if is_down? && !@now_down
      @now_down = true
    end
    damage_point
  end

  def is_down?
    life_point <= 0
  end

  def is_active?
    !is_down?
  end

  def now_down?
    @now_down
  end

  def recover_now_down!
    @now_down = false
  end

  def set_stray!(floor)
    @is_now_stray = true
    @floor_in_lonely = floor
  end

  def set_escape_dungeon!(floor)
    @floor_in_lonely = floor
  end

  def decrease_level_by_san!
    return if in_resist? # 抵抗中
    # 残っていない = 陵辱中
    level_from_san = ((@san_level - DEFAULT_SAN_LEVEL) / 10) + 1 # 切り上げ
    @level = [level_from_san, 1].max # 最低でも１は保証
  end

  def full_life_point?
    life_point_limit == life_point
  end

  def battle_virgin?
    total_exp_point == 0
  end

  def in_resist?
    # 装備値が残っているか、正気値が設定されていない
    @equip_point > 0 || @san_level == 0
  end

  def is_now_stray?
    @is_now_stray
  end

  def set_captured!(floor, initial_count, monster, betrayed_servant)
    @is_now_captured = true
    @captured_initial_count = initial_count
    @captured_monster = monster
    @floor_in_lonely = floor
    @captured_betrayed_servant = betrayed_servant
  end

  def leave_captured_betrayed_servant!
    @captured_betrayed_servant = nil
  end

  def set_to_merge!(identify_code, floor)
    @is_to_merge = true
    @party_identify_code = identify_code
    @floor_in_lonely = floor
  end

  def make_sold_identify_code!
    @sold_identify_code = make_identify_code
  end

  def to_merge?
    @is_to_merge
  end

  def is_now_captured?
    @is_now_captured
  end

  def clear_floor_in_lonely!
    @is_now_captured = false
    @captured_initial_count = 0
    @captured_monster = 0
    @is_now_stray = false
    @floor_in_lonely = 0
    @is_to_merge = false
    @party_identify_code = nil
    @in_lonely_adventure = false
    @in_lonely_adventure_by_trap = false
  end

  def print_status
    text = short_name + "（Lv#{level}"
    if is_down?
      text += '、戦闘不能'
    end
    short_status_bar_text = short_status_bar
    if short_status_bar_text != ''
      "、#{short_status_bar_text}"
    end
    text += '）'
    text
  end

  # ライフポイントを扱う必要があるのでここに
  def decrease_by_orgasm!
    if @life_point >= get_this_turn_orgasms_sum
      @life_point = 1
    else
      @life_point -= get_this_turn_orgasms_sum
    end
  end

  def jail_situation_script(by_adventurer=false)
    @adventurer_script.jail_situation_script(self, by_adventurer)
  end

  def jail_status_script
    daily_status_in_jail +
      status_detail_with_header
  end

  def daily_status_in_jail
    @adventurer_script.daily_status_in_jail(self)
  end

  def status_detail_with_header
    @adventurer_script.status_detail_with_header(self)
  end

  def passenger_status_detail_with_header
    @adventurer_script.passenger_status_detail_with_header(self)
  end

  def status_script
    if in_jail_room?
      jail_status_script
    else
      @adventurer_script.status_detail(self)
    end
  end

  def set_buyer_type(buyer_type)
    @buyer_type = buyer_type
  end

  # 売却済み？
  def in_sold?
    !(@buyer_type.nil?)
  end

  def buyer_type_name
    case buyer_type
      when 'brothel'
        '娼館'
      when 'gang'
        'ならず者'
      when 'rich'
        '富豪'
      when 'beast_man'
        '獣人'
      when 'tentacle'
        '触手苗床'
      else
        'その他'
    end
  end

  def get_bad_status!(skill)
    if :aphrodisiac == skill && random_roll(15, 100)
      get_bad_status!(:aphrodisiac_fever_holic)
    end
    @bad_status << skill unless @bad_status.include?(skill)
  end

  def get_bad_status_in_dive!(skill)
    if :aphrodisiac == skill && random_roll(15, 100)
      get_bad_status_in_dive!(:aphrodisiac_fever_holic)
    end
    @bad_status_in_dive << skill unless has_bad_status_in_dive?(skill)
  end

  # 脱走確率
  def escape_rate
    default_rate = 10 # デフォルトは10%
    bonus_by_level = [level, 30].max # レベルに応じてボーナス
    (default_rate + (bonus_by_level * 0.3)).to_i
  end

  def few_monster_escape_rate(enemies_count)
    (100 - (enemies_count * 0.8)).to_i
  end

  # 最初に捕まった三妖精用
  def be_initial_slave!(monster, floor)
    self.lost_virgin!('monster_force_sex', monster.name)
    initial_count = rand(10..30)
    self.set_captured!(floor, initial_count, monster, nil)
  end

  # 賞金かけて指名手配
  def bounty_price
    base_price = 3_000
    vanilla_price = base_price +
      level * 200 +
      max_level * 200 +
      honor * 150 +
      max_honor * 150

    if is_down?
      vanilla_price *=
        if (level > 60)
          0.25
        elsif (level > 45)
          0.3
        elsif (level > 30)
          0.4
        else
          0.5
        end
    end

    if in_lonely_adventure? # 孤立中。戦闘は可能
      vanilla_price *=
        if (level > 60)
          0.35
        elsif (level > 45)
          0.4
        elsif (level > 30)
          0.45
        else
          0.5
        end
    end

    vanilla_price.to_i
  end

  def set_bounty!
    @bounty_target = true
  end

  # 15ターンたつか、捕獲されると失効する
  def revoke_bounty!
    @bounty_target = false
  end

  def is_bounty_target?
    bounty_target
  end

  def set_in_jail!
    @in_jail = true
  end

  def release_from_jail!
    # 神の恩寵あれば回復
    cure_by_grace_in_jail_break_if

    @is_now_captured = false
    @in_jail = false
    @captured_betrayed_servant = nil
  end

  def in_jail_room?
    @in_jail
  end

  #============= 刻印系はじめ =============
  def has_tattoo?(tattoo_key)
    tattoos.include?(tattoo_key)
  end

  def brand_tattoo!(tattoo_key)
    @tattoos << tattoo_key unless has_tattoo?(tattoo_key)
  end

  def in_brain_washing?
    has_tattoo?(:brainwashing)
  end
  #============= 刻印系おわり =============

  def get_trap_damage_by_orgasm!
    @this_turn_ero_trap_damage_point = 0
    @this_turn_ero_trap_orgasm_count = get_this_turn_orgasms_sum
    damage_point = @sex_status.get_trap_damage_by_orgasm

    @this_turn_ero_trap_damage_point = damage_point
    @life_point -= damage_point
    @life_point = 1 if life_point < 1 # エロトラップでライフゼロ回避
    nil
  end

  # スライムによる陵辱の副作用
  # レベル１引いて、体力は30と最大値の半分の大きい方
  def drain_level_n_recover_by_slime!
    if @level > 1
      @level = @level - 1
      @life_point_limit -= LIFE_POINT_GROWTH
    end

      @life_point = [(life_point_limit / 2), 30].min
  end

  # 女淫魔による陵辱の副作用
  # レベル１引いて、体力は50と最大値の70%の大きい方
  def drain_level_n_recover_by_ero_tunnel!
    if @level > 1
      @level = @level - 1
      @life_point_limit -= LIFE_POINT_GROWTH
    end

    @life_point = [
      [(life_point_limit * 0.7).to_i, 50].max,
      life_point_limit
    ].min
  end

  def drain_magic_power!
    magic_power = generated_magic_power.to_i
    @generated_magic_power = 0
    magic_power
  end

  def have_generated_magic_power?
    generated_magic_power.nonzero?
  end

  # sex_statusに渡すメソッドを刻印を考慮した上で上書き
  #=======================はじめ====================
  def add_c_voltage!(voltage)
    voltage = _sensitive_bonus_if(voltage)
    voltage = _reward_bonus_if(voltage)
    @sex_status.add_c_voltage!(voltage)
  end

  def add_v_voltage!(voltage)
    voltage = _sensitive_bonus_if(voltage)
    voltage = _reward_bonus_if(voltage)
    @sex_status.add_v_voltage!(voltage)
  end

  def add_a_voltage!(voltage)
    voltage = _sensitive_bonus_if(voltage)
    voltage = _reward_bonus_if(voltage)
    voltage = (voltage * 1.5).to_i if has_tattoo?(:anal_sex)
    @sex_status.add_a_voltage!(voltage)
  end

  def add_b_voltage!(voltage)
    voltage = _sensitive_bonus_if(voltage)
    voltage = _reward_bonus_if(voltage)
    @sex_status.add_b_voltage!(voltage)
  end

  def add_v_sperm_shot!(count)
    _add_v_orgasm!(count) if has_tattoo?(:sperm_orgasm)
    @sex_status.add_v_orgasm_by_sperm_shot!(count) if has_bad_status_in_body?(:sperm_shot_orgasm)
    @sex_status.add_v_sperm_shot!(count)
  end

  def add_a_sperm_shot!(count)
    _add_a_orgasm!(count) if has_tattoo?(:sperm_orgasm)
    @sex_status.add_a_sperm_shot!(count)
  end

  def add_oral_sperm_shot!(count)
    @sex_status.add_orgasm_by_oral_sperm_shot!(count) if has_bad_status_in_body?(:oral_sperm_shot_orgasm)
    @sex_status.add_oral_sperm_shot!(count)
  end

  def _sensitive_bonus_if(voltage)
    if has_tattoo?(:sensitive)
      voltage * 2
    else
      voltage
    end
  end

  def _reward_bonus_if(voltage)
    if has_tattoo?(:reward_of_ecstasy)
      voltage * 2
    else
      voltage
    end
  end
  #======================おわり=====================

  def active_level
    if is_down?
      0
    else
      level
    end
  end

  def _level_up_betray_params!
    @dark_power += 5 # 洗脳従属サーヴァントは闇の力強化
    @lightness += 5 # 洗脳従属サーヴァントは敏捷アップ
    @equip_point += 5 # 洗脳従属サーヴァントは装備アップ
    @betrayed_battle_bonus += 1
  end

  def level_up_by_betray_servant!
    unless max_level_limit_over?
      _level_up_common
      _level_up_betray_params!
      @life_point = @life_point_limit # 全回復
    end
  end

  def _level_up_common
    @level += 1
    @max_level = @level if @max_level < @level

    if max_level_limit_over?
      @max_level = @level
      return
    end

    @life_point_limit += LIFE_POINT_GROWTH

    if scout? || (level % 2).zero?
      @lightness += 1
    end

    if wizard? || (level % 3).zero?
      @wisdom += 1
    end

    if fighter? || (level % 3).zero?
      @weapon_technique += 1
    end

    if scout?
      @dexterity += 1 if (level % 2).zero?
      @cunning += 1   if (level % 3).zero?
    end

    if wizard?
      @magic += 1 if (level % 3).zero?
    end
  end

  def max_level_limit_over?
    @level >= MAX_LEVEL_LIMIT
  end

  def set_stray_adventure!(floor)
    set_stray!(floor)
    set_lonely_adventure!
  end

  def active_spy?
    in_brain_washing? && !is_down?
  end

  HOOTER_LEVEL_MAX = 6
  def boom_hooter_level!
    unless hooter_level_max?
      @hooter_level += 1
    end
  end

  def hooter_level_max?
    @hooter_level == HOOTER_LEVEL_MAX
  end

  def has_growth_hooter?
    @hooter_level > @base_hooter_level
  end

  def base_hooter_level_name
    hooter_level_name(@base_hooter_level)
  end

  def hooter_level_name_plus
    text = hooter_level_name
    text += '☆' if has_growth_hooter?
    text
  end

  def hooter_level_short_name_plus
    text = hooter_level_short_name
    text += '☆' if has_growth_hooter?
    text
  end

  def hooter_level_short_name(level=@hooter_level)
    case level
      when 0
        '絶'
      when 1
        '貧'
      when 2
        '並'
      when 3
        '巨'
      when 4
        '爆'
      when 5
        '超'
      when 6
        '膨'
    end
  end

  def hooter_level_name(level=@hooter_level)
    case level
      when 0
        '絶壁'
      when 1
        '貧乳'
      when 2
        '並乳'
      when 3
        '巨乳'
      when 4
        '爆乳'
      when 5
        '超乳'
      when 6
        '膨乳'
    end
  end

  def hierarchy_name
    case hierarchy
      when 'royal'
        '王族'
      when 'noble'
        '貴族'
      when 'rich'
        '富裕'
      when 'middle'
        '中流'
      when 'poor'
        '貧民'
      else
        ''
    end
  end

  def noble_or_more?
    %w(royal noble).include?(hierarchy)
  end

  def sensitive_name(position)
    name = PERMANENT_ERO_BAD_STATUS_NAMES["#{position}_sensitive".to_sym]
    puts "NOT FOUND SENSITIVE! (#{position}_sensitive)" unless name
    name || ''
  end

  def boom_sensitive_position!(boom_sensitive_position)
    status = "#{boom_sensitive_position}_sensitive".to_sym
    add_bad_status_in_body!(status)
  end

  def drain_magic_power
    base_magic_power = 3_000

    base_power =
      if @max_level > 60
        (
        base_magic_power +
          @level * 700 +
          @max_level * 600
        ).to_i
      elsif @max_level > 40
        (
        base_magic_power +
          @level * 500 +
          @max_level * 400
        ).to_i
      elsif @max_level > 20
        (
        base_magic_power +
          @level * 300 +
          @max_level * 200
        ).to_i
      else
        (
        base_magic_power +
          @level * 250 +
          @max_level * 180
        ).to_i
      end

    bonus = 0
    if @max_level > 20
      max_level_bonus = @max_level - 20
      bonus += max_level_bonus * max_level_bonus * 300
    end
    if @level > 20
      level_bonus = @level - 20
      bonus += level_bonus * level_bonus * 200
    end
    (base_power + bonus).to_i
  end

  def add_friendly_adv_list!(adventurer_id)
    @friendly_adv_list << adventurer_id
  end

  def is_friendly_adventurer?(adventurer_id)
    @friendly_adv_list.include?(adventurer_id)
  end

  def add_best_friendly_adv_list!(adventurer_id)
    @best_friendly_adv_list << adventurer_id
  end

  def is_best_friendly_adventurer?(adventurer_id)
    @best_friendly_adv_list.include?(adventurer_id)
  end
end
