# coding: utf-8

require_rrk 'src/app/model/dungeon/magic_factory/tentacle_lab'
require_rrk 'src/app/helper/forwardable'

class MagicFactory
  extend Forwardable

  attr_reader :slaves,
              :maternity_room, :tentacle_lab, :slave_brothel,
              :brothel_by_type, :magic_power_drain_list

  def initialize(slaves)
    @slaves = slaves
    @maternity_room = {}
    @tentacle_lab = {}
    @slave_brothel = {}
    @slave_brothel_popular = 0

    @brothel_by_type = {}
    @magic_power_drain_list = { total: 0 } # 魔力吸収リスト

    @tentacle_lab_service = TentacleLab.new
  end

  # 新たな奴隷がやってきたよ！
  def prison_slave!(adventurer)
    puts "== alert!! その奴隷はもうすでにいます！" if @slaves[adventurer.id]

    slave = Slave.new(
      id: adventurer.id,
      original_id: adventurer.original_id,
      name: adventurer.name,
      short_name: adventurer.short_name,
      bad_status: adventurer.bad_status,
      bad_sex_status: adventurer.bad_sex_status,
      adventurer: adventurer,
    )
    @slaves[adventurer.id] = slave
  end

  def release_slave(slave_id)
    slave = @slaves[slave_id]
    slave.set_status(nil)
    slave.set_tentacle_lab_type(nil)

    @maternity_room.delete(slave_id)
    @tentacle_lab.delete(slave_id)
    _slave_brothel_delete(slave_id)
  end

  def set_maternity_room(slave_id)
    slave = @slaves[slave_id]
    slave.set_status('maternity_room')
    slave.set_tentacle_lab_type(nil)

    @maternity_room[slave_id] = slave
    @tentacle_lab.delete(slave_id)
    _slave_brothel_delete(slave_id)
  end

  def set_tentacle_lab(slave_id, type)
    slave = @slaves[slave_id]
    slave.set_status('tentacle_lab')
    slave.set_tentacle_lab_type(type)

    @maternity_room.delete(slave_id)
    @tentacle_lab[slave_id] = slave
    _slave_brothel_delete(slave_id)
  end

  def set_slave_brothel(slave_id, spring_type)
    slave = @slaves[slave_id]

    _slave_brothel_delete(slave_id) # 一旦リセット

    slave.set_status('slave_brothel')
    slave.set_tentacle_lab_type(nil)

    slave.set_sell_type!(spring_type)
    @brothel_by_type[spring_type] ||= []
    @brothel_by_type[spring_type] << slave_id

    @maternity_room.delete(slave_id)
    @tentacle_lab.delete(slave_id)
    @slave_brothel[slave_id] = slave
  end

  def add_tentacle_effect_if!
    @tentacle_lab_service.add_tentacle_effect_if!(@tentacle_lab)
  end

  BASE_BROTHEL_GUEST_COUNT = 10 # 基本の客の数
  ADD_WHORE_POPULAR_RATE = 0.2
  ADD_BROTHEL_POPULAR_RATE = 0.3
  ADD_BROTHEL_POPULAR_RATE_IN_FULL = 0.1
  def sale_spring_in_brothel!(dungeon_master)
    sales = 0 # 今日の売上
    today_guest_count = 0
    slaves_popular_bonus = (0.1 * @slave_brothel.values.map {|slave| slave.popular_in_brothel }.inject(:+)).to_i
    base_guest_count = BASE_BROTHEL_GUEST_COUNT + @slave_brothel_popular + slaves_popular_bonus
    guest_capacity = 0
    @slave_brothel.each do |_, slave|
      guest_capacity += slave.spring_capacity
    end

    if base_guest_count > guest_capacity
      # guest_count = guest_capacity
      sufficiency_rate = 1.0
    else
      guest_count = base_guest_count
      sufficiency_rate = guest_count.to_f / guest_capacity
    end

    @slave_brothel.each do |_, slave|
      guest_number = (slave.spring_capacity * sufficiency_rate).to_i

      if slave.show_spring?
        slave.set_tent_spring_type!
      end

      slave.add_spring_count!(dungeon_master, guest_number)
      slave.add_popular_in_brothel!(guest_number * ADD_WHORE_POPULAR_RATE)

      today_sales = guest_number * slave.spring_price
      slave.set_today_sales(today_sales)

      today_guest_count += guest_number
      sales += today_sales
    end
    growth_popular_rate =
      if sufficiency_rate < 1.0
        ADD_BROTHEL_POPULAR_RATE
      else
        ADD_BROTHEL_POPULAR_RATE_IN_FULL
      end
    _add_slave_brothel_popular!(today_guest_count * growth_popular_rate)

    abort_guest_count =
      if (sufficiency_rate == 1.0)
        base_guest_count - today_guest_count
      else
        0
      end

    # 人気が高ければ売上増加
    if @slave_brothel_popular > 200
      sales *= 1.25
    elsif @slave_brothel_popular > 100
      sales *= 1.1
    end
    return [sales.to_i, today_guest_count.to_i, abort_guest_count] # 売上と客数を返す
  end

  SLAVE_BROTHEL_POPULAR_LIMIT = 300
  def _add_slave_brothel_popular!(gain)
    return if @slave_brothel_popular >= SLAVE_BROTHEL_POPULAR_LIMIT

    @slave_brothel_popular += gain.to_i
    if @slave_brothel_popular > SLAVE_BROTHEL_POPULAR_LIMIT
      @slave_brothel_popular = SLAVE_BROTHEL_POPULAR_LIMIT
    end
  end

  def slave_brothel_script(sales, today_guest_count, abort_guest_count)
    sample_slave = @slave_brothel.values.sample
    prelude = sample_slave.brothel_prelude_script
    finale = sample_slave.brothel_finale_script(sales, today_guest_count, abort_guest_count)

    # 見世物売春小屋
    show_tent_ids = []
    # 見世物売春のステージ
    pole_dance_ids = []
    # 輪姦売春
    group_ids = []
    # 娼婦売春
    whore_ids = []
    # そのほか
    force_sex_ids = []
    @slave_brothel.each do |key, slave|
      if slave.in_show_tent_spring?
        # 見世物売春小屋
        show_tent_ids << key
      elsif slave.in_pole_dance_spring?
        # 見世物売春のステージ
        pole_dance_ids << key
      elsif slave.in_group_spring?
        # 輪姦売春
        group_ids << key
      elsif slave.in_whore_spring?
        # 娼婦売春
        whore_ids << key
      else
        # そのほか
        force_sex_ids << key
      end
    end
    scripts = []
    show_tent_ids.each do |key|
      slave = @slave_brothel[key]
      scripts += slave.brothel_script_by_type(slave)
    end
    pole_dance_ids.each do |key|
      slave = @slave_brothel[key]
      scripts += slave.brothel_script_by_type(slave)
    end
    group_ids.each do |key|
      slave = @slave_brothel[key]
      scripts += slave.brothel_script_by_type(slave)
    end
    whore_ids.each do |key|
      slave = @slave_brothel[key]
      scripts += slave.brothel_script_by_type(slave)
    end
    force_sex_ids.each do |key|
      slave = @slave_brothel[key]
      scripts += slave.brothel_script_by_type(slave)
    end

    # 精神崩壊した奴隷娼婦がいたら公衆便女に配置換え
    @slave_brothel.values.each do |slave|
      if slave.broken_mentally_exhaust? && !slave.flags[:mental_broken]
        # 公衆便女に配置換え
        set_slave_brothel(slave.id, 'sex_toilet')
        slave.flags[:mental_broken] = true
        scripts += slave.broken_mentally_exhaust_script(slave)
      end
    end

    prelude + scripts + finale
  end

  def generate_magic_power_by_maternity!
    @maternity_room.values.each do |slave|
      monster_count = slave.today_maternity_sex_monster_count
      slave.generate_magic_power_by_maternity!(monster_count)
    end
  end

  def decrease_resist_for_spring_by_maternity!
    @maternity_room.values.each do |slave|
      slave.dec_resist_by_maternity_room!
    end
  end

  def maternity_room_script
    maternity_slave = @maternity_room.values.sample
    script = maternity_slave.maternity_room_script(maternity_slave)
    script
  end

  def _slave_brothel_delete(slave_id)
    slave = @slaves[slave_id]
    @brothel_by_type[slave.spring_type].delete(slave_id) if slave.spring_type
    slave.clear_spring_status!
    @slave_brothel.delete(slave_id)
  end

  def spring_room_type_of(slave_id)
    slave = @slaves[slave_id]
    spring_type = slave.spring_type
    return nil if spring_type.nil? # 未配属
    room_type = room_type_by(spring_type)
    room_type
  end

  def room_type_by(spring_type)
    room_name =
      case spring_type
        when 'sex_toilet'
          'force_jail'
        when 'wall_hole_spring'
          'force_jail'
        when 'v_force_spring'
          'force_jail'
        when 'a_force_spring'
          'force_jail'
        when 'all_force_spring'
          'force_jail'
        when 'v_whore_spring'
          'brothel_room'
        when 'a_whore_spring'
          'brothel_room'
        when 'all_whore_spring'
          'brothel_large_room'
        when 'pole_dance_spring'
          'brothel_pole_dance_tent'
        when 'show_tent_spring'
          'brothel_show_tent'
        else
          puts "spring_type is INVALID!!(#{spring_type})"
          ''
      end
    room_name
  end

  def now_brothel_using_sizeof(spring_room_type)
    case spring_room_type
      when 'brothel_room'
        (brothel_by_type['v_whore_spring'] || []).size + (brothel_by_type['a_whore_spring'] || []).size
      when 'brothel_large_room'
        (brothel_by_type['all_whore_spring'] || []).size
      when 'brothel_pole_dance_tent'
        (brothel_by_type['pole_dance_spring'] || []).size
      when 'brothel_show_tent'
        (brothel_by_type['show_tent_spring'] || []).size
      when 'force_jail'
        0
      else
        puts "INVALID SPRING ROOM TYPE!!(#{spring_room_type})"
        100
    end
  end

  def rest_mentally_exhaust_of_slaves
    @slaves.each do |slave_key, slave|
      slave.rest_mentally_exhaust! unless @slave_brothel[slave_key]
    end
  end

  def collection_magic_power(dungeon_master)
    magic_power_list = {
      total: 0,
      slave_brothel: {},
      tentacle_lab: {},
      maternity_room: {},
    }
    @slaves.each do |id, slave|
      next if slave.status.nil?
      next unless slave.have_generated_magic_power?
      magic_power = slave.drain_magic_power!
      magic_power_list[slave.status.to_sym][id] = magic_power
      magic_power_list[:total] += magic_power
    end
    @magic_power_drain_list = magic_power_list

    return if no_drain_magic_power?
    dungeon_master.add_magic_power_from_magic_factory!(magic_power_list[:total].to_i)
  end

  def no_drain_magic_power?
    @magic_power_drain_list[:total].zero?
  end
end
