# coding: utf-8
require_rrk 'src/app/model/dungeon/luida_bar'
require_rrk 'src/app/model/dungeon/jail'
require_rrk 'src/app/model/dungeon/dungeon_master'
require_rrk 'src/app/service/dungeon/script_service'
require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/service/dungeon/night_hunt/night_service'
require_rrk 'src/app/util/debug_util'
require_rrk 'src/defines/script/dungeon/progress_script'
require_rrk 'src/app/helper/forwardable'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/app/helper/game_define_helper'

class GameMaster
  extend Forwardable
  include DungeonCommon
  include TrapList
  include GameDefineHelper

  attr_reader :parties, :jail, :dungeon_master
  include DebugUtil

  def_delegators :@dungeon_master,
                 :set_maternity_room, :set_tentacle_lab , :set_slave_brothel, :release_slave
  def_delegators :@bar,
                 :bounty_list, :revoke_bounties_by_adv,
                 :build_chairs_for_append_adventurers_for_original,
                 :build_chairs_for_append_adventurers_for_fgo,
                 :lost_adventurer_ids, :add_lost_adventurer_ids, :set_lost_adventurer_ids,
                 :remove_using_name
  attr_reader :long_day_mode

  def initialize
    @long_day_mode = false

    @bar = LuidaBar.new
    @jail = Jail.new
    @parties = {}
    @dungeon_master = DungeonMaster.new(@bar.slave_in_magic_factory)

    @append_adventurer_flag = true # 冒険者の追加投入可能フラグ

    @dp_service = DungeonProgressService.new(@bar, @dungeon_master)
    @script_service = ScriptService.new(adventurers)
    @night_service = NightService.new
    @progress_script = ProgressScript.new
  end

  def set_hour_glass(hour_glass)
    @dungeon_master.set_hour_glass(hour_glass)
  end

  def add_clear_bonus!
    @long_day_mode = true

    @repeat_count ||= 0
    @repeat_count += 1
    @dungeon_master.add_clear_bonus!(@repeat_count)
  end

  def adventurers
    bar_adventurers
  end

  # 冒険者の様子を見る、用
  def parties_list_by_floor
    list_by_floor = {}
    @parties.each do |key, party|
      list_by_floor[party.floor] ||= {}
      list_by_floor[party.floor][key] = party
    end
    sorted_list = Hash[*list_by_floor.sort.flatten]
    sorted_list
  end

  def jails_list_by_floor
    list_by_floor = {}
    @jail.jail_rooms.each do |key, jail_room|
      list_by_floor[jail_room.floor_number] ||= {}
      list_by_floor[jail_room.floor_number][key] = jail_room
    end
    list_by_floor
  end

  def auction_room_slaves
    @dungeon_master.auction_room_slaves
  end

  #======== dungeon_masterの魔術工房周り
  def magic_factory_slaves
    @dungeon_master.slave_in_magic_factory
  end

  def maternity_room_slaves
    @dungeon_master.maternity_room_slaves
  end

  def tentacle_lab_slaves
    @dungeon_master.tentacle_lab_slaves
  end

  def slave_brothel_slaves
    @dungeon_master.slave_brothel_slaves
  end
  #======== dungeon_masuterの魔術工房周り

  def boot_from_list!
    @bar.boot_from_list!
  end

  def before_build?
    bar_adventurers.nil? || bar_adventurers.empty?
  end

  APPEND_ADVENTURER_DAY = 30
  def append_adventurers_time?(hour_glass)
    !hour_glass.new_dawn? &&
      @append_adventurer_flag && hour_glass.day >= APPEND_ADVENTURER_DAY
  end

  def append_adventurers_time_for_original?(hour_glass)
    !hour_glass.new_dawn? &&
      ORIGINAL_MODE && ((hour_glass.day % 10) == 9) && !hour_glass.night?
  end

  def append_adventurers_time_for_fgo?(hour_glass)
    FGO_MODE &&
      ((hour_glass.day % 5) == 4) &&
      !hour_glass.new_dawn? && !hour_glass.night?
  end

  def build_game!
    @bar.test_build_chairs
  end

  def build_chairs_for_append_adventurers
    @append_adventurer_flag = false
    @bar.build_chairs_for_append_adventurers
  end

  def progress_by_turn(dungeon_master, bard, hour_glass)
    element =
      if hour_glass.new_dawn?
        progress_by_turn_in_new_dawn(dungeon_master, bard, hour_glass)
      elsif hour_glass.night?
        progress_by_turn_in_night(dungeon_master, bard, hour_glass)
      else
        progress_by_turn_in_day(dungeon_master, bard, hour_glass)
      end
    element
  end

  def progress_by_turn_in_day(dungeon_master, bard, hour_glass)
    puts "START TURN #{hour_glass.day_count} DAY" if DEVELOP_MODE

    scripts = []

    @parties = _remake_parties

    @parties.each do |_, party|
      party.cure_by_grace!
    end

    scripts += @progress_script.turn_prelude(dungeon_master, hour_glass)

    if append_adventurers_time_for_original?(hour_glass)
      scripts += @progress_script.new_adventurers_arrived
    end

    # Barでパーティー編成＆Barの様子
    new_parties = @bar.build_parties!(@jail, false)
    scripts += @progress_script.build_parties(new_parties)
    new_parties.each { |key, value| @parties[key] = value }

    # 廃止: リーダーが変更されたパーティーのキー変更処理
    # a1.ダンジョンの進捗
    scripts += _progress_by_turn_in_dungeon(hour_glass.day)

    # 廃止: パーティーの再合流処理
    # scripts += _merge_parties(hour_glass.day)

    # a3.構成員を全て失ったパーティの解散処理
    _break_blank_parties

    # a4.合流判定に成功した冒険者の収容処理
    scripts += _to_merge_adventures

    # a5.離脱した冒険者の収容処理
    scripts += _on_stray_adventures

    # a6.健在パーティーの、遭難者収容処理
    scripts += _merge_strayer_to_active_party

    # a7.孤立、遭難者同士の合流処理
    scripts += _merge_strayer_to_lonely_party

    # a8.深層守備モンスターの補充配置
    scripts += _deploy_monster_guard

    # 帰投したパーティーのchairsへの還元処理
    @parties = @bar.recycle_parties(@parties)

    # 所属不明の冒険者のリサイクル処理
    _recycle_adventurer

    # 今回倒れた冒険者の「今ターン倒れた」フラグの直し処理
    _fix_now_down_flag

    # 追跡中かつ補足できなかった洗脳済み従属奴隷の回復措置
    @dungeon_master.rest_of_stalkers!

    scripts += [' ',' ',] # 調整用に最後に空行を入れる
    bard.hear_story_in_dungeon!(scripts)
    scripts
  end

  # 今回倒れた冒険者の「今ターン倒れた」フラグの直し処理
  def _fix_now_down_flag
    adventurers.each { |_, adventurer| adventurer.recover_now_down! }
  end

  def _recycle_adventurer
    adventurer_ids = bar_adventurers.keys # 全体
    adventurer_ids -= @dungeon_master.sold_slaves.keys # 売られてない
    adventurer_ids -= bar_chairs.keys # 酒場にはいない

    # パーティーに所属 or パーティーに随伴
    party_adv_ids = @parties.values.map{ |party| (party.adventurers.values + party.passengers).map { |adv| adv.id } }.inject(:+)
    adventurer_ids -= party_adv_ids unless party_adv_ids.nil?

    adventurer_ids -=  # 牢獄にいない
      @jail.jail_rooms.keys.map { |adv_id| adv_id }

    adventurer_ids -=  # 奴隷商人のところにもいない
      auction_room_slaves.keys

    adventurer_ids -=  # 魔術工房のところにもいない
      magic_factory_slaves.keys

    adventurer_ids -=  # 洗脳サーヴァント小屋のところにもいない
      @dungeon_master.betrayed_slaves.keys

    if DEBUG_MODE
      unless adventurer_ids.empty?
        puts "!!!!!!!!=====lonely missing: #{adventurer_ids}"
        status_texts = adventurer_ids.map {|adv_id|
          adv = bar_adventurers[adv_id]; puts adv.is_now_stray?; "#{adv.short_name}:#{adv.status_text}"
        }
        puts status_texts.join(', ')
      end
    end
    adventurer_ids.each do |adv_id|
      adv = bar_adventurers[adv_id]
      bar_chairs[adv_id] = adv
      adv.become_rest!
    end
  end

  def progress_by_turn_in_new_dawn(dungeon_master, bard, hour_glass)
    # 特に何もなし
  end

  def progress_by_turn_in_night(dungeon_master, bard, hour_glass)
    puts "START TURN #{hour_glass.day_count} NIGHT" if DEVELOP_MODE

    # 事前作業
    @jail.clean_status_of_rooms!

    header_scripts = @progress_script.turn_prelude(dungeon_master, hour_glass)

    # お楽しみタイム
    jail_elements = @jail.progress!(hour_glass.day, dungeon_master.escape_penalty) # scriptのみを返す

    # b0. 地下牢からの魔力回収。jail_roomが存在するうちにやらないといけない
    drain_magic_power_from_jail = @jail.collection_magic_power(dungeon_master)

    body_scripts = []
    # b1.奴隷宣言した冒険者を売却or魔術工房行きにストック
    body_scripts += @jail.sell_slaves_if!(@dungeon_master)

    # b2.脱走した冒険者のパーティー独立処理（こっそり）
    body_scripts += _on_stray_adventures_in_escaping # ここは基本は空

    # b3.賞金首たちの削除処理
    _revoke_bounties_by_turn(hour_glass)

    # b4. 隷属サーヴァントたちへの維持コスト消費
    dungeon_master.pay_maintenance_magic_cost!

    # そのほか（売られた後の近況）

    # 宿屋への夜這い
    night_hunt_script = @night_service.hunt!(bar_chairs.values)
    bard.hear_story_in_midnight!(night_hunt_script)

    # 魔術工房
    magic_factory_scripts = @dungeon_master.magic_factory_in_night
    bard.hear_story_in_brothel!(magic_factory_scripts)

    bard.hear_story_in_jail!({
      header_scripts: header_scripts,
      jail_elements: jail_elements,
      body_scripts: body_scripts,
    })
  end

  EVENT_PARTY_MAIN_LIMIT = 1
  def _progress_by_turn_in_dungeon(day_count)
    result = {
      break_party: [], # 構成員の喪失あり
      engage_jails_journey: [], # 地下牢に遭遇
      hit_trap_journey: [], # トラップにひっかかった
      battle_journey: [], # 戦闘あり
      simple_journey: [], # 順調な進行
      inactive_parties: [], # 無効化済み
      parties_by_floors: @script_service.get_party_summaries(parties_list_by_floor), # フロアごとの地下牢の収容状況
      jails_by_floors: @script_service.get_jail_summaries(jails_list_by_floor), # フロアごとの地下牢の収容状況
    }
    event_by_party = {}

    # どのパーティにトラップを発動させるか
    shuffle_box_for_trap, shuffle_box_for_ero_trap, shuffle_box_for_pool_trap, shuffle_box_for_betray_servant, shuffle_box_for_recover_slime =
      @dp_service.shuffle_for_traps_n_betrays(@dungeon_master, parties)

    trap_result = _trap_shuffle_select(shuffle_box_for_trap, shuffle_box_for_ero_trap, shuffle_box_for_pool_trap)
    _betray_servant_shuffle_select(parties, shuffle_box_for_betray_servant)
    _recover_slime_shuffle_select(parties, shuffle_box_for_recover_slime)

    # パーティー毎の進撃
    parties.each do |party_id, party|
#      puts "party.leader.name:#{party.leader.name}"
      event_by_party[party_id] =
        @dp_service.adventure_progress_by(party, @jail, self, party_id, result, trap_result)
    end

    script = []

    # 収容状況
    if result[:jails_by_floors].size.nonzero? # 収容ゼロならパス
      script += @script_service.capture_summaries_header(day_count)
      script +=
        result[:jails_by_floors].map do |identify_code, params|
          @script_service.jail_room_status(@jail, identify_code, params[:adventurer_id], params[:text])
        end
      script += @script_service.capture_summaries_footer(day_count)
    end

    # パーティー進行状況
    if result[:parties_by_floors].size.nonzero?
      script += @script_service.party_summaries_header(day_count)
      party_status_texts =
        result[:parties_by_floors].map do |identify_code, party_params|
          short_status_text = party_params[0]
          party_status_text = party_params[1]
          @script_service.party_progress_status(@parties, identify_code, short_status_text, party_status_text)
        end

      script += party_status_texts.compact
      script += @script_service.party_summaries_footer(day_count)
    end

    result[:break_party].each { |party_id| script += @script_service.make_script_from(event_by_party[party_id]) }
    result[:engage_jails_journey].each { |party_id| script += @script_service.make_script_from(event_by_party[party_id]) }

    size_event_by_phase = (result[:break_party] + result[:engage_jails_journey]).size

    script, size_event_by_phase = _add_script_from_result(script, size_event_by_phase, result[:hit_trap_journey], event_by_party)
    script, size_event_by_phase = _add_script_from_result(script, size_event_by_phase, result[:battle_journey], event_by_party)
    script, size_event_by_phase = _add_script_from_result(script, size_event_by_phase, result[:simple_journey], event_by_party)
    script, _ = _add_script_from_result(script, size_event_by_phase, result[:inactive_parties], event_by_party)

    script
  end

  def _trap_shuffle_select(shuffle_box_for_trap, shuffle_box_for_ero_trap, shuffle_box_for_pool_trap)
    dive_parties_count = parties.values.select{|party| party.in_dungeon? }.count

    # 物理トラップはアバウト10パーティに1パーティがひっかかる。ただしフロア補正あり
    trap_candidate_count = (dive_parties_count * 0.07 * rand(0.5..2.0)).to_i
    # エロトラップはアバウト2.5パーティに1パーティがひっかかる。ただしフロア補正あり
    ero_trap_candidate_count = (dive_parties_count * 0.5 * rand(0.5..1.7)).to_i
    # プールトラップは解禁されているトラップを１つずつ割り当てる方式
    pool_trap_candidates = dungeon_master.unlocked_pool_traps

    trap_result = {} # ハマるparty_id => ハマったtrap_noの構図
    trap_candidate_count.times do
      party_id = shuffle_box_for_trap.sample
      shuffle_box_for_trap.delete(party_id)
      target_party = @parties[party_id]

      # 最深部にいたら除外
      next if target_party.in_max_floor?

      # これは落とし穴か転送陣か
      base_trap_type = [:hole, :teleporter].sample

      penalty_rate = target_party.scout_is_active? ? 1.0 : 0.7
      base_hit_rate = 20.0 + (target_party.floor / 2.0) # 基本発動確率

      max_skip_rate =
        if base_trap_type == :hole
          0.9 # 落とし穴ならスカウトがいれば避けられる
        else
          0.5 # 転送陣は別にかかってもいい
        end
      base_skip_rate = (target_party.scout_level * penalty_rate * max_skip_rate) # 回避確率
      # 落とし穴系は
      skip_rate = [base_skip_rate, (base_hit_rate * max_skip_rate)].min # 回避率は発動確率の90%が上限
      trap_hit_rate = ((base_hit_rate - skip_rate) * 100) + dungeon_master.physical_trap_bonus
      is_hit = random_roll(trap_hit_rate.to_i, 100) # 発動した？
      next unless is_hit

      # ここからさきは発動する前提
      # これは高性能トラップ？
      trap_no = dungeon_master.get_trap_or_alternative(base_trap_type)
      trap_result[party_id] = trap_no
    end

    ero_trap_candidate_count.times do
      party_id = shuffle_box_for_ero_trap.sample
      shuffle_box_for_ero_trap.delete(party_id)
      target_party = @parties[party_id]

      base_hit_rate = [(target_party.floor / 40.0) * 100.0, 100.0].max # 基本発動確率
      base_skip_rate = (target_party.trap_scout_level * 0.5) # 回避確率
      skip_rate = [base_skip_rate, (base_hit_rate * 0.5)].min # 回避率は発動確率の半減が上限
      ero_trap_hit_rate = ((base_hit_rate - skip_rate) * 100) + dungeon_master.ero_trap_bonus
      is_hit = random_roll(ero_trap_hit_rate.to_i, 100) # 発動した？
      next unless is_hit

      # ここからさきは発動する前提
      trap_list = ERO_TRAP_DEPLOY_LIST[target_party.floor]
      puts "ERO_TRAP_DEPLOY_LIST[#{target_party.floor}] IS NILL" unless trap_list
      next if trap_list.empty?
      base_trap_type = trap_list.sample
      trap_no = dungeon_master.get_trap_or_alternative(base_trap_type)
      trap_result[party_id] = trap_no
    end

    pool_trap_candidates.each do |pool_trap_no|
      party_id = shuffle_box_for_pool_trap.sample
      shuffle_box_for_pool_trap.delete(party_id)
      target_party = @parties[party_id]

      next unless target_party

      base_hit_rate = 50 + [(target_party.floor / 40.0) * 50.0, 50.0].max # 基本発動確率
      base_skip_rate = (target_party.trap_scout_level * 0.5) # 回避確率
      skip_rate = [base_skip_rate, (base_hit_rate * 0.5)].min # 回避率は発動確率の半減が上限
      ero_trap_hit_rate = ((base_hit_rate - skip_rate) * 100) + dungeon_master.ero_trap_bonus
      is_hit = random_roll(ero_trap_hit_rate.to_i, 100) # 発動した？
      next unless is_hit

      trap_result[party_id] = pool_trap_no
    end

    trap_result
  end

  def _betray_servant_shuffle_select(parties, shuffle_box_for_betray_servant)
    slave_servants = @dungeon_master.select_active_slave_servants
    return if slave_servants.size.zero?

    slave_servants.each do |slave_servant|
      if shuffle_box_for_betray_servant.empty?
        slave_servant.rest_by_idle_in_slave_cage!
        slave_servant.set_betrayed_status(:rest)
      else
        target_party_identify_code = shuffle_box_for_betray_servant.sample
        shuffle_box_for_betray_servant.delete(target_party_identify_code)
        target_party = parties[target_party_identify_code]
        target_party.set_betrayed_servant_target!(slave_servant.id)
        slave_servant.set_stalking_target_status!(target_party.leader.short_name)
        slave_servant.set_betrayed_status(:stalking)
      end
    end
  end

  def _recover_slime_shuffle_select(parties, shuffle_box_for_recover_slime)
    # まあだいたいは10パーティーに2隊はかかって欲しいね。
    target_count =
      if parties.count.zero?
        0
      elsif (@parties.count < 10)
        1
      else
        (@parties.count / 10 * 2)
      end

    target_count.times do
      target_party_identify_code = shuffle_box_for_recover_slime.sample
      next unless target_party_identify_code
      shuffle_box_for_recover_slime.delete(target_party_identify_code)
      target_party = parties[target_party_identify_code]
      target_party.set_recover_slime_target!
    end
  end

  def _add_script_from_result(script, size_event_by_phase, results_by_phase, event_by_party)
    return [script, size_event_by_phase] if results_by_phase.empty?

    size_event_by_phase = size_event_by_phase + results_by_phase.size
    if size_event_by_phase > EVENT_PARTY_MAIN_LIMIT
      results_by_phase.each { |party_id|
        events = event_by_party[party_id] # １ターン、１パーティー単位のscriptを生成
        script += @script_service.make_shorten_script_from(events)
      }
    else
      results_by_phase.each { |party_id|
        events = event_by_party[party_id]
        script += @script_service.make_script_from(events) }
    end
    return [script, size_event_by_phase]
  end

  def _to_merge_adventures
    script = []
    bar_adventurers.each do |adventurer_id, adventurer|
      if adventurer.to_merge?
        party = @parties[adventurer.party_identify_code]
        if party.nil?
          # 合流に失敗
          new_party = @bar.build_in_dungeon(adventurer, adventurer.floor_in_lonely)
          @parties[new_party.identify_code] = new_party
          adventurer.status_text += "；#{new_party.floor}Fではぐれたため、パーティー結成"
          script += @progress_script.break_party(adventurer)
        else
          if party.lonely?
            adventurer.initialize_in_join_party! # いろいろクリア
            party.add_if(adventurer)
            add_party_by_adventurer_id(adventurer.id, party)
            adventurer.status_text += "；#{party.floor}Fではぐれたあと、#{party.leader.short_name}隊と合流（１人隊）"
            script += @progress_script.rebuild_lonely_party(adventurer)
          else
            # 合流に成功
            adventurer.initialize_in_join_party! # いろいろクリア
            party.add_if(adventurer)
            add_party_by_adventurer_id(adventurer.id, party)
            adventurer.status_text += "；#{party.floor}Fではぐれたあと、#{party.leader.short_name}隊と合流"
            script += @progress_script.is_merged(adventurer)
          end
        end
        adventurer.clear_floor_in_lonely!
      end
    end
    script
  end

  def _on_stray_adventures
    script = []
    bar_adventurers.each do |adventurer_id, adventurer|
      if adventurer.is_now_stray?
        new_party = @bar.build_in_dungeon(adventurer, adventurer.floor_in_lonely)
        @parties[new_party.identify_code] = new_party
        lonely_adventure_by_trap = adventurer.in_lonely_adventure_by_trap?
        adventurer.clear_floor_in_lonely!
        if lonely_adventure_by_trap
          script += @progress_script.break_party_by_trap(adventurer)
        else
          script += @progress_script.break_party(adventurer)
        end
      end

      if adventurer.is_now_captured?
        @jail.capture!(adventurer, adventurer.floor_in_lonely)
        adventurer.clear_floor_in_lonely!
      end
    end
    script
  end

  def _on_stray_adventures_in_escaping
    adventurers = @jail.escaped_adventurers
    adventurers.each do |adventurer|
      new_party = @bar.build_in_dungeon(adventurer, adventurer.floor_in_lonely)
      @parties[new_party.identify_code] = new_party
      adventurer.clear_floor_in_lonely!
    end
    # セリフ表示はここではなし
    []
  end

  # a6.健在パーティーの、遭難者収容処理
  def _merge_strayer_to_active_party
    scripts = [' ', ' ',]
    lonely_parties = {}
    parties_in_dive = {}
    parties_in_return = {}
    @parties.values.each do |party|
      next if party.floor < 1
      if party.lonely?
        lonely_parties[party.floor] ||= []
        lonely_parties[party.floor] << party
      else
        if party.gently? # 親切なパーティーしか拾ってくれない
          if party.in_dive?
            parties_in_dive[party.floor] ||= {}
            parties_in_dive[party.floor][party.identify_code] = party
          else
            parties_in_return[party.floor] ||= {}
            parties_in_return[party.floor][party.identify_code] = party
          end
        end
      end
    end

    lonely_parties.each do |floor, parties|
      parties.each do |party|
        # 先に撤退中のパーティーから
        if parties_in_return[floor].nil?
          # 同じフロアに収容可能なパーティーがいない
          next if parties_in_dive[floor].nil?

          rescue_party = parties_in_dive[floor].values.sample
          rescue_in_dive = true
        else
          rescue_party = parties_in_return[floor].values.sample
          rescue_in_dive = false
        end

        # 合流判定成功？
        next if rescue_party.nil? || party.leader.nil?
        if party.leader.is_bounty_target? && random_roll(90, 100)
          # 賞金首なんぞと組みたがる奴はいない
          next
        elsif random_roll(65, 100)
          next
        end

        # 戦闘可能かつ空きがあればadventurerに。なければpassengerに
        rescue_party.add_if(party.leader)
        # 合流したよセリフ
        scripts += @progress_script.merge_strayer_to_active_party(rescue_party.leader, party.leader, floor)

        # 空になったパーティーは解散
        @parties.delete(party.identify_code)
        # 合流済みパーティーも多重合流を防ぐため削除
        if rescue_in_dive
          parties_in_dive[floor].delete(rescue_party.identify_code)
        else
          parties_in_return[floor].delete(rescue_party.identify_code)
        end
      end
    end
    scripts
  end

  # a7.孤立、遭難者同士の合流処理
  def _merge_strayer_to_lonely_party
    scripts = [' ',]

    lonely_parties = {}
    @parties.values.each do |party|
      next if party.floor < 1
      next unless party.lonely?

      lonely_parties[party.floor] ||= {}
      lonely_parties[party.floor][party.identify_code] = party
    end

    lonely_parties.each do |floor, parties|
      scripts = _merge_strayer_to_lonely_party_recursive(floor, parties, scripts)
    end

    scripts
  end

  # a8.深層への強力なモンスターの補充配置
  GUARD_DEPLOY_SUCCESS_RATE = 50
  def _deploy_monster_guard
    # 高度召喚の魔道書が必要
    return [] unless dungeon_master.has_book?(:summon_high_monster)
    # 精鋭モンスターが一定以上いると召喚は打ち止め
    return [] if @dungeon_master.has_many_guards?
    # 一定確率で精鋭部隊を配備
    return [] unless random_roll(GUARD_DEPLOY_SUCCESS_RATE, 100)

    floor = @dungeon_master.add_elite_guards_in_deep!
    @progress_script.summon_urk_script(floor)
  end

  def _select_target_party(parties_box, base_party)
    (parties_box - [base_party]).reject {|party| party.has_passenger? }.sample
  end

  def _select_lonely_target_party(parties_box, base_party)
    base_parties_box = (parties_box - [base_party])

    shuffled_parties_box = (base_parties_box.select {|party| party.is_lonely? && !party.leader.is_bounty_target? }) * 3
    shuffled_parties_box += base_parties_box.select {|party| party.is_lonely? && party.leader.is_bounty_target? }

    shuffled_parties_box.sample
  end

  def _merge_strayer_to_lonely_party_recursive(floor, parties, scripts)
    return scripts if parties.size < 2 # 合流可能なパーティーがいない
    parties_box = parties.values
    base_parties = parties_box.select {|party| party.has_passenger? } # 同行者つきを優先
    base_party =
      if base_parties.empty?
        parties_box.sample
      else
        base_parties.sample
      end

    if random_roll(70, 100) # 合流成功
      target_party = _select_lonely_target_party(parties_box, base_party) || _select_target_party(parties_box, base_party)

      return scripts unless target_party

      # メンバー加入。ダウンしていても配置を許す
      allow_down = true
      base_party.add_if(target_party.leader, allow_down)
      # 同行者も一緒に
      target_party.passengers.each do |passenger|
        base_party.add_if(passenger, allow_down)
      end

      scripts += @progress_script.merge_strayer_to_lonely_party(base_party.leader, target_party.leader, floor)

      # 空になったパーティーは解散
      @parties.delete(target_party.identify_code)
      # 合流済みパーティーは多重合流を防ぐため除外
      parties.delete(base_party.identify_code)
      parties.delete(target_party.identify_code)
    else
      parties.delete(base_party.identify_code)
    end

    scripts = _merge_strayer_to_lonely_party_recursive(floor, parties, scripts)
    scripts
  end

  def _break_blank_parties
    # パーティーの解体処理
    broken_party_ids = []
    parties.each do |party_id, party|
      # リーダーが存在しない＝選出可能なメンバーがいない＝全滅済みパーティーの解体処理
      if party.leader_is_withdrawn?
        broken_party_ids << party_id
        # 冒険者の退去
        party.adventurers.values.each do |adv|
          puts "#{party.leader.short_name}隊から冒険者#{adv.short_name}が退去" if DEVELOP_MODE
          adv.set_stray_adventure!(party.floor)
        end
        # 同行者の退去
        party.passengers.each do |adv|
          puts "#{party.leader.short_name}隊から同行者#{adv.short_name}が退去" if DEVELOP_MODE
          adv.set_stray_adventure!(party.floor)
        end
      end
      #      puts "======="
    end
    broken_party_ids.each {|id| @parties.delete(id) }
  end

  # ターン最初のパーティ再編成処理。リーダー交代とか
  def _remake_parties
    new_parties = {}
    @parties.each do |leader_id, party|
      next if party.adventurers.size.zero?
      party.select_new_leader! if party.leader_is_withdrawn?
      new_parties[party.identify_code] = party
    end
    new_parties
  end

  def set_party_for_test(parties)
    @parties = parties
  end

  def bar_chairs
    @bar.chairs
  end

  def bar_chairs_bookmarked
    result = {}
    @bar.chairs.each do |key, adventurer|
      if adventurer.bookmarked?
        result[key] = adventurer
      end
    end
    result
  end

  def bar_adventurers
    @bar.adventurers
  end

  def adventurers_in_dungeon_bookmarked
    result = {}
    adventurers_in_dungeon.each do |key, params|
      if params[:adventurer].bookmarked?
        result[key] = params
      end
    end
    result
  end

  def adventurers_in_dungeon
    list_by_floor = {}
    parties.values.each do |party|
      party.adventurers.values.each do |adv|
        list_by_floor[party.floor] ||= {}

        list_by_floor[party.floor][adv.id] = {
          adventurer: adv,
          floor: party.floor,
          leader_name: party.leader.short_name,
        }
      end

      party.passengers.each do |adv|
        list_by_floor[party.floor] ||= {}

        list_by_floor[party.floor][adv.id] = {
          adventurer: adv,
          floor: party.floor,
          leader_name: party.leader.short_name,
        }
      end
    end
    sorted_list_by_floor = Hash[*list_by_floor.sort.flatten]
    adv_list = {}
    sorted_list_by_floor.each do |_, adv_params_list|
      adv_params_list.each do |adv_id, adv_params|
        adv_list[adv_id] = adv_params
      end
    end
    adv_list
  end

  def adventurers_in_jail_bookmarked
    result = {}
    adventurers_in_jail.each do |key, params|
      if params[:adventurer].bookmarked?
        result[key] = params
      end
    end
    result
  end

  def adventurers_in_jail
    list_by_floor = {}
    jail.jail_rooms.each do |adv_id, room|
      adv = room.adventurer
      list_by_floor[room.floor] ||= {}

      monster = room.monster_party.monster
      monster_count_text = "#{room.monster_party.rest_enemies_count}#{monster.unit_name}"
      adv.set_jail_summary_text!(
        monster.short_name || monster.name,
        monster_count_text,
        room.jail_day_count
      )
      list_by_floor[room.floor][adv.id] = {
        adventurer: adv,
        floor: room.floor,
        monster_name: monster.name,
        monster_short_name: monster.short_name || monster.name,
        monster_count: monster_count_text,
        day_count: room.jail_day_count,
        force_sex_day_count: room.force_sex_day_count,
        in_resist: room.in_resist?,
        betrayed_servant: room.monster_party.betrayed_servant,
        fall_level: room.fall_level,
      }
    end
    sorted_list_by_floor = Hash[*list_by_floor.sort.flatten]
    adv_list = {}
    sorted_list_by_floor.each do |_, adv_params_list|
      adv_params_list.each do |adv_id, adv_params|
        adv_list[adv_id] = adv_params
      end
    end
    adv_list
  end

  def adventurers_in_magic_factory_bookmarked
    result = {}
    adventurers_in_magic_factory.each do |key, params|
      if params[:adventurer].bookmarked?
        result[key] = params
      end
    end
    result
  end

  def adventurers_in_magic_factory
    result = {}
    magic_factory_slaves.each do |slave_id, slave|
      result[slave_id] = {
        adventurer: slave,
      }
    end
    result
  end

  def adventurers_of_sold_bookmarked
    result = {}
    adventurers_of_sold.each do |key, params|
      if params[:adventurer].bookmarked?
        result[key] = params
      end
    end
    result
  end

  def adventurers_of_sold
    result = {}
    # 最初はオークション中の冒険者
    auction_room_slaves.each do |slave_id, params|
      result[slave_id] = {
        adventurer: params[:slave],
        buyer_name: :slave_merchant,
      }
    end
    # 次に売却済みの冒険者
    dungeon_master.sold_slaves.each do |slave_id, slave|
      result[slave_id] = {
        adventurer: slave,
        buyer_name: slave.buyer_type_name,
      }
    end
    result
  end

  def sold_slave_ids
    @dungeon_master.sold_slave_ids
  end

  def get_party_by_leader_id(adventurer_id)
    # リーダーIDからパーティーを引っ張る
    parties.values.find {|p| p.leader.id == adventurer_id }
  end

  def get_jail_room_by_id(adventurer_id)
    jail.jail_rooms[adventurer_id]
  end

  def adventurer_by_id(adventurer_id)
    bar_adventurers[adventurer_id]
  end

  def party_by_adventurer_id
    @bar.party_by_adventurer_id
  end

  def find_party_by_adventurer_id(adventurer_id)
    party = party_by_adventurer_id[adventurer_id]
    party
  end

  def add_party_by_adventurer_id(adventurer_id, party)
    party_by_adventurer_id[adventurer_id] = party
  end

  # セーブ＆ロード時
  def care_backward_compatibility!(save_version)
    # do nothing
  end

  def wallet_money
    @dungeon_master.money
  end

  def write_to_bounty_list!(adventurer_id, day_count)
    @bar.write_to_bounty_list!(adventurer_id, day_count)
  end

  def _revoke_bounties_by_turn(hour_glass)
    @bar.revoke_bounties_by_day(hour_glass.day)
    @jail.clear_now_revoke_bounty_flag_n_rtn.each do |adv_id|
      @bar.revoke_bounties_by_adv(adv_id)
    end
  end
end

=begin
master = GameMaster.new
master.boot_from_list!
master.build_game

master.parties.values.each do |party |
  puts "#{party.identify_code}, #{party.leader.short_name}"
end


master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.progress_by_turn
master.bar_adventurers.values.each do |adventurer|
  adventurer.print_status_for_debug
end

master.jail.print_jail_room
=end
