# coding: utf-8
require_rrk 'src/game_define/rrk/module/method_prefix_helper'
require_rrk 'src/app/model/define/rrk/spring_define_concern'

class SpringMenuService
  include MethodPrefixDefinesHelper
  include SpringDefineConcern

  def show_house_cage_menu(house)
    spring_slave_status = house.status_of_spring_cages(house.house_cage.keys)
    elements =
      spring_slave_status.map do |state|
        slave = house.house_cage[state['slave_key']]

        {
          'type' => 'button',
          'text' => "#{slave.short_name} (#{state['name']}、 体：#{slave.strength})",
          'action' => SELECT_SPRING_ASSIGN_SLAVE_PREFIX + slave.id.to_s,
        }
      end

    house.empty_brothel_cage_size.times do
      elements <<
        {
          'type' => 'button',
          'text' => "売春部屋(娼婦売春用) 空いています",
          'action' => 'show_empty_spring_cage_message',
        }
    end
    house.empty_force_cage_size.times do
      elements <<
        {
          'type' => 'button',
          'text' => "地下室(便器売春用) 空いています",
          'action' => 'show_empty_spring_cage_message',
        }
    end

    house_cage = house.house_cage
    wait_elements = []
    disable_elements = []
    (house_cage.keys - house.slave_keys_by_spring_type.keys).each do |slave_key|
      slave = house_cage[slave_key]
      if _accept_spring?(slave)
        wait_elements << {
          'type' => 'button',
          'text' => "#{slave.short_name} (売春可能)",
          'action' => SELECT_SPRING_ASSIGN_SLAVE_PREFIX + slave.id.to_s,
        }
      else
        disable_elements  << {
          'type' => 'button',
          'text' => "#{slave.short_name}",
          'action' => 'disable_for_spring',
        }
      end
    end

    elements = {
      'spring_cage' => elements,
      'other_members' => (wait_elements +  disable_elements),
    }
    elements
  end

  def select_spring_assign(slave, house)
    elements = []
    SPRING_TYPE_NAME.each do |action, name|
      if (_full_in_cage?(slave.id, action, house) && _accept_spring_by_type?(slave, house, action))
        elements <<
          {
            'type' => 'button',
            'text' => "#{slave.short_name} に#{name}をさせる",
            'action' => SPRING_ASSIGN_TO_PREFIX + action + '_'  + slave.id.to_s,
          }
      end
    end

    elements << {
      'type' => 'button',
      'text' => "#{slave.short_name} に売春をさせない",
      'action' => SPRING_RELEASE_FROM_ALL_PREFIX + slave.id.to_s,
    }
  end

  def _accept_spring?(slave)
    slave.lust > 400 &&
      slave.obedience > 280
  end

  # v1のみ
  def _accept_brothel_spring_v1?(slave)
    slave.lust > 400 &&
      slave.obedience > 400
  end

  def _full_in_cage?(slave_id, action, house)
    if action[0, 5] == 'force'
      !house.full_in_force_cage?(slave_id)
    else
      !house.full_in_brothel_cage?(slave_id)
    end
  end

  def _accept_spring_by_type?(slave, house, type)
    case type
      when 'force_oral' then
        accept_force_oral_spring?(slave)
      when 'force_v' then
        accept_force_v_spring?(slave)
      when 'force_wall' then
        accept_force_wall_spring?(slave, house)
      when 'force_all' then
        accept_force_all_spring?(slave)
      when 'whore_oral' then
        accept_whore_oral_spring?(slave)
      when 'whore_v' then
        accept_whore_v_spring?(slave)
      when 'whore_all' then
        accept_whore_all_spring?(slave)
      when 'whore_hypno' then
        accept_whore_hypno_spring?(slave, house)
      when 'whore_outdoor' then
        accept_whore_outdoor_spring?(slave, house)
      else
        puts "#spring_type: #{type} is invalid"
        false
    end
  end

  def accept_spring?(slave)
    slave.lust > 350 &&
      slave.obedience > 300 &&
      !(slave.forbid_spring_flag)
  end

  def accept_force_oral_spring?(slave)
    accept_spring?(slave)
  end

  def accept_force_v_spring?(slave)
    slave.lust > 400 &&
      slave.obedience > 400 &&
      slave.v_experience_count > 20 &&
      (!slave.virgin?)
  end

  def accept_force_wall_spring?(slave, house)
    house.possible_to_wall_spring?(slave.id) &&
      accept_force_v_spring?(slave)
  end

  def accept_force_all_spring?(slave)
    slave.lust > 500 &&
      slave.obedience > 500 &&
      slave.v_experience_count > 50 &&
      (!slave.virgin?)
  end

  def accept_whore_oral_spring?(slave)
    slave.lust > 400 &&
      slave.obedience > 400 &&
      slave.submit > 250
  end

  def accept_whore_v_spring?(slave)
    slave.lust > 400 &&
      slave.obedience > 450 &&
      slave.v_experience_count > 20 &&
      slave.sell_spring_count >= 3 &&
      (!slave.virgin?)
  end

  def accept_whore_a_spring?(slave)
    slave.lust > 400 &&
      slave.obedience > 450 &&
      slave.a_experience_count > 45 &&
      slave.sell_spring_count >= 5
  end

  def accept_whore_all_spring?(slave)
    slave.lust > 500 &&
      slave.obedience > 500 &&
      slave.v_experience_count > 50 &&
      slave.sell_spring_count >= 7 &&
      (!slave.virgin?)
  end

  def accept_whore_hypno_spring?(slave, house)
    house.possible_to_hypno_spring?(slave.id) &&
    accept_whore_v_spring?(slave)
  end

  def accept_whore_outdoor_spring?(slave, house)
    house.possible_to_outdoor_spring?(slave.id) &&
      accept_whore_all_spring?(slave)
  end

  SPRING_STRENGTH_COST = 400
  def make_spring_report(house)
    total_sales = 0
    rest_report = []
    brothel_report = []
    force_report = []

    house.brothel_spring_cage.each do |slave_key, spring|
      slave = house.house_cage[slave_key]
      if slave.strength > 500
        guest_number = _min_guest_number(spring['spring_type']) + rand(10) + _popular_bonus(slave)
        sales = guest_number * _spring_price(slave, spring['spring_type'])
        total_sales += sales

        _add_experience_count(slave, guest_number, spring['spring_type'])

        slave.strength -= (SPRING_STRENGTH_COST + (guest_number * 7)).to_i
        popular_gain = _spring_popular(slave, guest_number, true)
        slave.add_spring_gain!(slave, popular_gain, guest_number)

        brothel_report <<
          {
            'type' => spring['spring_type'],
            'type_name' => spring['spring_type_name'],
            'slave' => slave,
            'sales' => sales,
            'guest_number' => guest_number,
            'popular' => popular_gain,
          }
      else
        slave.strength += (slave.base_strength * 0.7).to_i
        rest_report <<
          {
            'slave' => slave,
          }
      end
    end

    house.force_spring_cage.each do |slave_key, spring|
      slave = house.house_cage[slave_key]
      if slave.strength > 500
        guest_number = _min_guest_number(spring['spring_type']) + rand(8) + _popular_bonus(slave)
        sales = guest_number * _spring_price(slave, spring['spring_type'])
        total_sales += sales

        _add_experience_count(slave, guest_number, spring['spring_type'])
        slave.strength -= (SPRING_STRENGTH_COST + (guest_number * 6.5)).to_i # 肉便器売春の方が消耗が高い
        popular_gain = _spring_popular(slave, guest_number, false)
        slave.add_spring_gain!(slave, popular_gain, guest_number)

        force_report <<
          {
            'type' => spring['spring_type'],
            'type_name' => spring['spring_type_name'],
            'slave' => slave,
            'sales' => sales,
            'guest_number' => guest_number,
            'popular' => popular_gain,
          }
      else
        slave.strength += (slave.base_strength * 0.7).to_i
        rest_report <<
          {
            'slave' => slave,
          }
      end
    end

    {
      'brothel_report' => brothel_report,
      'force_report' => force_report,
      'rest_report' => rest_report,
      'total_sales' => total_sales,
    }
  end

  def _spring_popular(slave, guest_number, is_whore)
    base_rate = 0.3

    lust_bonus_rate =
      if slave.lust > 700
        0.10
      elsif slave.lust > 500
        0.03
      else
        0
      end

    technique_bonus_rate =
      if slave.technique > 700
        0.15
      elsif slave.technique > 600
        0.10
      elsif slave.technique > 500
        0.05
      else
        0
      end

    type_rate =
      if is_whore
        1.0
      else
        0.4
      end

    guest_minus_bonus =
      if guest_number > 50
        ((guest_number - 50) * (0.7)) + 14
      elsif guest_number > 30
        ((guest_number - 30) * (0.5)) + 4
      elsif guest_number > 20
              (guest_number - 20) * (0.3)
      else
        0
      end
    value = (base_rate + lust_bonus_rate + technique_bonus_rate) * type_rate * (guest_number - guest_minus_bonus)
    value += _fixed_popular_bonus(slave)
    value.to_i
  end

  def _fixed_popular_bonus(slave)
    lust_bonus =
      if slave.lust > 700
        3
      elsif slave.lust > 600
        2
      elsif slave.lust > 500
        1
      else
        0
      end

    technique_bonus =
      if slave.technique > 700
        4
      elsif slave.technique > 600
        3
      elsif slave.technique > 500
        2
      elsif slave.technique > 400
        1
      else
        0
      end
    (lust_bonus + technique_bonus)
  end

  def _add_experience_count(slave, guest_number, spring_type)
    bonus =
      case spring_type
        when 'whore_oral' then
          {
            'v' => 0.3,
            'a' => 0.3,
          }
        when 'whore_v' then
          {
            'v' => 2,
          }
        when 'whore_hypno' then
          {
            'v' => 2,
          }
        when 'whore_all' then
          {
            'v' => 3,
            'a' => 3,
          }
        when 'whore_outdoor' then
          {
            'v' => 3,
            'a' => 3,
          }
        when 'force_oral' then
          {
            'v' => 2,
            'a' => 2,
          }
        when 'force_v' then
          {
            'v' => 2.5,
          }
        when 'force_wall' then
          {
            'v' => 2.7,
          }
        when 'force_all' then
          {
            'v' => 4,
            'a' => 4,
          }
      end

    bonus.each do |point, bonus_rate|
      count = (guest_number * bonus_rate).to_i
      slave.add_count(point, count)
    end

    slave.clear_today_sex_count
    slave.today_sex_count =
      if %w(whore_oral force_oral).include?(spring_type)
        0
      else
        (bonus['v'] * guest_number).to_i
      end
    slave.add_spring_sex_count!

    slave.sell_spring_count = 1 + slave.sell_spring_count
  end

  def _popular_bonus(slave)
    spring_popular = slave.spring_popular
    bonus =
      if spring_popular > 300
        30
      elsif spring_popular > 100
        20 + ((spring_popular - 100) / 20)
      elsif spring_popular > 50
        14 + ((spring_popular - 50) / 8)
      elsif spring_popular > 30
        10 + ((spring_popular - 30) / 5)
      else
        spring_popular / 3
      end
    bonus.to_i
  end

  def _spring_price(slave, spring_type)
    base_price =
      case spring_type
        when 'force_oral' then
          50
        when 'force_v' then
          75
        when 'force_wall' then
          80
        when 'force_all' then
          60
        when 'whore_oral' then
          70
        when 'whore_v' then
          100
        when 'whore_all' then
          80
        when 'whore_hypno' then
          120
        when 'whore_outdoor' then
          90
        else
          puts "alert!! in spring_price spring_type: #{type} is invalid"
          50
      end

    popular_bonus = [slave.spring_popular * 0.2, 100].min # 指名料とか？
    (base_price + popular_bonus).to_i
  end

  def _min_guest_number(spring_type)
    case spring_type
      when 'force_oral' then
        5
      when 'force_v' then
        10
      when 'force_wall' then
        12
      when 'force_all' then
        20
      when 'whore_oral' then
        10
      when 'whore_v' then
        15
      when 'whore_all' then
        25
      when 'whore_hypno' then
        17
      when 'whore_outdoor' then
        27
      else
        puts "alert!! spring_type: #{type} is invalid"
        5
    end
  end
end
