# coding: utf-8
require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/util/debug_util.rb'
require_rrk 'src/defines/script/dungeon/jail_script'
require_rrk 'src/defines/script/dungeon/merchant_script'
require_rrk 'src/app/service/dungeon/jail/merchant_service'

class JailService
  include DungeonCommon
  include DebugUtil

  def initialize
    @jail_script = JailScript.new
    @merchant_service = MerchantService.new
    @merchant_script = MerchantScript.new
  end

  def summary_base_text(room)
    @jail_script.summary_base_text(room)
  end

  def summary_text(room, text)
    @jail_script.summary_text(room, text)
  end

  def jail_summary_border(day_count)
    @jail_script.jail_summary_border(day_count)
  end

  def jail_summary_prelude_text(jail_count)
    @jail_script.jail_summary_prelude_text(jail_count)
  end

  def set_jail_escape_common(adventurer, floor_number)
    adventurer.clear_jail_flash_flags! # 監禁一回ごとのフラグクリア
    adventurer.decrease_level_by_san!
    adventurer.set_escape_dungeon!(floor_number)
    adventurer.set_jail_detail_script( # 脱走
      @jail_script.situation_in_escape(adventurer, floor_number)
    )
  end

  def set_jail_rescued_common(adventurer, leader, is_lonely, floor_number)
    adventurer.leave_captured_betrayed_servant!
    adventurer.clear_jail_flash_flags! # 監禁一回ごとのフラグクリア
    adventurer.set_jail_detail_script(
      @jail_script.situation_in_rescued(adventurer, leader, is_lonely, floor_number)
    )
    adventurer.decrease_level_by_san!
  end

  def decrease_resist(room, escape_penalty)
    script = []
    situation_in_resist_script = [] # set_jail_detail_script に入れるべき台詞
    room.refresh_by_turn!

    # 脱走判定
    situation_in_resist_script += @jail_script.ready_to_escape(room)

    adventurer = room.adventurer
    if adventurer.release_jail_mode? # 即時解放モード
      situation_in_resist_script += @jail_script.release_n_escape_success_situation(room)
      script += @jail_script.release_n_escape_success_situation(room)
      room.escape!
      adventurer.set_jail_detail_script(
        situation_in_resist_script
      )
      return script
    elsif _escape_from_jail?(room, escape_penalty)
      room.set_failed_reason!(nil)

      if adventurer.has_tattoo?(:rape_desire)
        room.set_failed_reason!('rape_desire_tattoo')
      else
        _abort_by_bad_status_if!(room)
      end

      if room.failed_reason.nil? # 特に妨害なく脱出
        situation_in_resist_script += @jail_script.ready_to_escape_success(room)
        script += @jail_script.ready_to_escape_n_success(room)
        room.escape!

        adventurer.set_jail_detail_script(
          situation_in_resist_script
        )
        return script
      else
        situation_in_resist_script += @jail_script.abort_to_escape_by_bad_status(room)
        script += @jail_script.ready_to_escape_n_failed_by_bad_status(room)
      end
    else
      situation_in_resist_script += @jail_script.ready_to_escape_failed(room)
      script += @jail_script.ready_to_escape_n_failed(room)
    end

    # 賞金首ならモンスター召喚
    if room.adventurer.is_bounty_target?
      room.revoke_bounty!
      old_monster_count = room.monster_count
      room.summon_companions_by_bounty!
      new_monster_count = room.monster_count
      situation_in_resist_script = @jail_script.summon_by_bounty(room, old_monster_count, new_monster_count)
    end

    # 手出しはしないモード
    if adventurer.left_jail_mode?
      return _adventurer_left_jail_mode(room, script, situation_in_resist_script)
    end

    old_resist_in_jail = adventurer.resist_point
    room.decrease_resist!
    new_resist_in_jail = adventurer.resist_point

    old_monster_count = room.monster_count
    script += @jail_script.situation_in_resist_shortly(room, old_resist_in_jail, new_resist_in_jail)
    situation_in_resist_script +=
      @jail_script.situation_in_resist(room, old_resist_in_jail, new_resist_in_jail)

    room.summon_companions!
    new_monster_count = room.monster_count
    diff_monster = new_monster_count - old_monster_count

    adventurer.status_text = @jail_script.room_status_text(room)

    epilogue_script = []
    epilogue_script_shortly = []
    if room.give_up_to_resist?
      adventurer.lost_honor!
      room.add_train_exp!
      adventurer.status_text += "；#{room.day_count}日目に、陵辱開始"

      epilogue_script += @jail_script.give_up_to_resist(room)
      epilogue_script_shortly += @jail_script.give_up_to_resist_shortly(room)
    else
      epilogue_script +=
        if room.over_flow?
          @jail_script.over_flow_monster(room, new_monster_count)
        else
          @jail_script.summon_monster(room, diff_monster, new_monster_count)
        end
      epilogue_script_shortly +=
        @jail_script.summon_monster_shortly(room, diff_monster, new_monster_count)
    end

    script += epilogue_script_shortly
    situation_in_resist_script += epilogue_script
    # 監禁中かつ抵抗中なら台詞を入れる
    adventurer.set_jail_detail_script(
      situation_in_resist_script
    )

    script += [' ',]
    script
  end

  def _adventurer_left_jail_mode(room, script, situation_in_resist_script)
    script += @jail_script.situation_in_resist_n_left_shortly(room)

    epilogue_script = []
    epilogue_script_shortly = []

    old_monster_count = room.monster_count
    room.summon_companions!
    new_monster_count = room.monster_count
    diff_monster = new_monster_count - old_monster_count

    epilogue_script +=
      if room.over_flow?
        @jail_script.over_flow_monster(room, new_monster_count)
      else
        @jail_script.summon_monster(room, diff_monster, new_monster_count)
      end
    epilogue_script_shortly +=
      @jail_script.summon_monster_shortly(room, diff_monster, new_monster_count)

    script += epilogue_script_shortly
    situation_in_resist_script += epilogue_script
    # 監禁中かつ抵抗中なら台詞を入れる
    room.adventurer.set_jail_detail_script(
      situation_in_resist_script
    )

    script += [' ',]
    script
  end

  # バステによる脱出失敗の可能性の計算
  def _abort_by_bad_status_if!(room)
    adventurer = room.adventurer
    return if adventurer.bad_status_in_dive.empty?
    status = adventurer.bad_status_in_dive.find {|status| random_roll(25, 100) }
    return unless status # 脱出失敗は起きず
    room.set_failed_reason!(status.to_s)
  end

  # 脱走確率
  include DungeonCommon
  def _escape_from_jail?(room, escape_penalty)
    return true if room.empty_monster?
    rate =
      if room.few_monsters? && !room.monster_has_arrest_skill?
        room.adventurer.few_monster_escape_rate(room.monster_party.rest_enemies_count)
      else
        room.adventurer.escape_rate + escape_penalty
      end

    random_roll(rate, 100)
  end

  def progress_of_training(room)
    adventurer = room.adventurer

    if adventurer.release_jail_mode? # 即時解放モード
      script = @jail_script.release_n_escape_success_situation(room)
      room.escape!
      adventurer.set_jail_detail_script(
        script
      )
      return script
    elsif adventurer.left_jail_mode? # 手出ししないモード
      script = @jail_script.situation_in_training_n_left_shortly(room)
      adventurer.set_jail_detail_script(
        script
      )
      return script
    end

    room.add_train_exp!

    old_level = adventurer.san_level
    room.decrease_san_level!
    new_level = adventurer.san_level

    adventurer.status_text = @jail_script.room_status_text_in_train(room)

    script = @jail_script.situation_in_training(room, old_level, new_level)

    # 仲間を呼んだ！
    old_monster_count = room.monster_count
    room.summon_companions!
    new_monster_count = room.monster_count
    diff_monster = new_monster_count - old_monster_count
    script += @jail_script.summon_monster(room, diff_monster, new_monster_count) unless room.over_flow?

    # バステ取得判定
    success_skill_personality = []
    room.monster_skill_personality.each do |status, default_rate|
      base_rate = default_rate + room.force_sex_full_day_count
      rate = adventurer.has_tattoo?(:reward_of_infection) ? (base_rate * 2) : base_rate
      success_skill_personality << status if random_roll(rate, 100)
    end
    unless success_skill_personality.empty?
      skill_personality = success_skill_personality.sample
      skill_personality_result = room.adventurer.add_skill_personality!(skill_personality)
      # バステ取得script
      if skill_personality_result
        script += @jail_script.get_skill_personality(room.adventurer, skill_personality)
      end
    end

    # バステ取得判定
    success_skill_holics = []

    bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
    room.monster_skill_holic.each do |status, rate|
      success_skill_holics << status if random_roll((rate * bonus_rate), 100)
    end
    unless success_skill_holics.empty?
      skill_holic = success_skill_holics.sample
      skill_holic_result = room.adventurer.add_skill_holic!(skill_holic)
      # バステ取得script
      if skill_holic_result
        script += @jail_script.get_skill_holic(room.adventurer, skill_holic)
      end
    end

    success_bad_status_in_body = []
    room.monster_bad_status_in_body.each do |status, rate|
      success_bad_status_in_body << status if random_roll(rate, 100)
    end
    unless success_bad_status_in_body.empty?
      bad_status_in_body = success_bad_status_in_body.sample
      bad_status_in_body_result = room.adventurer.add_bad_status_in_body!(bad_status_in_body)
      # バステ取得script
      if bad_status_in_body_result
        script += @jail_script.get_skill_holic(room.adventurer, bad_status_in_body)
      end
    end

    if adventurer.in_brain_washing? && room.day_count >= 1
      # 洗脳済みの場合のみ、２日目の時点で脱走する
      script += @jail_script.escape_in_brain_washing(room)
      room.escape!
      return script
    elsif room.break_san_level?
      room.is_now_slave_call!
      room.adventurer.status_text += "；陵辱#{room.force_sex_day_count}日目に奴隷宣言"
      script += @jail_script.down_to_slave(room)
    end

    script += [' ',]
    script
  end

  def calculate_sell_info(slave)
    @merchant_service.calculate_sell_info(slave)
  end

=begin
  def _calculate_price_script(slave, price_elements)
    @merchant_service.price_script(slave, price_elements)
  end
=end
  # ======= 売却処理 start ==================
  def _fallen_rooms_elements(rooms)
    fallen_rooms = rooms.values.select { |room| room.is_now_slave_call? }
    elements = fallen_rooms.inject({}) do |result, room|
      slave = room.adventurer
      result[slave.id] = {
        slave: slave,
        sell_info: calculate_sell_info(slave),
      }
      result
    end
    elements
  end

  def sell_slaves_if!(jail, dungeon_master)
    jail_rooms = jail.jail_rooms
    elements = _fallen_rooms_elements(jail_rooms)
    return [] if elements.empty?

    elements.each do |_, value|
      dungeon_master.add_slave_to_auction_room(value[:slave], value[:sell_info])
    end

    fallen_adventurer_ids = []

    jail_rooms.each do |id, room|
      if room.is_now_slave_call?
        fallen_adventurer_ids << id
        room.clear_slave_call!
      end
    end

    scripts = @merchant_script.auction_slave_script(elements)

    fallen_adventurer_ids.each do |id|
      puts "(#{id})が売却されました" if DEBUG_MODE

      room = jail_rooms[id]
      adventurer = room.adventurer
      jail.delete_jail_room!(adventurer)
    end
    unless scripts.empty?
      scripts += [' ',]
    end
    scripts
  end
  # ======= 売却処理 end ==================
end
