# coding: utf-8

require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/defines/script/dungeon/trap_script'
require_rrk 'src/defines/dungeon/skill_list'

class TrapService
  include DungeonCommon
  include MonsterListByType
  include TrapList
  include DungeonSkillList

  def initialize
    @trap_script = TrapScript.new
  end

  def hit_traps_by_party_if(party, trap_no, game_master)
    # トラップの影響は？
    trap_effect = TRAP_DEFINE_LIST[trap_no]
    trap_params = _calculate_trap_effect(trap_no, trap_effect, party.floor, party, game_master)
    return ['hit_ero_trap', trap_params]
  end

  # トラップ追加時にはここに定義追加すること（3/3）
  def _calculate_trap_effect(trap_no, trap, floor, party, game_master)
    event = nil

    party.get_trap_sign!

    case trap_no
      when 1  # 落とし穴
        fail_point = party.lonely? ? 80 : 50
        if false && random_roll(fail_point, 100)
          victim = party.adventurers.values.sample
          event = {
            event: 'trap_event_failed',
            trap_no: trap_no,
            target: victim,
            floor: floor,
            effect: {},
          }
          return event
        end

        drop_floor_count = rand(1..2)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        adventurer = party.adventurers.values.sample
        event = {
          event: 'trap_event',
          trap_no: trap_no,
          target: adventurer,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count: drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }
        party.trap_dropped_adventurer!(adventurer, new_floor)
        adventurer.add_trap_count!
        event
      when 2  # 転移方陣
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_2_failed_script(party)
          return script
        end

        drop_floor_count = rand(1..2)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        event = {
          event: 'trap_event',
          trap_no: trap_no,
          target: party,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }
        party.drop_floor!(drop_floor_count)
        party.leader.add_trap_count!
        event
      when 3  # 落とし穴Lv2
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_1_failed_script(party, party.lonely?)
          return script
        end

        drop_floor_count = rand(2..3)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        adventurer = party.adventurers.values.sample

        event = {
          event: 'trap_event',
          trap_no:  trap_no,
          target: adventurer,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.trap_dropped_adventurer!(adventurer, new_floor)
        adventurer.add_trap_count!
        event
      when 4  # 転移方陣Lv2
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_2_failed_script(party)
          return script
        end

        drop_floor_count = rand(2..3)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        event = {
          event: 'trap_event',
          trap_no: trap_no,
          target: party,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.drop_floor!(drop_floor_count)
        party.leader.add_trap_count!
        event
      when 5  # 落とし穴Lv3
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_1_failed_script(party, party.lonely?)
          return script
        end

        drop_floor_count = rand(3..4)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end
        adventurer = party.adventurers.values.sample

        event = {
          event: 'trap_event',
          trap_no:  trap_no,
          target: adventurer,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.trap_dropped_adventurer!(adventurer, new_floor)
        adventurer.add_trap_count!
        event
      when 6  # 落とし穴Lv4
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_1_failed_script(party, party.lonely?)
          return script
        end

        drop_floor_count = rand(4..5)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end
        adventurer = party.adventurers.values.sample

        event = {
          event: 'trap_event',
          trap_no:  trap_no,
          target: adventurer,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.trap_dropped_adventurer!(adventurer, new_floor)
        adventurer.add_trap_count!
        event
      when 7  # 転移方陣Lv3
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_2_failed_script(party)
          return script
        end

        drop_floor_count = rand(3..4)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        event = {
          event: 'trap_event',
          trap_no: trap_no,
          target: party,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.drop_floor!(drop_floor_count)
        party.leader.add_trap_count!
        event
      when 7  # 転移方陣Lv3
        fail_point = party.lonely? ? 90 : 66
        if false && random_roll(fail_point, 100)
          script += @trap_script.trap_2_failed_script(party)
          return script
        end

        drop_floor_count = rand(4..5)
        new_floor = party.floor + drop_floor_count
        if new_floor > party.get_max_floor
          new_floor = party.get_max_floor
          drop_floor_count = new_floor - party.floor
        end

        event = {
          event: 'trap_event',
          trap_no: trap_no,
          target: party,
          floor: floor,
          leader: party.leader,
          is_lonely: party.is_only?,
          effect: {
            drop_count:  drop_floor_count,
            old_floor: party.floor,
            new_floor: new_floor,
          },
        }

        party.drop_floor!(drop_floor_count)
        party.leader.add_trap_count!
        event
      when 11 # スライムプール
        adventurer = party.adventurers.values.sample

        event = {
          event:    'trap_event',
          trap_no:  trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:   adventurer,
          floor:    floor,
          effect:   {},
        }
        adventurer.lost_equip!
        adventurer.add_trap_count!
        event
      when 12 # 麻痺効果つきスライムプール
        adventurer = party.adventurers.values.sample

        event = {
          event:     'trap_event',
          trap_no:   trap_no,
          leader:    party.leader,
          is_lonely: party.is_only?,
          target:    adventurer,
          floor:     floor,
          effect:    {},
        }
        adventurer.lost_equip!
        adventurer.add_trap_count!
        event
      when 13 # 媚薬効果つきスライムプール
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)

        event = {
          event:    'trap_event',
          trap_no:  trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:   adventurer,
          floor:    floor,
          effect:   {},
        }
        adventurer.lost_equip!
        adventurer.add_trap_count!
        event
      when 21 # 催淫ガス
        adventurer = party.adventurers.values.sample
        duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
        adventurer.add_bad_status_in_dive!(:aphrodisiac)

        if duplicate || party.lonely?
          adventurer.masturbation!(rand(1..3))
        end
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          effect: {
            duplicate: duplicate,
            is_party_lonely: party.lonely?,
          },
        }
        event
      when 22 # 集団催淫ガス
        if party.lonely?
          adventurer = party.adventurers.values.first
          adventurer.add_bad_status_in_dive!(:aphrodisiac)
          adventurer.masturbation!(rand(5..8))
          adventurer.add_trap_count!
        else
          party.adventurers.values.each do |adventurer|
            duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
            adventurer.add_bad_status_in_dive!(:aphrodisiac)
            masturbation_count =
              if duplicate
                5
              else
                2
              end
            adventurer.masturbation!(rand(masturbation_count..(masturbation_count * 2)))
            adventurer.add_trap_count!
          end
        end
        result = {}
        party.adventurers.values.each do |adv|
          result[adv.id] = {
            adventurer: adv,
            adv_life_point: adv.life_point,
            this_turn_orgasms: adv.get_this_turn_orgasms,
          }
        end
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          floor:   floor,
          adventurers:  result,
        }
        event
      when 23 # 広範囲催淫ガス
        if party.lonely?
          adventurer = party.adventurers.values.first
          adventurer.add_bad_status_in_dive!(:aphrodisiac)
          adventurer.masturbation!(rand(5..8))
          adventurer.add_trap_count!
        else
          party.all_members_with_passengers.each do |adventurer|
            duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
            adventurer.add_bad_status_in_dive!(:aphrodisiac)
            masturbation_count =
              if duplicate
                5
              else
                2
              end
            adventurer.masturbation!(rand(masturbation_count..(masturbation_count * 2)))
            adventurer.add_trap_count!
          end
        end
        result = {}
        party.adventurers.values.each do |adv|
          result[adv.id] = {
            adventurer: adv,
            adv_life_point: adv.life_point,
            this_turn_orgasms: adv.get_this_turn_orgasms,
          }
        end

        floor_parties = game_master.parties_list_by_floor[floor].values - [party]
        target_parties = floor_parties.sample(3)
        target_parties.each do |party|
          party.all_members_with_passengers.each do |adventurer|
            duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
            adventurer.add_bad_status_in_dive!(:aphrodisiac)
            masturbation_count =
              if duplicate
                5
              else
                2
              end
            adventurer.masturbation!(rand(masturbation_count..(masturbation_count * 2)))
            adventurer.add_trap_count!
          end
        end

        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          floor:   floor,
          adventurers:  result,
          parties: target_parties,
        }
        event
      when 24 # フロア全体催淫ガス
        if party.lonely?
          adventurer = party.adventurers.values.first
          adventurer.add_bad_status_in_dive!(:aphrodisiac)
          adventurer.masturbation!(rand(5..8))
          adventurer.add_trap_count!
        else
          party.all_members_with_passengers.each do |adventurer|
            duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
            adventurer.add_bad_status_in_dive!(:aphrodisiac)
            masturbation_count =
              if duplicate
                5
              else
                2
              end
            adventurer.masturbation!(rand(masturbation_count..(masturbation_count * 2)))
            adventurer.add_trap_count!
          end
        end
        result = {}
        party.adventurers.values.each do |adv|
          result[adv.id] = {
            adventurer: adv,
            adv_life_point: adv.life_point,
            this_turn_orgasms: adv.get_this_turn_orgasms,
          }
        end

        target_parties = game_master.parties_list_by_floor[floor].values - [party]
        target_parties.each do |party|
          party.all_members_with_passengers.each do |adventurer|
            duplicate = adventurer.has_bad_status_in_dive?(:aphrodisiac) # すでに催淫？
            adventurer.add_bad_status_in_dive!(:aphrodisiac)
            masturbation_count =
              if duplicate
                5
              else
                2
              end
            adventurer.masturbation!(rand(masturbation_count..(masturbation_count * 2)))
            adventurer.add_trap_count!
          end
        end

        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          floor:   floor,
          adventurers:  result,
          parties: target_parties,
        }
        event
      when 31 # モンスターハウス
        # 未完成
        party.leader.add_trap_count!
        party.leader.add_bad_status_in_dive!(:induction)
      when 41 # 注射針つき触手
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.add_trap_count!

        is_hooter_level_max = adventurer.hooter_level_max?
        boom_hooter_success = random_roll(trap[:boom_hooter], 100)
        if boom_hooter_success
          adventurer.boom_hooter_level!
        end

        event = {
          trap_no: trap_no,
          event:   'trap_event',
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
          boom_hooter_success: boom_hooter_success,
          is_hooter_level_max: is_hooter_level_max,
        }
        event
      when 42 # 大型の注射針つき触手
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.add_trap_count!

        is_hooter_level_max = adventurer.hooter_level_max?
        boom_hooter_success = random_roll(trap[:boom_hooter], 100)
        if boom_hooter_success
          adventurer.boom_hooter_level!
        end

        event = {
          trap_no: trap_no,
          event:   'trap_event',
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
          boom_hooter_success: boom_hooter_success,
          is_hooter_level_max: is_hooter_level_max,
        }
        event
      when 43 # 特濃薬液入り注射針つき触手
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.add_trap_count!

        is_hooter_level_max = adventurer.hooter_level_max?
        boom_hooter_success = random_roll(trap[:boom_hooter], 100)
        if boom_hooter_success
          adventurer.boom_hooter_level!
        end

        event = {
          trap_no: trap_no,
          event:   'trap_event',
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
          boom_hooter_success: boom_hooter_success,
          is_hooter_level_max: is_hooter_level_max,
        }
        event
      when 51 # 秘所愛撫触手
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_tentacle')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          trap_no: trap_no,
          event:   'trap_event',
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 52 # 秘所愛撫触手ブラシ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_tentacle_brush')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 53 # 触手肉バイブ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_tentacle_vibe')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 54 # 触手イボ付き肉バイブ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_tentacle_warts_vibe')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 61 # アナル愛撫触手
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_a_tentacle')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 62 # アナル愛撫触手ブラシ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 63 # 触手肉アナルバイブ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 64 # 触手肉アナルビーズ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 71 # クリ愛撫触手
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 72 # クリ愛撫触手ブラシ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 73 # 乳房愛撫触手
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 74 # 乳房愛撫触手ブラシ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 75 # クリ敏感改造触手オナホール
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 76 # 乳房搾乳触手ブラシ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
          life_point: adventurer.life_point,
        }
        event
      when 81 # 催眠種付けフェラ
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_hypno_sex')
        adventurer.decrease_by_orgasm!
        head_count = 1 # 召喚人数は一人
        add_breed_man_exp!(adventurer, head_count)
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 82 # 催眠種付けセックス
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        head_count = 1 # 召喚人数は一人
        add_breed_man_oral_exp!(adventurer, head_count)
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 91 # ふたなりオナホオナニー
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 92 # ふたなり搾精オナホオナニー
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 93 # 淫霊憑依レズ乱交
        target_and_result = []
        # 被害者
        party.adventurers.values.each do |adventurer|
          result = _trap_common(adventurer, trap)
          _lost_virgin_if(adventurer, result, 'trap_possession_lesbian_sex')
          adventurer.decrease_by_orgasm!
          adventurer.add_trap_count!
          target_and_result << {
            target: adventurer,
            effect:  result,
            this_turn_orgasms: adventurer.get_this_turn_orgasms,
            this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
            life_point: adventurer.life_point,
          }
        end
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          floor:   floor,
          target_and_result: target_and_result,
          effect:  result,
        }
        event
      when 94 # 淫霊憑依ふたなり乱交
        target_and_result = []
        party.adventurers.values.each do |adventurer|
          result = _trap_common(adventurer, trap)
          _lost_virgin_if(adventurer, result, 'trap_possession_penis_sex')
          adventurer.decrease_by_orgasm!
          adventurer.add_trap_count!
          target_and_result << {
            target: adventurer,
            effect:  result,
            this_turn_orgasms: adventurer.get_this_turn_orgasms,
            this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
            life_point: adventurer.life_point,
          }
        end
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target_and_result: target_and_result,
          floor:   floor,
          effect:  result,
        }
        event
      when 95 # 淫霊憑依オナニー
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 96 # 淫霊憑依公開露出オナニー
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 101 # 地雷
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
        }
        event
      when 102 # 痺れ矢
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
        }
        event
      when 103 # 機械式くすぐりトラップ
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          floor:   floor,
          effect:  result,
          leader: party.leader,
          target: adventurer,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 121 # 膨乳効果つきスライムプール
        # 被害者
        adventurer = party.adventurers.values.sample

        companion_is_brainwashed = !party.is_only? && (party.adventurers.values - [adventurer]).all?(&:in_brain_washing?)
        without_rescue = party.is_only? || companion_is_brainwashed
        result = _trap_common(adventurer, trap, without_rescue)

        is_hooter_level_max = adventurer.hooter_level_max?
        boom_hooter_success = without_rescue || random_roll(25, 100)
        if boom_hooter_success
          adventurer.boom_hooter_level!
        end

        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          party: party,
          floor:   floor,
          leader: party.leader,
          adventurer: adventurer,
          is_only: party.is_only?,
          effect:  result,
          companion_is_brainwashed: companion_is_brainwashed,
          without_rescue: without_rescue,
          boom_hooter_success: boom_hooter_success,
          is_hooter_level_max: is_hooter_level_max,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 122 # 感度上昇効果スライムプール
        # 被害者
        adventurer = party.adventurers.values.sample

        companion_is_brainwashed = !party.is_only? && (party.adventurers.values - [adventurer]).all?(&:in_brain_washing?)
        without_rescue = party.is_only? || companion_is_brainwashed
        result = _trap_common(adventurer, trap, without_rescue)

        bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
        boom_sensitive_success = without_rescue || random_roll((25 * bonus_rate), 100)
        if boom_sensitive_success
          boom_sensitive_position = %w(c v a b).sample
          boom_sensitive_success = adventurer.boom_sensitive_position!(boom_sensitive_position)
        end

        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          party: party,
          floor:   floor,
          leader: party.leader,
          adventurer: adventurer,
          is_only: party.is_only?,
          effect:  result,
          boom_sensitive_success: boom_sensitive_success,
          boom_sensitive_position: boom_sensitive_position,
          companion_is_brainwashed: companion_is_brainwashed,
          without_rescue: without_rescue,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 131 # 触手だまり
        # 被害者
        adventurer = party.adventurers.values.sample

        companion_is_brainwashed = !party.is_only? && (party.adventurers.values - [adventurer]).all?(&:in_brain_washing?)
        without_rescue = party.is_only? || companion_is_brainwashed
        result = _trap_common(adventurer, trap, without_rescue)

        bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
        if (without_rescue && random_roll((50 * bonus_rate), 100))|| random_roll((20 * bonus_rate), 100)
          tentacle_holic_success = adventurer.add_skill_holic!(:tentacle_sex_holic)
        end

        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          party: party,
          floor:   floor,
          leader: party.leader,
          adventurer: adventurer,
          is_only: party.is_only?,
          effect:  result,
          companion_is_brainwashed: companion_is_brainwashed,
          tentacle_holic_success: tentacle_holic_success,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 132 # 触手の巣
        # 被害者
        adventurer = party.adventurers.values.sample
        companion_is_brainwashed = !party.is_only? && (party.adventurers.values - [adventurer]).all?(&:in_brain_washing?)
        without_rescue = party.is_only? || companion_is_brainwashed
        result = _trap_common(adventurer, trap, without_rescue)

        bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
        if (without_rescue && random_roll((60 * bonus_rate), 100))|| random_roll((30 * bonus_rate), 100)
          tentacle_holic_success = adventurer.add_skill_holic!(:tentacle_sex_holic)
        end

        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          party: party,
          floor:   floor,
          leader: party.leader,
          adventurer: adventurer,
          effect:  result,
          is_only: party.is_only?,
          companion_is_brainwashed: companion_is_brainwashed,
          tentacle_holic_success: tentacle_holic_success,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 141 # バイブ付き貞操帯
        # 被害者
        adventurer = party.adventurers.values.sample
        adventurer.add_equip_belt_in_dive!(trap_no)
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_vibe')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 142 # 極太バイブ付き貞操帯
        # 被害者
        adventurer = party.adventurers.values.sample
        adventurer.add_equip_belt_in_dive!(trap_no)
        result = _trap_common(adventurer, trap)
        _lost_virgin_if(adventurer, result, 'trap_v_vibe')
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 143 # ローター付き貞操帯
        # 被害者
        adventurer = party.adventurers.values.sample
        adventurer.add_equip_belt_in_dive!(trap_no)
        result = _trap_common(adventurer, trap)
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: result[:orgasms],
          this_turn_sperm_shots: result[:sperm_shot_points],
        }
        event
      when 151 # 性感魔法弾
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 152 # 性感拷問機
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 161 # 拘束強制鞭打ち
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 162 # 拘束触手椅子
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
      when 163 # 拘束丸呑み触手肉袋
        # 被害者
        adventurer = party.adventurers.values.sample
        result = _trap_common(adventurer, trap)
        adventurer = result[:adventurer]
        adventurer.decrease_by_orgasm!
        adventurer.add_trap_count!
        event = {
          event:   'trap_event',
          trap_no: trap_no,
          leader: party.leader,
          is_lonely: party.is_only?,
          target:  adventurer,
          floor:   floor,
          effect:  result,
          this_turn_orgasms: adventurer.get_this_turn_orgasms,
          this_turn_sperm_shots: adventurer.get_this_turn_sperm_shot_points,
          life_point: adventurer.life_point,
        }
        event
    end
    event
  end

  def _lost_virgin_if(adventurer, result, type)
    return unless result[:lost_virgin]
    adventurer.lost_virgin!(type)
  end

  def _trap_common(adventurer, trap, without_rescue=false)
    # 精液＆絶頂全部クリア
    adventurer.clear_this_turn_orgasms_and_sperm_shot!

    result = {
      bad_status_in_dive: [],
      bad_status_in_body: [],
      exp: {},
      damage: 0,
      adventurer: adventurer,
    }

    if trap[:insert_v] && adventurer.virgin?
      result[:lost_virgin] = true
    end

    if %w(enchant_pool tentacle_pool).include?(trap[:type]) && without_rescue
      if trap[:performance_without_rescue] # 快楽付与
        trap[:performance_without_rescue].each do |key, voltage_range|
          voltage = voltage_range.to_a.sample
          case key
            when :c
              adventurer.add_c_voltage!(voltage)
            when :v
              adventurer.add_v_voltage!(voltage)
            when :a
              adventurer.add_a_voltage!(voltage)
            when :b
              adventurer.add_b_voltage!(voltage)
          end
        end
      end
    else
      if trap[:performance] # 快楽付与
        trap[:performance].each do |key, voltage_range|
          voltage = voltage_range.to_a.sample
          case key
            when :c
              adventurer.add_c_voltage!(voltage)
            when :v
              adventurer.add_v_voltage!(voltage)
            when :a
              adventurer.add_a_voltage!(voltage)
            when :b
              adventurer.add_b_voltage!(voltage)
          end
        end
      end
    end

    if trap[:sperm_shot]
      trap[:sperm_shot].each do |key, shot_count_range|
        shot_count = shot_count_range.to_a.sample
        case key
          when :v
            adventurer.add_v_sperm_shot!(shot_count)
          when :a
            adventurer.add_a_sperm_shot!(shot_count)
          when :oral
            adventurer.add_oral_sperm_shot!(shot_count)
          when :skin
            adventurer.add_skin_sperm_shot!(shot_count)
        end
      end
    end

    if trap[:exp] # 各種経験
      trap[:exp].each do |key, exp_range|
        count = exp_range.to_a.sample
        case key
          when :c
            adventurer.add_c_exp!(count)
          when :v
            adventurer.add_v_exp!(count)
          when :a
            adventurer.add_a_exp!(count)
          when :b
            adventurer.add_b_exp!(count)
          when :oral
            adventurer.add_oral_exp!(count)
            result[:exp][key] = count
          when :v_sex
            adventurer.add_v_sex_exp!(count)
            result[:exp][key] = count
          when :a_sex
            adventurer.add_a_sex_exp!(count)
            result[:exp][key] = count
        end
      end
    end

    # 使ってるっぽい
    if trap[:bad_status_in_dive] # バステ付与
      bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
      trap[:bad_status_in_dive].each do |status, rate|
        if random_roll((rate * bonus_rate), 100)
          adventurer.add_bad_status_in_dive!(status)
          bad_status_name = DungeonSkillList::BAD_STATUS_IN_DIVE_NAMES[status]

          puts "不明(#{status}) in BAD_STATUS_IN_DIVE_NAMES" unless bad_status_name

          result[:bad_status_in_dive] << (bad_status_name || "不明(#{status})")
        end
      end
    end

    # 使ってるっぽい
    if trap[:bad_status_in_body] # バステ付与
      bonus_rate = adventurer.has_tattoo?(:reward_of_infection) ? 2 : 1
      trap[:bad_status_in_body].each do |status, rate|
        if random_roll((rate * bonus_rate), 100)
          adventurer.add_bad_status_in_body!(status)
          bad_status_name = DungeonSkillList::BAD_STATUS_IN_BODY_NAMES[status]

          puts "不明(#{status}) in BAD_STATUS_IN_BODY_NAMES" unless bad_status_name

          result[:bad_status_in_body] << (bad_status_name || "不明(#{status})")
        end
      end
    end

    if trap[:damage] # ダメージ付与
      damage = trap[:damage].to_a.sample
      adventurer.get_trap_damage!(damage)
      result[:damage] = adventurer.this_turn_trap_damage_point
    end

    if trap[:ero_develop_rate] # 性感帯開発
      trap[:ero_develop_rate].any? do |key, param|
        if random_roll(param[:success], 1) && !adventurer.is_max_ero_develop_rate?(key)
          adventurer.add_ero_develop_rate!(key, param[:rate])
          true
        end
      end
    end

    # 絶頂によるダメージ付与
    adventurer.get_trap_damage_by_orgasm!

    result[:orgasms] = adventurer.get_this_turn_orgasms
    result[:sperm_shot_points] = adventurer.get_this_turn_sperm_shot_points
    adventurer.clear_this_turn_orgasms_and_sperm_shot!
    result
  end

  def _ero_equip_trap_common(adventurer, trap)
    # 精液＆絶頂全部クリア
    adventurer.clear_this_turn_orgasms_and_sperm_shot!

    result = {
      exp: {},
      damage: 0,
      adventurer: adventurer,
    }

    if trap[:performance] # 快楽付与
      trap[:performance].each do |key, voltage_range|
        voltage = voltage_range.to_a.sample
        case key
          when :c
            adventurer.add_c_voltage!(voltage)
          when :v
            adventurer.add_v_voltage!(voltage)
          when :a
            adventurer.add_a_voltage!(voltage)
          when :b
            adventurer.add_b_voltage!(voltage)
        end
      end
    end

    if trap[:sperm_shot]
      trap[:sperm_shot].each do |key, shot_count_range|
        shot_count = shot_count_range.to_a.sample
        case key
          when :v
            adventurer.add_v_sperm_shot!(shot_count)
          when :a
            adventurer.add_a_sperm_shot!(shot_count)
          when :oral
            adventurer.add_oral_sperm_shot!(shot_count)
          when :skin
            adventurer.add_skin_sperm_shot!(shot_count)
        end
      end
    end

    if trap[:exp] # 各種経験
      trap[:exp].each do |key, exp_range|
        count = exp_range.to_a.sample
        case key
          when :c
            adventurer.add_c_exp!(count)
          when :v
            adventurer.add_v_exp!(count)
          when :a
            adventurer.add_a_exp!(count)
          when :b
            adventurer.add_b_exp!(count)
          when :oral
            adventurer.add_oral_exp!(count)
            result[:exp][key] = count
          when :v_sex
            adventurer.add_v_sex_exp!(count)
            result[:exp][key] = count
          when :a_sex
            adventurer.add_a_sex_exp!(count)
            result[:exp][key] = count
        end
      end
    end

    if trap[:damage] # ダメージ付与
      damage = trap[:damage].to_a.sample
      adventurer.get_trap_damage!(damage)
      result[:damage] = adventurer.this_turn_trap_damage_point
    end

    result
  end

  # 種付けおじさんによる経験値増加
  def add_breed_man_exp!(adventurer, head_count)
    v_exp = rand((5 * head_count)..(10 * head_count))
    a_exp = rand((2 * head_count)..(5 * head_count))
    oral_exp = rand((2 * head_count)..(4 * head_count))

    v_sex_exp = v_exp
    a_sex_exp = a_exp

    v_sperm_shot = (v_sex_exp * rand(0.8..1.5)).to_i
    a_sperm_shot = (a_sex_exp * rand(0.4..0.8)).to_i
    oral_sperm_shot = (oral_exp * rand(0.3..0.6)).to_i
    skin_sperm_shot = (v_sperm_shot * rand(0.1..0.3)).to_i

    force_sex_count = ((v_exp + a_exp + oral_exp) * 0.7).to_i
    adventurer.add_men_exp!(head_count)
    adventurer.add_force_sex_exp!(force_sex_count)
    adventurer.add_v_exp!(v_exp)
    adventurer.add_a_exp!(a_exp)
    adventurer.add_oral_exp!(oral_exp)
    adventurer.add_v_sex_exp!(v_sex_exp)
    adventurer.add_a_sex_exp!(a_sex_exp)
    adventurer.add_v_sperm_shot!(v_sperm_shot)
    adventurer.add_a_sperm_shot!(a_sperm_shot)
    adventurer.add_oral_sperm_shot!(oral_sperm_shot)
    adventurer.add_skin_sperm_shot!(skin_sperm_shot)
  end

  def add_breed_man_oral_exp!(adventurer, head_count)
    oral_exp = rand((5 * head_count)..(10 * head_count))

    oral_sperm_shot = (oral_exp * rand(0.8..1.5)).to_i
    skin_sperm_shot = (oral_sperm_shot * rand(0.3..0.5)).to_i

    adventurer.add_men_exp!(head_count)
    adventurer.add_oral_exp!(oral_exp)
    adventurer.add_oral_sperm_shot!(oral_sperm_shot)
    adventurer.add_skin_sperm_shot!(skin_sperm_shot)
  end

  def ero_damage_from_recover_slime_result_if(party)
    return {} unless party.is_recover_slime_target?

    # ターゲット指定の解除
    party.unset_recover_slime_target!

    target_adventurer = party.adventurers.values.sample
    return {} unless target_adventurer
    is_success = party.is_all_down?
    unless is_success # 追い払われた
      # ターゲットにされたが標的から解除
      return {
        target: target_adventurer,
        target_life_point: target_adventurer.life_point,
        is_success: is_success,
        is_lonely: party.is_only?,
        this_turn_orgasms: target_adventurer.get_this_turn_orgasms,
      }
    end

    c_exp = rand(20..70)
    v_exp = rand(20..50)
    a_exp = rand(15..30)
    b_exp = rand(20..30)
    oral_exp = rand(20..30)

    v_sex_exp = v_exp
    a_sex_exp = a_exp

    c_voltage = c_exp * rand(5..20)
    v_voltage = v_exp * rand(10..20)
    a_voltage = a_exp * rand(10..20)
    b_voltage = b_exp * rand(5..20)

    v_sperm_shot = (v_sex_exp * rand(0.8..1.5)).to_i
    a_sperm_shot = (a_sex_exp * rand(0.4..0.8)).to_i
    oral_sperm_shot = (oral_exp * rand(0.3..0.6)).to_i
    skin_sperm_shot = (v_sperm_shot * rand(0.1..0.3)).to_i

    target_adventurer.add_c_voltage!(c_voltage)
    target_adventurer.add_v_voltage!(v_voltage)
    target_adventurer.add_a_voltage!(a_voltage)
    target_adventurer.add_b_voltage!(b_voltage)

    target_adventurer.add_c_exp!(c_exp)
    target_adventurer.add_v_exp!(v_exp)
    target_adventurer.add_a_exp!(a_exp)
    target_adventurer.add_b_exp!(b_exp)
    target_adventurer.add_oral_exp!(oral_exp)
    target_adventurer.add_v_sex_exp!(v_sex_exp)
    target_adventurer.add_a_sex_exp!(a_sex_exp)
    target_adventurer.add_v_sperm_shot!(v_sperm_shot)
    target_adventurer.add_a_sperm_shot!(a_sperm_shot)
    target_adventurer.add_oral_sperm_shot!(oral_sperm_shot)
    target_adventurer.add_skin_sperm_shot!(skin_sperm_shot)

    target_adventurer.drain_level_n_recover_by_slime!

    # 絶頂によるダメージ付与はなし
    result = {
      target: target_adventurer,
      target_life_point: target_adventurer.life_point,
      is_success: is_success,
      is_lonely: party.is_only?,
      this_turn_orgasms: target_adventurer.get_this_turn_orgasms,
    }
    result
  end

  def ero_damage_from_equip_if(party)
    result = {}
    party.all_members_with_passengers.each do |adventurer|
      next unless adventurer.has_any_equip_belt_in_dive?

      result[adventurer.id] = {}
      result[adventurer.id][:target] = adventurer

      adventurer.clear_this_turn_orgasms_and_sperm_shot!
      adventurer.equip_belt_in_dive.each do |equip_no|
        result[adventurer.id][:equip_no] = equip_no
        trap_effect = EQUIP_ERO_TRAP_DEFINE_LIST[equip_no]
        _trap_common(adventurer, trap_effect)
      end

      # 絶頂によるダメージ付与
      adventurer.get_trap_damage_by_orgasm!
      result[adventurer.id][:this_turn_orgasms] = adventurer.get_this_turn_orgasms
      adventurer.clear_this_turn_orgasms_and_sperm_shot!
      result
    end

    result
  end
end

