# coding: utf-8

module MonsterListByType
  require_rrk 'src/defines/dungeon/monster_list_v5'
  require_rrk 'src/defines/dungeon/monster_list_fgo'
  require_rrk 'src/app/helper/game_define_helper'
  require_rrk 'src/defines/dungeon/utils/version_helper'

  include GameDefineHelper
  include VersionHelper

  def master_monster_list
    return MonsterListFGO::MASTER_MONSTERS if FGO_MODE
    return MonsterListV5::MASTER_MONSTERS  if TRIAL_MODE
    return MonsterListV5::MASTER_MONSTERS  if _version_int < 5

    list =
      case _version_int
        when 5
          MonsterListV5::MASTER_MONSTERS
        else
          MonsterListV5::MASTER_MONSTERS
      end
    return list
  end

  def default_monsters_list
    return MonsterListFGO::DEFAULT_MONSTERS if FGO_MODE
    return MonsterListV5::DEFAULT_MONSTERS  if TRIAL_MODE
    return MonsterListV5::DEFAULT_MONSTERS  if _version_int < 5

    list =
      case _version_int
        when 5
          MonsterListV5::DEFAULT_MONSTERS
        else
          MonsterListV5::DEFAULT_MONSTERS
      end
    return list
  end

  def monster_define_list
    return MonsterListFGO::MONSTER_DEFINE_LIST if FGO_MODE
    return MonsterListV5::MONSTER_DEFINE_LIST  if TRIAL_MODE
    return MonsterListV5::MONSTER_DEFINE_LIST  if _version_int < 5

    list =
      case _version_int
        when 5
          MonsterListV5::MONSTER_DEFINE_LIST
        else
          MonsterListV5::MONSTER_DEFINE_LIST
      end
    return list
  end

  def self.elite_monster_define_list
    return [] if TRIAL_MODE

    if version_int < 5
      []
    else
      MonsterListV5::GUARD_MONSTER_DEFINE_LIST
    end
  end

  def guard_monster_define_list
    MonsterListV5::GUARD_MONSTER_DEFINE_LIST
  end

  def floor_enemies_list
    return MonsterListFGO::FLOOR_ENEMIES_LIST if FGO_MODE
    return MonsterListV5::FLOOR_ENEMIES_LIST  if TRIAL_MODE
    return MonsterListV5::FLOOR_ENEMIES_LIST  if _version_int < 5

    list =
      case _version_int
        when 5
          MonsterListV5::FLOOR_ENEMIES_LIST
        else
          MonsterListV5::FLOOR_ENEMIES_LIST
      end
    return list
  end

  # 1.1モンスターを追加したら定義をここに追加
  def dungeon_master_summons_list
    return MonsterListFGO::DUNGEON_MASTER_SUMMONS_LIST if FGO_MODE
    return MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST  if TRIAL_MODE
    return MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST  if _version_int < 5

    list =
      case _version_int
        when 5
          MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST
        else
          MonsterListV5::DUNGEON_MASTER_SUMMONS_LIST
      end
    return list
  end

  def self.version_int
    DUNGEON_VERSION.floor
  end

  def _version_int
    DUNGEON_VERSION.floor
  end
end
