# coding: utf-8
require_rrk 'src/defines/script/dungeon/script_utils'
require_rrk 'src/app/service/dungeon/module/dungeon_common'

class BaseScript
  include ScriptUtils
  include DungeonCommon

  def jail_summary_text(monster_name, monster_count, day_count)
    monster_name + monster_count + 'によって監禁中' +
      "#{half_number_to_full(day_count)}日目"
  end

  def get_teleport_pass(floor, adventurer)
    [
      "#{adventurer.short_name}は初めて#{floor_view(floor)}にたどり着いた。",
      'ワープ装置を発見することができた！ ' +
        "#{adventurer.short_name}は高速転送魔術装置の石板の、魔術的な文字が刻まれたその一片を切り出した。",
      '１０Ｆごとに設置されたダンジョン内を深い階層へと一方通行で移動する高速転送魔術装置を使うことで、次の冒険から' +
        "#{floor_view(floor)}に一瞬でたどり着くことができるだろう",
    ]
  end

  def get_teleport_pass_shorten(floor, adventurer)
    [
      "#{adventurer.short_name}は初めて#{floor_view(floor)}にたどり着き、ワープ装置を発見することができた！",
      '１０Ｆごとに設置されたダンジョン内を深い階層へと一方通行で移動する高速転送魔術装置を使い、次の冒険から' +
        "#{floor_view(floor)}に一瞬でたどり着くことができるだろう",
    ]
  end

  def teleport_for_rescue_script(floor, leader, is_lonely)
    [
      "仲間の救出のために、#{_party_leader_short_names_by(leader.short_name, is_lonely)}は高速転送魔術装置の魔晶石の欠片を使い装置を起動した！",
      "淡い光とともに#{leader.short_name}は高速転送魔術装置の力で一瞬で#{floor_view(floor)}にたどり着いた。",
    ]
  end

  def teleport_script(floor, leader, is_lonely)
    [
      "#{_party_leader_short_names_by(leader.short_name, is_lonely)}は高速転送魔術装置の魔晶石の欠片を使い装置を起動した！",
      "淡い光とともに#{leader.short_name}は高速転送魔術装置の力で一瞬で#{floor_view(floor)}にたどり着いた。",
    ]
  end

  def get_treasure_script(leader, is_lonely, money_amount, master_money)
    script = [
      "#{_party_leader_short_names_by(leader.short_name, is_lonely)}は最下層をさまよううちに、隠された小部屋を見つけた！",
      leader.short_name + 'は小部屋の中で光り輝く財宝を見つけることができた！',
    ]
    script +=
      if is_lonely
        [
          '単独行ながらも莫大な財宝を手に入れることができた' + leader.short_name + 'は素直に喜ぶことができなかった。',
          'ここからこの財宝を持ったまま無事に脱出できるかどうかも疑わしいと思ったからだ。',
        ]
      else
        [
          leader.short_name + 'は仲間たちと共に公平に財宝を分配した。公平な態度は仲間たちの尊敬も集めているようだ。',
        ]
      end
    script += [
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'に略奪されたことであなたの資金は' +
        _add_comma(money_amount) + '減った',
      money_status(master_money),
    ]
    script
  end

  def resist_status_script_in_jail(slave)
    if slave.resist_rate >= 80
      [
        [
          slave.short_name + 'は襲いかかる魔物たちをあしらい続けている。',
          '素手とはいえ、' + _adventurer_type + 'である' + slave.short_name + 'にとっては造作もないことなのだろう。',
          'その媚肉が目当ての魔物たちも、殺さぬよう手加減していることも大きいようだ。',
        ],
        [
          '部屋の中には、多数の魔物たちが美女を目当てにぐるりと囲んでいる',
          '（相手にもならないとはいえ）',
          slave.short_name + 'は犯そうと襲い掛かる魔物たちをあしらい続けている',
          '（数が多く、しつこいっ、このっ）',
          'だが数が多く、全てを倒してこの地下牢から逃れることは難しいだろう。いまできることは、ここで抵抗し破滅を先延ばしにしながら、チャンスを待ち続けることだけだ。',
        ],
      ].sample
    elsif slave.resist_rate >= 60
      [
        [
          '（いい加減っ、しつこいっ！）',
          '連日休みなく繰り返される怪物たちの攻めを' + slave.short_name + 'はあしらい続けている。',
          'だが休みもなく抗いつづける' + slave.short_name + 'の疲労はたまり続けるばかりだが、魔物たちには交代し、休む余裕もある。',
          slave.short_name + 'の行き着く先が見え始めているのは明らかだった。',
        ],
        [
          'まだ、魔物たちに' + slave.short_name + 'は遅れをとることはない。',
          'だが' + slave.short_name + 'の体は体力を失い、満足な食事も取れず、弱っていくばかりだ。',
          '一方で魔物たちは仲間を呼び、数を増やしつつある。今なら、まだ負けはしない。',
          'だが先行きを考えれば、楽観はできなかった。',
        ],
      ].sample
    elsif slave.resist_rate >= 40
      [
        [
          '夜も更けた頃、どうやら魔物たちは今日は諦めたようだ。',
          slave.short_name + 'は体力を回復させるべく、床に身を横たえている。',
          '冷たく、硬い石の床に横たわっても疲れはほとんど取れはしない。',
          'だがそれでも' + slave.short_name + 'は休めなければいけない。助けがくる、あるいは逃げ出す' + 
          'チャンスが巡ってくるまでは体力を一欠片でも温存しておかねばならなかった。',
          '夜、体を休めるのは魔物たちも同じだ。',
          'だが彼らはボロ布やわらでできた思い思いのねぐらで休んでいる。外から襲われる可能性もないねぐらで休む魔物たちは、翌朝には十分に英気を養い、体力を回復させることができる。' + 
          slave.short_name + 'とは大違いだった。',
          '……時間の経過が、' + slave.short_name + 'の不利になることは明らかだった。',
        ],
        [
          '魔物たちをあしらい続ける' + slave.short_name + 'の動きにも、かつてのキレは見えない。',
          'ここ数日で溜まりに溜まった疲労が、' + slave.short_name + 'の肉体を蝕んでいた……。',
          'キレが落ちたとはいえ、' + slave.short_name + 'は未だ魔物たちに屈することはない。' + 
          'だが限界が迫りつつあるのは、' + slave.short_name + '自身も理解していた……',
        ],
      ].sample
    elsif slave.resist_rate >= 20
      [
        [
          '夜も更けた頃、どうやら魔物たちは今日は諦めたようだ。',
          slave.short_name + 'は体力を回復させるべく、床に身を横たえている。',
          '（助けは、本当に来ないの……）',
          '弱音が、心の奥から湧き上がってくる。',
          slave.short_name + 'の心身は、限界に近付きつつあった。',
        ],
        [
          'まだ、魔物たちに' + slave.short_name + 'は遅れをとることはない。',
          'だが' + slave.short_name + 'の体は体力を失い、満足な食事も取れず、弱っていくばかりだ。',
          'すでに手足には鉛のような重さを覚えている。自分が切れ味を失いつつあるのは明らかだ。',
          '一方で魔物たちは仲間を呼び、数を増やしつつある。',
          '今なら、まだ負けはしないだろう。だが限界は、すぐ目の前にまで迫りつつあった。',
        ],
        [
          '重い疲労を全身に感じながら、' + slave.short_name + 'は魔物たちを睨みつける。',
          'まだ、こんなところで負けるわけにはいかなかった。',
          slave.short_name + 'は己を叱咤する。自分で自分を励まさねば、折れてしまいそうだった。',
        ],
      ].sample
    else
      [
        [
          '魔物の一匹がついに' + slave.short_name + 'を押し倒した。',
          'ブチブチと服が乱暴に破られる。押しのけようとした右腕が、別の魔物に押さえつけられる。',
          '「くぅぅっ、このっ、離れろっ！」',
          '押し倒された' + slave.short_name + 'は、必死の力で、魔物の腹を蹴り上げる。うまくみぞおちに入ったことで、魔物は苦しそうに呻きとびのいた。',
          '腕を押さえていた魔物も顔に手刀を入れ、その隙に' + slave.short_name + 
          'は離れることができた。',
          'かろうじて危地を脱することはできたが、崖っぷちにまで自分が追い込まれたのだ。',
          '押し倒され、おしのけることもできず犯されるのは遠い未来ではない。今日か、それとも明日か。',
          slave.short_name + 'の背筋に恐ろしい予感が走り抜けた。',
        ],
        [
          'ぜぇぜぇと' + slave.short_name + 'は荒い息を吐く。',
          '魔力を封じられてさえいなければ、武器さえあれば、仲間さえいれば、体力さえあれば。',
          'こんな雑魚たちなど束になっても相手になるはずがなかった。それが、こんなやつらに追い詰められている。',
          slave.short_name + 'は忌々しく思う。',
          '次の魔物が襲いかかろうとするのをみて、' + slave.short_name + 'は構えをとる。こんなところで、こんなところでこんな奴らに犯されるわけにはいかなかった。',
        ],
      ].sample
    end
  end

  def status_script_in_jail(slave)
    script =
      if slave.san_rate > 100
        [
          [
            '「うぎぃぃぃィッ、殺すっ……　お前たちっ、全員殺してやるぅぅ……」',
            slave.short_name + 'は心の中に溜まった憎悪をぶちまけるように、底冷えのような声で呻く',
            '「ゲヒャヒャッ、なぁにが殺すだよ。たっぷりブチ犯されるだけの肉便器がよぉ」',
            'もちろん魔物たちがそれに動じるはずもない。いまや' + slave.short_name + 'は無様に犯されるだけの肉穴なのだ。',
            '今までの借りを返すかのように、男たちは' + slave.short_name + 'を執拗にいたぶっている……',            
          ],
          [
            '「おごっ、んぐぶぅぅぅぅぅ！？」',
            slave.short_name + 'の頭をがっちりとホールドした男が、' + slave.short_name + 'の喉奥まで肉棒でしっかりと味わう。',
            '呼吸口を塞がれた' + slave.short_name + 'は男の腰を降参だというように何度もタップするが、男はもちろん気にするはずもない。',
            '肉穴も尻穴も口も、それぞれ男たちが己の欲望を吐き出すために塞がれている。',
            slave.short_name + 'はわずかな休みもなくたっぷりと嬲られている。',
          ],
        ].sample
      elsif slave.san_rate > 50
        [
          [
            '「いい加減に……　いい加減にして……」',
            slave.short_name + 'は陵辱者たちを睨みつけてはいるが、その顔には怯えの色が見える……',
            '繰り返し刻み込まれた陵辱が' + slave.short_name + 'を蝕み続けている……',
          ],
          [
            '「早くここから出してよ……」',
            slave.short_name + 'は男たちの顔も見たくないというように顔を背けた',
            'しかしそれは、睨みつける気力が萎えてしまったことを悟らせないようにするための無駄な抵抗でもあった。',
          ],
        ].sample
      elsif slave.san_rate > 25
        [
          [
            slave.short_name + 'の白い胸には無数の歯型がついている。',
            'かつては愛らしいピンク色だった乳首は、乱暴に扱われ、何度も男たちに歯を立てられたことでいまでは痛々しく真っ赤に腫れ上がっている。',
            'すでに触れられれば痛みしか感じないその胸に、男の毛むくじゃらの手が伸びる。もちろん男がたっぷりと揉みしだき感触を楽しむためだ。そこに、' + slave.short_name + 'への思いやりなどひとかけらもありはしない……',
          ],
          [
            '「おっ、お願いします……　すこしだけっ、少しでいいんですっ、５分でいいから、休ませてくださいっ！」',
            slave.short_name + 'は男たちに懇願する。朝からか。いや前の日からかもしれない。',
            slave.short_name + 'にはわずかばかりの休みも与えられることなく男たちに犯され続けていた。',
            '精神や肉体が悲鳴を上げ、容赦なくすり減っていく。だが男たちは交代してながら犯すことで体力をたっぷりと回復させ、有り余る獣欲をたっぷりと' + slave.short_name + 'にぶつけていた。',
            '「いやっ、いやぁぁぁぁぁぁ！」',
            'また次の男が' + slave.short_name + 'を貫き始めたようだ。',
            'この地獄は、まだまだ終わらない。なにしろ夜が更けるまで時間はたっぷりとあるのだから。',
          ],
        ].sample
      else
        [
          [
            '「いやぁぁぁ……　許して、もう許してよぉぉ……」',
            '口を塞いでいた男が離れると、' + slave.short_name + 'は悲痛な声を上げ、許しを乞おうとする。',
            'かつて、' + _adventurer_type + 'として無数のモンスターを屠ってきた凛々しい面影はそこにはない。',
            '限界を超えた暴力に屈しかけ、ひび割れた精神が悲鳴を上げている少女の姿がそこにあった……',
            slave.short_name + 'が限界を迎える時も近いだろう',
          ],
          [
            '「……もう、やめて……　限界なの……」',
            '奇妙に平板な声で' + slave.short_name + 'はつぶやく。疲労の限界を超え、もうまともな感情の起伏を起こす余力もないのだ',
            '「本当に、死んでしまうの……　だから……」',
            '次の男が' + slave.short_name + 'を組み敷いた。たとえ死んだとしても許されるはずはない。この陵辱が終わるとすれば、それは' + slave.short_name + 'が奴隷宣言を行った時だけだ。',
            'そして、彼女にとって不幸なことに、' + _adventurer_type + 'としての優れた' + slave.short_name + 'の頑健な肉体は、まだそう簡単には壊れはしないだろう……',
          ],
        ].sample
      end
    script
  end

  def ero_trap_effect_script(adventurer)
    orgasm_sum_count = adventurer.get_this_turn_orgasms_sum
    return [] unless orgasm_sum_count > 0

    orgasm_sum = half_number_to_full(adventurer.get_this_turn_orgasms_sum)
    [
      _this_turn_orgasms_chars(adventurer) + 'は' + orgasm_sum + '回絶頂し、体力を' +
        orgasm_sum + '消費した（体力：' + half_number_to_full(adventurer.life_point) + '）',
      "（Ｃ絶頂：#{half_number_to_full(adventurer.this_turn_c_orgasm_count)}、" +
        "Ｖ絶頂：#{half_number_to_full(adventurer.this_turn_v_orgasm_count)}、" +
        "Ａ絶頂：#{half_number_to_full(adventurer.this_turn_a_orgasm_count)}、" +
        "Ｂ絶頂：#{half_number_to_full(adventurer.this_turn_b_orgasm_count)}）",
    ]
  end

  def ero_trap_sperm_n_orgasms_script(adventurer, sperm_shot_params, orgasms_params, life_point_at_that_time)
    ero_trap_sperm_shot_script(adventurer, sperm_shot_params) +
      ero_trap_orgasms_by_params_script(adventurer, orgasms_params, life_point_at_that_time)
  end

  def ero_trap_sperm_shot_script(adventurer, params)
    return [] if params.nil? || params.empty?
    this_turn_sperm_shot_sum = params.values.inject(0) {|sum, n| sum + n}

    max_count = params.values.max
    return [] if max_count.nil? || max_count.zero?

    point_script = []
    point_script << "膣内#{half_number_to_full(params[:v])}回" if params[:v]
    point_script << "直腸#{half_number_to_full(params[:a])}回" if params[:a]
    point_script << "口内#{half_number_to_full(params[:oral])}回" if params[:oral]
    point_script << "ぶっかけ#{half_number_to_full(params[:skin])}回" if params[:skin]

    script = ''
    if max_count >= 5
      script += '精液まみれになる程'
    elsif max_count >= 3
      script += '呆れるほどたっぷりと'
    elsif max_count >= 2
      script += '大量に'
    end
    script += '射精'

    [
      adventurer.short_name + "は#{half_number_to_full(this_turn_sperm_shot_sum)}回#{script}された" + '（' + point_script.join('、') + '）',
    ]
  end

  def ero_trap_orgasms_by_params_script(adventurer, params, life_point_at_that_time)
    return [] if params.empty?
    this_turn_orgasms_sum = params.values.inject(0) {|sum, n| sum + n}

    orgasms_detail_text = []
    orgasms_detail_text << "クリ：#{half_number_to_full(params[:c])}回" if params[:c]
    orgasms_detail_text << "膣：#{half_number_to_full(params[:v])}回" if params[:v]
    orgasms_detail_text << "アナル：#{half_number_to_full(params[:a])}回" if params[:a]
    orgasms_detail_text << "胸：#{half_number_to_full(params[:b])}回" if params[:b]

    max_count = params.values.max
    orgasms_text = ''
    orgasms_text += 'Ｃ' if params[:c]
    orgasms_text += 'Ｖ' if params[:v]
    orgasms_text += 'Ａ' if params[:a]
    orgasms_text += 'Ｂ' if params[:b]

    if max_count >= 5
      orgasms_text += '極大'
    elsif max_count >= 3
      orgasms_text += '特大'
    elsif max_count >= 2
      orgasms_text += '大'
    end
    orgasms_text += '絶頂'

    orgasm_sum = half_number_to_full(this_turn_orgasms_sum)
    script =
      if life_point_at_that_time
        scr = [
          orgasms_text + '（' + orgasms_detail_text.join('、') + '）',
        ]
        scr <<
          if this_turn_orgasms_sum > 25
            adventurer.short_name + 'は' + orgasm_sum + '回絶頂し、快楽の前に完全屈服した。'
          elsif this_turn_orgasms_sum > 15
            adventurer.short_name + 'は' + orgasm_sum + '回絶頂し、快楽の前に屈服した。'
          elsif this_turn_orgasms_sum > 10
            adventurer.short_name + 'は' + orgasm_sum + '回絶頂し、快楽の前に敗北した。'
          else
            adventurer.short_name + 'は' + orgasm_sum + '回絶頂した'
          end
        scr += [
          '体力を' + half_number_to_full(orgasm_sum) + '消費した（体力：' + half_number_to_full(life_point_at_that_time) + '）',
        ]
        scr
      else
        [
          orgasms_text + '（' + orgasms_detail_text.join('、') + '）',
          adventurer.short_name + 'は' + orgasm_sum + '回絶頂した',
        ]
      end
    script
  end

  def _this_turn_orgasms_chars(adventurer)
    max_count = adventurer.get_this_turn_orgasms_max_count
    return nil if max_count.zero?

    result = adventurer.get_this_turn_orgasms
    script = ''
    script += 'Ｃ' if result[:c]
    script += 'Ｖ' if result[:v]
    script += 'Ａ' if result[:a]
    script += 'Ｂ' if result[:b]

    if max_count >= 5
      script += '極大'
    elsif max_count >= 3
      script += '特大'
    elsif max_count >= 2
      script += '大'
    end
    script += '絶頂'

    script
  end

  def lost_virgin_script(type, monster_name)
    case type
      when 'monster_force_sex'
        monster_name + 'たちに輪姦され、処女を無惨に奪われた'
        # 罠で処女喪失系
      when 'ero_tunnel_sex'
        'ふたなりの肉棒を生やした女淫魔の魅了にかかり、偽りの幸福の中で何度も絶頂しながら処女穴を容赦なく犯された'
        # 罠で処女喪失系
      when 'trap_v_tentacle' # 秘所愛撫触手
        'エロトラップ（秘所愛撫触手）によって太い触手を秘裂にねじ込まれ、処女を無惨に奪われた'
      when 'trap_v_tentacle_brush' # 秘所愛撫触手ブラシ
        'エロトラップ（秘所愛撫触手ブラシ）によって太い触手を秘裂にねじ込まれ、処女を無惨に奪われたあとたっぷりと快楽地獄を味あわされた'
      when 'trap_v_tentacle_vibe' # 触手肉バイブ
        'エロトラップ（触手肉バイブ）によって触手バイブを秘裂にねじ込まれ、処女を無惨に奪われた'
      when 'trap_v_tentacle_warts_vibe' # 触手イボ付き肉バイブ
        'エロトラップ（触手イボ付き肉バイブ）によってえげつない形状の触手バイブを秘裂にねじ込まれ、処女を無惨に奪われながら情け容赦のない快楽を味あわされた'
      when 'trap_hypno_sex'
        'エロトラップによる催眠中、種付けおじさんによって処女を奪われた。たっぷりと特濃精液を子宮に流し込まれ、初めて知る女の悦びに震えながら何度も絶頂した'
      when 'trap_possession_lesbian_sex'
        'エロトラップによって淫霊に憑依され、仲間たちにバイブで何度も犯され処女を失った'
      when 'trap_possession_penis_sex'
        'エロトラップによって淫霊に憑依され、仲間たちとふたなりペニスでお互いを貪りあいながら処女を失った'
      when 'trap_v_vibe'
        'エロトラップ（バイブ付き貞操帯）によってゴツいバイブを秘裂にねじ込まれ、処女を無残に失った'
      else
        puts "Invalid lost virgin type!!(#{type})"
        ''
    end
  end

  # ====== 地下牢で凌辱中 はじめ ===================
  def down_to_slave_script(adventurer, monster_party)
    monster = monster_party.monster
    script = [
      [
        '精神に嫌という程叩き込まれた' + monster.name + 'たちの容赦のない陵辱は、' + adventurer.short_name + 'の内部に存在した誇りや理性、矜持や知性というものを徹底的に粉砕してしまった。',
        '精神の内側にぽっかりと空いてしまった黒々とした空洞には、かわりに強者への服従と恐怖が強引に押し込まれてしまった。',
        'いまの' + adventurer.short_name + 'にとって男たちの命令とは絶対であり、女とは男たちのために肉の全てで奉仕するための存在。子孕み袋であり慰み穴であり、都合の良い肉奴隷である。',
        '深刻な後遺症が残るほどの過酷な陵辱を休みなく受け続けたことで、思考能力のすべてを失っていた' + adventurer.short_name + 'は、今や男たちに叩き込まれた「常識」を疑うことなく信じ込んでいた。',
        '「いやらしい変態女である' + adventurer.self_call + 'に、肉奴隷としての常識を教え込んでいただきありがとうございました……。」',
        '「皆様のおかげで、' + adventurer.self_call + 'は、本当の自分に気づくことができました」',
        adventurer.short_name + 'は陵辱者である' + monster.name + 'たちに土下座をしながら礼を述べる。',
        'すでに壊れかけていた' + adventurer.short_name + 'の精神は、' + monster.name + 'たちが差し出した条件。肉奴隷になることを受け入れれば解放してやる、という条件を、疑いもせずにあっさり受け入れた。',
        'この終わりのない肉の地獄から抜け出せるのであればもうあとはなんでもよかった。少し先のことを考えるための思考能力すら、もはや' + adventurer.short_name + 'には残ってはいなかった。',
        'そう。「解放」とは、この地下牢からの解放を意味する。もちろん行き先は奴隷商人のもとであり、さらに行き着く先は様々だが、いずれも女としてのまともな人生とは対極の場所に行くことを意味する。',
        'それを知らない' + adventurer.short_name + 'は、純粋にこの場所から離れられることにわずかな安堵の表情を見せてた。',
        ' ',
        '行き着く先の地獄を知らない陵辱され尽くした美女の姿を' + monster.name + 'たちはただニヤニヤと眺めていた。',
        'いかなる美女といえど、何日も犯し続ければ飽きもする。飽きてしまえばもう用はない。あとは、奴隷商人に売り払い金にするだけだ。',
        '慰み者として女日照りの' + monster.name + 'たちの便利な性欲処理先として使い潰したあげくに、売り払えば金にもなる。まったく' + _adventurer_type + 'というものは便利な存在だと、' + monster.name + 'たちはせせら笑った。',
        'しばらくは手に入れた金で贅沢をしよう。次の獲物がまた来れば金などまた手に入るのだから。',
        '（' + adventurer.short_name + 'は奴隷宣言しました。）',
      ],
      [
        '「な、なります……　奴隷に、肉奴隷になります……」',
        '精液が水たまりのようになっている汚い床に、' + adventurer.short_name + 'は自分が汚れるのも構わずに額を擦り付けるようにして土下座をしている。',
        '「' + adventurer.self_call + 'はお、思い上がっていました。男の人に逆らうなんて、とんでもない思い違いでした」',
        'かつての凛々しさや威厳などかけらも残ってはいない惨めな態度で、' + adventurer.short_name + 'は' + monster_party.monster.name + 'に許しを懇願している' ,
        '「女は、男の人のおちんぽをハメていただくための肉便器です、オナホールです。それを' +
          adventurer.self_call + 'のような愚かな女に、身の程を教え込んでいただき、本当にありがとうございました。」',
        adventurer.short_name + 'は震えながら言葉を絞り出した。その態度が、言葉が心からのものではないことを示している。',
        'だが' + adventurer.short_name + 'はいやらしい笑いを浮かべる男たちの命令のままに嘲笑と侮蔑に晒されながら、心にもない言葉を述べさせられている。',
        '逆らえば何をされるかわからない。命令に従っておけば被害は少なくなる。その恐怖に' + adventurer.short_name + 'は完全に支配されていた。',
        '（' + adventurer.short_name + 'は奴隷宣言しました。）',
      ],
      [
        '「ああ゛あ゛ぁぁ……　あぁぁ……」',
        '全身の穴という穴から精液を垂れ流し、' + adventurer.short_name + 'は白眼を剥いてビクビクと痙攣している。',
        '時折ビクンビクンと手脚が跳ね上がってはそのまま乱暴に床に落ちる。その姿は壊れかけたバネ仕掛けの人形を思わせた。',
        '最上質の絹のように美しかった肌は白濁液まみれでドロドロに穢されており、シミひとつなかった全身はアザと歯型だらけでかつての美しさは見る影もない。' + 
        '膣穴と尻穴はぽっかりと大きく入り口を広げており、中には大量に流し込まれた溢れるほどの精液が覗いていた。',
        '美しかった髪にも顔にも大量の精液をぶちまけられたその顔は涙と鼻水と涎が垂れ流しで、焦点の合わぬ瞳のまま薄れた意識で痴呆のように呆けている。もはや彼女の女としての人生そのものが「終わって」いることは明らかだった。',
        '限界を超え、後戻りできぬほど徹底的に調教された彼女は、二度と' + _adventurer_type + 'として復帰することはおろか、まともな日常生活にも戻れないだろう。',
        monster.name + 'たちは使い潰してしまった' + adventurer.short_name + 'にはもう興味もないというように、奴隷商人に引き渡した。',
        '「おぉぉぉ……　ああ゛あ゛ぁぁ……　いやっ、もういやあぁぁ……」',
        '意識の薄れたまま、うわごとのように' + adventurer.short_name + 'はつぶやく。桁違いの性暴力の前に、精神のタガが大きく外れてしまったのだ。',
        'だがもう心配はないだろう。',
        '奴隷商人は彼女に精神の回復魔術をかけるつもりだった。それはもちろん完治には程遠く、単に高値で奴隷として売るに十分な程度のレベルに過ぎないものではあったが。',
        '奴隷商人が連れてきた屈強な男たちが' + adventurer.short_name + 'を担ぎ上げる。目的地は' + adventurer.short_name + 'を売却するための奴隷オークションの会場だった。',
        '（' + adventurer.short_name + 'は奴隷宣言しました。）',
      ],
      [
        '休みのない手加減なしの陵辱によって冒険者としてのプライドをすべて砕かれ、' + adventurer.short_name + 'の理性はドロドロに溶かされてしまった。',
        '激しい陵辱の末に、ついに' + adventurer.short_name + 'は冒険者としての気概をすべて失ってしまった……',
        '女とは男に犯されるための肉穴であり、精液を排泄されるための都合の良い性処理便器なのだ。そう悟った' + adventurer.short_name +
          'は、虚ろな目で性奴隷になることを宣言した。',
        monster.name + 'たちはそれを聞くとニヤニヤと笑いながら奴隷商人に引き渡した……',
        '（' + adventurer.short_name + 'は奴隷宣言しました。）',
      ],
    ].sample
    script
  end

  def jail_training_short_script(slave, monster_party)
    monster = monster_party.monster

    [
      [
        '「へへっ、次は俺だぜぇ。くっせぇザーメンたっぷり飲ませてやっから、せいぜいそのチンポしごき穴締めつけろやっ！」',
        monster.name + 'の罵声に' + slave.short_name + 'は答えない。',
        '言葉を聞き、認識して、思考し、適切な回答をするという脳の高度な機能はすでに彼女からは失われていた。',
        'ぐぢり、という湿った音を立てながら剛直が彼女の穴に侵入する。',
        'うああ゛ぁぁ……、という不明瞭な声だけが彼女の唇から漏れた。',
        '鼻の穴は半ば精液で塞がれろくに呼吸もできない。酸素不足で脳細胞がじわじわと死んでいくせいか、さらに思考が不明瞭になる。',
        '呼吸の自由を奪われながら、腕は縛られていてどうすることもできない。' +
          '鼻水やよだれのように精液混じりの汚らしいものを垂れ流しながら、' + slave.short_name + 'はただ' + monster.name + 'たちのされるがままになっていた……',
      ],
      [
        '「あっ、うああぁぁっ！ いやっ、いやだぁぁ、またぁぁぁ……」',
        slave.short_name + 'が細かく激しい痙攣を始める。絶頂が近いのだ。',
        '「うああぁぁぁっ、ひぃぃっ、うあぁっ、あああぁぁぁぁ………」',
        'ついに耐え切れず、' + slave.short_name + 'は絶頂した。すると' + monster.name + 'たちの一' + monster.unit_name  +
          'が' + slave.short_name + 'の前髪をつかみ、顔を無理やり上げさせる。',
        '絶頂の余韻と多幸感が押し寄せたことで、幸せそうに目を細め表情を緩ませている顔が、' + monster.name + 'たちの前にあらわになった。',
        '『へへっ、犯されてんのにそんだけ気持ちよさそうな顔するとは、さすが淫乱な' + slave.short_name + 'だぜ！』',
        '『どんなチンポがきても、穴はギチギチ嬉しそうに締め付けてくるからなぁ。やっぱり犯されたくて仕方ないんだろぉ？』',
        monster.name + 'たちの嘲弄が、絶頂直後の無謀になった' + slave.short_name + 'の精神へと、絶頂の背徳感とともに流れ込み、' +
          '屈服感と被虐という後ろ暗い喜びへと変わっていく……',
        monster.name + 'たちは' + slave.short_name + 'の肉体だけでなく精神まで、陵辱によって徹底的に嬲り尽くした。',
      ],
      [
        '捕らえられ陵辱の宴に投げ込まれた肉贄、' + slave.short_name + 'の理性を砕き、矜持をへし折るための休みのない陵辱がわずかな休みもなく繰り返される。',
        '魔物たちは' + slave.short_name + 'の全ての穴で楽しみ、思い思いに白濁液を吐き出し、矜持をぐちゃぐちゃに辱め、肉体を徹底的に穢し、容赦なくその知性を踏みにじっていく。',
      ],
      [
        '膣穴はおろか、尻の穴と口、胸や手まで使って' + slave.short_name + 'は' + monster.name + 'たちへの奉仕を強いられている。',
        '「オラッ、休んでる暇なんかねぇぞぉ！ 次は俺だぁ！」',
        '一' + monster.unit_name + 'の' + monster.name + 'が精液を吐き出し終え、離れるとたちまちそこに次の魔物が現れ肉奉仕を強要する。' +
          '休む暇など与えられるはずもない。なにしろ' + monster.name + 'たちには' + slave.short_name + 'への気遣いなど最初からないのだから。',
        monster.name + 'たちの嗜虐の宴は、朝から夜中まで続いていく……',
      ],
      [
        '「おげぇ、うげぇぇ…… おごっ、おごぼお゛お゛お゛ぉぉぉっ！」',
        '口を肉槍で塞がれ、聞き苦しい声を上げながら' + slave.short_name + 'は半ば意識を手放しかけた状態で全身をビクビクと痙攣させる。' +
          monster.name + 'たちは' + slave.short_name + 'のことを一切気遣う様子もなく、思い思いに腰を動かし、その媚肉を貪る。',
        '情け容赦のない徹底的な肉嬲りが、' + slave.short_name + 'の精神に亀裂をいれていく……',
      ],
      [
        '「おい、次はこの便女穴を使うのは誰だぁ？」',
        monster.name + 'の１' + monster.unit_name + 'が仲間たちに呼びかける。' +
          'その言葉には、一切の' + slave.short_name + 'への遠慮というものが含まれてはいなかった。',
        '徹底的にふみにじり、犯し抜き、屈服させる対象。それこそが今の' + slave.short_name + 'の立場だった……',
      ],
      [
        '「うあああ゛あ゛あ゛ぁぁっ！ またっ、まだぁあ゛ぁ、いくっ、イグゥゥ！！！」',
        '聞き苦しい声を上げながら、' + slave.short_name + 'は絶頂した。',
        '愛してもいない相手との望まぬセックス。嫌悪すらする相手による肉悦がもたらす絶頂を、' +
          slave.short_name + 'は今日だけで数え切れぬほど迎えていた。' + '一体今日だけで何度絶頂しただろうか。一体どれほどの時間、' +
        monster.name + 'たちと交わっているのだろうか。',
        slave.short_name + 'には分からなかった。快楽地獄の泥沼のような甘い苦悦が、' + slave.short_name + 'の思考をズタズタに引き裂いていた……',
      ],
    ].sample
  end

  def jail_training_script(slave, monster_party)
    if slave.in_brain_washing?
      return jail_training_in_brain_washing_script(slave, monster_party)
    end

    monster = monster_party.monster

    case monster.race_type
    when 'goblin'
      return jail_training_goblin_script(slave, monster_party)
    when 'ork'
      return jail_training_ork_script(slave, monster_party)
    when 'kobold'
      return jail_training_kobold_script(slave, monster_party)
    when 'lizard'
      return jail_training_lizard_script(slave, monster_party)
    when 'mage'
      return jail_training_mage_script(slave, monster_party)
    when 'daemon'
      return jail_training_daemon_script(slave, monster_party)
    when 'beastman'
      return jail_training_beastman_script(slave, monster_party)
    when 'urk'
      return jail_training_urk_script(slave, monster_party)
    else
      puts "OTHER monster.race_type is #{monster.race_type}/#{monster.name}"
    end

    scripts = [
      [
        "薄暗い地下室の暗がりの中で複数の#{monster.name}の影が蠢き、そのたびに女の悲痛な叫びがあがる。",
        slave.short_name + 'は' + monster_name_and_unit_name(monster, monster_party.rest_enemies_count) + 'によって徹底的に嬲られていた。',
        monster_name_and_unit_name(monster, 1) + 'が汚らわしい男の白い滾りをどくどくと吐き出し終えたようだ。' +
          '絶望と諦めの表情を浮かべ涙をこぼす' + slave.short_name + 'とは対照的に、' +
          monster.name + 'が満足そうな表情でそこをどくと、' +
          'すぐに次の' + monster.name + 'が' + slave.short_name + 'にのしかかる',
      ],
      [
        "ランプに照らされた薄暗い地下室の暗がりの中で複数の#{monster.name}の影の隙間から女の白い肌が覗く。まるで砂糖菓子に群がる蟻のような有様だ。",
        slave.short_name + 'は' + monster_name_and_unit_name(monster, monster_party.rest_enemies_count) + 'によって徹底的に嬲られていた。',
        'かつては雪のように白く、絹のように滑らかだった肌は、いまや' + monster.name + 'たちがつけた痣や歯型によって見るも無残な有様になっている。',
        '乾いた精液や得体の知れないドロドロとした液体が半ば乾いてこびりついた髪や顔には、かつての凛々しかった威厳などひとかけらも残ってはいない。',
        'それでも' + monster.name + 'たちは、' + slave.short_name + 'の人格や尊厳の最後のひとかけらまでも蹂躙してやらんとばかりに徹底的な陵辱を休みなく続けている。',
      ],
    ]

    if slave.get_this_turn_orgasms_sum > 3
      scripts += [
        [
          '「おごっ、うごぉぉぉっ！ またっ、まだぁぁぁぁ、イッ、イグゥゥッ！」',
          slave.short_name + 'は数え切れぬほど迎えた望まぬ絶頂に再び達せさせられた。ビクビクと白い裸身を震わせる' +
            slave.short_name + 'のすべての穴を、' + monster.name + 'の肉槍が貫き休みなく嬲り抜いている。',
        ],
      ]
    end

    script = scripts.sample
    script
  end

  def jail_training_in_brain_washing_script(slave, monster_party)
    monster = monster_party.monster

    [
      [
        '「うふふ、いつもご主人様のために働いているのですから……」',
        slave.short_name + 'は自ら服をはだけ、生まれたままの姿となった。輝くほどの白い肌が、' + monster.name + 'たちの欲望にギラついた視線に晒される。',
        '「たまにはこんな素敵な息抜きも、許されますよね？」',
        '床にぺたんと座り込んだ' + slave.short_name + 'は、さあ召し上がれとでもいうように' + monster.name + 'たちの前で誘うように大きく脚を開いた。',
        '飢えた犬たちの群れの中に、極上の肉を投げ込んだようなものだった。目の前の媚肉に、' + monster.name + 'たちが群がっていく。思い思いにペニスを穴にねじ込み、手に握らせ、口にしゃぶらせる。',
        '肉の狂宴は激しく燃え上がっていく。',
        '全ての穴という穴が異形のペニスによって激しく耕され、全身がどろどろとした精液によって汚されながらも' + slave.short_name + 'は荒々しく満たされる肉悦に歓喜の表情を浮かべている。',
        '激しく陵辱されながらも情熱的に肉奉仕する' + slave.short_name + 'の姿は、' + monster.name + 'たちを大いに満足させたようだ……',
      ],
      [
        '「うふふ、こんなにおちんぽをガチガチにして……」',
        '自ら服をはだけ、一糸まとわぬ姿になった' + slave.short_name + 'は' + monster.name + 'のペニスに目を釘付けにされながら舌なめずりする。',
        '「どうぞ、この卑しい肉奴隷の体を、皆様の欲望のためにお使いください♡」',
        '刻み込まれた淫らな欲望への飢えを満たす絶好の機会を、' + slave.short_name + 'は逃すつもりはなかった。',
        'それはもちろん、' + monster.name + 'たちも同じだった。あっという間に' + monster_name_and_unit_name(monster, monster_party.rest_enemies_count) + 'が白い肌へと群がっていく。',
        '全ての穴という穴が異形のペニスによって激しく耕され、全身がどろどろとした精液によって汚されながらも' + slave.short_name + 'は荒々しく満たされる肉悦に歓喜の表情を浮かべている。',
        '激しく陵辱されながらも情熱的に肉奉仕する' + slave.short_name + 'の姿は、' + monster.name + 'たちを大いに満足させたようだ……',
      ],
    ].sample
  end

  def jail_training_goblin_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        'ゴブリン。どんな初心者であっても、ダンジョンに潜る者であれば存在は知っているだろう。もちろんそれは雑魚として、だ。大人が子供に喧嘩で負けないように、こんな相手に、まともに戦って負ける者がいるはずがない。多くの者はそう考えている。',
        'だが例外は、どこにでも存在する。',
        slave.short_name + 'が、まさにいい例だった。',
        '「ひいぃぃぃっ、いやぁぁぁ……　ううぅ、もうやめてぇぇ……」',
        slave.short_name + 'の悲鳴を嗜虐心へのスパイスとでも考えているのだろう。',
        monster.name + 'たちはさも可笑しそうに嘲笑うと、さらに' + slave.short_name + 'への陵辱を激しくした。',
        'どうして自分がこんな目に。なぜ自分がこんな雑魚相手に。自分がこんな目に会わねばならない理由があるわけがない。',
        slave.short_name + 'の疑問と理不尽への不満に答えがあるわけがない。',
        '今ここにあるのは、目の前の媚肉に目をつけた陵辱者たちと、その生贄に捧げられた哀れな' + slave.short_name + 'という生贄があるだけだ。',
        slave.short_name + 'の嗚咽をよそに、陵辱は夜遅くまで続いた……',
      ],
      [
        slave.short_name + 'にむらがる' + monster.name + 'たちは、久しぶりのメスの柔肉を味わい思うがままに子種汁を吐き出していく。',
        'もちろん自分たちの子を、' + slave.short_name + 'の胎に孕ませるための容赦のない生殖陵辱。',
        '流し込まれた精液の熱が' + slave.short_name + 'の理性をドロドロに溶かし、かわりに刻まれた快楽の味が奴隷としての屈服の悦びを刻み込んでいく……',
      ],
    ]

    if slave.san_rate < 50
      scripts += [
        [
          monster.name + 'たちは' + slave.short_name + 'に群がりその媚肉を存分に貪っている',
          '「うあぁぁ……　いやぁぁ、あぁぁ……　またっ、またナカにぃぃ……」',
          '朦朧とした意識のままぼんやりと呻く' + slave.short_name + 'の態度に、かつての威厳や気の強さは欠片も存在しない。',
          '本来であれば取るに足らない雑魚相手の終わりのない陵辱。',
          '矜持は踏み砕かれ、理性は引き裂かれた。どぷどぷと流し込まれた精液が子宮内で掻き混ぜられる感覚が精神を苛み、どす黒く染め上げていく。',
          '終わりのない陵辱。終わるとすれば、それは' + slave.short_name + 'が屈服と隷従の言葉を述べたときだけ。',
          '望まぬ快楽を容赦なく送り込まれる絶望に、' + slave.short_name + 'の心はどす黒く染まっていった……',
        ],
      ]
    end

    if slave.san_rate < 30
      scripts += [
        [
          '「おまんこでも、お尻でも、口でも……　好きに使って良いですから、だから、乱暴にしないでください……」',
          slave.short_name + 'は地面に頭を擦り付けるようにして平身低頭し、懇願する。',
          '実際に頭を地面に擦り付けることができないのは、後ろから片手を押さえられた上で' + monster.name + 'に犯されているからだ。',
          '本来であれば取るに足らない雑魚たちに本心より懇願したいわけではない。だが今の彼女にはもはや抵抗する術もなかった。ただ己の破滅を先延ばしにするために' + monster.name + 'たちに媚を売り、情けを乞う。',
          'そしてそのたびに屈辱によって心は大きくひび割れていく。',
          '完全な悪循環だった……',
          slave.short_name + 'はただ、目の前の破滅をなんとか避けようと、卑劣な陵辱者たちに矜持を捨てて懇願する。',
          'だが、容赦なく絶望の淵は' + slave.short_name + 'に迫りつつあった……',
        ],
      ]
    end

    script = scripts.sample
    script
  end

  def jail_training_kobold_script(slave, monster_party)
    monster = monster_party.monster

    scripts = [
      [
        monster.name + 'たちは' + slave.short_name + 'に群がりその媚肉を存分に貪っている',
        '「いやっ、いやぁぁぁ……　やだっ、もうやだぁぁ……　助けてぇ、誰か助けてよぉ……」',
        '朦朧とした意識のまま弱々しくつぶやく' + slave.short_name + 'の態度に、かつての威厳や気の強さは欠片も存在しない。',
        '終わりのない陵辱に心を砕かれ、それでも望まぬ快楽を容赦なく送り込まれる絶望に、' + slave.short_name + 'の心はどす黒く染まっていった……',
      ],
      [
        '「いやっ、いやぁぁぁ……　' + slave.self_call + 'っ、こんな、獣みたいにぃぃ……」',
        slave.short_name + 'は犬のように四つん這いの姿勢に拘束されていた。手脚をしっかりと戒める鉄製の拘束具は頑丈でどれほど力を込めようともビクともしない。',
        '前後の穴を貫く' + monster.name + 'の雄々しい肉槍が、容赦なく' + slave.short_name + 'の内側を削り落とし、代わりに恐ろしいほどの快楽を刻み込んでいく。',
        '犬獣人特有の大きく張り出したエラが膣襞を優しく撫で上げるたび、腰が砕けそうになるほどの快楽を生み出していく。その快楽の味は麻薬のようで、一度知ってしまえば人間の貧弱なペニスでは満足できなくなると言われるほどだ。',
        'そして' + monster.name + 'たちはその威力を存分に知っていた。だからこそセックスの快楽の味を容赦なく' + slave.short_name + 'に教え込み、犬獣人のペニスの形と味を覚えさせていく。' +
        '膣内に、そして腸内に大量の射精を何度も何度も執拗に行うことで獣の臭いが取れなくなるほどのザーメンマーキングを繰り返しているのだ。',
        '「おおぉぉっ、うあぁぁぁっ……　' + slave.self_call + '、' + slave.self_call + 'っ、犬チンポでまたぁぁ……ッ！」',
        slave.short_name + 'は体をこわばらせ、無意識のうちに目の前の' + monster.name + 'のたくましい胸板に抱きつき、ガクガクと四肢を痙攣させる。再び、何十度目かの望まぬセックスによる絶頂に近づいているのだ。',
        '絶頂を重ねるたび、子宮内に、そして直腸に大量の精液をぶちまけられるたび、' + slave.short_name + 'は自分が犬獣人とのセックスに溺れ、' + _adventurer_type + 'としての誇りを失っていくことを感じていた。',
        'そのことへの絶望と嫌悪が、だんだんと薄れていく。自分は、いつまでこの快楽を受け入れずに居られるだろうかと、絶頂直後の余韻に浸りながら' + slave.short_name + 'はぼんやりと思った……',
      ],
    ]

    if slave.san_rate < 30
      scripts += [
        [
          monster.name + 'に前後の穴と口を犯されながら' + slave.short_name + 'はなすがままになっている。',
          '射精を終えた' + monster.name + 'が口から肉棒を引き抜き、次の' + monster.name + 'が肉棒を顔に突きつけると、' + slave.short_name + 'は何も言われずとも自分から男の者を頬張る。',
          '（' + monster.name + 'の精液ぃ……　こんなに獣臭くて、気持ち悪いのにぃ……　なんだか、これがクセになって……）',
          'すでに理性はまともな働きをしていない。数日間休みなく陵辱され続けた' + slave.short_name + 'は、徹底的に叩き込まれた肉奉仕の技術で当然のように陵辱者たちの肉槍をもてなし、欲望のたぎりを注ぎ込まれる。',
          '肉体はすでに堕ち、精神も屈服しつつあった……。',
          '救いのこない地下牢で、肉贄を貪る宴が今日も続く……',
        ],
      ]
    end

    script = scripts.sample
    script
  end

  # ペニスはザラザラしていて舌のように動くよ
  def jail_training_lizard_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        '「ひぃぃィィッ！　ナカっ、ナカがぁぁぁ！　動くぅぅっ、動いてぇ！　ナカを舐めないでぇぇぇ！」',
        slave.short_name + 'は度の外れた声をあげ、体を大きく跳ねさせる。',
        '頑丈な拘束具によって逃れることは妨げられたが、その激しさは彼女が感じている衝撃を物語っていた。',
        monster.name + 'のペニスは爬虫類らしく、ウロコ状の突起が生えたザラザラとしたヤスリ状になっている。これが抽迭のたびに膣襞の一つ一つを荒々しく愛撫するのだ。',
        '「うあああぁぁぁぁ！　こんなっ、動いてるぅぅ！　反則っ、こんなの反則だからぁぁぁっ！」',
        'しかもヤスリ状のペニスは、短躯の蛇のように自在に動かすことができるのだ。膣内で' + slave.short_name + 'の弱点を探り当て、そこを徹底的にヤスリ状の表面で削り落とすように責めたてる' + monster.name + 'の責めに' + slave.short_name + 'は面白いように乱れた。',
        'やがて体位を変えられ、前の穴を貫く' + monster.name + 'の上に' + slave.short_name + 'はうつ伏せになる姿勢を取らされる。すでに膝はガクガクと震え、腰は抜けかけており、抵抗のしようもない。',
        '尻穴に、別の' + monster.name + 'のヤスリ状ペニスがあてがわれる。',
        'これから自分が何をされるか悟った' + slave.short_name + 'は必死に暴れ始める。',
        '「いやだっ、いやだぁぁぁ！　こんなっ、こんなのお尻に挿れられたらぁぁぁ！　本当におかしくなるっ、' + slave.self_call + '、狂っちゃうからぁぁぁぁ！　いやぁ、やめてぇぇぇ！」',
        'しかし' + monster_name_and_unit_name(monster, 2) + 'によってがっちりと押さえつけられたことで逃れることもできず尻穴への侵入を許してしまう。',
        '直腸のひだの一つ一つを舐めあげられ、ヤスリによって削りおとされるような背徳の快楽に' + slave.short_name + 'は何度も絶頂を味合わされ、狂わされ続けた。',
      ],
      [
        '「ひぃぃぃっ！　ほおぉぉっ、おああぁぁぁっ！　ナカっ、ナカがぁぁぁぁ！」',
        'ザラザラとしたウロコによってヤスリ状になった' + monster.name + 'のペニスが' + slave.short_name + 'の膣内をこすりあげるたび、暴力的なまでの快楽が生み出される。その快楽電流は背骨を通り、' + 
        slave.short_name + 'の脳を直撃するたび思考回路が焼き切られ、彼女の頭から知性や思慮という高尚なものが壊されていく。',
        'ストロークのたびに快楽電流は起き、そのたびに' + slave.short_name + 'は快楽に吠え狂い、壊されていく。',
        'まるでそれは、ヤスリ状の' + monster.name + 'のペニスによって' + slave.short_name + 'の理性が内側から削り落とされていくようでもあった。',
      ],
      [
        '「うああぁぁぁっ、トカゲッ、トカゲチンポすごいぃぃぃ……ッ！　ナカっ、ナカで擦れてぇぇぇ！」',
        slave.short_name + 'は髪を振り乱し、狂ったように叫ぶ。',
        monster.name + 'のペニスは、内側から女の理性を削り落とし己の所有物に堕とすために最適化された' + 
        'ザラザラとしたヤスリ状になっている。' + 
        'しかもそれらは指のようにオスの意思によって曲げることもできるのだ。',
        '膣内や直腸を掘削しながら目ざとく見つけ出した弱点を曲げることで徹底的に擦りあげ、ゴシゴシと削り落とすように責めたてる凶悪な肉凶器は、ペニスというよりは舌に近いかもしれない。',
        '太く器用な舌で膣内と直腸を責められるという経験を持つ女などいるはずがない。すなわちそれは、女が耐えられるはずのない快楽の濁流を直接叩きつけてくるということでもあった。',
        '「ああ゛あ゛ぁぁー！　こんなっ、こんなっ、おっ、うああぁっ！　おっ、お゛お゛お゛ぉぉぉぉっ！」',
        'もはや' + slave.short_name + 'の態度には余裕や知性というものはひとかけらも残ってはいない。獣のように吠えながら恐ろしい快楽に溺れ、必死にもがく。',
        'それでも' + monster.name + 'たちは責めを緩めるつもりもない、いやむしろ何の手加減もなく、許容限度を超えた快楽に白眼を剥く' + slave.short_name + 'を使い潰すかのように一層激しく陵辱していく。',
        monster.name + 'たちがズボズボと激しくストロークするたび、' + slave.short_name + 'は叫び、啼き狂い、絶頂を迎える。そのたびに' + slave.short_name + 'の脳の中で知性や矜持という高尚な機能がブチブチと音を立てて壊れていく。',
        'それはもはや、輪姦というよりも肉のリンチとでもいうべき代物だった。',
      ],
    ]
    script = scripts.sample
    script
  end

  # 洗脳、言葉責め
  def jail_training_daemon_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        '「ヒヒヒッ、あの' + slave.short_name + 'サマが、一日中俺たちのちんぽハメられてヨガってるとはねぇ」',
        monster.name + 'たちは' + slave.short_name + 'を揶揄するように嘲笑う。',
        '「オイィ、まぁたイッたのかよ淫乱の' + slave.short_name + 'ちゃんはぁ。レイプされて感じるとかどうしようもねぇ変態だな、こいつは」',
        '口を塞がれ、反論もできぬまま、言葉で精神を嬲られる屈辱に' + slave.short_name + 'は歯噛みする。',
        'もちろん嬲るものは精神だけではない。' + monster.name + 'たちは、感度を上げる淫紋をいくつも' + slave.short_name + 'の肌に刻むことで快楽を増幅させ、' + slave.short_name + 'が肉悦に溺れるよう謀っている。',
        '精神をいたぶり、肉体に快楽をたたき込み' + slave.short_name + 'を都合の良い肉穴として容赦なく' + monster.name + 'たちは作り変えていった。',
        '「ちんぽシゴき穴としての立場ってもんを徹底的に教え込んでやるから覚悟しておけよオラッ」',
        '地下牢では陵辱の宴が夜通し続いた……',
      ],
      [
        '「うああぁぁぁっ、卑怯っ！　こんなの卑怯だからぁぁっ、うああぁぁ！」',
        slave.short_name + 'はガクガクと震えながら何十度目かの絶頂を迎えた。',
        '全身に感度を上げる淫紋を刻まれたいまの' + slave.short_name + 'の全身は性感帯に等しい感度を持っていた。',
        monster.name + 'の肌が擦れるたびに恐ろしいほどの快楽が発生する。',
        '乳首を摘ままれたり、乱暴に陰核を押し潰されでもすればそれだけで意識がトぶほどの快楽に飲み込まれ絶頂してしまう。',
        monster.name + 'たちはそんな' + slave.short_name + 'をいたぶるように、じっくりと陵辱し尽くしていく。',
        '精神が折れ、屈服し、肉奴隷に自分からなると認めるまで、この淫惨な陵辱劇は続くのだ……',
      ],
    ]

    if slave.san_rate > 50
      scripts += [
        [
          '「うあぁぁぁっ、お尻っ、' + slave.self_call + 'っ、お尻でイくっ！　イくぅぅぅぅっ」',
          slave.short_name + 'はガクガクと震え、尻穴で背徳の絶頂を迎えた。',
          'その様子を見て周囲の' + monster.name + 'たちは一斉に嘲りの言葉を投げかける。',
          '尻穴でイく変態。犯されて感じる淫乱。イく姿を見られて喜ぶ色情狂。',
          '「いやぁぁ……　' + slave.self_call + '、お尻なんかで……　イヤだったのに……　イキたくなかったのに、うぅぅ……」',
          '嗚咽を漏らす' + slave.short_name + 'に、さらに悪魔に似た姿を持つ' + monster.name + 'たちは嗜虐の言葉を投げかける。',
          '彼らは女を貶めるのに、陵辱だけでなく揶揄の言葉が有効だと知っているのだ。',
          '肉体と精神を嬲られながら、' + slave.short_name + 'の精神は削られていった……',
        ],
      ]
    end

    if slave.san_rate < 30
      scripts += [
        [
          '「オラッ、しっかりちんぽを締めつけろやっ！　セックス以外に能のない淫乱メス豚なんだからセックスくらい気合い入れろやっ！」',
          monster.name + 'は容赦なく' + slave.short_name + 'を罵り、肉奴隷として貶める言葉を投げかける。',
          '「はいぃぃ、締めます！　だからっ、だからぁ！　' + slave.short_name + 'のおちんぽシゴき穴をもっとズボズボしてくださいぃぃぃ！」',
          'だが' + slave.short_name + 'はその言葉に迎合し、自らを貶める言葉をさらに上書きする。',
          '理由は' + slave.short_name + 'の首筋に刻まれた淫紋にある。' + slave.short_name + 'を辱め貶める言葉を聞くたびに感じる倒錯した屈服快楽を、えげつなく増幅し、それを至高の悦びに増幅してしまうのだ。',
          'そしてそれに重ねられた、倒錯した屈服快楽への従順と渇望の洗脳。もはや' + slave.short_name + 'にとって、貶められ、辱められることは女として、肉奴隷としての最高の幸福だった。',
          'しかし一度発した言葉は精神に染み込み、貶められたことで悦んだという事実は魂に刻まれる。',
          'たとえ洗脳を解いたとしても、変質した精神や魂は二度と元に戻りはしないのだ。',
          '洗脳と淫紋によって、' + slave.short_name + 'は二度と元に戻れぬよう徹底的に、屈服快楽によって作り変えられていった。',
        ],
      ]
    end

    script = scripts.sample
    script
  end

  def jail_training_ork_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        '「おぉぉっ、またっ、またぁぁぁ………　ああぁぁ、イクッ、イクゥゥっ！」',
        slave.short_name + 'は自分に突き刺さる欲情した' + monster.name + 'の視線を感じながら、何十度目かの絶頂を迎えた。',
        'こんな奴らには屈しない。自分の誇りは、こんな奴らに穢されはしない。',
        'そう信じていた' + slave.short_name + 'の誇りは、とっくの昔に執拗な陵辱の前に踏み砕かれていた。',
        '何度も何度も、執拗に、徹底的に、容赦なく繰り返された陵辱の前に、もはや' + slave.short_name + 'の精神は、限界の瀬戸際に達していた。',
        'もちろん、救出や休息があれば回復もするだろう。だが。そんなものを与える慈悲など' + monster.name + 'が持つはずがないのだ',
        '暴力的な快楽に晒され、己がオスに屈服するべき肉便器だという「肉奴隷」の常識を叩き込まれながら、' + slave.short_name + 'はゆっくりと、' + monster.name + 'にとって都合の良い存在として、作り替えられていった……',
      ],
      [
        '「おああぁぁぁぁっ！　' + monster.name + 'ッ、' + monster.name + 'ちんぽヤバいっ！　こんなのっ、こんなのぉぉぉっ！　ズルいぃぃっ、卑怯よこんなのっ！　気持ちよすぎてぇぇ、' + slave.self_call + 'ッ、うわぁぁぁ！」',
        '前後の穴をズボズボと容赦なくほじくられながら' + slave.short_name + 'は知性のかけらもない声を上げる。',
        'それも当然だろう。' + monster.name + 'たちは朝から夜も更けつつある今の今まで、徹底的に' + slave.short_name + 'を嬲り尽くしてきたのだ。',
        '他種族の女を陵辱し、犯し潰し、快楽に隷属させ、自分たちの子供を孕ませるための肉袋に堕とすためのテクニックを、' + monster.name + 'は恐ろしく高いレベルで備えていた。',
        'それを嫌という程味あわされればどんな女でもチンポの奴隷に堕ちてしまう。ましてや、このダンジョンに潜るまで、男女の交わりというものを全く知らず快楽に免疫のない' + slave.short_name + 'であればそれは仕方のないことなのだ。',
        '圧倒的な実力差のある大人と子供が、本気で勝負をするようなもの。もちろん、強者の側には手加減をする理由など欠片もない。そうであれば、結果など火を見るより明らかだった。',
        monster.name + 'たちは、' + slave.short_name + 'の乱れ狂う反応を楽しみながら、じっくりと女肉を味わい、いたぶり尽くしていった……',
      ],
      [
        '「奥ぅ、奥にぃぃぃっ！　すごいっ、こんな逞しいものでズボズボされたら' + slave.self_call + 'ぃっ、うあぁぁぁ！」',
        slave.short_name + 'は髪を振り乱し、叫ぶようにして快楽の大きさを訴える。',
        slave.short_name + 'を後ろからしっかりと押さえつけ、前後の穴を貫く' + monster.name + 'の二本の野太い肉凶器が凄まじいほどの快楽を引き起こし、' + slave.short_name + 'の理性を容赦なく削り落とし、たっぷりと肉悦の味を刻み込んでいく。',
        monster.name + 'が気持ちよさそうに吠えると、肉棒がひときわ大きく膨らむ。射精。女の子宮に子種汁を流し込み、己の所有物に,堕とすための獣欲の濃縮液。',
        'どぷどぷと煮えたぎる白いマグマが' + slave.short_name + 'の膣内に流し込まれていく。もちろん' + monster.name + '特有の、メスを己の肉棒に忠誠を誓わせるための中毒性のある催淫効果を持つ媚薬精液をこれほど流し込まれては、後遺症が残らないはずがなかった。',
        'だが肉欲に飲まれた' + slave.short_name + 'に理性があげる警鐘を気にすべき知性はとっくの昔に淫熱の前に溶け落ちていた……。',
        '汚らしい' + monster.name + 'の獣じみた遺伝子が自分の遺伝子と混ぜ合わされるおぞましい感覚が膣内射精の本能的な喜悦を呼び起こし、' + slave.short_name + 'は種付けの屈服快楽に酔いしれている。',
      ],
    ]

    if slave.san_rate < 60
      scripts += [
        [
          '「おおオッ、出るっ、射精るぞっ！　全部飲み込めよぉぉっ！」',
          monster.name + 'は高く吠えると肉棒から欲望の滾りを' + slave.short_name + 'の口の中にほとばしらせる。びゅくびゅくと白濁液が、舌に叩きつけられる。生臭い味がたっぷりと' + slave.short_name + 'の味蕾を通じて知覚野を埋め尽くし、脳に濃厚な味覚情報として届けられる。',
          '（生臭くて、ひどい臭いで、腐ったような味……　こんなの、絶対に慣れたくない……）',
          slave.short_name + 'は屈辱と、暴力的な味と臭いの前に顔をしかめるが、頭をがっちりとホールドされていては逃れられるはずもない。',
          'せめて呼吸だけは邪魔されまいと' + slave.short_name + 'は必死に喉を鳴らし、強烈な嫌悪感に耐えながら濃厚な子種汁を飲み干していく。',
          '（絶対に、絶対に……　' + slave.self_call + 'は堕ちない……　こんなの絶対に受け入れられるはずがない、慣れるはずがないもの……）',
          slave.short_name + 'はぜぇぜぇと荒い息を吐きながら、自分が' + monster.name + 'の精液に強烈な嫌悪感を覚えていることに、どこか安堵感を覚えていた。',
          '……だが、' + slave.short_name + 'は理解していなかった。最初の頃こそあれほど嫌悪していた' + monster.name + 'の精液を飲み込むことに、いまや自分がまったく抵抗を覚えていないことを……',
          '自覚のないまま、' + slave.short_name + 'はゆっくりと、だが確実に堕ちていった……。',
        ],
      ]
    end

    if slave.san_rate < 30
      scripts += [
        [
          '「ひぃぃ……　やります、やりますから……　だから、だからせめて痛くしないで……」',
          slave.short_name + 'は自分から' + monster.name + 'の上に跨ると、雄々しすぎる肉槍を己の秘裂に咥え込んだ。',
          '常人よりもふたまわりは大きい肉槍はギチギチと' + slave.short_name + 'の肉壷を内側から押し広げている。擦れるたびに、表面のイボイボが膣襞を刺激し、狂おしいほどの快楽が湧き上がっていく。',
          'それでも' + slave.short_name + 'は己の体を叱咤し、腰を上下に揺り動かす。円を描くように腰をくねらせ、前後に揺することで相手を喜ばせるように肉奉仕する。',
          'ペニスからぬらぬらと滲み出る先走り液はその効果をはっきりと知ればどんな女であっても顔を背けたくなるほどの効果があった。女をペニスの持ち主の所有物、家畜、そして肉オナホに堕とすための悪魔のごとき媚薬。',
          '女を強制的に発情させる効果、精神に作用しオスに逆らえぬ従順さを刻み込む洗脳作用、さらにはオスチンポなしにはいられなくなる禁断効果すらもあった。',
          'それをたっぷりと塗りこまれ、さらに暴力と快楽で躾けられた' + slave.short_name + 'は、いまや' + monster.name + 'にとって、手懐けられた肉玩具も同然なのだ。',
          'これからの陵辱は、調教し、肉奴隷としての礼儀を叩き込み、二度と元に戻ることも出来ぬようにプライドと理性の残骸を粉々に踏み砕く仕上げにすぎない。',
          slave.short_name + 'は、本物の肉奴隷に堕ちる瀬戸際まで追い込まれつつあった……',
        ],
      ]
    end

    script = scripts.sample
    script
  end

  # 洗脳
  def jail_training_mage_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        "薄暗い地下室の暗がりの中で複数の#{monster.name}の影が蠢き、そのたびに女の甘ったるい嬌声が響く。",
        '「うぁぁっ、いいっ、おちんぽしゅごいぃぃぃ……　こんなのっ、こんなのおかしいのぉっ……」',
        '快楽に溺れたトロ顔で' + slave.short_name + 'はつぶやいた。膣内と直腸の中で' + monster.name + 'の肉槍がうごめくたび、背筋が折れそうになるほどの快楽が脳めがけて直撃する。',
        'これほどの暴力的快楽を普通の望まぬセックスで生み出せる訳がなかった。',
        '「らめぇっ、感じちゃッ、感じちゃらめなのにぃぃ……♡」',
        'その声にはもはや確かな抵抗の色は少ない。それもそのはずだった。腹の上に描かれた淡く輝く淫紋。それこそが' + slave.short_name + 'の媚肉を狂わせ、何倍にも増幅された法外な快楽を注ぎ込んでいる源泉だった。',
        '快楽に流されつつある' + slave.short_name + 'に、もう一体の' + monster.name + 'は近く。',
        '「ヒヒッ、まだ言葉を話せるとは随分頑張るねぇ。ほら、ご褒美をやるよ。舌を出しな」',
        monster.name + 'の言葉を、かすむ意識の中で' + slave.short_name + 'は必死に拒否しようとする。しかし' + monster.name + 'が手を' + slave.short_name + 'の頭にかざし、何事か呟く。',
        'それだけで拒絶の意思が霧散した' + slave.short_name + 'は素直に舌を出した。' + monster.name + 'は舌に指先で字を描くようにして、悪魔文字を刻み込む。',
        '「ヒヒヒヒッ。いいねぇ。こいつは精液が舌先に触れただけで絶頂するほどの快楽に変換される淫紋だぜぇ。さぁて、この状態で口で奉仕してもらおうかぁ？」',
        monster.name + 'が取り出した肉凶器を、' + slave.short_name + 'はぼぉっとした頭のままに素直にしゃぶり始めた。舌の上で肉棒を転がし熱心に吸いたてる。',
        '「おおぉぉっ、いいじゃねえか。すぐに射精しちまいそうだぜ」',
        '歪められた意識の中で、' + slave.short_name + 'はじっくりと嬲られていった。',
      ],
      [
        '「へへへっ、次は尻穴の感度をさらに３倍増しだぜぇ。どうだぁ、これやべえだろぉ？」',
        monster.name + 'は' + slave.short_name + 'を嬲るように耳元で囁く。',
        'その効果は悪魔的で、尻穴をえぐる' + monster.name + 'が腰を振るたび、' + slave.short_name + 'はガクガクと快楽に震える。',
        slave.short_name + 'の全身にはすでに何重にも淫紋が刻み込まれていた。常人であれば一つ刻み込まれた他だけで快楽の濁流に屈服してしまう悪魔の刻印。',
        'それを' + monster.name + 'たちは面白半分に' + slave.short_name + 'の全身に刻み込んでいるのだ。',
        'もちろん鍛え抜かれた冒険者である' + slave.short_name + 'はその程度では屈しはしないだろう。',
        'だからこそ、それがいいのだ。',
        monster.name + 'は嬲り甲斐のあるおもちゃを手に入れた悪童のように嘲笑い、' + slave.short_name + 'を陵辱する。早く肉奴隷に堕ちてしまえばそれでいいし、長持ちすればその分長く遊べる。',
        monster.name + 'たちの、底のない悪意に晒されながら、' + slave.short_name + 'はゆっくりと、徹底的に嬲り抜かれていく……',
      ],
    ]

    if slave.san_rate < 60
      scripts += [
        [
          slave.short_name + 'は陵辱者たちに秘穴と尻穴の前後に加え、両手で二本の肉茎をシゴきつつ、口で' + monster.name + 'の肉棒に奉仕している',
          'じゅるっ、じゅぶぶぶぶっ、ぐぶぅ、ぐぽぉっ……　ぐぷぅ、じゅぼぼっ、じゅるるるっ……',
          '唾液の混じった卑猥な音を響かせながら必死に口腔奉仕する' + slave.short_name + 'は、まるで淫乱のようだ。だがそれも当然のことだった。',
          '「ヒヒヒッ、俺をイカせねぇと、お前もイケねぇんだぜぇ？　フェラが下手で、イくこともできずお預けされすぎて頭が壊れたやつは何人もいるからなぁ。お前はそうならねぇといいなぁ、へへっ」',
          monster.name + 'は' + slave.short_name + 'を嘲笑うようにいった。' + slave.short_name + 'に刻まれた淫紋は、飲精の淫紋。',
          '口内に射精されなければ、決してイクこともできない呪い。それでいて、絶頂するはずだった快楽は肉体に蓄積し、口内に射精された瞬間何倍にも増幅され襲いかかるという悪辣な代物だった。',
          '「オラオラ、何回分絶頂を我慢してんだぁ？　そろそろイカねぇと、本当に頭が壊れちまうかもなぁ？　おらっ、真面目に奉仕しろや」',
          monster.name + 'はそういうが、すでに全身に鉛のように疲労が蓄積している。' + slave.short_name + 'にとってはもはや気力だけで頭を前後させ、必死に口で奉仕している状態なのだ。',
          '「チッ、しょうがねぇなぁ。本物のフェラ奉仕ってやつを、俺が仕込んでやるぜェ」',
          'そういうと' + monster.name + 'は' + slave.short_name + 'の頭をがっちり掴む。抗議の言葉を言う余裕もなかった。',
          monster.name + 'は、まるでオナホを使うように' + slave.short_name + 'の喉奥に肉槍を叩きつけ始めた。喉奥の粘膜をえぐられる苦しみ、恐ろしい嘔吐感。',
          'だが' + monster.name + 'は止まらない。己の快楽だけを追い求め、' + slave.short_name + 'を使い潰すように使い続ける。',
          '……',
          '一時間もする頃には、' + slave.short_name + 'は単なる肉玩具、あるいはそれ以下の存在として' + monster.name + 'に使われていた。',
          'フェラチオ奉仕から解放されれば、必死に絶頂を懇願する肉奴隷。口内に射精されるたびに壊れたばね仕掛けのおもちゃのようにデタラメに痙攣する射精のための玩具。口を使われるたび、滑稽なほど必死に奉仕するフェラチオ奴隷。',
          '刻まれた淫紋のままに、必死に精液を求めて男たちに全身で奉仕する' + slave.short_name + 'で、' + monster.name + 'たちはたっぷりと楽しんだ……',
        ],
      ]
    end  

    if slave.san_rate < 30
      scripts += [
        [
          '「うあああ、あひっ、あひぃぃ……」',
          '「ちんぽっ、ちんぽすごいっ、いひぃぃぃぃ……」',
          slave.short_name + 'は全身を大きく弓のように反らせると、ブルリと震えた',
          '「ああぁぁぁっ、イクっ、' + slave.self_call + '、またっ、イクゥっ！」',
          slave.short_name + 'は甘く吠えると、ガクガクと痙攣しながら激しく絶頂した。',
          '繰り返された陵辱により、与えられる快楽を受け入れ絶頂することも、憎い敵の前で絶頂を訴えることも、もはや抵抗がなくなっていた。',
          'いや抵抗するだけ無駄だと思い知らされてしまったのだ。',
          '女の肉体は容赦のない快楽を休みなく与えられることによって男にとって都合の良いメスへと作り変えられていく。',
          '「ヒヒヒッ、またイッちまうとはどうしようもねぇ淫乱だなァ。ほおら罰ゲームだぜぇ？」',
          monster.name + 'は' + slave.short_name + 'の白い肌に魔術紋様を刻み込んでいく。それが女体にとって有害でまずい代物であるということは' +
            '詳しい効果を知らない者でも想像がつくだろう。ましてや何度も刻み込まれていた' + slave.short_name + 'は、その悪魔的な効果にブルリと震える。',
          '「あぁぁぁ…… いやっ、もういやなのぉ……」',
          '体がゾクリと震える。魔術紋様の効果、感度の増幅だ。',
          '先ほどから' + monster.name + 'は' + slave.short_name + 'が絶頂するたびに、様々な魔術紋様を刻み込んで来た。快楽の増幅、感度の鋭敏化、精液愛好の味覚変化。' +
            'その効果は容赦なく' + slave.short_name + 'の肉体に影響していく。これほど何度も刻まれては、効果が永続する副作用も起きるだろう。',
          '「へへっ、いやならイクのを我慢すればいいだけだろぉ？ 簡単なことじゃねえか。それとも、イキまくるのを我慢できねえ淫乱なのかよ、' + slave.short_name + 'チャンはよぉ？」',
          monster.name + 'は意地悪く笑う。',
          'すでに呆れるほどの絶頂を迎え、肉の悦びの味を覚えてしまった' + slave.short_name + 'の肉体は、とっくの昔にブレーキが効かなくなっていた。',
          'ましてやこれほどまでに快楽増幅や感度敏感の効果を刻まれているのだ。絶頂をおさえることなどできるはずがない。',
          'それを知っていて、' + monster.name + 'は結果の見えている出来試合を' + slave.short_name + 'に強要しているのだ。',
          'だが' + slave.short_name + 'にはイカサマだらけの不公平な出来試合を拒否する権利も逃れる術もない。',
          'ただただ、凄惨な肉のリンチによって快楽に狂わされる。快楽が快楽を呼び、絶頂のたびに状況が悪化する。',
          'ただひたすらに甘く肉を狂わされ、叫びたくなるような肉悦地獄の中で、' + slave.short_name + 'はゆっくりと嬲られていった',
        ],
      ]
    end  

    scripts.sample
  end  

  # 巨根、絶倫、大量射精
  def jail_training_beastman_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        monster.name + 'は射精をまた迎えたようだ。今日何十度目かになる' + slave.short_name + 'の子をなすための場所への欲望の白濁汁がたっぷりと吐き出されていく。',
        '「うぁぁぁ……　熱い、熱いぃぃ、精液がぁぁ、こんなにいっぱいぃぃ……」',
        monster.name + 'を含む獣人種族は種族の特徴としてオスしかいない。種族として存続するために人間やエルフの女をさらい、自分たちの子供を孕ませる子袋として使うのだ。',
        'そのためにその繁殖力は凄まじく、人間の女であれば一月と持たずに確実に妊娠してしまう。こんなやつらの子供を孕む。それは' + slave.short_name + 'にとって想像するだけでも恐ろしいことだった……',
        '「いやっ、いやぁぁぁ……　こんなっ、こんなやつらの子供を孕むなんていやぁぁ……」',
        'どぷどぷと大量の精液が' + slave.short_name + 'の膣内に注ぎ込まれていく感覚に' + slave.short_name + 'はおぞましさと妊娠の恐怖を感じながら震えていた……',
      ],
      [
        '「ひぃぃぃっ、いやっ、いやぁぁぁ……」',
        '地下牢には' + slave.short_name + 'の悲鳴と、' + monster.name + 'たちの興奮した声と息遣いが響く。',
        monster.name + 'たちは、久々にありついた肉穴、それもとびきりの美女である' + slave.short_name + 'の穴を争うように何度も何度も貫き、耕し、掻き回している。',
        '痛みと屈辱、そしてだんだんと痛みにとってかわるようになった被虐の肉悦に支配されている' + slave.short_name + 'にとってそれは地獄の苦しみにちがいない。',
        'だが' + monster.name + 'には、そんなことは気にすることではない。ただ目の前の肉を味わい、肉襞を堪能し、その穴に白濁汁を注ぎ込むことで相手の肉体を陵辱し、屈服させ、己の子供を孕ませようとしている。',
        '原始的で本能に基づいた暴力的な快楽に晒されながら、強酸に溶かされる美しい大理石の彫像のように' + slave.short_name + 'の精神は、削られていった……',
      ],
      [
        'どぷ、びゅるるるっ……　びゅく、びゅるるっ、どぷどぷどぷどぷっ……',
        '他種族のメスを孕ませるための、強烈な繁殖力を持つオスの精液が、' + slave.short_name + 'の膣内へと注ぎ込まれていく……',
        '「あぁぁ……　うあぁぁ……　ああ゛ぁ、おあ゛あ゛あぁぁぁぁ……」',
        '膣内で、獣人の精子が自分の卵子を食い荒らし、' + slave.short_name + 'の遺伝子が、凶暴な陵辱者のそれと混ぜ合わされていく感覚をたしかに' + slave.short_name + 'は覚えていたが、まともな言葉を発することはできなかった。',
        '朝から' + monster.name + 'に叩き起こされ、何度も何度も呆れるほど執拗に犯され、徹底的に陵辱され、意識は原型を留めぬほどぐちゃぐちゃにされてしまっていた。体力はとっくに底を尽き、指一本すらも動かすこともできない。',
        'それでも' + monster.name + 'たちは' + slave.short_name + 'に、何の容赦もなくその媚肉を貪り続けていた。',
        'ただですら女のいない地下深くのダンジョンに閉じ込められたまま、無尽蔵の精力と体力を持て余してきた' + monster.name + 'にとって、' + slave.short_name + 'はようやく与えられた美味なる獲物だった。',
        '手加減などされるはずもなく徹底的に嬲られ続けながら、' + slave.short_name + 'は、自分の精神が壊れていく音を、どこか他人事のように聞いていた……',
      ],

    ]

    if slave.san_rate < 30
      scripts += [
        [
          '「うあぁぁぁ……　獣くさい精液ぃぃ……　また' + slave.self_call + 'の中に注ぎ込まれてるぅ……」',
          slave.short_name + 'は己の膣内をこってりとした獣人の精液でマーキングされる感覚に、目を細めつぶやいた。',
          'その顔には以前ほどの嫌悪感はなく、むしろ背徳と屈服の倒錯した悦びに飲まれかけているようにも見えた……',
        ],
      ]
    end  

    scripts.sample
  end

  # ウルクは基本オークなので、オークベース
  def jail_training_urk_script(slave, monster_party)
    monster = monster_party.monster
    scripts = [
      [
        '「おぉぉっ、またっ、またぁぁぁ………　ああぁぁ、イクッ、イクゥゥっ！」',
        slave.short_name + 'は自分に突き刺さる欲情した' + monster.name + 'の視線を感じながら、何十度目かの絶頂を迎えた。',
        'こんな奴らには屈しない。自分の誇りは、こんな奴らに穢されはしない。',
        'そう信じていた' + slave.short_name + 'の誇りは、とっくの昔に執拗な陵辱の前に踏み砕かれていた。',
        '何度も何度も、執拗に、徹底的に、容赦なく繰り返された陵辱の前に、もはや' + slave.short_name + 'の精神は、限界の瀬戸際に達していた。',
        'もちろん、救出や休息があれば回復もするだろう。だが。そんなものを与える慈悲など' + monster.name + 'が持つはずがないのだ',
        '暴力的な快楽に晒され、己がオスに屈服するべき肉便器だという「肉奴隷」の常識を叩き込まれながら、' + slave.short_name + 'はゆっくりと、' + monster.name + 'にとって都合の良い存在として、作り替えられていった……',
      ],
      [
        '「おああぁぁぁぁっ！　' + monster.name + 'ッ、' + monster.name + 'ちんぽヤバいっ！　こんなのっ、こんなのぉぉぉっ！　ズルいぃぃっ、卑怯よこんなのっ！　気持ちよすぎてぇぇ、' + slave.self_call + 'ッ、うわぁぁぁ！」',
        '前後の穴をズボズボと容赦なくほじくられながら' + slave.short_name + 'は知性のかけらもない声を上げる。',
        'それも当然だろう。' + monster.name + 'たちは朝から夜も更けつつある今の今まで、徹底的に' + slave.short_name + 'を嬲り尽くしてきたのだ。',
        '他種族の女を陵辱し、犯し潰し、快楽に隷属させ、自分たちの子供を孕ませるための肉袋に堕とすためのテクニックを、' + monster.name + 'は恐ろしく高いレベルで備えていた。',
        'それを嫌という程味あわされればどんな女でもチンポの奴隷に堕ちてしまう。ましてや、このダンジョンに潜るまで、男女の交わりというものを全く知らない免疫のない' + slave.short_name + 'であればそれは仕方のないことなのだ。',
        '圧倒的な実力差のある大人と子供が、本気で勝負をするようなもの。もちろん、強者の側には手加減をする理由など欠片もない。そうであれば、結果など火を見るより明らかだった。',
        monster.name + 'たちは、' + slave.short_name + 'の乱れ狂う反応を楽しみながら、じっくりと女肉を味わい、いたぶり尽くしていった……',
      ],
      [
        '「奥ぅ、奥にぃぃぃっ！　すごいっ、こんな逞しいものでズボズボされたら' + slave.self_call + 'ぃっ、うあぁぁぁ！」',
        slave.short_name + 'は髪を振り乱し、叫ぶようにして快楽の大きさを訴える。',
        slave.short_name + 'を後ろからしっかりと押さえつけ、前後の穴を貫く' + monster.name + 'の二本の野太い肉凶器が凄まじいほどの快楽を引き起こし、' + slave.short_name + 'の理性を容赦なく削り落とし、たっぷりと肉悦の味を刻み込んでいく。',
        monster.name + 'が気持ちよさそうに吠えると、肉棒がひときわ大きく膨らむ。射精。女の子宮に子種汁を流し込み、己の所有物に,堕とすための獣欲の濃縮液。',
        'どぷどぷと煮えたぎる白いマグマが' + slave.short_name + 'の膣内に流し込まれていく。もちろん' + monster.name + '特有の、メスを己の肉棒に忠誠を誓わせるための中毒性のある催淫効果を持つ媚薬精液をこれほど流し込まれては、後遺症が残らないはずがなかった。',
        'だが肉欲に飲まれた' + slave.short_name + 'に理性があげる警鐘を気にすべき知性はとっくの昔に淫熱の前に溶け落ちていた……。',
        '汚らしい' + monster.name + 'の獣じみた遺伝子が自分の遺伝子と混ぜ合わされるおぞましい感覚が膣内射精の本能的な喜悦を呼び起こし、' + slave.short_name + 'は種付けの屈服快楽に酔いしれている。',
      ],
    ]

    if slave.san_rate < 60
      scripts += [
        [
          '「おおオッ、出るっ、射精るぞっ！　全部飲み込めよぉぉっ！」',
          monster.name + 'は高く吠えると肉棒から欲望の滾りを' + slave.short_name + 'の口の中にほとばしらせる。びゅくびゅくと白濁液が、舌に叩きつけられる。生臭い味がたっぷりと' + slave.short_name + 'の味蕾を通じて知覚野を埋め尽くし、脳に濃厚な味覚情報として届けられる。',
          '（生臭くて、ひどい臭いで、腐ったような味……　こんなの、絶対に慣れたくない……）',
          slave.short_name + 'は屈辱と、暴力的な味と臭いの前に顔をしかめるが、頭をがっちりとホールドされていては逃れられるはずもない。',
          'せめて呼吸だけは邪魔されまいと' + slave.short_name + 'は必死に喉を鳴らし、強烈な嫌悪感に耐えながら濃厚な子種汁を飲み干していく。',
          '（絶対に、絶対に……　' + slave.self_call + 'は堕ちない……　こんなの絶対に受け入れられるはずがない、慣れるはずがないもの……）',
          slave.short_name + 'はぜぇぜぇと荒い息を吐きながら、自分が' + monster.name + 'の精液に強烈な嫌悪感を覚えていることに、どこか安堵感を覚えていた。',
          '……だが、' + slave.short_name + 'は理解していなかった。最初の頃こそあれほど嫌悪していた' + monster.name + 'の精液を飲み込むことに、いまや自分がまったく抵抗を覚えていないことを……',
          '自覚のないまま、' + slave.short_name + 'はゆっくりと、だが確実に堕ちていった……。',
        ],
      ]
    end

    if slave.san_rate < 30
      scripts += [
        [
          '「ひぃぃ……　やります、やりますから……　だから、だからせめて痛くしないで……」',
          slave.short_name + 'は自分から' + monster.name + 'の上に跨ると、雄々しすぎる肉槍を己の秘裂に咥え込んだ。',
          '常人よりもふたまわりは大きい肉槍はギチギチと' + slave.short_name + 'の肉壷を内側から押し広げている。擦れるたびに、表面のイボイボが膣襞を刺激し、狂おしいほどの快楽が湧き上がっていく。',
          'それでも' + slave.short_name + 'は己の体を叱咤し、腰を上下に揺り動かす。円を描くように腰をくねらせ、前後に揺することで相手を喜ばせるように肉奉仕する。',
          'ペニスからぬらぬらと滲み出る先走り液はその効果をはっきりと知ればどんな女であっても顔を背けたくなるほどの効果があった。女をペニスの持ち主の所有物、家畜、そして肉オナホに堕とすための悪魔のごとき媚薬。',
          '女を強制的に発情させる効果、精神に作用しオスに逆らえぬ従順さを刻み込む洗脳作用、さらにはオスチンポなしにはいられなくなる禁断効果すらもあった。',
          'それをたっぷりと塗りこまれ、さらに暴力と快楽で躾けられた' + slave.short_name + 'は、いまや' + monster.name + 'にとって、手懐けられた肉玩具も同然なのだ。',
          'これからの陵辱は、調教し、肉奴隷としての礼儀を叩き込み、二度と元に戻ることも出来ぬようにプライドと理性の残骸を粉々に踏み砕く仕上げにすぎない。',
          slave.short_name + 'は、本物の肉奴隷に堕ちる瀬戸際まで追い込まれつつあった……',
        ],
      ]
    end

    script = scripts.sample
    script
  end
  # ====== 地下牢で凌辱中 おわり ===================

  # slave_servant用
  def betrayed_servant_battle_opening_script(slave)
    [
      [
        'かつての仲間を、自分の「ご主人様」のために肉奴隷に堕とす。それこそが自らの崇高な使命であると' + slave.short_name +
          'は今や本気で信じていた。',
        '男に犯され、肉体で奉仕し、子種汁を注ぎ込んでいただく。女として至尊の幸福を仲間たちに教えこむべく、' + slave.short_name +
          'は武器を構えた。かつての仲間たちが、かつての自分と同様にどうしようもなく頑迷で愚昧であることはわかっていた。荒療治は仕方がないのだ。',
      ],
      [
        '女とは、男には逆らえぬものだ。 ',
        '女とは、男に屈服し、服従し、奉仕するために生まれてきたものだ。',
        'それがこの世の真理なのだ。それを教え込んでやろうというのに、どうして不満や反抗の態度をかつての仲間たちが示すのか' +
        slave.short_name + 'には本気で分からなかった。',
        'まあいい。愚か者の意思など重要ではない。犯され、快楽で躾けられ、身の程を教え込まれれば、いずれ分かることなのだから。',
        slave.short_name + 'はニコリと、底の知れぬ悪意に満ちた笑いを浮かべた。',
      ],
      [
        slave.short_name + 'は驚きによって表情が固まるかつての仲間たちの前で、不敵に笑った。',
      ],
    ].sample
  end

  def battle_prelude_script(adventurer, is_only, brain_washing_active_member_is_majority)
    []
  end

  def battle_prelude_only_n_washed_script(adventurer)
    []
  end

  def battle_prelude_all_washed_script(adventurer)
    []
  end
end
