# coding: utf-8
require 'src/defines/script/dungeon/script_utils.rb'

class CaptureScript
  include ScriptUtils

  def capture_event(adventurer, party_info, monster_party_info, capture_count)
    party = party_info[:party]
    script = [
      '（捕獲逃走判定：失敗）',
    ]
    if party_info[:is_only_down_members]
      script += [
        monster_party_info[:monster_name] + 'の迫り来る魔の手から、動かぬ体を叱咤しながら' + adventurer.short_name + 'は必死に逃げる',
        adventurer.short_name + 'は仲間が助けてはくれないかと周囲をみるが、既に仲間も倒れており助ける余裕はないようだ',
      ]
    end

    script += [
      half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
        'に襲われた' + adventurer.short_name + 'は抵抗も虚しく捕まってしまった。',
    ]

    unless party_info[:is_no_companions]
      script += [
        '仲間たちは' + adventurer.short_name + 'を助けようとしたが、他の' + monster_party_info[:monster_name] + 'に阻まれ助けることができなかった',
      ]
    end
    script
  end

  def _capture_event_passenger_message_common(adventurer, is_lonely)
    return [] if adventurer.nil?

    script = [
      '仲間が倒されたことで、戦線が崩れ始めた。' +
        '傷つき、戦うことのできない' + adventurer.short_name + 'に魔物たちが気づくと、そちらにも向かっていく。',
      ]

    unless is_lonely
      script += [
        '他の仲間たちも' + adventurer.short_name + 'を守ろうとするが、敵に押され手が出せない！',
      ]
    end
    script += [
      '敵が' + adventurer.short_name + 'に向かって群がり始めた',
    ]
    script
  end

  def already_down_capture_message_common(adventurer, is_lonely)
    _capture_event_passenger_message_common(adventurer, is_lonely)
  end

  def capture_event_of_passenger(adventurer, party_info, monster_party_info, capture_count)
    script = _capture_event_passenger_message_common(adventurer, party_info[:is_lonely]) + [
      '（捕獲逃走判定：失敗）',
    ]
    if !(party_info[:is_no_companions]) && party_info[:is_only_down_members]
      script += [
        monster_party_info[:monster_name] + 'の迫り来る魔の手から、動かぬ体を叱咤しながら' + adventurer.short_name + 'は必死に逃げる',
        adventurer.short_name + 'は仲間が助けてはくれないかと周囲をみるが、既に仲間も倒れており助ける余裕はないようだ',
      ]
    end

    script += [
      half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
        'に襲われた' + adventurer.short_name + 'は抵抗も虚しく捕まってしまった。',
    ]

    unless party_info[:is_no_companions]
      script += [
        '仲間たちは' + adventurer.short_name + 'を助けようとしたが、他の' + monster_party_info[:monster_name] + 'に阻まれ助けることができなかった',
      ]
    end
    script
  end

  def capture_escape_event(adventurer, party_info, monster_party_info, capture_count, is_merged)
    script =
      if monster_party_info[:is_indifferent]
        [
          '（捕獲逃走判定：成功）',
          half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
            'に打ち倒された' + adventurer.short_name + 'は敵の手から逃げ出した',
          monster_party_info[:monster_name] + 'は追撃してこない。女の体には興味がないようだ……',
        ]
      else
        if capture_count <= 0
          [
            '（捕獲逃走判定：成功）',
            adventurer.short_name + 'は' + monster_party_info[:monster_name] +
              'に襲われたが、自分に向かってきた敵が少なかったおかげで辛くも魔の手から逃げ出した',
          ]
        else
          [
            '（捕獲逃走判定：成功）',
            half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
              'に襲われた' + adventurer.short_name + 'は辛くも魔の手から逃げ出した',
          ]
        end
      end

=begin
    script +=
      if is_lonely
        [
          'もはや疲れ果て、今にも倒れそうな体を引きずりながら' + adventurer.short_name + 'はフラフラと地上を目指して再び歩みを進め始めた……',
        ]
      else
        [
          '無事逃げおおせた' + adventurer.short_name + 'は' +
          '他の仲間も無事に地上にたどり着けるよう祈りながら、' + adventurer.short_name + 'は地上を目指して歩き始めた……',
        ]
      end
=end
    party = party_info[:party]
    real_is_merged = is_merged &&
      # パーティーが消滅していたり、構成員が全滅したケースは合流予定はなしになる
      !(party.nil? || party.no_companions?)
    _merge_or_not_script(adventurer, real_is_merged)
    script
  end

  def _merge_or_not_script(adventurer, is_merged)
    if is_merged
      [
        '（合流判定：成功）',
        '逃げおおせた' + adventurer.short_name + 'は首尾よく他の仲間たちと合流することができた。',
      ]
    else
      [
        '（合流判定：失敗）',
        adventurer.short_name + 'は不安げに周囲を見回す。',
      ]
    end
  end

  def capture_escape_event_of_passenger(adventurer, monster_party_info, capture_count, is_lonely)
    script =
      if monster_party_info[:is_indifferent]
        [
          '（捕獲逃走判定：成功）',
          half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
            'に打ち倒された' + adventurer.short_name + 'は敵の手から逃げ出した',
          monster_party_info[:monster_name] + 'は追撃してこない。女の体には興味がないようだ……',
        ]
      else
        if capture_count <= 0
          [
            '（捕獲逃走判定：成功）',
            adventurer.short_name + 'は' + monster_party_info[:monster_name] +
              'に襲われたが、自分に向かってきた敵が少なかったおかげで辛くも魔の手から逃げ出した',
          ]
        else
          [
            '（捕獲逃走判定：成功）',
            half_number_to_full(capture_count) + monster_party_info[:unit_name] + 'の' + monster_party_info[:monster_name] +
              'に襲われた' + adventurer.short_name + 'は辛くも魔の手から逃げ出した',
          ]
        end
      end

    script +=
      if is_lonely
        [
          'もはや疲れ果て、今にも倒れそうな体を引きずりながら' + adventurer.short_name + 'はフラフラと地上を目指して再び歩みを進め始めた……',
        ]
      else
        [
          '無事逃げおおせた' + adventurer.short_name + 'は' +
            '他の仲間も無事に地上にたどり着けるよう祈りながら、' + adventurer.short_name + 'は地上を目指して歩き始めた……',
        ]
      end
    script
  end

  def get_party_summaries(parties_list_by_floor)
    res = {}
    parties_list_by_floor.sort.each do |floor, parties|
      parties.values.each do |party|
        party_status_text =  _party_status(party)
        floor_text =
          if floor.zero?
            '地上'
          else
            "#{half_number_to_full(floor)}階"
          end
        res[party.identify_code] = [party_status_text, ("#{floor_text}　#{party.leader.short_name}隊（#{_party_status(party)}）　" + party_status_text)]
      end
    end
    res
  end

  def _party_status_name(party)
    text = "結成#{half_number_to_full(party.turn_count)}日目"
    text
  end

  def _party_status(party)
    if party.in_rescue?
      '救出行動'
    elsif party.in_dive?
      '探索中'
    elsif party.in_distress?
      '遭難中'
    elsif party.lonely?
      '孤立中'
    else
      '撤退中'
    end
  end

  def get_jail_summaries(jails_list_by_floor)
    res = {}
    jails_list_by_floor.sort.each do |floor, rooms|
      rooms.values.each do |room|
        res[room.identify_code] = {
          adventurer_id: room.adventurer.id,
          text: ("#{half_number_to_full(floor)}階　#{room.adventurer.short_name}　" + _capture_status_name(room))
        }
      end
    end
    res
  end

  def _capture_status_name(room)
    text = "監禁#{half_number_to_full(room.day_count)}日目"
    text += "（陵辱調教#{half_number_to_full(room.force_sex_day_count)}日目）" unless room.in_resist?
    text
  end

  def party_progress_status(parties, identify_code, status_text, party_text)
    party = parties.values.find { |party| party.identify_code == identify_code }
    status_name =
      if party.is_blank_members? # メンバーなし＝全滅。
        ['全滅']
      else
        _party_progress_status_name(party)
      end

    if status_name.empty?
      if %w(遭難中 孤立中).include?(status_text)
        nil
      else
        "#{party_text}"
      end
    else
      "#{party_text} →#{status_name.join('、')}"
    end

  end

  def _party_progress_status_name(party)
    status = []
    status << '反転' if party.return_in_this_turn?
    status << '敗北' if party.lose_n_lost_member_in_this_turn?
    status << '罠で被害' if party.get_trap_sign_in_this_turn?
    status << '落とし穴！' if party.lost_member_by_trap?
    status << '地上に到着' if party.in_ground?
    status
  end

  def jail_room_status(jail, identify_code, adventurer_id, target_room_text)
    jail_rooms = jail.jail_rooms
    room = jail_rooms.values.find { |room| room.identify_code == identify_code }
    status_name =
      if room.nil? # 解放済み
        rescue_party_leader = jail.success_rescue_list[adventurer_id]
        rescue_party_leader.short_name + '隊が救出に成功'
      else
        _jail_room_status_name(room)
      end
    text =
      if status_name.empty?
        "#{target_room_text}"
      else
        "#{target_room_text} （#{status_name}）"
      end
    if room && room.betrayed_servant
      room.betrayed_servant.set_betrayed_jail_text(room.adventurer, room.monster_party.monster)
      text = '[♂]' + text
    end
    text
  end

  def _jail_room_status_name(target_room)
    if target_room.status_in_this_turn.nil?
      ''
    elsif target_room.status_in_this_turn == 'abort'
      target_room.rescue_party_leader.short_name +
        '隊は救出を断念'
    elsif target_room.status_in_this_turn == 'repulse'
      target_room.rescue_party_leader.short_name +
        '隊は返り討ちにあった'
    else
      puts "======! alert !======== target_room.status is #{target_room.status_in_this_turn}"
      '(不明)'
    end
  end

  def capture_summaries_header(day_count)
    [
      "＝＝＝＝＝＝＝＝＝＝＝＝#{half_number_to_full(day_count)}日目の地下牢＝＝＝＝＝＝＝＝＝＝＝＝"
    ]
  end

  def capture_summaries_footer(day_count)
    return [] if true

    text =
      if day_count < 10
        [
          '＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝'
        ]
      else
        [
          '＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝'
        ]
      end
    text + [
      ' ',
    ]
  end

  def party_summaries_header(day_count)
    [
      "＝＝＝＝＝＝＝＝＝＝＝＝#{half_number_to_full(day_count)}日目のパーティーたち＝＝＝＝＝＝＝＝＝＝＝＝",
    ]
  end

  def party_summaries_footer(day_count)
    if day_count < 10
      [
        '＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝',
        ' ',
      ]
    else
      [
        '＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝＝',
        ' ',
      ]
    end
  end
end