# coding: utf-8
require 'src/defines/script/dungeon/script_utils.rb'
require_rrk 'src/app/helper/game_define_helper'

class MenuSubScript
  include ScriptUtils
  include GameDefineHelper

  def _adv_display_name(adventurer)
    text =
      if FGO_MODE
        "#{adventurer.name}[#{adventurer.short_class_type}]"
      else
        adventurer.name
      end
    text = text + "[#{adventurer.hooter_level_short_name_plus}]"
    if adventurer.in_brain_washing?
      text = '[洗]' + text
    end
    text
  end

  def adventurer_in_bar_text(adventurer)
    return adventurer_original_in_bar_text(adventurer) if ORIGINAL_MODE

    if adventurer.in_rest_or_ready?
      _adv_display_name(adventurer) + "（休養中#{half_number_to_full(adventurer.rest_turns)}ターン：Lv.#{adventurer.level}）"
    else
      _adv_display_name(adventurer) + "（#{_level_text(adventurer)}）"
    end
  end

  def adventurer_original_in_bar_text(adventurer)
    original_status_text = ''
    if adventurer.in_brain_washing?
      original_status_text += '[洗]'
    end
    if adventurer.has_talent?(:royal)
      original_status_text += '[王族]'
    end
    original_status_text +=
      if adventurer.in_rest_or_ready?
        adventurer.class_name + _adv_display_name(adventurer) + "（休養中#{half_number_to_full(adventurer.rest_turns)}ターン：#{_level_text(adventurer)}）"
      else
        adventurer.class_name + _adv_display_name(adventurer) + "（#{_level_text(adventurer)}）"
      end
    original_status_text
  end

  def _level_text(adventurer)
    "Lv.#{adventurer.level}"
  end

  def adventurer_in_dungeon_text(params, party)
    adventurer = params[:adventurer]
    floor_text =
      if params[:floor].zero?
        '地上'
      else
        "#{params[:floor]}F"
      end
    squad_text =
      if party.nil?
        "#{params[:leader_name]}隊"
      elsif party.lonely?
        if adventurer.is_down?
          '遭難中'
        else
          '孤立中'
        end
      else
        "#{params[:leader_name]}隊"
      end

    status_text =
      if ORIGINAL_MODE
        original_status_text = ''
        if adventurer.in_brain_washing?
          original_status_text += '[洗]'
        end
        if adventurer.has_talent?(:royal)
          original_status_text += '[王族]'
        end
        original_status_text +=
          adventurer.class_name + _adv_display_name(adventurer) +
          "（#{floor_text}、Lv.#{adventurer.level}、#{squad_text}）"
        original_status_text
      else
        _adv_display_name(adventurer) +
          "（#{floor_text}、Lv.#{adventurer.level}、#{squad_text}、#{adventurer.class_name}）"
      end
    if adventurer.is_bounty_target?
      status_text = '[賞金]' + status_text
    end
    status_text
  end

  def adventurer_in_jail_text(params, show_brands_mode=false)
    adventurer = params[:adventurer]
    brand_tattoo_text =
      if show_brands_mode
        if adventurer.tattoos.empty?
          ''
        else
          '【' + adventurer.tattoos.map {|t| tattoo_short_name(t) }.join(', ') + '】'
        end
      else
        ''
      end

    text =
      _adv_display_name(adventurer) +
        brand_tattoo_text +
        "（#{params[:floor]}F、Ｌｖ．#{adventurer.level} #{_fall_level_text(params)}、#{jail_summary_text(params)}）"

    if params[:betrayed_servant]
      text = '[♂' + params[:betrayed_servant].short_name.slice(0..3) + ']' + text
    end

    if ORIGINAL_MODE
      text = adventurer.class_name + text
    end
    text
  end

  def jail_summary_text(params)
    "#{params[:monster_short_name]}#{params[:monster_count]}が監禁（#{_jail_day_text(params)}日目"
  end

  def _jail_day_text(params)
    if params[:in_resist]
      "監禁#{params[:day_count]}"
    else
      "陵辱調教#{params[:day_count]}"
    end
  end

  def betray_servant_text(slave)
    text =
      _adv_display_name(slave) +
        "[ＨＰ：#{slave.life_point} ／ #{slave.life_point_limit}]" +
        "（#{_level_text(slave)}、#{slave.betrayed_status_text}）"

    text = '[♂]' + text if slave.has_betrayed_penis?

    text
  end

  def betray_servant_penis_text(slave)
    base_text =
      _adv_display_name(slave) +
        "[ＨＰ：#{slave.life_point} ／ #{slave.life_point_limit}]" +
        "（#{_level_text(slave)}）"

    text = base_text +
      if slave.has_betrayed_penis?
        'の陰核を元に戻す'
      else
        'をふたなりにする'
      end

    text = '[ふたなり]' + text if slave.has_betrayed_penis?

    text
  end

  def adventurer_in_magic_factory_text(params)
    slave = params[:adventurer]
    status_text = slave.status_text_in_bar(slave.status)
    "#{_adv_display_name(slave)}（#{status_text}）"
  end

  def adventurer_in_slave_text(params)
    rumor_text =
      if params[:buyer_name] == :slave_merchant
        '奴隷オークションにかけられるという噂'
      else
        "#{params[:buyer_name]}に売られたという噂"
      end
    adventurer = params[:adventurer]
    "#{_adv_display_name(adventurer)}（#{rumor_text}。元#{_level_text(adventurer)}、#{adventurer.class_name}）"
  end

  def _fall_level_text(params)
    if params[:in_resist]
      "抵抗：#{half_number_to_full(params[:fall_level])}"
    else
      "正気：#{half_number_to_full(params[:fall_level])}"
    end
  end

  def tattoo_short_name(tattoo)
    case tattoo
      when :sensitive
        '敏感'
      when :sperm_orgasm
        '精液'
      when :rat
        '発情'
      when :rape_desire
        '願望'
      when :anal_sex
        '肛虐'
      when :brainwashing
        '洗脳'
      when :reward_of_ecstasy
        '快楽'
      when :reward_of_infection
        '汚染'
      else
        puts "INVALID TATTOO NAME:#{tattoo}, #{tattoo.class}"
        ''
    end
  end
end
