# coding: utf-8
require 'src/defines/script/dungeon/script_utils'
require 'src/app/helper/game_define_helper'

class ProgressScript
  include ScriptUtils
  include GameDefineHelper

  def in_rest(party)
    [
      "(#{party.leader.short_name}たちは地上で休憩中)",
    ]
  end

  def is_merged(adventurer)
    [
      '（合流判定：成功）',
      '逃げおおせた' + adventurer.short_name + 'は首尾よく他の仲間たちと合流することができた。',
      '彼女たちは意図して明るく振る舞いながら地上を目指して歩き始めた。',
      'だがすでに戦う力の大半を失い傷つき疲れ果てた彼女たちが無事に地上にたどり着くことができるのか。',
      '目の前に迫った恐ろしすぎる不安を口に出す者はいなかった……',
    ]
  end

  def break_party(adventurer)
    [
      '（合流判定：失敗）',
      adventurer.short_name + 'は不安げに周囲を見回す。',
      'あのあと間一髪、無事に恐ろしい敵から逃げおおせることには成功したもの、' + adventurer.short_name +
        'は他の仲間たちとの合流には失敗してしまった。',
      '他の仲間も無事に地上にたどり着けるよう祈りながら、' + adventurer.short_name + 'は地上を目指して歩き始めた……',
    ]
  end

  def break_party_by_trap(adventurer)
    [
      adventurer.short_name + 'は不安げに周囲を見回す。',
      '罠により他のパーティーの仲間と切り離され、単身放り出されてしまった。控えめに言って明るくはない状況だが、' +
      adventurer.short_name + 'は己自身の心を奮い立たせる。こんなところで挫けている暇はないのだ。まずは、無事に地上にたどり着かねば。',
      '他の仲間も無事に地上にたどり着けるよう祈りながら、' + adventurer.short_name + 'は地上を目指して歩き始めた……',
    ]
  end

  def merge_strayer_to_active_party(leader, strayer, floor)
    script = [
      half_number_to_full(floor) + '階で、' + leader.short_name + '隊は孤立していた' +
        strayer.short_name + 'と合流した。',
    ]
    if strayer.is_down?
      script += [
        [
          '陵辱の恐怖に怯え、迷宮内をさまよっていた' + strayer.short_name + 'の表情には、怯えの色が濃い。' + leader.short_name + 'は言葉をかけ励まそうとしていた。',
        ],
        [
          strayer.short_name + 'は、' + leader.short_name + 'に素直に礼を述べてパーティーに加わった',
        ],
        [
          '消耗し尽くし、戦う力を失っていた' + strayer.short_name + 'は、他の冒険者と合流した事で安堵した表情を見せている。',
        ],
      ].sample
    else
      script += [
        [
          '何が起きるかわからない迷宮の中で、戦力が増える事は悪い事ではないはずだ。',
        ],
        [
          strayer.short_name + 'を加えた事で、' + leader.short_name + '隊はさらに戦力を増強した。' +
            '戦えるものはいくらいても困りはしないはずだ。',
        ],
      ].sample
    end
    script
  end

  def merge_strayer_to_lonely_party(leader, strayer, floor)
    script = [
      half_number_to_full(floor) + '階で、孤立していた' + leader.short_name + 'は同じく孤立していた' +
        strayer.short_name + 'と合流した。',
    ]

    script += [
      [
        '孤立している事は、この迷宮の中では大きな危険となる。一人でも仲間が増える事は悪い事ではないはずだった。',
      ],
      [
        '消耗した冒険者が一人から二人に増えたところで、安心はそうは増えはしない。だがそれでも。孤独感を感じないだけマシなのだ。' +
          '口には出さずとも' + leader.short_name + 'は' + strayer.short_name + 'と同じ思いである事は表情から察せられた。',
      ],
      [
        'たとえ戦力としては大きく増えずとも、ともに地上を目指す仲間ができたことに、' + leader.short_name + 'は素直に安堵した。',
      ],
      [
        '合流したあと二人は、不器用な言葉でお互いを励ましあった',
      ],
    ].sample

    script
  end

  def rebuild_lonely_party(adventurer)
    [
      '（逃走判定：成功）',
      adventurer.short_name + 'は不安げに周囲を見回す。',
      'あのあと間一髪、無事に恐ろしい敵から逃げおおせることには成功した。',
      'だが、戦闘能力のない今の' + adventurer.short_name + 'にとって、敵と出会ってしまえば何度も都合よく逃げられるとは限らない。',
      '自分が無事に地上にたどり着けるよう祈りながら、' + adventurer.short_name + 'は地上を目指して歩き始めた……',
    ]
  end

  def turn_prelude(dungeon_master, hour_glass)
    scripts =
      if hour_glass.night?
        [
          '太陽が沈んだ',
          half_number_to_full(hour_glass.day) + '日目の夜。薄暗いダンジョンの中で、遠くから何かの唸り声や女の悲痛な声、すすり泣くような声が響く。',
          '太陽の射さぬこの地下では、女にとって生まれたことを後悔するような地獄絵図が始まることもある',
        ]
      else
        [
          'また日が昇る。',
          half_number_to_full(hour_glass.day) + '日目の昼。薄暗いダンジョンの中ではモンスターやならず者、' +
            _adventurer_type + 'たちが蠢いている。',
          '様々な悲喜劇を生み出してきたこの悪魔の洞窟の中で、今日も何かが起きる',
        ]
      end

    scripts +
      money_status_script(dungeon_master, dungeon_master.today_brothel_sales, hour_glass.night?) +
      [' ']
  end

  def new_adventurers_arrived
    [
      '宿屋がにわかに騒がしくなる。 ',
      'どうやら新たな冒険者たちが到着したようだ。',
      '若く、はつらつとした女冒険者たち。',
      '新参者たちは物珍しげに辺りをキョロキョロと眺めている。その顔には期待と希望が満ちあふれている。',
      'その顔が、現実を知り絶望に染まるのまでに一体何日かかるだろうか',
      '新顔たちを冷たい目で眺める酒場の男たちはそう考えていた。',
      ' ',
    ]
  end

  def money_status_script(dungeon_master, today_brothel_sales, is_night)
    scripts = [
      "所持金： #{_add_comma(dungeon_master.money)}",
      "魔力：  #{magic_power_print(dungeon_master.magic_power)}",
    ]
    unless is_night
      scripts += [
        '[昨日の結果]',
        "娼館の売上： #{_add_comma(today_brothel_sales)}",
        "地下牢に監禁された冒険者から奪った魔力： #{magic_power_print(dungeon_master.today_magic_power_from_jail)}",
        "魔術工房の奴隷たちから奪った魔力： #{magic_power_print(dungeon_master.today_magic_power_from_magic_factory)}",
      ]

      scripts <<
        if FGO_MODE
          "洗脳した隷属サーヴァントたちの維持に消費された魔力： −#{magic_power_print(dungeon_master.magic_maintenance_cost)}"
        else
          "洗脳した隷属奴隷たちの維持に消費された魔力： −#{magic_power_print(dungeon_master.magic_maintenance_cost)}"
        end
    end

    scripts
  end

  def build_parties(new_parties)
    scripts = [
      '酒場では' + _adventurer_type + 'たちが次の冒険に備えて装備を整え、情報を交換し、また酒を適度に飲み干している。',
    ]

    scripts += [
      [
        '酒場の中央では吟遊詩人が今ここにいない' + _adventurer_type + 'のダンジョンでの活躍を、たっぷりと尾ひれをつけた英雄譚として語っている。',
        '無責任な観客たちはその話に聞き入っている。どうやら盛り上がっているようだ。',
      ],
      [
        'ダンジョンの入り口の結界は男の出入りを防ぐため、女のみが入ることができる。',
        'それはすなわち、内部のモンスターたちが外にあふれることを防いでいるということでもある。問題は、その結界が解かれれば惨事になることは間違いない。',
        'この世界の人間にとって、ダンジョンは滅ぼさねばならないものなのだ。',
      ],
      [
        '酒場の隅にいる酔漢たちが、このダンジョンを挟んだ山脈の向こうにあるという娼館街の話をしている。',
        'どんなプレイでも受け入れるという娼館の噂を好き勝手に話しては男たちは大笑いする。あまりにひどい猥談ゆえに女たちは眉をひそめているが、酔漢たちは気づいていないようだ。',
      ],
      [
        'このダンジョンに集まる者は、もちろんこの酒場にいる女たちだけではない。',
        '冒険者相手の商売人。あるいはモンスターの素材を目当てにした怪しげな薬師。さらには集まった人間相手に商売をする店など。',
        'ダンジョンの麓にある冒険者向けの街は賑わっているようだ。',
      ],
      [
        '表がにわかに騒がしくなる。ダンジョンから冒険者が生還したのだ。もちろん無傷というわけにはいかない。白い肌が大きくあらわになっている者もいる',
        '街の男たちはその様子を野次馬のように眺めている……',
      ],
      [
        '酒場に新しい客がやってきた。武器屋に目当てのものがほぼ売り切れていたと文句を言っている。',
        '途切れなく新たにやってくる冒険者によってこの街はいずれも賑わっているようだ',
      ],
    ].sample

    unless new_parties.empty?
      scripts += [
        '酒場がにわかに騒がしくなる。',
        'しばらくの騒ぎの後、何組かの' + _adventurer_type + 'たちが酒場を後にした。どうやら新たなパーティーが組まれたようだ',
      ]
      scripts += new_parties.values.map { |party| party.status }
    end

    scripts += [
      ' ',
    ]
    scripts
  end

  def summon_urk_script(floor)
    [
      '迷宮の深く昏い闇の底で、魔法陣が、ひときわ妖しく輝く……',
      "深層#{half_number_to_full(floor)}階層に、深淵の魔物、ウルクたちが配置されました。",
      'あなたが全幅の信頼を置く強力な魔物たちは、他の魔物たちとは比べ物にならないほど強力です。' +
        '冒険者たちを苦しめ、必ずあなたの望みを叶えてくれるでしょう……',
    ]
  end
end