# coding: utf-8
require_rrk 'src/defines/script/dungeon/script_utils'
require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/util/debug_util'
require_rrk 'src/defines/dungeon/skill_list'

class TrapScript
  include DungeonCommon
  include ScriptUtils
  include DebugUtil

  def trap_1_script(leader, is_lonely, adventurer, drop_count, new_floor)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は落とし穴トラップの仕掛けを踏んでしまった。',
      '足元でカチリという音がすると、' + adventurer.short_name + 'が穴に吸い込まれてしまった。' +
        '他の仲間たちは必死に呼びかけるとスキマから返事が聞こえたことからどうやら無事のようだ。',
      'しばらくの悪戦苦闘の末、合流を諦めた' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'はそれぞれ別行動することに決めた',
      '（' + adventurer.short_name + 'は' + half_number_to_full(drop_count) + '階分落ちて、' + floor_view(new_floor) + 'にいる）',
    ]
  end

  def trap_1_failed_script(leader, is_lonely)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は落とし穴トラップの仕掛けを踏んでしまった。',
      '（回避判定成功）',
      '機転を利かせ、武器をひっかけたおかげで落ちずに済んだようだ……',
    ]
  end

  def trap_2_script(leader, is_lonely, adventurer, drop_count, new_floor)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は強制転移トラップの仕掛けを踏んでしまった。',
      '足元でカチリという音がすると、周囲から眩しい光が湧き上がっていく。気がついた時には' + _party_leader_short_names_by(leader.short_name, is_lonely) +
        'は別のフロアに転送されていた。',
      '（' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'は' + half_number_to_full(drop_count) + '階分下層にある、' + floor_view(new_floor) + 'に転送された）',
    ]
  end

  def trap_2_failed_script(party)
    [
      '！おおっと！',
      _party_leader_short_names(party) + 'は強制転移トラップの仕掛けを踏んでしまった。',
      '（回避判定成功）',
      '機転を利かせ、転移魔導陣の外に走り出たおかげで転移されずに済んだようだ……',
    ]
  end

  def trap_3_script(leader, is_lonely, adventurer, drop_count, new_floor)
    # セリフは共通
    trap_1_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_4_script(leader, is_lonely, adventurer, drop_count, new_floor)
    trap_2_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_5_script(leader, is_lonely, adventurer, drop_count, new_floor)
    # セリフは共通
    trap_1_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_6_script(leader, is_lonely, adventurer, drop_count, new_floor)
    # セリフは共通
    trap_1_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_7_script(leader, is_lonely, adventurer, drop_count, new_floor)
    trap_2_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_8_script(leader, is_lonely, adventurer, drop_count, new_floor)
    trap_2_script(leader, is_lonely, adventurer, drop_count, new_floor)
  end

  def trap_11_script(leader, is_lonely, floor, target)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はスライムプールの仕掛けを踏んでしまった。',
      '強溶性のスライムでたっぷりと満たされたプールに' + target.short_name + 'は落ちてしまった。',
      '苦労してプールから抜け出すが、装備がすべてドロドロに溶かされてしまった',
      '（' + target.short_name + 'の装備ポイントがゼロになりました）',
    ]
  end

  def trap_12_script(leader, is_lonely, floor, target)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は麻痺効果付きスライムプールの仕掛けを踏んでしまった。',
      '強溶性のスライムでたっぷりと満たされたプールに' + target.short_name + 'は落ちてしまった。',
      '苦労してプールから抜け出すが、装備がすべてドロドロに溶かされてしまった',
      '（' + target.short_name + 'の装備ポイントがゼロになりました）',
    ]
  end

  def trap_13_script(leader, is_lonely, floor, target)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は媚薬効果付きスライムプールの仕掛けを踏んでしまった。',
      '強溶性のスライムでたっぷりと満たされたプールに' + target.short_name + 'は落ちてしまった。',
      '苦労してプールから抜け出すが、装備がすべてドロドロに溶かされてしまった',
      '（' + target.short_name + 'の装備ポイントがゼロになりました）',
    ]
  end

  def trap_21_script(leader, is_lonely, floor, adventurer, duplicate)
    return [] unless adventurer

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は催淫ガス噴出トラップの仕掛けを踏んでしまった。',
    ]

    if duplicate
      script += [
        'すでに催淫状態に侵され、正常な判断もままならないまま耐え難い疼きを必死にこらえていた' + adventurer.short_name +
          'にさらにたっぷりと催淫ガスが浴びせられる。美しい眉をハの字に曲げ、必死に何かをこらえるような' + adventurer.short_name +
          'の表情がたちまち発情したメスそのものの顔へと変わっていくことからも、その残酷なまでの強力さは明らかだ。',
        'の腰の奥から甘い疼きが沸き上り、たちまち全身に甘美な波紋が広がっていく……',
      ]
      script +=
        if is_lonely
          [
            adventurer.short_name + 'は発情した獣のような声をあげると、秘所へと指を何本もズブリと潜り込ませる。',
            '周囲に人がいないことをいいことに、肉の疼きを鎮めるべく' + adventurer.short_name + 'は一心不乱に己の秘所を指でぐちゅぐちゅとかき回している。' +
              'いつ誰が通るかもわからない通路でこのような痴女めいた真似をするスリルすらも、度を超えた淫熱に正常な判断を奪われた' + adventurer.short_name +
              'にとってはもはや思考の中には入っていないようだった……',
            '（自慰経験 ＋１）',
          ]
        else
          [
            adventurer.short_name + 'は吠えるような声をあげると、心配する仲間の前で大胆にも自慰行為を始めた。' +
              '周囲の仲間たちは軽蔑と怯えの表情を向けているが、' + adventurer.short_name + 'は、女であれば何か刺激をもらえねば狂ってしまうほどの' +
              '肉の渇きのままに、腰をカクカクと震わせながら自慰行為を続けている……',
          ]
        end
    else
      script += [
        '催淫ガスを浴びた' + adventurer.short_name + 'の腰の奥から甘い疼きが沸き上り、たちまち全身に甘美な波紋が広がっていく……',
      ]
      script +=
        if is_lonely
          [
            adventurer.short_name + 'は何かに耐えかねたかのように周囲をキョロキョロと見回すと秘所へとそっと指を伸ばした。',
            '周囲に人がいないことをいいことに、肉の疼きを鎮めるべく' + adventurer.short_name + 'は一心不乱に己の秘所を指でぐちゅぐちゅとかき回している。' +
              'いつ誰が通るかもわからない通路でこのような痴女めいた真似をするスリルすらも、淫熱に正常な判断を奪われた' + adventurer.short_name +
              'にとっては興奮を高めるスパイスでしかない……',
            '（自慰経験 ＋１）',
          ]
        else
          [
            '心配する仲間たちに、' + adventurer.short_name + 'は何でもないと手を振り強がる。' +
              '周囲の仲間たちも若干訝しがりながらも本人が大丈夫というのでそれを信じることにしたようだ。' +
              'しかし' + adventurer.short_name + 'は切なげに内腿を擦り合わせながら体の熱を持て余しているようだ……',
          ]
        end
      script += [
        '（' + adventurer.short_name + 'のステータスに、「催淫」が付与されました）',
      ]
    end

    script += adventurer.ero_trap_effect_script
    script
  end

  def trap_22_script(leader, is_lonely, floor, adventurers)
    return [] unless adventurers

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は催淫ガス大量噴出トラップの仕掛けを踏んでしまった。',
    ]

    script +=
      if is_lonely
        adventurer = adventurers.values.first[:adventurer]
        [
          '大量の催淫ガスをまともに浴びた' + adventurer.short_name + 'は、胸いっぱいに女を狂わせる淫瘴気を吸い込んでしまった',
          '吸い込んだだけで正気を保てなくなる催淫ガスをこれほど大量に吸い込んでは、正気など根こそぎ吹き飛ばされてしまうのも当然だった。',
          '雌犬のような叫び声をあげると、狂ったように' + adventurer.short_name + 'はその場で自慰を始めた。',
          '腰がすぐにガクガクと震え、えび反りになって' + adventurer.short_name + 'は絶頂した。それでも己を嬲る指の動きは止まらない。',
          adventurer.short_name + 'は何度も何度も、己の腰の奥の肉の渇きに命じられるままにたっぷりと自涜快楽を貪り続けた……',
        ]
      else
        first_adv = adventurers.values.sample
        second_adv = (adventurers.values - first_adv).sample
        first_adventurer = first_adv[:adventurer]
        second_adventurer = second_adv[:adventurer]
        [
          _party_leader_short_names_by(leader.short_name, is_lonely) + 'の周囲に大量の催淫ガスが充満する！',
          '息を止める暇もなく、' + _party_leader_short_names_by(leader.short_name, is_lonely) +  'は魔性のガスを吸い込んでしまった。',
          first_adventurer.short_name + 'は必死に暴れ狂う肉の飢えを理性で押さえつけ周囲の仲間に冷静になるよう叫ぶ。',
          'しかしその声は突然に遮られた。',
          '催淫ガスによって狂わされた' + second_adventurer.short_name + 'によって押し倒されたからだ。' + first_adventurer.short_name + 'は必死に抵抗するが、力が入らず、淫欲に呑まれた仲間たちによってたちまちに狂わされてしまう。',
          'しばらくの間、女同士で快楽を貪り合う' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'の甘い嬌声が' + floor_view(floor) + '中に響き渡った。',
        ]
      end

    script += adventurers.map do |adventurer|
      '（' + adventurer.short_name + 'のステータスに、「催淫」が付与されました）'
    end

    adventurers.each do |adventurer|
      script += adventurer.ero_trap_effect_script
    end
    script
  end

  def trap_23_script(leader, is_lonely, floor, adventurers, parties)
    return [] unless adventurers

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は広範囲催淫ガス噴出トラップの仕掛けを踏んでしまった。',
    ]

    script +=
      if is_lonely
        adventurer = adventurers.values.first[:adventurer]
        [
          '大量の催淫ガスをまともに浴びた' + adventurer.short_name + 'は、胸いっぱいに女を狂わせる淫瘴気を吸い込んでしまった',
          '吸い込んだだけで正気を保てなくなる催淫ガスをこれほど大量に吸い込んでは、正気など根こそぎ吹き飛ばされてしまうのも当然だった。',
          '雌犬のような叫び声をあげると、狂ったように' + adventurer.short_name + 'はその場で自慰を始めた。',
          '腰がすぐにガクガクと震え、えび反りになって' + adventurer.short_name + 'は絶頂した。それでも己を嬲る指の動きは止まらない。',
          adventurer.short_name + 'は何度も何度も、己の腰の奥の肉の渇きに命じられるままにたっぷりと自涜快楽を貪り続けた……',
        ]
      else
        first_adv = adventurers.values.sample
        second_adv = (adventurers.values - first_adv).sample
        first_adventurer = first_adv[:adventurer]
        second_adventurer = second_adv[:adventurer]
        [
          _party_leader_short_names_by(leader.short_name, is_lonely) + 'の周囲に大量の催淫ガスが充満する！',
          '息を止める暇もなく、' + _party_leader_short_names_by(leader.short_name, is_lonely) +  'は魔性のガスを吸い込んでしまった。',
          first_adventurer.short_name + 'は必死に暴れ狂う肉の飢えを理性で押さえつけ周囲の仲間に冷静になるよう叫ぶ。',
          'しかしその声は突然に遮られた。',
          '催淫ガスによって狂わされた' + second_adventurer.short_name + 'によって押し倒されたからだ。' + first_adventurer.short_name + 'は必死に抵抗するが、力が入らず、淫欲に呑まれた仲間たちによってたちまちに狂わされてしまう。',
          'しばらくの間、女同士で快楽を貪り合う' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'の甘い嬌声が' + floor_view(floor) + '中に響き渡った。',
        ]
      end

    script += adventurers.map do |adventurer|
      '（' + adventurer.short_name + 'のステータスに、「催淫」が付与されました）'
    end

    adventurers.each do |adventurer|
      script += adventurer.ero_trap_effect_script
    end

    script += [
      'ブシュウウウウッ、シュウ、シュウゥゥゥとガスは際限なく吐き出されていく。' +
      '床の穴から勢いよく噴き出した催淫ガスが、フロア中に充満していく！！！'
    ]

    if parties.empty?
      script += [
        'フロア中に広がるほどの恐ろしい量の催淫ガスを浴びた' + _party_leader_short_names_by(leader.short_name, is_lonely) +
          'は正気でいられるはずもない。浅ましく乱れるう声が、大きく響き続けた。',
      ]
    else
      parties.each do |party|
        target_adventurers = party.all_members_with_passengers.select {|adv| adv.has_bad_status_in_dive?(:aphrodisiac) }
        script += [
          _party_leader_short_names_by(party.leader.short_name, party.is_only?) + 'はフロアに充満する催淫ガスを吸い込んでしまった！',
          '（' + target_adventurers.map(&:short_name).join('、') + 'のステータスに、「催淫」が付与されました）',
        ]
      end
    end

    script
  end

  def trap_24_script(leader, is_lonely, floor, adventurers, parties)
    # 基本同じなので使い回す
    trap_23_script(leader, is_lonely, floor, adventurers, parties)
  end

  def trap_31_script(leader, is_lonely, floor)
    [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は誘導フェロモンの仕掛けを踏んでしまった。',
      'モンスターを引き寄せるガスが' + leader.short_name + 'に吹きかけられた！',
      '（次回の遭遇率が上がります。）',
    ]
  end

  # 注射針つき触手
  def trap_41_script(leader, is_lonely, floor, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は注射針つき触手の仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '脚に巻きつき獲物の動きを封じると、鋭い針のついた触手が' + adventurer.short_name + 'に狙いを定める！',
      '恐ろしいほどの速さで鋭い針が' + adventurer.short_name + 'の体に突き刺さった',
    ]
    script = _trap_4x_script(script, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script
  end

  # 大型の注射針つき触手
  def trap_42_script(leader, is_lonely, floor, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は大型の注射針つき触手の仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '脚に巻きつき獲物の動きを封じると、鋭い針のついた触手が' + adventurer.short_name + 'に狙いを定める！',
      '恐ろしいほどの速さで矢尻のように大きく鋭い針が' + adventurer.short_name + 'の体に突き刺さった',
    ]
    script = _trap_4x_script(script, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script
  end

  # 特濃薬液入り注射針つき触手
  def trap_43_script(leader, is_lonely, floor, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は特濃薬液入り注射針つき触手のトラップを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '脚に巻きつき獲物の動きを封じると、恐ろしい薬毒液を溜め込んだことで根元をこぶのようにボコリと膨らませた、鋭い針のついた触手が' + adventurer.short_name + 'に狙いを定める！',
      '恐ろしいほどの速さで大きく鋭い針が' + adventurer.short_name + 'の体に突き刺さった',
    ]
    script = _trap_4x_script(script, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    script
  end

  def _trap_4x_script(script, adventurer, effect, boom_hooter_success, is_hooter_level_max)
    if effect[:bad_status_in_dive].empty? && effect[:bad_status_in_body].empty? && !boom_hooter_success
      script += [
        '運良く触手の針は、' + adventurer.short_name + 'の鎧の部分に突き刺さったようだ。',
        adventurer.short_name + 'は脚に巻きついた触手を引きちぎり、鎧に刺さった触手を引き抜くと足早にそこを離れた',
      ]
    else
      # 突き刺さって薬液注入
      script += [
        '体に突き刺さった触手の針によって、' + adventurer.short_name + 'の体内に危険な触手毒液が注入されていく。',
        '深々と突き刺さっているはずなのに、痛みはない、しかし体を走り抜ける悪寒が、その薬液の危険さをはっきりと示していた。',
      ]

      unless effect[:bad_status_in_dive].empty?
        bad_status_names = effect[:bad_status_in_dive].map do |bad_status_name|
          bad_status_name
        end
        script += [
          '即効性の薬効によって、' + adventurer.short_name + 'の肉体には危険な効果が刻み込まれていく。',
          adventurer.short_name + 'は触手薬毒の効果によって「' + bad_status_names.compact.join('、') + '」を取得した',
        ]
      end
      unless effect[:bad_status_in_body].empty?
        bad_status_names = effect[:bad_status_in_body].map do |bad_status|
          DungeonSkillList::BAD_STATUS_IN_BODY_NAMES[bad_status]
        end
        script += [
          '即効性の薬効によって、' + adventurer.short_name + 'の肉体は徹底的に容赦なく作り変えられていく。',
          adventurer.short_name + 'は触手薬毒の効果によって「' + bad_status_names.compact.join('、') + '」を取得した',
        ]
      end
      if boom_hooter_success # 胸が膨らんだ
        script +=
          if is_hooter_level_max
            [
              'もはや限界を超えて' + adventurer.short_name + 'の胸は、さらに豊満になったようだ……',
              'たとえ妊婦といえどもこれほどには大きくならぬだろう。すべての動きを阻害する巨大な胸は、まさに乳牛のような外観になっている。',
              '疼きを覚える' + adventurer.short_name + 'の胸の先端からは母乳すら噴き出していた。',
            ]
          else
            [
              '触手が胸に流し込んだ薬毒液の効果は、' + adventurer.short_name + 'の肉体に不可逆の効果をもたらしたようだ。',
              adventurer.short_name + 'は消えぬ違和感を訴える胸をみると、そこが先ほどよりも大きくなっていることに気づいた……',
              '（' + adventurer.short_name + 'の胸は、' + adventurer.hooter_level_name + 'サイズになった）',
            ]
          end
        if adventurer.hooter_level < 3
          script += [
            '気のせいかもしれないが、' + adventurer.short_name + 'はどことなく嬉しそうだ……',
          ]
        elsif adventurer.hooter_level > 4
          script += [
            '明らかに並の女ではありえないサイズになった自分の胸を見た、' + adventurer.short_name + 'は絶望の表情を浮かべる。',
            '冒険や戦闘にはもはや邪魔にしかならない胸。こんなものを手に入れるために、自分は迷宮に挑んだわけではないのだ。',
            'いまの' + adventurer.short_name + 'の肉体を一目見た男達は冒険者への敬意ではなく、セックスへの劣情を抱くことは間違いないだろう。',
          ]
        end
      end

      script += [
        'たっぷりと薬毒液を' + adventurer.short_name + 'の体内に注ぎ込み終えた触手は、ずるりと針を引き抜くと' +
          '壁や床の隙間へと消えていった。',
        '太い針が刺さったはずなのに' + adventurer.short_name + 'の体に残った傷跡は小さなものだ。' +
          'それは薬毒液に治癒効果のあるものが含まれていた事を意味する。',
        'つまり触手たちの狙いは、獲物を傷つける事ではなく、弱らせ、作り変える事が目的であることなのだ。' +
          'それはすなわち魔物たちに媚肉として貪らせる悪辣な狙いのために作られた事を意味していた。',
      ]
    end
    script
  end

  # 秘所愛撫触手
  def trap_51_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は秘所愛撫触手の仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '四肢を拘束し、身動きを封じると大量の触手が愛液を求めて' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '秘所を埋め尽くすミミズのようなグロテスクな見た目とは裏腹に、触手たちは陰核をコリコリと優しく責め立て、舐めるように秘裂を愛撫する。',
      adventurer.short_name + 'ははしたない声をあげ、腰をカクつかせながら快楽に悶え狂った……',
    ]
    if effect[:bad_status_in_body].include?('c_sensitive')
      script += [
        '凶悪な形状の触手による陰核への徹底的で執拗な人外の刺激は' + adventurer.short_name + 'の肉体と快楽を求める本能を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「陰核敏感」を取得しました。',
      ]
    end
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 秘所愛撫触手ブラシ
  def trap_52_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は秘所愛撫触手ブラシの仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '四肢を拘束し、身動きを封じると大量の触手が愛液を求めて' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '触手が先端の口にあたる部分を開くと中には肉色のブラシのようなものがびっしりと生え揃っているのが見えた。こんなもので責められたら、間違いなく狂ってしまう。',
      adventurer.short_name + 'はゴクリと息を飲んだ',
      'これからこの肉凶器でお前を狂わせてやる、そんな予告めいた一本の触手の動きを皮切りに、無数の肉色のブラシが' + adventurer.short_name + 'の陰核と秘所へと群がっていく',
      '甘い嬌声はやがて切羽詰まった悲鳴へと変わり、さらに低い獣じみた吠え声へと変わっていく。脳を直接ゴシゴシとこすられるような暴力的快楽が' + adventurer.short_name + 'を狂わせているのだ。',
      '許容限度を超えた快楽電流を受けた腰が不自然に痙攣する。しかし触手たちは、逃げるな、と言いたげにがっちりと押さえ込み身じろぎすらも出来なくした上で更に執拗にゴシゴシと肉ブラシで責めたてる。',
      '無様そのものの表情を浮かべた' + adventurer.short_name + 'はいままさに快楽で壊されながら、情けなく許しを乞い続けたが、触手たちはまるで気にもとめず、たっぷりと' + adventurer.short_name + 'を嬲り、愛液をすすり続けた',
    ]

    if effect[:bad_status_in_body].include?('c_sensitive')
      script += [
        '凶悪な形の肉のブラシによる陰核への執拗な刺激は' + adventurer.short_name + 'の肉体と快楽を求める本能を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「陰核敏感」を取得しました。',
      ]
    end
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 触手肉バイブ
  def trap_53_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は触手肉バイブの仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の秘所めがけて飛び出していく。',
      'とっさに避けようとした' + adventurer.short_name + 'だが、他の触手に足を絡め取られ、秘裂に喰らいつかれた。',
      '触手は服の隙間に潜り込み、下着を横にずらすとついに肉壷の中へと侵入した！',
    ]

    script +=
      if effect[:lost_virgin]
        [
          '信じられないほどの激痛が' + adventurer.short_name + 'を襲う。',
          '未踏の処女地を、恐ろしいほど器用に体をくねらせた触手が強引にこじ開け、中に入り込んでいるのだ。',
          '触手に処女を踏みにじられたという恐ろしい現実に叫び声を上げながら' + adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は' +
            '柔らかい体をくねらせ、ギチギチと万力のような力で締め上げる膣内へとスルスルと侵入していく。',
          '半狂乱の' + adventurer.short_name + 'がめちゃくちゃに武器で触手を叩くと、ついに触手は切れた。いや意図的に切り離したのだ。' +
            '太く長い先端部を膣内に残るように切り離すと本体は再び地中へと消えていった。',
          '膣内で甘く震える触手バイブの存在感と違和感が' + adventurer.short_name + 'に恐ろしい違和感を覚えさせる。しかし、肉バイブの表面から分泌' +
            'される高濃度の媚薬原液によって痛みと嫌悪感を急速に遠ざけていく。',
          '振動が与えられるたび' + adventurer.short_name + 'は顎を上げ甘い息を吐き出す。肉バイブが強く震えるたび、' + adventurer.short_name + 'を腰砕けにしてしまう。膝がガクガクと甘く震え、ついには座り込んでしまった。',
          'それでも許さぬ肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'のつい先ほどまで処女だった肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたてた。',
        ]
      else
        [
          adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は太く長い先端部を膣内に残るように切り離すと本体は再び地中へと消えていった。',
          '膣内で甘く震える触手バイブの振動が' + adventurer.short_name + 'を腰砕けにしてしまう。膝がガクガクと甘く震え、ついには座り込んでしまう。',
          'それでも許さぬ肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'の肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたてた。',
        ]
      end

    if effect[:bad_status_in_body].include?('v_sensitive')
      script += [
        '高濃度の媚薬と凶悪な形の肉バイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「肉壷敏感」を取得しました。',
      ]
    end
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_54_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は触手イボ付き肉バイブの仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の秘所めがけて飛び出していく。',
      'とっさに避けようとした' + adventurer.short_name + 'だが、他の触手に足を絡め取られ、秘裂に喰らいつかれた。',
      '触手は服の隙間に潜り込み、下着を横にずらすとついに凶悪な形のイボがたっぷりとついた先端を肉壷の中へと侵入した！',
    ]

    script +=
      if effect[:lost_virgin]
        [
          '信じられないほどの激痛が' + adventurer.short_name + 'を襲う。',
          '未踏の処女地を、恐ろしいほど器用に体をくねらせた触手が強引にこじ開け、中に入り込んでいるのだ。',
          '触手に処女を踏みにじられたという恐ろしい現実に叫び声を上げながら' + adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は' +
            '柔らかい体をくねらせ、ギチギチと万力のような力で締め上げる膣内へとスルスルと侵入していく。',
          '半狂乱の' + adventurer.short_name + 'がめちゃくちゃに武器で触手を叩くと、ついに触手は切れた。いや意図的に切り離したのだ。' +
            '太く長い先端部を膣内に残るように切り離すと本体は再び地中へと消えていった。',
          '膣内で甘く震える触手バイブの存在感と違和感が' + adventurer.short_name + 'に恐ろしい違和感を覚えさせる。しかし、肉バイブの表面から分泌' +
            'される高濃度の媚薬原液によって痛みと嫌悪感を急速に遠ざけていく。',
          '肉バイブ全体が回転するように動き、肉イボで膣内を甘く撫で回し、柔らかくほぐしていく。もちろん表面からあふれる毒媚薬をたっぷりとなすりつけながら、だ。',
          '振動と回転が与えられるたび' + adventurer.short_name + 'は顎を上げ甘い息を吐き出す。肉バイブが強く震えるたび、' + adventurer.short_name + 'を腰砕けにしてしまう。膝がガクガクと甘く震え、ついには座り込んでしまった。',
          'それでも許さぬイボつきの肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'のつい先ほどまで処女だった肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたて、' +
            'そこを快楽のためだけの都合の良い肉穴として容赦なく作り変えていった。',
        ]
      else
        [
          adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は太く長い先端部を膣内に残るように切り離すと本体は再び地中へと消えていった。',
          '肉バイブ全体が膣内で回転するように動き、肉イボで膣内を甘く撫で回し、柔らかくほぐしていく。もちろん表面からあふれる毒媚薬をたっぷりとなすりつけながら、だ。',
          '膣内で甘く震える触手バイブの振動が' + adventurer.short_name + 'を腰砕けにしてしまう。膝がガクガクと甘く震え、ついには座り込んでしまう。',
          'それでも許さぬ肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'の肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたて、' +
            'そこを快楽のためだけの都合の良い肉穴として容赦なく作り変えていった。',
        ]
      end

    if effect[:bad_status_in_body].include?('v_sensitive')
      script += [
        '高濃度の媚薬と凶悪な形の肉バイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「肉壷敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_61_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はアナル愛撫触手の仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '四肢を拘束し、身動きを封じると大量の触手が' + adventurer.short_name + 'に狙いを定め、その下半身に一気に襲いかかった',
      '無数の触手が衣服の隙間から' + adventurer.short_name + 'の下半身に潜り込もうとする。',
      '必死に秘所をガードする' + adventurer.short_name + 'をあざ笑うように、一本の触手は背徳の穴へとぬぷりと潜り込んだ。',
      '固く閉じたはずの排泄穴をあっさりとこじ開けた柔らかい触手は、直腸の中に潜り込むと内部を優しく撫で上げる。その背徳の感覚に思わず' + adventurer.short_name + 'は体をこわばらせてしまう。' +
        'その隙をついて無数の肉色のブラシが' + adventurer.short_name + 'の菊門へと群がっていく',
      'アナルを埋め尽くす触手たちはミミズのようなグロテスクな見た目とは裏腹に、直腸の襞を舐めあげるように優しく責め立て、中を押し広げ、器用に掘削し快楽器官として作り変えていく。',
      adventurer.short_name + 'は下品な喘ぎ声をあげ、腰をカクつかせながら快楽に悶え狂った……',
    ]

    if effect[:bad_status_in_body].include?('a_sensitive')
      script += [
        '触手から分泌される高濃度の媚薬と触手による器用で徹底的な刺激は' + adventurer.short_name + 'の直腸の快楽神経を目覚めさせ、第二の性器として肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「尻穴敏感」を取得しました。',
      ]
    end
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_62_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はアナル愛撫触手ブラシの仕掛けを踏んでしまった。',
      '地面から飛び出した無数の触手は、' + adventurer.short_name + 'に狙いを定めると一気に襲いかかった',
      '四肢を拘束し、身動きを封じると大量の触手が' + adventurer.short_name + 'に狙いを定め、その下半身に一気に襲いかかった',
      '触手が先端の口にあたる部分を開くと中には肉色のブラシのようなものがびっしりと生え揃っているのが見えた。こんなもので責められたら、間違いなく狂ってしまう。',
      adventurer.short_name + 'はゴクリと息を飲んだ',
      'これからこの肉ブラシでお前を狂わせてやる、そんな予告めいた一本の触手が' + adventurer.short_name + 'の下半身に潜り込もうとする。',
      '必死に秘所をガードする' + adventurer.short_name + 'をあざ笑うように、触手は背徳の穴へとぬぷりと潜り込んだ。',
      '直腸の襞を優しく撫で上げる背徳の感覚に、思わず' + adventurer.short_name + 'は体をこわばらせてしまう。' +
        'その隙をついて無数の肉色のブラシが' + adventurer.short_name + 'の菊門へと群がっていく',
      '甘い嬌声はやがて切羽詰まった悲鳴へと変わり、さらに低い獣じみた吠え声へと変わっていく。脳を直接ゴシゴシとこすられるような暴力的快楽が' + adventurer.short_name + 'を狂わせているのだ。',
      '許容限度を超えた快楽電流を受けた腰が不自然に痙攣する。しかし触手たちは、逃げるな、と言いたげにがっちりと押さえ込み身じろぎすらも出来なくした上で更に執拗に直腸全体をゴシゴシと肉ブラシで責めたてる。',
      '無様そのものの表情を浮かべた' + adventurer.short_name + 'はいままさに快楽で壊されながら、情けなく許しを乞い続けたが、触手たちはまるで気にもとめず、たっぷりと' + adventurer.short_name + 'を嬲り、愛液をすすり続けた',
    ]
    if effect[:bad_status_in_body].include?('a_sensitive')
      script += [
        '触手ブラシから分泌される高濃度の媚薬とブラシによるえげつなく圧倒的な刺激は' + adventurer.short_name + 'の直腸の快楽神経を目覚めさせ、第二の性器として肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「尻穴敏感」を取得しました。',
      ]
    end
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_63_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は触手肉アナルバイブの仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の菊門めがけて飛び出していく。',
      'とっさに避けようとした' + adventurer.short_name + 'だが、他の触手に足を絡め取られ、尻穴に喰らいつかれた。',
      '触手は服の隙間に潜り込み、下着を横にずらすとついにぼっこりと頼もしく膨らんだ先端を直腸の中へとねじり入れた！',
      adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は太く長い先端部を尻穴の中に残るように切り離すと本体は再び地中へと消えていった。',
      '直腸内で甘く震える触手バイブの振動が' + adventurer.short_name + 'を腰砕けにしてしまう。背徳の快楽に膝がガクガクと甘く震え、ついには座り込んでしまう。',
      'それでも許さぬ肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'の直腸を高濃度の媚薬を分泌しながらたっぷりと責めたてた。',
    ]
    if effect[:bad_status_in_body].include?('a_sensitive')
      script += [
        '高濃度の媚薬と凶悪な形の肉バイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        '高濃度の媚薬と動く肉張り型によるえげつなく圧倒的な刺激は' + adventurer.short_name + 'の直腸の快楽神経を目覚めさせ、第二の性器として肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「尻穴敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_64_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は触手肉アナルビーズの仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の菊門めがけて飛び出していく。',
      'とっさに避けようとした' + adventurer.short_name + 'だが、他の触手に足を絡め取られ、尻穴に喰らいつかれた。',
      '触手は服の隙間に潜り込み、下着を横にずらすとついにぼっこりとえげつない肉イボをびっしりと生やした肉バイブが、頼もしく膨らんだ先端から直腸の中へとねじり入れた！',
      adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は太く長い先端部を尻穴の中に残るように切り離すと本体は再び地中へと消えていった。',
      'ねじりこむように回転し、肉イボで直腸内を激しく嬲りながら直腸内で甘く震える触手バイブの振動が' + adventurer.short_name + 'を腰砕けにしてしまう。背徳の快楽に膝がガクガクと甘く震え、ついには座り込んでしまう。',
      'それでも許さぬ肉バイブは背中を丸め甘い声で啼く' + adventurer.short_name + 'の直腸をたっぷりと高濃度の媚薬を分泌しながら肉イボで襞の一つ一つにそれをなじませ、作り変えながら尻穴をたっぷりと責めたてた。',
    ]
    if effect[:bad_status_in_body].include?('a_sensitive')
      script += [
        '高濃度の媚薬と凶悪な形の肉バイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        '高濃度の媚薬と動く肉張り型によるえげつなく圧倒的な刺激は' + adventurer.short_name + 'の直腸の快楽神経を目覚めさせ、第二の性器として肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「尻穴敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_71_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はクリ愛撫触手の仕掛けを踏んでしまった。',
      '地面から大量の触手たちが姿をあらわすと、どうやら触手たちは' + adventurer.short_name + 'に狙いをつけたようだ！',
      '群がる触手たちを振り払おうとするが、触手たちは' + adventurer.short_name + 'の秘所へと死角から容赦なく潜り込む。' +
        '陰核に触手が触れ、甘い電気刺激が走ると' + adventurer.short_name + 'は武器を取り落としてしまった。',
      'ついに淫惨な快楽ショーが幕を開けた。',
      '触手が淫核をえげつなく責め苛むたび、' + adventurer.short_name + 'の尻が震え、薄闇の中で淫らなダンスを踊っていた。',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_72_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はクリ愛撫触手ブラシの仕掛けを踏んでしまった。',
      '群がる触手たちを振り払おうとするが、触手たちは' + adventurer.short_name + 'の秘所へと死角から容赦なく潜り込む。' +
        '陰核に触手が触れ、甘い電気刺激が走ると' + adventurer.short_name + 'は武器を取り落としてしまった。',
      'ついに淫惨な快楽ショーが幕を開けた。',
      '肉ブラシがゴシゴシと削り取るように淫核を刺激する。本来であれば激痛が走るはずだが、触手が分泌する媚薬原液によってそれはすべて甘い快楽に置き換えられていた。',
      '「ひぃぃっ、ひぃィィィィッ！？　すごいっ、これぇぇぇぇっ！」',
      'にちゃにちゃ、ぐちゃぐちゃという粘ついた咀嚼音が響くたび、法外な快楽の濁流が' + adventurer.short_name + 'の脳を襲っていることがわかる。',
      'バネ仕掛けの人形のように、デタラメに手足が跳ね回り、その快楽のえげつなさをはっきりと示していた。',
      '触手が淫核をえげつなく責め苛むたび、' + adventurer.short_name + 'の尻が震え、薄闇の中で淫らなダンスを踊る。',
            adventurer.short_name + 'は、触手が満足するまでたっぷりといたぶられた。'
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_73_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は乳房愛撫触手の仕掛けを踏んでしまった。',
      '群がる触手たちを振り払おうとするが、触手たちは' + adventurer.short_name + 'の秘所へと死角から容赦なく潜り込む。' +
        '鎧や服の隙間から潜り込んだ触手が乙女の胸の突起に触れ、甘い電気刺激が走ると' + adventurer.short_name + 'は武器を取り落としてしまった。',
      'ついに淫惨な快楽ショーが幕を開けた。',

    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_74_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は乳房愛撫触手ブラシの仕掛けを踏んでしまった。',
      '地面や周囲の壁の隙間から這い出してきた触手たちが' + adventurer.short_name + 'の胸へと群がっていく。',
      '服の隙間から入り込み、胸をこね回し人の手では不可能な繊細さで愛撫する触手のテクニックに' + adventurer.short_name + 'は腰砕けになりながら甘い声で啼く。',
      '振り払おうとしても次から次へと群がる触手たちの前に抵抗する' + adventurer.short_name + 'の手指から力が抜けていく。それをいいことに、触手たちは' + adventurer.short_name + 'の胸をたっぷりと楽しんだ。',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_75_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はクリ敏感改造触手オナホールの仕掛けを踏んでしまった。',
      '壁の隙間から這い出してきた触手が、' + adventurer.short_name + 'の陰核に絡みつく。何かを刺されたような感触を感じ、そこに注意を向けた時にはもうすべてが手遅れだった。',
      adventurer.short_name + 'の陰核がむくむくと大きく膨らんでいく',
      '「いっ、いやぁぁ……　なにこれぇ……」',
      '男の肉槍、と呼ぶにはややサイズが小さく可愛らしいが、それでもそれはペニスとして十分な機能を備えていた。衝撃に絶句する' + adventurer.short_name + 'の偽陽根を、大きく口を開けた触手オナホールが飲みこんだ',
      '「うひぃぃぃっ、なにっ、なにこれぇぇ！」',
      adventurer.short_name + 'の声に、切羽詰まったような色が大きく混じりこむ。',
      '敏感な快楽神経がたっぷりと詰まった陰核をこれほどまでに肥大化させられた上に、ぐちゅぐちゅという音を響かせるオナホールに飲み込まれたのだ。',
      'しかも中には媚薬原液がたっぷりと分泌させられており、偽のペニスにそれらがゆっくりと丹念に肉ブラシによってなじませられていく。',
      '「ほああぁぁぁっ、あひぃぃっ、これっ、これぇぇぇぇっ！　あたまっ頭おかしくっ、おがじぐなるぅぅぅぅっ！」',
      adventurer.short_name + 'は声にならない甘い声を上げながら、女として味わったことのないオスの快楽に酔いしれた……',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_76_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は乳房搾乳触手ブラシの仕掛けを踏んでしまった。',
      '地面や周囲の壁の隙間から這い出してきた触手たちが' + adventurer.short_name + 'の胸へと群がっていく。',
      '服の隙間から入り込み、胸をこね回し人の手では不可能な繊細さで愛撫する触手のテクニックに' + adventurer.short_name + 'は腰砕けになりながら甘い声で啼く。',
      '振り払おうとしても次から次へと群がる触手たちの前に抵抗する' + adventurer.short_name + 'の手指から力が抜けていく。それをいいことに、触手たちは' + adventurer.short_name + 'の胸をたっぷりと楽しんだ。',
      '女のシンボルである乳房の先端の、ぷっくりと勃起した尖りに群がった触手たちはそこに巻きつき、搾り出すような動きでシゴキあげていく。',
      '不意に限界を超えた' + adventurer.short_name + 'の胸から、大量の母乳がほとばしる。',
      '驚愕の表情を浮かべる' + adventurer.short_name + 'に構わず、触手たちは容赦のない動きで次々と母乳を絞り出し、吸い上げていく。',
      '胸に塗りこまれていた触手たちの分泌液には魔力を母乳に変換する成分が含まれていたのだ。',
      'すでに何度も噴乳させられ、腰砕けになりながらぜぇぜぇと荒い息を吐く' + adventurer.short_name + 
      'の胸に、チューブがついたワイングラスのような形状のカップが押し当てられる。それがなにか、正体を' + 
      adventurer.short_name + 'は己の肉体で悟らされた。',
      'びゅるびゅると、今までとは比べ物にならないほどの勢いで母乳が噴きだし、搾り取られていく。',
      '搾乳カップの内側にびっしりと張り付いた触手が胸に巻きつき、さらに魔力をたっぷりと含んだ母乳を搾り出してやろうと容赦なく蠢めく。',
      '敏感化した乳首が母乳が噴き出すたびに快楽を感じる' + adventurer.short_name + 'は何度も何度も快楽の甘い嬌声をあげながら、噴乳快楽に溺れていった。',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  def trap_81_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    monster_name = '種付けおじさん'
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は' + monster_name + '召喚の仕掛けを踏んでしまった。',
      '音とともに紫色の煙が吹き出し、' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'は急激な眠りに落ちてしまった。',
      'どうやら' + monster_name + 'は、' + adventurer.short_name + 'に狙いを定めたようだ',
    ]
    script += [
      monster_name + 'は眠りに落ちた無防備な' + adventurer.short_name + 'にのしかかるとすぐに激しく犯し始めた。',
    ]
    unless is_lonely
      script += [
        '深い眠りに落ちた仲間たちは、' + monster_name + 'によって好き放題貪られている' + adventurer.short_name + 'の惨状に気づいてもいない。',
      ]
    end

    # 催眠
    hypnosis = random_roll(50, 100)
    if effect[:lost_virgin]
      lost_virgin = true
      script +=
        if hypnosis
          [
            adventurer.short_name + 'は突然の感覚に目を覚まし抵抗しようとするが、先ほどのガスに含まれた媚薬成分、そして強力な催眠成分によってすぐに抵抗する気力を奪われてしまう。',
            'しかし' + monster_name + 'のテクニックと催眠成分が' + adventurer.short_name + 'の認識を原型を留めぬほど歪めていく……',
            'どうやら' + adventurer.short_name + 'は' + monster_name + 'を最愛の男という認識を刻み込まれてしまったようだ',
            '（処女喪失！）',
            adventurer.short_name + 'の純潔は、' + monster_name + 'によって散らされてしまった。',
            monster_name + 'は' +
              adventurer.short_name + 'の処女を蹂躙したことに気づくと、これほどの美女を自分のものにした征服感に顔を緩ませている……',
            'しかし' + adventurer.short_name + 'は自分の初めてを最愛の男に捧げられた無上の喜びに涙さえ流している……',
            '処女を失ったばかりだというのに手加減なしに膣内を耕し続ける男の圧倒的な充足感に、' + adventurer.short_name + 'は頼もしささえも覚えていた。',
          ]
        else
          [
            '（処女喪失！）',
            adventurer.short_name + 'の純潔は、' + monster_name + 'によって無惨に散らされてしまった。' + monster_name + 'は' +
              adventurer.short_name + 'が処女だったことに気づくと、これほどの美女を自分のものにした征服感に顔を緩ませている……',
          ]
        end
    end

    if hypnosis
      script +=
        if lost_virgin
          []
        else
          [
            adventurer.short_name + 'は突然の感覚に目を覚まし抵抗しようとするが、先ほどのガスに含まれた媚薬成分、そして強力な催眠成分によってすぐに抵抗する気力を奪われてしまう。',
          ]
        end
      script += [
        [
          adventurer.short_name + 'は' + monster_name + 'のテクニックによってすぐにセックスに溺れていった……',
          'ダンジョンの一角にはしばらくの間女を貪る' + monster_name + 'の荒い息と、肉の喜びを強制的に刻み込まれている' + adventurer.short_name + 'の甘い声が響いていた……',
          '（どぷっ、ぐぷっ、びゅるるるるるっ）',
        ],
        [
          adventurer.short_name + 'は' + monster_name + 'のテクニックによってすぐにセックスに溺れていった……',
          '催眠成分によって' + monster_name + 'を最愛の男として意識に刻み込まれた' + adventurer.short_name + 'は、' +
            'ハゲかけた中年男と愛おしげに舌を絡め、唾液を飲み干し、自分から脚を絡め自ら腰を揺すぶる。',
          '手の指を絡めながら、およそ美醜の対極にある男に対してさらなる行為を懇願する' + adventurer.short_name + 'の姿は、一種異様な光景でもあった',
          'ダンジョンの一角にはしばらくの間女を貪る' + monster_name + 'の荒い息と、最愛の相手と愛を確かめ合うような' + adventurer.short_name + 'の甘い声が響いていた……',
          '（どぴゅぴゅっ、ぶびゅるるっ、どぴゅどぴゅどぴゅどぴゅっ）',
        ],
      ].sample
    else
      script += [
        [
          adventurer.short_name + 'は突然の感覚に目を覚まし抵抗しようとするが、先ほどのガスに含まれた媚薬成分と' +
            monster_name + 'のテクニックによってすぐにセックスに溺れていった……',
          'ダンジョンの一角にはしばらくの間女を貪る' + monster_name + 'の荒い息と、肉の喜びを強制的に刻み込まれている' + adventurer.short_name + 'の甘い声が響いていた……',
          '（どぷっ、ぐぷっ、びゅるるるるるっ）',
        ],
      ].sample
    end

    script += [
      [
        monster_name + 'は満足げに呻くと、' + adventurer.short_name + 'を全体重をかけてがっちりと押さえつけ、膣内の一番奥、子宮口に' +
          '肉棒の先端、鈴口をしっかりと押し付けながら' + half_number_to_full(adventurer.this_turn_v_sperm_shot_count) + '回目の射精となる生命の滾りを放出した。',
      ],
    ].sample

    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)

    script += [
      monster_name + 'は' + adventurer.short_name + 'の膣内にプリップリの精液をたっぷりと吐き出すと、最後に' + adventurer.short_name + 'に口を使って肉棒を綺麗になるまでじっくりと清めさせた。',
      'たっぷりと媚肉を味わった後、' + monster_name + 'は' + '満足したように立ち去った。'
    ]
    script
  end

  def trap_82_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    monster_name = '種付けおじさん'
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は' + monster_name + '召喚の仕掛けを踏んでしまった。',
      '音とともに紫色の煙が吹き出し、' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'は急激な眠りに落ちてしまった。',
      'どうやら' + monster_name + 'は、' + adventurer.short_name + 'に狙いを定めたようだ',
    ]
    script += [
      monster_name + 'は眠りに落ちた無防備な' + adventurer.short_name + 'を座らせると、その美しい顔に肉棒を押し付け、強引に口に肉棒を咥えさせた。',
    ]
    unless is_lonely
      script += [
        '深い眠りに落ちた仲間たちは、' + monster_name + 'によって好き放題口を使われている' + adventurer.short_name + 'の惨状に気づいてもいない。',
      ]
    end

    # 催眠
    hypnosis = random_roll(50, 100)

    if hypnosis
      script += [
        [
          adventurer.short_name + 'は突然の衝撃に目を覚まし抵抗しようとするが、先ほどのガスに含まれた媚薬成分と呼吸もままならぬ朦朧とする意識、' +
            monster_name + 'の強い力によってすぐにされるがままになった……',
          'ダンジョンの一角にはしばらくの間女の口オナホを貪る' + monster_name + 'の荒い息と、屈服を刻み込まれながら' + adventurer.short_name + 'されるがままになる唾液によるいやらしい音に時折混じる咽せる声が響いていた……',
          '（どぷっ、ぐぷっ、びゅるるるるるっ）',
        ],
      ].sample
    else
      script += [
        [
          adventurer.short_name + 'は突然の衝撃に目を覚まし抵抗しようとするが、先ほどのガスに含まれた媚薬成分と強力な催眠成分によって抵抗しようという意識は霧散してしまう。',
          'たちまち呼吸もままならぬ朦朧とする意識、' +
            monster_name + 'の強い力によってすぐにされるがままになった……',
          'いやむしろ、' + adventurer.short_name + 'は' + monster_name + 'を最愛の相手とさえ悪魔のような催眠の力によって、誤認すらしていた',
        ],
      ].sample
      script += [
        [
          adventurer.short_name + 'は自分から男のグロテスクな肉棒に舌をはわせ、恥垢を舐め取り、丁寧にカリ首を刺激しながら口腔奉仕を続けている。',
          '白魚のような指が陰嚢を刺激し、精子をたっぷりと吐き出して欲しいと催促するようなたまらない刺激を与える。',
          'もちろん' + monster_name + 'も奉仕されるだけでなく、女の頭をがっちりとおさえ、ズボズボと強引なイマラチオ奉仕を強要する。',
          adventurer.short_name + 'は苦しさを伴うその奉仕行為に嬉しげに笑い、積極的に応えた。',
          'ダンジョンの一角にはしばらくの間女の口オナホを貪る' + monster_name + 'の荒い息と、屈服を刻み込まれながら' + adventurer.short_name + 'されるがままになる唾液によるいやらしい音に時折混じる咽せる声が響いていた……',
          '（どぷっ、ぐぷっ、びゅるるるるるっ）',
        ],
      ].sample
    end

    script += [
      [
        monster_name + 'は満足げに呻くと、' + adventurer.short_name + 'の頭を腕でしっかりとロックし、柔らかい舌の上に' +
          '肉棒の先端、鈴口をしっかりと押し付けながら' + half_number_to_full(adventurer.this_turn_v_sperm_shot_count) + '回目の射精となる生命の滾りをたっぷりと放出した。',
        'もちろん吐き出すことはできない。' +
          monster_name + 'は頭を押さえつけ、最後の一滴を飲み干すまで呼吸すらも許さないからだ',
        '結局' + adventurer.short_name + 'は喉を鳴らして、絡みつくほどに濃い男の精液を飲み干してしまった。',
      ],
    ].sample

    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)

    script += [
      monster_name + 'は' + adventurer.short_name + 'の口腔内にプリップリの精液をたっぷりと吐き出すと、満足したように立ち去った。'
    ]
    script += _this_turn_orgasms_and_sperm_shots_chars(adventurer, this_turn_orgasms, this_turn_sperm_shots)
    script
  end

  # ふたなりオナホオナニー
  def trap_91_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はふたなりオナホオナニーの仕掛けを踏んでしまった。',
      '床から飛び出した触手が、' + adventurer.short_name + 'の秘所にまとわりつく。クリトリスが肥大化し、あっという間に肉体と同化したふたなりが生えてしまった！',
      '同時に床から薄いガスが吹き出るととともに通路の進路と退路が壁で塞がれる。',
      'どうやらここから出るには、目の前の壁に埋まったオナホールでオナニーし、射精するしかないらしい。',
      '「よくあるトラップだ。なにもおかしくない。」',
      '常識改変ガスの影響下にある' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'たちは自然とそう思っていた。',
      adventurer.short_name + 'が意を決してオナホールの中に己の偽肉槍を突き入れる。',
      'ゾクゾクゾクッ、と側から見ていてもわかるほどに' + adventurer.short_name + 'の背筋が震えた。',
      'たまらない、きもちいいのだ。このオナホール、まるで生きているようだ。',
      adventurer.short_name + 'は今度は思い切り腰を前後させる。ゾクゾクゾクッ、危険なほどの甘い痺れが背筋を駆け抜け脳を直撃する。',
      'こんなことをこれ以上続けては、自分がおかしくなってしまう。そんな危機感が' + adventurer.short_name + 'の肩を掴む。',
      'だが止まらない、止まれないのだ。',
      'びゅるびゅるっ、どぷどぷっ',
      'ついにたまらず射精してしまう。股間に生暖かい感触。',
      'ふたなりがいつのまにか肉壷の中に伸び、オナホールの中に吐き出した量と同じだけの精液を、自分の肉壷の中に吐き出したのだ。',
      '精液がかかったあたりから熱くやけるような感触。媚薬効果もあるようだ。',
      '媚薬に狂わされたように腰をドスドスとオナホールに突きたて始める。',
      '「おほっ、おおおぉぉぉっ！ いいっ、いいのっ、オチンポッ、オナホすごいいいいぃぃいぃ！」',
      '間抜けな声をあげながら、' + adventurer.short_name + 'はスコスコと腰を振り続けた',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # ふたなり搾精オナホオナニー
  def trap_92_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    unless (leader && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はふたなり搾精オナホオナニーの仕掛けを踏んでしまった。',
      '床から飛び出した触手が、' + adventurer.short_name + 'の秘所にまとわりつく。クリトリスが肥大化し、あっという間に肉体と同化したふたなりが生えてしまった！',
      '同時に床から薄いガスが吹き出るととともに通路の進路と退路が壁で塞がれる。',
      'どうやらここから出るには、目の前の壁に埋まったオナホールでオナニーし、射精するしかないらしい。',
      '「よくあるトラップだ。なにもおかしくない。」',
      '常識改変ガスの影響下にある' + _party_leader_short_names_by(leader.short_name, is_lonely) + 'たちは自然とそう思っていた。',
      adventurer.short_name + 'が意を決してオナホールの中に己の偽肉槍を突き入れる。',
      'ゾクゾクゾクッ、と側から見ていてもわかるほどに' + adventurer.short_name + 'の背筋が震えた。',
      'たまらない、きもちいいのだ。このオナホール、まるで生きているようだ。',
      adventurer.short_name + 'は今度は思い切り腰を前後させる。ゾクゾクゾクッ、危険なほどの甘い痺れが背筋を駆け抜け脳を直撃する。',
      'こんなことをこれ以上続けては、自分がおかしくなってしまう。そんな危機感が' + adventurer.short_name + 'の肩を掴む。',
      'だが止まらない、止まれないのだ。',
      'びゅるびゅるっ、どぷどぷっ',
      'ついにたまらず射精してしまう。股間に生暖かい感触。',
      'ふたなりがいつのまにか肉壷の中に伸び、オナホールの中に吐き出した量と同じだけの精液を、自分の肉壷の中に吐き出したのだ。',
      '精液がかかったあたりから熱くやけるような感触。媚薬効果もあるようだ。',
      '媚薬に狂わされたように腰をドスドスとオナホールに突きたて始める。',
      '「おほっ、おおおぉぉぉっ！ いいっ、いいのっ、オチンポッ、オナホすごいいいいぃぃいぃ！」',
      '間抜けな声をあげながら、' + adventurer.short_name + 'はスコスコと腰を振り続ける。すぐに股間に何かが込み上げる感覚が湧き上がってくる。',
      'ふたたび射精の前兆。ぷりっぷりのザーメンが尿道を駆け上がっていくあのたまらない感覚。',
      'びゅるっ、びゅるるるっ、どびゅるるるるっ！',
      '熱いものが噴き出していく。',
      '「あひっ、あへぇぇぇぇっ！　ちんぽっ、ちんぽ射精すごいぃぃぃいぃぃぃ！」',
      'びゅるっ、どびゅるるるるっ、びゅるるるるっ',
      adventurer.short_name + 'の中にある魔力や生命力といったものが、白い精液に変換され、ふたなりからびゅるびゅる噴き出していく。',
      '「止まらないっ、腰っ、とまらないのぉぉぉぉおぉぉっ！」',
      adventurer.short_name + 'は叫ぶ。最後の一滴まで絞り出した後、再び腰を動かし始めた。',
      '「いひっ、いいぃぃぃ……　止められないっ、こんな気持ちいことっ、止められるはずがないっ！」',
      adventurer.short_name + 'は浅ましい肉欲の命ずるままに、未知の快楽を貪り続けた',
      ' ',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 淫霊憑依レズ乱交
  def trap_93_script(leader, is_lonely, floor, target_and_result, effect)
    unless leader
      puts "NIL!! leader_adv.nil?(#{leader.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は淫霊憑依レズ乱交の仕掛けを踏んでしまった。',
      '床からあやしいガスが噴き出していく。ガスというよりは、これは霊のたぐい、と気付いた時にはもう手遅れだった。',
    ]

    script +=
      if is_lonely
        [
          _party_leader_short_names_by(leader.short_name, is_lonely) + 'に取り付いた淫霊の催淫効果によって耐え難い肉の疼きが' + leader.short_name + 'を襲う。',
          '目の前に、淫霊が実体を伴った女の形を作る。',
          leader.short_name + 'から吸い出した魔力で分身を造ったのだ。',
        'たちまち淫霊の実体たちが' + leader.short_name + 'に襲いかかる。',
          '自らも催淫効果にかかった' + leader.short_name + 'は、淫霊によってたっぷりと弄ばれた……',
        ]
      else
        [
          _party_leader_short_names_by(leader.short_name, is_lonely) + 'に取り付いた淫霊の催淫効果によって耐え難い肉の疼きが' + leader.short_name + 'を襲う。',
          '他の仲間たちも同様のようだ。だれからそうしたのか、いや' +
            leader.short_name + 'がそうしたのかもしれない。',
          'お互いがお互いの肉の疼きを慰め合うために、お互いの秘所や陰核、胸、果ては尻穴まで、指と舌でたっぷりと責め立てる。',
          '経験豊富な淫霊のテクニックを借りてお互いを容赦なく追い詰める雌犬たちの群れは、たっぷりと快楽を貪りあった。',
        ]
      end

    target_and_result.each do |params|
      adventurer = params[:target]
      script += _this_turn_orgasms_and_sperm_shots_chars(adventurer, params[:this_turn_orgasms], params[:this_turn_sperm_shots])
      script += adventurer.ero_trap_effect_script
    end

    script
  end

  # 淫霊憑依ふたなり乱交
  def trap_94_script(leader, is_lonely, floor, target_and_result, effect)
    unless leader
      puts "NIL!! leader_adv.nil?(#{leader.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end
    leader_adv = leader

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は淫霊憑依レズふたなり乱交の仕掛けを踏んでしまった。',
      '床からあやしいガスが噴き出していく。ガスというよりは、これは霊のたぐい、と気付いた時にはもう手遅れだった。',
    ]

    script +=
      if is_lonely
        [
          _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'に取り付いた淫霊の催淫効果によって耐え難い肉の疼きが' + leader_adv.short_name + 'を襲う。',
          '目の前に、淫霊が実体を伴った女の形を作る。',
          leader_adv.short_name + 'から吸い出した魔力で分身を造ったのだ。',
          'いやそれだけではない。' + leader_adv.short_name + 'の股間には立派な男のものを模したものが生えていた。',
          'いまこれを弄ばれたら、大変なことになる。' + leader_adv.short_name + 'がゴクリと唾を飲んだ瞬間淫霊たちを目があった。',
        'たちまち淫霊の実体たちが' + leader_adv.short_name + 'に襲いかかる。',
          '偽肉棒をねちっこく弄び、女穴に迎え入れる。それでいてそう簡単に射精はさせないぞとばかりに徹底的にいたぶられる。',
          '自らも催淫効果にかかった' + leader_adv.short_name + 'は、淫霊によってたっぷりと弄ばれた……',
        ]
      else
        [
          _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'に取り付いた淫霊の催淫効果によって耐え難い肉の疼きが' + leader_adv.short_name + 'を襲う。',
          '他の仲間たちも同様のようだ。',
          'いやそれだけではない。己の股間には隆隆たる雄々しい肉槍が生えていた。どう使うのかは、淫霊たちが流し込んだ知識によって知っている。',
          'ブチ込みたい。これを女の穴の中にブチこんでやりたい。徹底的に嬲り、狂わせてやりたい。……。',
          'ふと顔をあげると、仲間と目があった。彼女も同じことを考えていることがわかった。',
          '誰からそうしたのか、いや' +
            leader_adv.short_name + 'がそうしたのかもしれない。',
          'お互いがお互いの肉の疼きを慰め合うために、お互いの秘所や尻穴に太いモノをねじ込んだ。射精しぐったりとしていると、仲間たちに襲われる。',
          '女ゆえにお互いの弱点を、感じる場所を知り尽くした相手によって徹底的に嬲られ辱められる喜び。' +
          '陰核、胸、果ては尻穴まで、指と舌でたっぷりと責め立てる。',
          '経験豊富な淫霊のテクニックを借りてお互いを容赦なく追い詰める雌犬たちの群れは、たっぷりと快楽を貪りあった。',
        ]
      end
    target_and_result.each do |params|
      adventurer = params[:target]
      script += _this_turn_orgasms_and_sperm_shots_chars(adventurer, params[:this_turn_orgasms], params[:this_turn_sperm_shots])
      script += adventurer.ero_trap_effect_script
    end
    script
  end

  # 淫霊憑依オナニー
  def trap_95_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は淫霊憑依オナニーの仕掛けを踏んでしまった。',
      '床からあやしいガスが噴き出していく。ガスというよりは、これは霊のたぐい、と気付いた時には' +
        adventurer.short_name + 'はもう手遅れだった。',
      '同時に床から薄いガスが吹き出るととともに通路の進路と退路が壁で塞がれる。',
      'どうやらここから出るには、ここでオナニーし、絶頂するしかないらしい。',
      '「よくあるトラップだ。なにもおかしくない。」',
      '常識改変ガスの影響下にある' + _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'たちは自然とそう思っていた。',
      'そぉっと指先を秘所に沿わせる。すぐに指が加速した。誰かが指を勝手に動かしていくような感覚。今までに何人もの女を狂わせてきた淫霊の指技の全力が、' + adventurer.short_name + 'を責め狂わせていく。',
      '「ひぃぃぃっ、いいっ、いいのぉぉぉぉっ！　もっともっとオナニーしたい！　クリトリス気持ちいいっ！　腰、浮くくらい気持ちいい！」',
      '自然といやらしい言葉が溢れる。その途端、羞恥心でさらに快楽が加速する。',
      '「おまんこっ、おまんこもいじりたいっ！　指っ、挿れているのにッ、細いっ、こんなのじゃ全然満足できないっ！」',
      adventurer.short_name + 'は卑猥な言葉を叫びながら、何度も何度も、扉が開いてさえも自慰行為を続けていた……',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 淫霊憑依公開露出オナニー
  def trap_96_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は淫霊憑依公開露出オナニーの仕掛けを踏んでしまった。',
      '床からあやしいガスが噴き出していく。ガスというよりは、これは霊のたぐい、と気付いた時には' +
        adventurer.short_name + 'はもう手遅れだった。',
      '同時に床から薄いガスが吹き出るととともに通路の進路と退路が壁で塞がれる。',
      'どうやらここから出るには、ここでオナニーし、絶頂するしかないらしい。',
      '「よくあるトラップだ。なにもおかしくない。」',
      '常識改変ガスの影響下にある' + _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'たちは自然とそう思っていた。',
      'そぉっと指先を秘所に沿わせる。すぐに指が加速した。誰かが指を勝手に動かしていくような感覚。今までに何人もの女を狂わせてきた淫霊の指技の全力が、' + adventurer.short_name + 'を責め狂わせていく。',
      '「ひぃぃぃっ、いいっ、いいのぉぉぉぉっ！　もっともっとオナニーしたい！　クリトリス気持ちいいっ！　腰、浮くくらい気持ちいい！」',
      '自然といやらしい言葉が溢れる。その途端、羞恥心でさらに快楽が加速する。',
      '仲間たちが自分の痴態を見ているのがわかる。視線を感じると、さらに快楽のギアが一段上がった。',
      adventurer.short_name + 'は大きく股を開き、ぐちょぐちょに濡れている秘所を曝け出しながら己を嬲り続ける。',
      '「おまんこっ、おまんこもいじりたいっ！　指っ、挿れているのにッ、細いっ、こんなのじゃ全然満足できないっ！」',
      '視線と注目と、そして僅かに混じった軽蔑と羨望を感じる。それがいい、それがいいのだ。人に見られるほど、快楽が加速し収拾がつかなくなっていく。',
      adventurer.short_name + 'は卑猥な言葉を叫びながら、何度も何度も、扉が開いてさえも激しい公開露出自慰行為を続けていた……',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 地雷
  def trap_101_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は地雷罠の仕掛けを踏んでしまった。',
      'これはまずいと思う間もなく、魔力を込められた罠の機構が発動した。',
      '足元で小規模な爆発。幸い鍛え上げられた冒険者である' + adventurer.short_name + 'はとっさに防御姿勢をとれたものの、ダメージは免れなかった。',
    ]
    script += _get_trap_damage_script(adventurer)
    script += [
      '……ダメージそのものは大したことはないが、全くのゼロでもない。このような卑劣な罠が' + _adventurer_type + 'を弱らせるためにこのダンジョンには無数に仕掛けられている……',
    ]
    life_point_at_that_time = nil
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 痺れ矢
  def trap_102_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は痺れ矢の仕掛けを踏んでしまったようだ。',
      'ドシュッ、という軽い音とともに壁の隙間から小ぶりな矢が放たれた。',
    ]
    script += _get_trap_damage_script(adventurer)
    script += [
      '……ダメージそのものは大したことはないが、全くのゼロでもない。このような卑劣な罠が' + _adventurer_type + 'を弱らせるためにこのダンジョンには無数に仕掛けられている……',
    ]

    life_point_at_that_time = nil
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point_at_that_time)
    script
  end

  # 機械式くすぐりトラップ
  def trap_103_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'は機械式くすぐりトラップの仕掛けを踏んでしまったようだ。',
      '床の隙間から無数の機械仕掛けの腕がせり上がってくる。',
      '機械の腕は' + adventurer.short_name + 'の体をがっちりと捉えると、さらに別の腕たちが' + adventurer.short_name +
        'の肉体を無遠慮にまさぐっていく！',
      adventurer.short_name + 'は必死に逃れようとするが、がっちりと掴まれている腕はそう簡単には外せず、' +
        'くすぐられたことで腕に込めたはずの力も抜けてしまう',
      '機械の腕たちは、腋や脇腹、太ももや腹だけでなく胸や秘所など敏感な場所も容赦なく弄っていく',
      'くすぐったさにいり混じる肉の悦びに、' + adventurer.short_name + 'は乱れた姿を晒してしまった。',
    ]
    script += adventurer.ero_trap_sperm_n_orgasms_script(this_turn_sperm_shots, this_turn_orgasms, life_point)
    script
  end

  # 膨乳効果つきスライムプール
  def trap_121_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots,
                      party, boom_hooter_success, without_rescue, companion_is_brainwashed, is_hooter_level_max)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      adventurer.short_name + 'はプールトラップの仕掛けを踏んでしまったようだ。',
      '足元の地面が音を立てて開くと、怪しげに輝くスライムで満たされたプールに' + adventurer.short_name + 'は落ちてしまった。',
      'なんとスライムプールの正体は、膨乳効果つきスライムプールだった！',
      'スライム達が胸を中心に' + adventurer.short_name + 'の全身を弄っていく！',
      '甘く切ない痺れにも似た感覚に、'+ adventurer.short_name + 'は甘い声を漏らしてしまう。',
      'その快楽の強さが、全身を揉みしだくスライムの数の多さで倍増される。一刻も早く抜け出さなければいけないと、本能が' +
        adventurer.short_name + 'に強く警告する。',
    ]

    script +=
      if is_lonely
        [
          '助けてくれる仲間もいない' + adventurer.short_name + 'は、スライムプールの中でもがく。' +
            'しかし粘度の高い泥沼のようなスライムプールの中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、スライム達が飽きて放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を呆れるほど味合わされ続けた' + adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができなかった。',
        ]
      elsif companion_is_brainwashed
        brainwashed_member_names = (party.adventurers.values - [adventurer]).map(&:short_name).join('、')
        [
          '助けを求めるように' + adventurer.short_name + 'は仲間の名を呼び、必死に手を伸ばすが' +
            brainwashed_member_names + 'たちはニヤニヤと笑うばかりで手を差し伸べようともしない。',
          '必死に助けを求めながら' + adventurer.short_name + 'は、スライムプールの中でもがく。' +
            'しかし粘度の高い泥沼のようなスライムプールの中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、スライム達が飽きて放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を呆れるほど味合わされ続けた' +
            adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができない。',
          'その無様な姿を見ても、洗脳された仲間達はただ不気味に笑うだけだった……',
        ]
      else
        [
          adventurer.short_name + 'は仲間の手を借りて膨乳効果つきスライムプールから脱出することができた。',
          'ベタベタと気色悪い感触に悩まされながら、' + adventurer.short_name + 'はその場を後にした',
        ]
      end

    if boom_hooter_success
      script +=
        if is_hooter_level_max
          [
            'もはや限界を超えて' + adventurer.short_name + 'の胸は、さらに豊満になったようだ……',
            'たとえ妊婦といえどもこれほどには大きくならぬだろう。すべての動きを阻害する巨大な胸は、まさに乳牛のような外観になっている。',
            '疼きを覚える' + adventurer.short_name + 'の胸の先端からは母乳すら噴き出していた。',
          ]
        else
          [
            'スライムプールの効果は、' + adventurer.short_name + 'の肉体に不可逆の効果をもたらしたようだ。',
            adventurer.short_name + 'は消えぬ違和感を訴える胸をみると、そこが先ほどよりも大きくなっていることに気づいた……',
            '（' + adventurer.short_name + 'の胸は、' + adventurer.hooter_level_name + 'サイズになった）',
          ]
        end
      if adventurer.hooter_level < 3
        script += [
          '気のせいかもしれないが、' + adventurer.short_name + 'はどことなく嬉しそうだ……',
        ]
      elsif adventurer.hooter_level > 4
        script += [
          '明らかに並の女ではありえないサイズになった自分の胸を見た、' + adventurer.short_name + 'は絶望の表情を浮かべる。',
          '冒険や戦闘にはもはや邪魔にしかならない胸。こんなものを手に入れるために、自分は迷宮に挑んだわけではないのだ。',
          'いまの' + adventurer.short_name + 'の肉体を一目見た男達は冒険者への敬意ではなく、セックスへの劣情を抱くことは間違いないだろう。',
        ]
      end
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 感度上昇効果スライムプール
  def trap_122_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots,
                      party, boom_sensitive_success, boom_sensitive_position, without_rescue, companion_is_brainwashed)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      adventurer.short_name + 'はプールトラップの仕掛けを踏んでしまったようだ。',
      '足元の地面が音を立てて開くと、怪しげに輝くスライムで満たされたプールに' + adventurer.short_name + 'は落ちてしまった。',
      'なんとスライムプールの正体は、感度上昇効果スライムプールだった！',
      '無数のスライム達が' + adventurer.short_name + 'の全身を弄っていく！',
      '甘く切ない痺れにも似た感覚に、'+ adventurer.short_name + 'は耐え切れず甘い喘ぎ声を漏らしてしまう。',
      '全身を溶かしていくようなその快楽の強さが、全身を揉みしだく数えきれぬスライムの数で倍増される。脳天を焼き切るような刺激に、一刻も早く抜け出さなければいけないと、本能が' +
        adventurer.short_name + 'に強く警告する。',
    ]

    script +=
      if is_lonely
        [
          '助けてくれる仲間もいない' + adventurer.short_name + 'は、スライムプールの中でもがく。' +
            'しかし粘度の高い泥沼のようなスライムプールの中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、スライム達が飽きて放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を呆れるほど味合わされ続けた' + adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができなかった。',
        ]
      elsif companion_is_brainwashed
        brainwashed_member_names = (party.adventurers.values - [adventurer]).map(&:short_name).join('、')
        [
          '助けを求めるように' + adventurer.short_name + 'は仲間の名を呼び、必死に手を伸ばすが' +
            brainwashed_member_names + 'たちはニヤニヤと笑うばかりで手を差し伸べようともしない。',
          '必死に助けを求めながら' + adventurer.short_name + 'は、スライムプールの中でもがく。' +
            'しかし粘度の高い泥沼のようなスライムプールの中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、スライム達が飽きて放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を嫌という程味合わされ続けた' +
            adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができない。',
          'その無様な姿を見ても、洗脳された仲間達はただ不気味に笑うだけだった……',
        ]
      else
        [
          adventurer.short_name + 'は仲間の手を借りて感度上昇効果スライムプールから脱出することができた。',
          'ベタベタと気色悪い感触に悩まされながら、' + adventurer.short_name + 'はその場を後にした',
        ]
      end

    if boom_sensitive_success
      position_name =
        case boom_sensitive_position
          when 'c'
            'クリトリス'
          when 'v'
            '肉壷'
          when 'a'
            'アナル'
          when 'b'
            '胸'
        end
      script += [
        'スライムプールの効果は、' + adventurer.short_name + 'の肉体に不可逆の効果をもたらしたようだ。',
        adventurer.short_name + 'は消えぬ違和感を訴える' + position_name + 'に軽く指を這わせてみる。' +
          '軽く触れただけにもかかわらず稲妻のような快楽が頭へと走り抜けた！',
        adventurer.short_name + 'の' + position_name + 'が快楽に従順で、敏感なセックス向きの場所へと作り変えられたことは明白だった……',
        '（' + adventurer.short_name + 'は、ステータス「' + adventurer.sensitive_name(boom_sensitive_position) + '」を取得した）',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 触手だまり
  def trap_131_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots,
                      party, companion_is_brainwashed, tentacle_holic_success)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      adventurer.short_name + 'は触手だまりの仕掛けを踏んでしまったようだ。',
      '足元の地面が音を立てて開くと、大きな穴の中で触手達がうねうねと蠢めく触手だまりに' + adventurer.short_name + 'は落ちてしまった。',
      '目ざとく獲物を見つけた無数の触手達が' + adventurer.short_name + 'の全身に群がり、そこを弄んでいく！',
      '甘く切ない痺れにも似た感覚に、'+ adventurer.short_name + 'は耐え切れず甘い喘ぎ声を漏らしてしまう。',
      '全身を溶かしていくようなその快楽の強さが、全身を揉みしだく数えきれぬ触手の数で倍増される。脳天を焼き切るような刺激に、一刻も早く抜け出さなければいけないと、本能が' +
        adventurer.short_name + 'に強く警告する。',
    ]

    script +=
      if is_lonely
        [
          '助けてくれる仲間もいない' + adventurer.short_name + 'は、触手で満たされた穴の中でもがく。' +
            'しかしぶにぶにとした柔らかいのような触手の中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、絞りカス同然になるまで魔力を根こそぎ奪われ、用済みと判断した触手達が放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を呆れるほど味合わされ続けた' + adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができなかった。',
        ]
      elsif companion_is_brainwashed
        brainwashed_member_names = (party.adventurers.values - [adventurer]).map(&:short_name).join('、')
        [
          '助けを求めるように' + adventurer.short_name + 'は仲間の名を呼び、必死に手を伸ばすが' +
            brainwashed_member_names + 'たちはニヤニヤと笑うばかりで手を差し伸べようともしない。',
          'しかしぶにぶにとした柔らかいのような触手の中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、絞りカス同然になるまで魔力を根こそぎ奪われ、用済みと判断した触手達が放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を嫌という程味合わされ続けた' +
            adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができない。',
          'その無様な姿を見ても、洗脳された仲間達はただ不気味に笑うだけだった……',
        ]
      else
        [
          adventurer.short_name + 'は仲間の手を借りて触手だまりから脱出することができた。',
          'ベタベタと気色悪い感触に悩まされながら、' + adventurer.short_name + 'はその場を後にした',
        ]
      end

    if tentacle_holic_success
      script += [
        '触手たちによる陵辱の効果は、' + adventurer.short_name + 'の肉体に不可逆の効果をもたらしたようだ。',
        adventurer.short_name + 'は限度を超えた快楽を注ぎ込まれたことでどこかが壊れてしまったようだ……',
        '意識せずに秘所を愛撫し続ける両手の与える快楽は児戯のようなもので触手による法外な快楽とは比べることすらできもしない。',
        adventurer.short_name + 'は触手による快楽の味を忘れられなくなってしまった。',
        '（' + adventurer.short_name + 'は、ステータス「' + adventurer.get_skill_holic_name(:tentacle_sex_holic) + '」を取得した）',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 触手の巣
  def trap_132_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots,
                      party, companion_is_brainwashed, tentacle_holic_success)
    leader_adv = leader || adventurer
    unless (leader_adv && adventurer)
      puts "NIL!! leader_adv.nil?(#{leader_adv.nil?}) || adventurer.nil?(#{adventurer.nil?}) in #{__method__}" if DEBUG_SCRIPT_MODE
      return []
    end

    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader_adv.short_name, is_lonely) + 'の足元でカチリという音がした。',
      adventurer.short_name + 'は「触手の巣」の仕掛けを踏んでしまったようだ。',
      '足元の地面が音を立てて開くと、大きな穴の中で触手達がうねうねと蠢めく触手の巣に' + adventurer.short_name + 'は落ちてしまった。',
      '目ざとく獲物を見つけた無数の触手達が' + adventurer.short_name + 'の全身に群がり、そこを弄んでいく！',
      '甘く切ない痺れにも似た感覚に、'+ adventurer.short_name + 'は耐え切れず甘い喘ぎ声を漏らしてしまう。',
      '全身を溶かしていくようなその快楽の強さが、全身を揉みしだく数えきれぬ触手の数で倍増される。脳天を焼き切るような刺激に、一刻も早く抜け出さなければいけないと、本能が' +
        adventurer.short_name + 'に強く警告する。',
    ]

    script +=
      if is_lonely
        [
          '助けてくれる仲間もいない' + adventurer.short_name + 'は、触手で満たされた穴の中でもがく。' +
            'しかしぶにぶにとした柔らかいのような触手の中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、絞りカス同然になるまで魔力を根こそぎ奪われ、用済みと判断した触手達が放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を呆れるほど味合わされ続けた' + adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができなかった。',
        ]
      elsif companion_is_brainwashed
        brainwashed_member_names = (party.adventurers.values - [adventurer]).map(&:short_name).join('、')
        [
          '助けを求めるように' + adventurer.short_name + 'は仲間の名を呼び、必死に手を伸ばすが' +
            brainwashed_member_names + 'たちはニヤニヤと笑うばかりで手を差し伸べようともしない。',
          'しかしぶにぶにとした柔らかいのような触手の中であがいても、ますます深みへと飲み込まれていくだけだ。',
          'ついに全身を飲み込まれた' + adventurer.short_name + 'は、絞りカス同然になるまで魔力を根こそぎ奪われ、用済みと判断した触手達が放り出すまでたっぷりと弄ばれた。',
          '限度を超えた快楽と、あまりにも深すぎる絶頂を嫌という程味合わされ続けた' +
            adventurer.short_name + 'は解放された後もガクガクと痙攣しそこから動くことができない。',
          'その無様な姿を見ても、洗脳された仲間達はただ不気味に笑うだけだった……',
        ]
      else
        [
          adventurer.short_name + 'は仲間の手を借りて触手の巣から脱出することができた。',
          'ベタベタと気色悪い感触に悩まされながら、' + adventurer.short_name + 'はその場を後にした',
        ]
      end

    if tentacle_holic_success
      script += [
        '触手たちによる陵辱の効果は、' + adventurer.short_name + 'の肉体に不可逆の効果をもたらしたようだ。',
        adventurer.short_name + 'は限度を超えた快楽を注ぎ込まれたことでどこかが壊れてしまったようだ……',
        '意識せずに秘所を愛撫し続ける両手の与える快楽は児戯のようなもので触手による法外な快楽とは比べることすらできもしない。',
        adventurer.short_name + 'は触手による快楽の味を忘れられなくなってしまった。',
        '（' + adventurer.short_name + 'は、ステータス「' + adventurer.get_skill_holic_name(:tentacle_sex_holic) + '」を取得した）',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  def trap_141_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はバイブ付き貞操帯の仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の両足をがっちりと押さえつける！、',
      '飛び出してきた触手をとっさに避けようとした' + adventurer.short_name + 'だが、多くの触手に足を絡め取られ身動き取れなくなってしまう！',
      '触手は服の隙間に潜り込み、下着を横にずらすといつの間にか取り出していた貞操帯を' + adventurer.short_name +
        'の秘所へと取り付けた！！！',
      ' ',
    ]

    script +=
      if effect[:lost_virgin]
        [
          '信じられないほどの激痛が' + adventurer.short_name + 'を襲う。',
          '未踏の処女地を、貞操帯に取り付けられた張り型が強引にこじ開け、中に入り込んでいるのだ。',
          '血の通わぬ玩具に処女を踏みにじられたという恐ろしい現実に叫び声を上げながら' + adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は' +
            '手足にまとわりつき、抵抗を許さぬままに奥までバイブを押し込んでいく。',
          '半狂乱の' + adventurer.short_name + 'がめちゃくちゃに武器で触手を叩くと、ついに触手は諦めたように引き上げていく。いや用が済んだと言わんばかりに引き上げたのだ。' +
            '貞操帯を、乙女の穴へとはめ込んだ触手たちは再び地中へと消えていった。',
          adventurer.short_name + 'のつい先ほどまで処女だった肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたて、' +
            'そこを快楽のためだけの都合の良い肉穴として容赦なく作り変えていった。',
        ]
      else
        [
          '貞操帯にはバイブが内側に取り付けられていたようだ。',
          '男の逸物を模した張型が、' + adventurer.short_name + 'の内側を執拗かつ念入りに耕し、甘い女の喜びを刷り込み手懐けていく。',
        ]
      end

    script += [
      '膣内で甘く震えるバイブの存在感と違和感が' + adventurer.short_name + 'に恐ろしい違和感を覚えさせる。',
      'バイブ全体がブルブルと揺れ動き、膣内を甘く震わせ、ゆっくりと、しかし確実に手懐けていく。振動と回転が与えられるたび' + adventurer.short_name + 'は顎を上げ甘い息を吐き出す。',
      '一歩でもそこから動こうとするたびに、耐えがたい肉の疼きを' + adventurer.short_name + 'に味あわせていく。だがこの迷宮の中では足を止めることは致命傷になり得る。',
      '背筋を震わすほどの肉悦を味あわせられながら' + adventurer.short_name + 'は震えて崩れそうになる膝を叱咤しながら、重い足取りで進んでいった、',
    ]

    if effect[:bad_status_in_body].include?('v_sensitive')
      script += [
        '凶悪な形のバイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「肉壷敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  def trap_142_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は極太バイブ付き貞操帯の仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の両足をがっちりと押さえつける！、',
      '飛び出してきた触手をとっさに避けようとした' + adventurer.short_name + 'だが、多くの触手に足を絡め取られ身動き取れなくなってしまう！',
      '触手は服の隙間に潜り込み、下着を横にずらすと、いつの間にか取り出していた貞操帯を' + adventurer.short_name +
        'の秘所へと取り付けた！！！',
      ' ',
    ]

    script +=
      if effect[:lost_virgin]
        [
          '信じられないほどの激痛が' + adventurer.short_name + 'を襲う。',
          '未踏の処女地を、貞操帯に取り付けられた張り型が強引にこじ開け、中に入り込んでいるのだ。',
          '血の通わぬ玩具に処女を踏みにじられたという恐ろしい現実に叫び声を上げながら' + adventurer.short_name + 'は必死に引き抜こうとするが、逆に触手は' +
            '手足にまとわりつき、抵抗を許さぬままに奥まで極太のバイブを押し込んでいく。',
          '並の男の逸物よりもふた回りは大きい、雄々しすぎる逸物に肉を引き裂かれる痛みに' + adventurer.short_name + 'は喉から獣のような声をあげて吠え狂う。' +
            '痛みに理性が引き裂かれ、思考が激痛で塗りつぶされていく。',
          '半狂乱の' + adventurer.short_name + 'がめちゃくちゃに武器で触手を叩くと、ついに触手は諦めたように引き上げていく。いや用が済んだと言わんばかりに引き上げたのだ。' +
            '貞操帯を、乙女の穴へとはめ込んだ触手たちは再び地中へと消えていった。',
          adventurer.short_name + 'のつい先ほどまで処女だった肉壷を高濃度の媚薬を分泌しながらたっぷりと責めたて、' +
            'そこを快楽のためだけの都合の良い肉穴として容赦なく作り変えていく。',
        ]
      else
        [
          '貞操帯には極太のバイブが内側に取り付けられていたようだ。',
          '雄々しすぎる男の逸物を模した張型が、' + adventurer.short_name + 'の内側を執拗かつ念入りに耕し、甘い女の喜びを刷り込み手懐けていく。',
        ]
      end

    script += [
      '膣内で甘く震える極太バイブの存在感と違和感が' + adventurer.short_name + 'に恐ろしい違和感を覚えさせる。',
      '極太バイブ全体がブルブルと揺れ動き、膣内を甘く震わせ、ゆっくりと、しかし確実に手懐けていく。振動と回転が与えられるたび' + adventurer.short_name + 'は顎を上げ甘い息を吐き出す。',
      '一歩でもそこから動こうとするたびに、耐えがたい肉の疼きを' + adventurer.short_name + 'に味あわせていく。だがこの迷宮の中では足を止めることは致命傷になり得る。',
      '背筋を震わすほどの肉悦を味あわせられながら' + adventurer.short_name + 'は震えて崩れそうになる膝を叱咤しながら、重い足取りで進んでいった、',
    ]

    if effect[:bad_status_in_body].include?('v_sensitive')
      script += [
        '凶悪な形の極太バイブの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「肉壷敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  def trap_143_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'はローター付き貞操帯の仕掛けを踏んでしまった。',
      '地面から飛び出してきた触手が、' + adventurer.short_name + 'の両足をがっちりと押さえつける！、',
      '飛び出してきた触手をとっさに避けようとした' + adventurer.short_name + 'だが、多くの触手に足を絡め取られ身動き取れなくなってしまう！',
      '触手は服の隙間に潜り込み、下着を横にずらすと、いつの間にか取り出していた貞操帯を' + adventurer.short_name +
        'の秘所へと取り付けた！！！',
      ' ',
      '貞操帯にはいくつものローターが内側に取り付けられていたようだ。',
      '卵型の肉玩具たちは、' + adventurer.short_name + 'の女性器や陰核を表面から内側へと快楽をすり込むように、執拗かつ念入りに耕し、甘い女の喜びを刷り込み、' +
        'その肉体を快楽の喜びによって手懐けていく。',
      '秘所で甘く震えるローターの存在感と違和感が' + adventurer.short_name + 'に甘い違和感を覚えさせる。',
      'ローターたちが反響するようにブルブルと揺れ動き、媚肉を甘く震わせ、ゆっくりと、しかし確実に手懐けていく。振動と回転が与えられるたび' + adventurer.short_name + 'は顎を上げ甘い息を吐き出す。',
      '一歩でもそこから動こうとするたびに、耐えがたい肉の疼きを' + adventurer.short_name + 'に味あわせていく。だがこの迷宮の中では足を止めることは致命傷になり得る。',
      '背筋を震わすほどの肉悦を味あわせられながら' + adventurer.short_name + 'は震えて崩れそうになる膝を叱咤しながら、重い足取りで進んでいった、',
    ]

    if effect[:bad_status_in_body].include?('c_sensitive')
      script += [
        '絶え間なく続く凶悪なローターの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「陰核敏感」を取得しました。',
      ]
    end
    if effect[:bad_status_in_body].include?('v_sensitive')
      script += [
        '絶え間なく続く凶悪なローターの刺激は' + adventurer.short_name + 'の肉体を容赦なく不可逆的に作り変えてしまったようだ。',
        adventurer.short_name + 'はステータス「肉壷敏感」を取得しました。',
      ]
    end

    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 性感魔道弾
  def trap_151_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は性感魔道弾の仕掛けを踏んでしまった。',
      '光り輝く性感魔道弾が床から大量に飛び出し、宙空に次々と不気味に静止した。',
      '宙に静止した性感魔道弾たちは' + adventurer.short_name + 'を取り囲むように位置どると、一斉に' + adventurer.short_name +
        'に向けて襲いかかった！',
      '最初の性感魔道弾が当たる瞬間、' + adventurer.short_name + 'は身をこわばらせた。爆発ダメージ系の罠だと考えたからだ。',
      '魔道弾が炸裂した瞬間、その考えは誤りだったと' + adventurer.short_name + 'は悟った。いや悟らされた。',
      '体に炸裂した場所を中心に甘い快楽が容赦なく走り抜け、背筋が反り返ってしまう。動きが止まったところにさらに性感魔道弾が炸裂する。',
      '顔にさらに一発の性感魔道弾が当たった。防具の上からでも強力な快楽が走る炸裂弾が肌に直接炸裂すれば、どうなるかなど明白だった。強力な' +
        '快楽の暴風雨が' + adventurer.short_name + 'の内側で荒れ狂う。',
      '性感魔道弾が弾けるたびに、' + adventurer.short_name + 'はビクビクと全身を戦慄かせ、快楽に震えながら何度も絶頂した。',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 性感拷問機
  def trap_152_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は性感拷問機の仕掛けを踏んでしまった。',
      '足元から人が入るカプセル型の機械が飛び出す！ 機械式のアームたちが' + adventurer.short_name + 'の体を掴むと無理やり不気味な機械の中へと引きずり込んだ。',
      '暴れる' + adventurer.short_name + 'の顔に向けてブシュッという音とともに勢いよくガスが噴きかけられる。',
      '思わず吸い込んでしまった' + adventurer.short_name + 'は、意識が遠のいていくのを感じる。その隙に機械アームたちは' + adventurer.short_name +
        'を逃げられぬようがっちりと拘束した！',
      '口元を覆うように、呼吸器に似たガス噴霧器が取り付けられる。口に押し込まれたチューブから流れ出すドロリとしたゲル状の液体は、' +
        '正体こそ知れぬものの、女を啼き狂わせるためのものであることは明らかだ。',
      adventurer.short_name + 'の抵抗が弱まったことで下準備が完了したと判断した機械たちはさらに次の段階に進むことにしたようだ。',
      'ブラシ、バイブレーターが' + adventurer.short_name + 'の体に群がっていく。',
      '「ひぃぃぃっ、いやっ、いやぁぁぁぁ！」',
      adventurer.short_name + 'は情けない声をあげる。ガスで鈍らされた意識が快楽で覚醒させられたのだ。薬毒で無防備になった' +
        'ところに、快楽本能へと訴えかけられる情熱的な律動を全身に叩きつけられてはなんの防御もできるはずがなかった。',
      '機械式カプセルがガタガタと内側からの激しい振動を伝える。だが外側からどれほど強い衝撃を与えても開かない扉は、内側からの' +
        '衝撃で開くはずがなかった。',
      '快楽に次ぐ快楽。どれほどイキ狂っても際限なく快楽が押し寄せ、' + adventurer.short_name + 'の思考を蝕んでいく。',
      adventurer.short_name + 'の反応が単調になったり弱々しくなると機械式の注射機ががっちりと拘束した腕に突き刺される。',
      '薬物の中身は高濃度の媚薬、精力剤、強心剤などだ。効果は恐ろしいほど強いが、その分副作用も強い。後遺症が残る可能性もある悪魔のような薬物を' +
        '機械たちはなんの容赦もなく' + adventurer.short_name + 'に投与した。',
      '時間が経つほどに、カプセルの振動は激しくなり、カプセルの中から聞こえる' + adventurer.short_name + 'の声は意味をなさない獣のようなものになっていく。',
      'どれほど' + adventurer.short_name + 'が敗北を認め、懇願し、止めてくれるよう喚いても、機械たちは無慈悲に行為を続けていく。',
      '....',
      '呆れるほどの回数の絶頂を重ね、呼吸を忘れるほど何度も徹底的にイキ狂わされた後、ようやく' + adventurer.short_name + 'はカプセルの中から解放された。',
      '地面にぐったりと倒れ伏し、口からは粘っこいよだれを垂れ流している' + adventurer.short_name + 'の姿は、彼女が味わった快楽が' +
        'どれほど法外だったかを雄弁に物語っていた……',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 拘束強制鞭打ち
  def trap_161_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は拘束強制鞭打ちの仕掛けを踏んでしまった。',

      '無数の触手たちが' + adventurer.short_name + 'に襲いかかり、四肢を絡め取っていく。',
      adventurer.short_name + 'はたちまちのうちに、脚を大きく開かされ尻を突き出すような姿勢を取らされる。' +
        adventurer.short_name + 'は力の限り必死にもがくが、触手たちを振りほどくことも抜け出すこともできない。',
      '太く重い触手たちが、' + adventurer.short_name + 'に狙いを定めた。',
      '大きくしなり、勢いをつけた触手たちが、' + adventurer.short_name + 'の尻に振り下ろされる。',
      'ビシィィッ！　小気味の良い、肉を叩く乾いた音が迷宮に響く。',
      adventurer.short_name + 'がたまらず悲鳴をあげる。',
      'ビシィッ！ 別の触手鞭が' + adventurer.short_name + 'の尻に叩きつけられる。',
      'ビシッ、ビシィィ！　ビシッ！ ピシッィ！　次々と' + adventurer.short_name + 'の尻、そして背中に何本もの触手鞭が叩きつけられる。',
      '触手鞭たちは息を合わせ、たっぷりとしなりをつけ勢いよく' + adventurer.short_name + 
      'に叩きつける。思い切り触手鞭を叩きつけることで肺腑の底まで酸素を吐き出させ、' + adventurer.short_name + 'が必死に息を吸い込んだところで、再び思い切り触手鞭を叩きつける。',
      '呼吸の自由すら奪われながら、' + adventurer.short_name + 'は痛みと屈辱の中で悶える。',
      'いや触手鞭たちが与えてくるものは痛みと屈辱だけではなかった。', 
      adventurer.short_name + 'の背中が勢いよく跳ね上がる。' + 
      'その悲鳴には甘いものも含まれていた。',
      '被虐快楽。間違いなく' + adventurer.short_name + 'は触手鞭が与える痛みの中に快楽を見出していた。',
      'それは荒れ狂う激痛の中で精神を守るための' + adventurer.short_name + 'の本能的な防御作用、というよりは触手たちが休みなく塗り込んでくる触手粘液の効果に違いなかった。',
      'たとえそれが薬物による効果だったとしても、痛みの中に快楽を見出してしまった経験を' + adventurer.short_name + 'は忘れることはないだろう。',
      'これから先も、' + adventurer.short_name + 'は痛みの中に快楽を見出すことになるだろう。',
      '刻み込まれた背徳の被虐快楽の経験は、' + adventurer.short_name + 'の肉体や精神をゆっくりと作り変えていった……。',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 拘束触手椅子
  def trap_162_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は拘束触手椅子の仕掛けを踏んでしまった。',
      '無数の触手たちが' + adventurer.short_name + 'に襲いかかり、四肢を絡め取っていく。',
      adventurer.short_name + 'はたちまちのうちに、肉色の触手が絡みついた不気味な椅子に座らされ、縛り付けられてしまった。。' +
        adventurer.short_name + 'は力の限り必死にもがくが、触手たちを振りほどくことも抜け出すこともできない。',
      '「ひぃぃぃっ！？　いやっ、なにっ、これぇぇ！」',
      '触手たちが隙間から服の中に潜り込んだのだ。' + adventurer.short_name + 
        'はたまらず甘い悲鳴をあげる。',
      '触手たちは' + adventurer.short_name + 'の性感帯を容赦なく刺激し快楽を好き放題流し込んでいく。',
      adventurer.short_name + 'は無防備に喉を晒し、悲鳴をあげ喘ぎ狂い、吠え叫ぶがそれでも触手たちは止まるはずがない。',
      '触手椅子の上で逃れることもできず、' + adventurer.short_name + 'は淫らに踊らされ続けた……',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  # 拘束丸呑み触手肉袋
  def trap_163_script(leader, is_lonely, floor, adventurer, effect, this_turn_orgasms, this_turn_sperm_shots, life_point_at_that_time)
    script = [
      '！おおっと！',
      _party_leader_short_names_by(leader.short_name, is_lonely) + 'は拘束丸呑み触手肉袋の仕掛けを踏んでしまった。',
      '壁から這い出してきた大蛇のような大きさの触手が、大きく口を広げると逃げる暇さえ与えずに' + adventurer.short_name + 'を飲み込んだ。',
      adventurer.short_name + 'を頭からつま先まで飲み込んだ触手は、大きく開いた壁の裂け目から物陰へと消えた……',
      '触手の中で必死にもがく' + adventurer.short_name + 'を気にも留めず、触手の内部の肉ひだが舐めしゃぶるように' + adventurer.short_name + 'を愛撫する。',
      '「ひぃっ、なにこれっ！？」',
      '狭い触手内部の空間に、デロリとした粘液が満ちていく。溺れ死ぬ危険を感じた' + 
      adventurer.short_name + 'は暴れるが、脚は絡め取られ、動くこともできない。。',
      '熱い。最初に粘液に触れた感覚はそれだった。粘液がもたらす発情と感覚の敏感化の効果により、' + 
      adventurer.short_name + 'は自分の体が触れられるだけで悶え狂うほど鋭敏化していることを悟った。',
      'このままではまずい、そう考えた瞬間。',
      '「いひぃぃっ！？」',
      adventurer.short_name + 'は情けない声を上げた。',
      '肉袋触手が再び' + adventurer.short_name + 'を舐めしゃぶり始めたのだ。',
      'それ自体は単なる刺激に過ぎない。',
      'だが、触れられただけで叫びそうになる程全身が敏感になった状態で、まんべんなく、一時も休みもなく責め続けられればそれは恐ろしい拷問になる。',
      'このままでは気が狂いそうになる。最初の５分間は' + adventurer.short_name + 'はそう考えた。',
      '助けがないと本当に狂ってしまう。次の５分間は' + adventurer.short_name + 'はそう考えた。',
      'おかしくなる。快楽で思考のすべてが塗りつぶされてしまう。さらに次の５分間の思考はそれだった。',
      'そのあと、自分が何を考え、何を叫んでいたのかもはや' + adventurer.short_name + 'は覚えていない。',
      '肉袋触手の外にまで聞こえるほどの叫び声が迷宮の狭い空間に響き続け、' + 
      '叫び声の合間に、ただ切れ切れに無様に許しを乞う声が響く……。',
      '肉袋触手による感覚鋭敏化の愛撫拷問によって、' + adventurer.short_name + 'はたっぷりと弄ばれた……',
    ]
    script += adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
    script
  end

  def trap_equip_141_script(adventurer, this_turn_orgasms)
    [
      '一歩一歩、歩みを進めるたびに膣内でバイブが擦れ己の存在感を主張する。貞操帯から生えたバイブから響く甘い振動によって' +
        adventurer.short_name + 'は快楽を掘り起こされていく。',
      '甘い浅く熱のこもった呼吸を繰り返している' + adventurer.short_name + 'が欲情し、快楽を感じていることは明らかだ',
    ] + adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
  end

  def trap_equip_142_script(adventurer, this_turn_orgasms)
    [
      '一歩一歩、歩みを進めるたびに極太バイブが膣内を荒々しくえぐり己の存在感を主張する。貞操帯から生えた極太バイブから響く凶悪なまでの快楽振動によって' +
        adventurer.short_name + 'は肉悦を叩き込まれていく。',
      'ぜぇぜぇと息も絶え絶えな熱のこもった呼吸を繰り返している' + adventurer.short_name + 'が欲情し、快楽に溺れていることは明らかだ',
    ] + adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
  end

  def trap_equip_143_script(adventurer, this_turn_orgasms)
    [
      '一歩一歩、歩みを進めるたびに秘部にローターが擦れ淫玩具の存在感を主張する。貞操帯に取り付けられたローターから響く甘い振動によって' +
        adventurer.short_name + 'は秘部に快楽を流し込まれていく。',
      '甘い浅く熱のこもった呼吸を繰り返している' + adventurer.short_name + 'が欲情し、快楽を感じていることは明らかだ',
    ] + adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms)
  end

  def ero_recover_slime_drain_script(adventurer, adventurer_life_point, is_success, is_lonely, this_turn_orgasms)
    return [] unless adventurer
    unless is_success
      return [
        _party_leader_short_names_by(adventurer.short_name, is_lonely) + 'の足元に、エロスライムがにじり寄ってくる。',
        '淫魔たちが好んで使役する、知能が低く非力なこの軟体生物は無力な冒険者を食い物にする魔法生物だった。',
        '卑劣な雑魚を、' + _party_leader_short_names_by(adventurer.short_name, is_lonely) + 'は難なく追い払った。',
      ]
    end

    script = [
      _party_leader_short_names_by(adventurer.short_name, is_lonely) + 'の足元に、エロスライムがにじり寄ってくる。',
      '淫魔たちが好んで使役する、知能が低く非力なこの軟体生物は無力な冒険者を食い物にする魔法生物だった。',
      'この卑劣な雑魚など、本来であればなんの問題にもならない、そのはずだった。',
      'だが、疲れ果て消耗しきった' + _party_leader_short_names_by(adventurer.short_name, is_lonely) + 'には単なる軟体生物を追い払う術も体力も残されていなかった。',
      'スライムは必死に逃げる' + adventurer.short_name + 'を標的に定めたようだ。俊敏な動きでたちまち追いついたスライムは大きく体をひろげ、飲み込むようにして' + adventurer.short_name + 'を包み込んだ！',
      adventurer.short_name + 'は必死に抵抗しようとするが、ぶよぶよとしたスライムの肉体は衝撃を吸収しダメージにならない。その間にもスライムは少女の全身にまとわりつき、表面を使って甘い官能を送り込んでくる。' +
        '悲痛だが甘さの混じった悲鳴をたっぷりと響かせながら、' + adventurer.short_name + 'は嬲られ続けた。',
    ]
    if adventurer.level > 1
      script += [
        'スライムに力を奪われたことで' + adventurer.short_name + 'のレベルが下がり、' + half_number_to_full(adventurer.level) + 'になった。',
      ]
    end

    script += [
      'スライムの淫辱の副作用として、' + adventurer.short_name + 'は力を取り戻すことができた。体力が' + half_number_to_full(adventurer_life_point) + 'ポイント回復した（ＨＰ：' + half_number_to_full(adventurer_life_point) + '）',
    ]
    merged_script = script + adventurer.ero_trap_orgasms_by_params_script(this_turn_orgasms, nil)
    merged_script
  end

  def _get_trap_damage_script(adventurer)
    if adventurer.this_turn_trap_damage_point > 0
      [
        adventurer.short_name + 'は' + half_number_to_full(adventurer.this_turn_trap_damage_point) +
          'のダメージを受けた',
      ]
    else
      []
    end
  end

  def _this_turn_orgasms_and_sperm_shots_chars(adventurer, this_turn_orgasms, this_turn_sperm_shots)
    scripts = []
    orgasms_chars = _this_turn_orgasms_chars(this_turn_orgasms)
    scripts += orgasms_chars.nil? ? [] : [ orgasms_chars ]
    sperm_shots_chars = _this_turn_sperm_shots_chars(this_turn_sperm_shots)
    scripts += sperm_shots_chars.nil? ? [] : [ sperm_shots_chars ]
    scripts
  end

  def _this_turn_orgasms_chars(this_turn_orgasms)
    return nil if this_turn_orgasms.nil?

    max_count = this_turn_orgasms.values.max
    return nil if max_count.nil? || max_count.zero?

    result = this_turn_orgasms
    script = ''
    script += 'Ｃ' if result[:c]
    script += 'Ｖ' if result[:v]
    script += 'Ａ' if result[:a]
    script += 'Ｂ' if result[:b]

    if max_count >= 5
      script += '極大'
    elsif max_count >= 3
      script += '特大'
    elsif max_count >= 2
      script += '大'
    end
    script += '絶頂'

    script
  end

  def _this_turn_sperm_shots_chars(this_turn_sperm_shots)
    return nil if this_turn_sperm_shots.nil?

    max_count = this_turn_sperm_shots.values.max
    return nil if max_count.nil? || max_count.zero?

    result = this_turn_sperm_shots
    script = ''
    script += '膣内' if result[:v]
    script += '直腸' if result[:a]
    script += '口内' if result[:oral]
    script += 'ぶっかけ' if result[:skin]

    if max_count >= 5
      script += '極大'
    elsif max_count >= 3
      script += '特大'
    elsif max_count >= 2
      script += '大量'
    end
    script += '射精'

    script
  end
end
