# coding: utf-8

%w(Button Label ).each do |source|
  java_import "javafx.scene.control.#{source}"
end

# Controllerの機能を使い、パーツを生成する
class ChainView
  include JRubyFX::Controller

  def convert_scripts_to_view_elements(scripts)
    script_view_elements = []

    scripts.each do |script|
      if script.size > _script_list_max_size
        line_number = (script.size / _script_list_max_size).to_i

        text_view_elements =
          (0..line_number).to_a.map do |ln|
            begin
              # scriptが_script_list_max_size文字数以上ある場合
              start_position = (ln * _script_list_max_size)
              split_script =
                if _have_max_size(script.size , start_position, _script_list_max_size)
                  # 最終行ではない
                  script[start_position, _script_list_max_size]
                else
                  # 最終行である
                  last_line_length = script.size - start_position
                  script[start_position, last_line_length]
                end
              {'type' => 'label', 'text' => split_script}
=begin
            rescue => e
              puts '===== Oops, some error has occurred in convert_scripts_to_view_elements ==================='
              puts e.message
              {'type' => 'label', 'error' => true, 'text' => script, 'error_message' => e.message}
=end
            end
          end

        script_view_elements += text_view_elements
      else
        script_view_elements += [{'type' => 'label', 'text' => script}]
      end
    end

    script_view_elements
  end

  def _script_list_max_size
    30
  end

  def _have_max_size(text_size, start_position, script_list_max_size)
    (text_size - start_position) >= script_list_max_size
  end

  # labelを加えるのはscript_listだけであり、ほかはこれで十分
  def refresh_view_element_buttons_to_xx_list(target_list, elements)
    clear_items_of(target_list)

    add_view_element_buttons_to_xx_list(target_list, elements)
  end

  def add_view_element_buttons_to_xx_list(target_list, elements)
    buttons =
      elements.map do |element|
        linked_button = button(
          :id => element[:id],
          :text => element[:text],
          :pref_width => element[:width]
        )
        linked_button.set_on_action do
          send(element[:action])
        end
      end
    add_items_to_xx_list(target_list, buttons)
  end

  # ボタンのみのリストを形成する
  def convert_buttons_from(elements, width)
    elements.map do |menu_key, element|
      _convert_to_button(
      {
        'text' => element['text'],
        'action' => element['action'],
        'width'  => width
      })
    end
  end

  def refresh_items_to_xx_list(target_list, items)
    clear_items_of(target_list)
    add_items_to_xx_list(target_list, items)
  end

  def add_items_to_xx_list(target_list, items)
    items.each { |item| target_list.items << item }
  end

  def clear_items_of(list)
    list.items.clear
  end

  def set_text(label, text)
    label.text = text
  end
end
