# coding: utf-8
# Controllerの機能を使い、パーツを生成する
module FxParts
  module GenerateHelper
    include JRubyFX::Controller

    def assign_to_lists(elements)
      elements.each do |list_name, value|
        target_list = get_list_from(list_name)
        if value['clear']
          target_list.items.clear
        end

        if value['house_map'] # このときだけは地図作るよ
          make_house_map(value['elements']['house'], value['elements']['brothel'])
        elsif value['world_map'] # このときだけは地図作るよ
          assign_elements_to(target_list, value['elements'])
          make_world_maps(value['map_elements'])
        else
          # デフォルト
          assign_elements_to(target_list, value['elements'])
        end
      end
    end

    def assign_items_to(target_list, items)
      items.each do |item|
        target_list.items << item
      end
    end

    def assign_elements_to(target_list, elements)
      if elements.nil?
        return ;
      end

      items = convert_by_type(elements)
      items.each do |item|
        target_list.items << item
      end
    end

    def convert_by_type(items)
      items.map do |item|
        if item['type'] == 'button'
          _convert_to_button(item)
        elsif item['type'] == 'label'
          _convert_to_label(item)
        else
          puts "invalid type: #{item['type']}"
        end
      end
    end

    def _convert_to_button(item)
      button_config = {
        :text       => item['text'],
        :pref_width => item['width'],
      }

      target_button = button(
#      :on_action => item['action'],
        button_config
      )

      target_button.set_on_action do
        eval(item['action'])
      end

      target_button
    end

    PROFILE_FONT_STYLE = '-fx-font-size: 13; -fx-text-fill: white'
    def _convert_to_label(item)
      label_config = {
        :text   => item['text'],
        :width  => item['width'],
      }
      label_config[:style] =
        if item['style_class'] == 'label_for_profile'
          PROFILE_FONT_STYLE
        end
      element = label(label_config)
      element.get_style_class.add('label')
      element
    end

    def get_list_from(list_name)
      case list_name
        when 'script_list' then
          return @script_list
        when 'group_menu_list' then
          return @group_menu_list
        when 'menu_list' then
          return @menu_list
        when 'left_status_list' then
          return @left_status_list
        when 'right_status_list' then
          return @right_status_list
        else
          puts 'LIST IS NOT FOUND!! : ' + list_name
          nil
      end
    end

    HOUSE_MAP_WIDTH  = 580
    def make_house_map(house_image_file_name, brothel_image_file_name)
      house_button = _make_view_tile(
        house_image_file_name, 'show_house_image', 'show_house')

      brothel_button = _make_view_tile(
        brothel_image_file_name, 'show_brothel_image', 'show_spring_house_menu')

      house_button.set_style('
        -fx-pref-height: 300;
        -fx-height: 300;
        -fx-padding: 0 0 0 0;
                               ')
      brothel_button.set_style('
        -fx-pref-height: 200;
        -fx-height: 200;
        -fx-padding: 0 0 0 0;
                               ')
      @script_list.items << house_button
      @script_list.items << brothel_button
    end

    def _make_view_tile(file_name, tile_id, action)
      view_image = image(resource_url(:images, "#{file_name}.png").to_s)
      iv = image_view(view_image)
      button_config = {
        :id   => tile_id,
        :text => '',
        :max_width  => HOUSE_MAP_WIDTH,
      }
      map_button = button(button_config) do
        set_graphic(iv)
      end

      map_button.set_on_action do
        eval(action)
      end

      map_button
    end

    # 地図タイル用
    MAP_TILE_MAX_WIDTH = 190
    X_SIZE_NUMBER = 3 # 横は３列
    def make_world_maps(elements)
      layout_x_map = [10, 200, 390]
      x_count = 0
      x_layout_box = nil
      elements.each do |element|
        map_tile = _make_world_map_tile(element)

        if (x_count == 0)
          x_layout_box = hbox(:max_width => (MAP_TILE_MAX_WIDTH * 3))
          @script_list.items << x_layout_box # 新たに一段追加
        end
        x_pane = pane(
          :layout_x => layout_x_map[x_count],
          :max_width => MAP_TILE_MAX_WIDTH,
          :width => MAP_TILE_MAX_WIDTH)
        x_pane.children.add(map_tile)
        x_layout_box.children.add(x_pane)

        x_count = (x_count + 1) % X_SIZE_NUMBER
      end
    end

    WORLD_MAP_WIDTH  = 180
    WORLD_MAP_HEIGHT = 170
    def _make_world_map_tile(element)
      view_image = image(resource_url(:images, "#{element['image_path']}.png").to_s)
      iv = image_view(view_image)
      button_config = {
        :id   => element['id'],
        :text => '',
        :max_width  => WORLD_MAP_WIDTH,
        :max_height => WORLD_MAP_HEIGHT,
      }
      map_button = button(button_config) do
        set_graphic(iv)
      end

      map_button.set_on_action do
        eval(element['action'])
      end

      map_button
    end
  end
end
