# coding: utf-8

require_rrk 'src/app/view_element_logic/dungeon/factory_view_element_logic'
require_rrk 'src/defines/script/dungeon/factory_menu_script'

# 魔術工房を扱うよ
class FactoryLogicController

  def initialize
    @factory_view_element_logic = FactoryViewElementLogic.new
    @factory_menu_script = FactoryMenuScript.new
    @view_element_logic = FactoryViewElementLogic.new
  end

  def show_magic_factory(game_master, hour_glass)
    scripts = @factory_menu_script.show_magic_factory_prelude(hour_glass)
    list = game_master.magic_factory_slaves
    scripts +=  @factory_menu_script.show_magic_factory(list)
    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_magic_factory,
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def show_slave_positions(game_master, hour_glass)
    list = game_master.magic_factory_slaves
    scripts =
      @factory_menu_script.show_slave_positions +
        @factory_menu_script.show_magic_factory(list)


    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_slave_positions(list),
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
  end

  def show_slave_position_detail(game_master, adventurer_id)
    slave = game_master.magic_factory_slaves[adventurer_id]
    scripts = @factory_menu_script.show_slave_positions_detail(slave)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_slave_position_detail(adventurer_id ,scripts),
      },
    }
  end

  def show_slave_in_brothel_detail(game_master, adventurer_id)
    slave = game_master.magic_factory_slaves[adventurer_id]
    scripts = @factory_menu_script.show_slave_in_brothel_detail(slave)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_slave_brothel_position_detail(
          adventurer_id, slave, game_master.dungeon_master, scripts),
      },
    }
  end

  def show_slave_in_brothel_detail_no_vacancy(game_master, adventurer_id)
    slave = game_master.magic_factory_slaves[adventurer_id]
    scripts = @factory_menu_script.show_slave_in_brothel_detail_in_no_vacancy(slave)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_slave_brothel_position_detail(
          adventurer_id, slave, game_master.dungeon_master, scripts),
      },
    }
  end

  def show_slave_in_brothel_detail_in_resist(game_master, adventurer_id)
    slave = game_master.magic_factory_slaves[adventurer_id]
    scripts = @factory_menu_script.show_slave_in_brothel_detail_in_resist(slave)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_slave_brothel_position_detail(
          adventurer_id, slave, game_master.dungeon_master, scripts),
      },
    }
  end

  def show_slave_brothel(game_master, hour_glass)
    status_scripts = _show_slave_brothel_list(game_master)
    scripts = @factory_menu_script.show_slave_brothel

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
  end

  def show_maternity_room(game_master, hour_glass)
    status_scripts = _show_maternity_room_list(game_master)
    scripts = @factory_menu_script.show_maternity_room

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
  end

  def show_tentacle_lab(game_master, hour_glass)
    status_scripts = _show_tentacle_lab_list(game_master)
    scripts = @factory_menu_script.show_tentacle_lab

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
  end

  def show_slave_in_tentacle_lab_detail(game_master, adventurer_id)
    slave = game_master.magic_factory_slaves[adventurer_id]
    scripts = @factory_menu_script.show_slave_in_tentacle_lab_detail(slave)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_slave_tentacle_lab_position_detail(
          adventurer_id, game_master.dungeon_master, scripts),
      },
    }
  end

  def release_slave(game_master, slave_id)
    game_master.release_slave(slave_id)
    slaves = game_master.magic_factory_slaves
    slave = slaves[slave_id]

    scripts = @factory_menu_script.release_slave(slave)

    lists = {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts),
      },
    }
    lists.merge(_show_magic_factory_in_menu_list(game_master))
  end

  def set_to_slave_brothel(game_master, slave_id, spring_type)
    game_master.set_slave_brothel(slave_id, spring_type)

    slaves = game_master.magic_factory_slaves
    slave = slaves[slave_id]
    scripts = @factory_menu_script.set_to_slave_brothel(slave, spring_type)
    status_scripts = _show_slave_brothel_list(game_master)

    lists = {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
    lists.merge(_show_magic_factory_in_menu_list(game_master))
  end

  def set_to_tentacle_lab(game_master, slave_id, type)
    game_master.set_tentacle_lab(slave_id, type)

    slaves = game_master.magic_factory_slaves
    slave = slaves[slave_id]
    scripts = @factory_menu_script.set_to_tentacle_lab(slave, type)
    status_scripts = _show_tentacle_lab_list(game_master)

    lists = {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
    lists.merge(_show_magic_factory_in_menu_list(game_master))
  end

  def set_to_maternity_room(game_master, slave_id)
    game_master.set_maternity_room(slave_id)

    slaves = game_master.magic_factory_slaves
    slave = slaves[slave_id]
    scripts = @factory_menu_script.set_to_maternity_room(slave)
    status_scripts = _show_maternity_room_list(game_master)

    lists = {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(scripts + status_scripts),
      },
    }
    lists.merge(_show_magic_factory_in_menu_list(game_master))
  end

  def _show_slave_brothel_list(game_master)
    slaves = game_master.slave_brothel_slaves
    status_scripts =
      if slaves.empty?
        @factory_menu_script.blank_slave_brothel
      else
        @factory_menu_script.list_header_text +
          slaves.map do |_, slave|
            slave.short_name + "（#{slave.status_text(slave)}）"
          end
      end
    status_scripts
  end

  def _show_tentacle_lab_list(game_master)
    slaves = game_master.tentacle_lab_slaves
    status_scripts =
      if slaves.empty?
        @factory_menu_script.blank_tentacle_lab
      else
        @factory_menu_script.list_header_text +
          slaves.map do |_, slave|
            slave.short_name + "（#{slave.status_text(slave)}）"
          end
      end
    status_scripts
  end

  def _show_maternity_room_list(game_master)
    slaves = game_master.maternity_room_slaves
    status_scripts =
      if slaves.empty?
        @factory_menu_script.blank_maternity_room
      else
        @factory_menu_script.list_header_text +
          slaves.map do |_, slave|
            slave.short_name + "（#{slave.status_text(slave)}）"
          end
      end
    status_scripts
  end

  def _show_magic_factory_in_menu_list(game_master)
    list = game_master.magic_factory_slaves
    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_slave_positions(list),
      },
    }
  end
end
