# coding: utf-8
# LogicControllerは本質的にControllerが担うべき役割を持つ
# 共通ロジックの管理が超煩雑になったため
# この子はメインメニュー画面を管轄する

#view_element_logic
%w(menu_view_element_logic).each do |src_name|
  require_rrk 'src/app/view_element_logic/dungeon/' + src_name
end

require_rrk 'src/app/logic_controller/dungeon/helper/menu_list_helper'
require_rrk 'src/defines/script/dungeon/menu_script'
require_rrk 'src/defines/script/dungeon/jail_room_script'

class MenuLogicController
  include MenuListHelper

  def initialize(options)
    @trial_mode = options[:trial_mode]
    @view_element_logic = MenuViewElementLogic.new(options)
    @menu_script = MenuScript.new
    @jail_room_script = JailRoomScript.new
  end

  def show_start_menus
    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_initial_menu,
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def opening_menu(dungeon_master, auction_room_slaves)
    script = @menu_script.opening_menu
    @view_element_logic.update_in_opening_menu(dungeon_master, auction_room_slaves)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_main_menu,
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def top_menu(hour_glass)
    script = @menu_script.show_top_menus_script(hour_glass)

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_main_menu,
      },
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
    }
  end

  def reload_top_menu
    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_main_menu,
      },
    }
  end

  def show_ending(is_trial_mode)
    script = @menu_script.show_ending(is_trial_mode)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.make_scripts_from(script),
      },
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.go_to_new_game,
      },
    }
  end

  # セーブ / ロード
  def show_save_load
    {
      'menu_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.show_save_load,
      },
    }
  end

  def show_save_slots
    scripts = @menu_script.show_save_slots

    {
      'script_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_save_slots(scripts)
      },
    }
  end

  def show_load_slots
    scripts = @menu_script.show_load_slots
    {
      'script_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_load_slots(scripts)
      },
    }
  end

  def show_load_slots_in_initial
    scripts = @menu_script.show_load_slots
    {
      'script_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_load_slots_in_initial(scripts)
      },
    }
  end

  def write_adventure_log(bard)
    scripts = @menu_script.write_adventure_log
    {
      'script_list' => {
        'clear'    => true,
        'elements' => @view_element_logic.write_adventure_log(scripts, bard),
      },
    }
  end

  def update_by_turn(game_master, is_night, is_new_dawn)
    @view_element_logic.update_by_turn(game_master, is_night, is_new_dawn)
  end

  def clear_script_list
    {
      'script_list' => {
        'clear'    => true,
      },
    }
  end

  def save_to_(slot_number, save_data)
    @view_element_logic.save_to_(slot_number, save_data)
    scripts = @menu_script.save_to_view_elements

    {
      'menu_list' => {
        'clear' => true,
        'elements' => @view_element_logic.show_main_menu,
      },
      'script_list' => {
        'clear' => true,
        'elements' => @view_element_logic.save_to_view_elements(scripts)
      },
    }
  end

  def load_from_(slot_number)
    save_data = @view_element_logic.load_from_(slot_number)
    scripts = @menu_script.load_from_view_elements

    game_master = save_data['game_master']
    hour_glass = save_data['hour_glass']
    update_by_turn(game_master, hour_glass.night?, hour_glass.new_dawn?)

    {
      'save_data' => save_data,
      'view_elements' => {
        'menu_list' => {
          'clear' => true,
          'elements' => @view_element_logic.show_main_menu,
        },
        'script_list' => {
          'clear' => true,
          'elements' => @view_element_logic.make_scripts_from(scripts),
        },
      },
    }
  end

  # 冒険者閲覧メニュー一覧
  def hear_adventurers_in_bar(game_master, hour_glass)
    scripts = @menu_script.hear_adventurers_in_bar(game_master, hour_glass)
    elements = @view_element_logic.hear_adventurers_in_bar(scripts)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # 洗脳サーヴァントを見る
  def show_betray_servant_cage(game_master, hour_glass)
    dungeon_master = game_master.dungeon_master

    elements =
      if dungeon_master.has_betrayed_servant?
        script = @menu_script.show_betray_servant_cage(game_master, hour_glass)
        manage_penis_text = @menu_script.manage_betray_servant_penis_text
        betrayed_slaves = game_master.dungeon_master.betrayed_slaves
        @view_element_logic.show_betray_servant_cage(script, manage_penis_text, betrayed_slaves)
      else
        script = @menu_script.show_betray_servant_cage_in_empty(game_master, hour_glass)
        @view_element_logic.make_scripts_from(script)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def manage_betray_servant_penis(game_master, hour_glass)
    script = @menu_script.manage_betray_servant_penis
    betrayed_slaves = game_master.dungeon_master.betrayed_slaves
    elements = @view_element_logic.manage_betray_servant_penis(script, betrayed_slaves)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # 冒険者単位
  def show_adventurers(game_master, hour_glass)
    _show_adventurers(game_master, hour_glass, 'default')
  end

  def show_adventurers_show_jails(game_master, hour_glass)
    _show_adventurers(game_master, hour_glass, 'show_jails')
  end

  def show_adventurers_only_prisoner(game_master, hour_glass)
    _show_adventurers(game_master, hour_glass, 'only_prisoner')
  end

  def show_adventurers_with_bookmark(game_master, hour_glass)
    _show_adventurers(game_master, hour_glass, 'with_bookmark')
  end

  # 冒険者単位
  def _show_adventurers(game_master, hour_glass, mode)
    text_list = {
      header_script: @menu_script.show_adventurers(hour_glass),
      footer_script: @menu_script.blank_jail_if(game_master),
      bar_header_text:     @menu_script.bar_header_text,
      dungeon_header_text: @menu_script.dungeon_header_text,
      jail_header_text:    @menu_script.jail_header_text,
      magic_factory_header_text: @menu_script.magic_factory_header_text,
      slave_header_text:   @menu_script.slave_header_text,
    }
    if 'show_jails' == mode
      text_list[:header_script] = @menu_script.show_adventurers_in_jails(game_master, hour_glass)
      text_list[:footer_script] = []
    elsif 'only_prisoner' == mode
      text_list[:header_script] = @menu_script.hear_rumor_of_adventurers_in_jails(game_master, hour_glass)
      text_list[:footer_script] = []
    end
    list =
      if 'with_bookmark' == mode
        {
          adventurers_in_bar:     game_master.bar_chairs_bookmarked,
          adventurers_in_dungeon: game_master.adventurers_in_dungeon_bookmarked,
          adventurers_in_jail:    game_master.adventurers_in_jail_bookmarked,
          adventurers_in_magic_factory: game_master.adventurers_in_magic_factory_bookmarked,
          adventurers_in_slave:   game_master.adventurers_of_sold_bookmarked,
        }
      else
        {
          adventurers_in_bar:     game_master.bar_chairs,
          adventurers_in_dungeon: game_master.adventurers_in_dungeon,
          adventurers_in_jail:    game_master.adventurers_in_jail,
          adventurers_in_magic_factory: game_master.adventurers_in_magic_factory,
          adventurers_in_slave:   game_master.adventurers_of_sold,
        }
      end

    is_all_empty = list.values.all?(&:empty?)
    text_list[:all_empty] =
      if 'with_bookmark' == mode
        @menu_script.bookmark_is_all_empty_script(is_all_empty)
      else
        @menu_script.is_all_empty_script(is_all_empty)
      end

    elements =
      case mode
        when 'with_bookmark'
          @view_element_logic.show_adventurers_with_bookmark(list, text_list, game_master.party_by_adventurer_id)
        when 'show_jails'
          @view_element_logic.show_adventurers_show_jails(list, text_list, game_master.party_by_adventurer_id)
        when 'only_prisoner'
          @view_element_logic.show_adventurers_only_prisoner(list, text_list, game_master.party_by_adventurer_id)
        else
          @view_element_logic.show_adventurers(list, text_list, game_master.party_by_adventurer_id)
      end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # パーティー単位
  def show_adventurers_in_dungeon(game_master, hour_glass)
    scripts = @menu_script.show_adventurers_in_dungeon(game_master, hour_glass)
    list = game_master.parties_list_by_floor
    elements_by_floor = @menu_script.show_adventurers_list_in_dungeon(list)
    elements = @view_element_logic.show_adventurers_in_dungeon(elements_by_floor, scripts)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # ダンジョンに入るパーティー単体を見る
  def show_party_detail(game_master, adventurer_id)
    party = game_master.get_party_by_leader_id(adventurer_id)
    scripts =
      party.adventurers_summary_status_script +
        party.passengers_summary_status_script +
        party.each_adventurer_status_script +
        party.each_passenger_status_script
    elements = @view_element_logic.show_party_detail(scripts)

    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def show_jail_room(game_master, adventurer_id)
    jail_room = game_master.get_jail_room_by_id(adventurer_id)
    scripts = @jail_room_script.jail_room_status(jail_room)
    elements = @view_element_logic.show_jail_room(scripts)
    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def show_jail_details_for_custom(game_master, adventurer_id)
    jail_room = game_master.get_jail_room_by_id(adventurer_id)
    scripts = @jail_room_script.jail_details_for_custom(jail_room, game_master.dungeon_master.magic_power)

    adventurer = game_master.adventurer_by_id(adventurer_id)
    elements = @view_element_logic.show_jail_details_for_custom(
      adventurer_id, adventurer, game_master.dungeon_master, scripts)

    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def show_betray_servant_detail(game_master, adventurer_id)
    slave = game_master.dungeon_master.betrayed_slaves[adventurer_id]
    scripts = slave.show_betray_servant_detail_script(slave, game_master.dungeon_master.magic_power)

    elements = @view_element_logic.show_betray_servant_detail(
      adventurer_id, slave, game_master.dungeon_master, scripts)

    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def toggle_betray_servant_penis(game_master, adventurer_id)
    slave = game_master.dungeon_master.betrayed_slaves[adventurer_id]
    # ふたなりチンコを付けたり外したり
    slave.toggle_betrayed_penis!

    # ペニスを外した後にセリフをつける
    scripts = slave.toggle_betray_servant_penis_scirpt(slave)

    scripts += @menu_script.manage_betray_servant_penis
    betrayed_slaves = game_master.dungeon_master.betrayed_slaves
    elements = @view_element_logic.manage_betray_servant_penis(scripts, betrayed_slaves)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def level_up_betray_servant(game_master, adventurer_id)
    slave = game_master.dungeon_master.betrayed_slaves[adventurer_id]
    magic_cost = slave.magic_power_betrayed_level_up(slave)
    dungeon_master = game_master.dungeon_master
    # 最大レベル超過 or 魔力が不足
    if slave.max_level_limit_over? || !dungeon_master.have_magic_power?(magic_cost)
      reason_script =
        if slave.max_level_limit_over?
          slave.level_up_betray_servant_in_over_limit_script(slave)
        elsif !dungeon_master.have_magic_power?(magic_cost)
          slave.level_up_betray_servant_in_lack_script(slave, magic_cost, dungeon_master.magic_power)
        else
          []
        end

      scripts =
          slave.show_betray_servant_detail_script(slave, dungeon_master.magic_power) +
            reason_script
      elements = @view_element_logic.show_betray_servant_detail(
        adventurer_id, slave, dungeon_master, scripts)

      return {
        'script_list' => {
          'clear' => true,
          'elements' => elements,
        },
      }
    end

    # 魔力引いてレベルアップ
    dungeon_master.decrease_magic_power!(magic_cost)
    slave.level_up_by_betray_servant!

    scripts =
      slave.show_betray_servant_detail_script(slave, dungeon_master.magic_power) +
        slave.level_up_betray_servant_script(slave, magic_cost, dungeon_master.magic_power)
    elements = @view_element_logic.show_betray_servant_detail(
      adventurer_id, slave, dungeon_master, scripts)

    return {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def brand_tattoo_to_prisoner(game_master, adventurer_id, tattoo)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    dungeon_master = game_master.dungeon_master

    dungeon_master.decrease_magic_power!(tattoo[:cost])
    adventurer.brand_tattoo!(tattoo[:key])

    scripts =
      @jail_room_script.brand_tattoo_to_prisoner(adventurer, tattoo[:name], dungeon_master.magic_power, tattoo[:cost])
    # もう一回再表示
    elements = @view_element_logic.show_jail_details_for_custom(
      adventurer_id, adventurer, game_master.dungeon_master, scripts)

    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def brand_detail_tattoo_to_prisoner(game_master, adventurer_id, tattoo)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    dungeon_master = game_master.dungeon_master

    scripts = tattoo[:description]

    elements =
      if dungeon_master.have_magic_power?(tattoo[:cost])
        scripts += @jail_room_script.brand_detail_tattoo_to_prisoner(
          adventurer, tattoo[:name], tattoo[:cost], dungeon_master.magic_power)
        @view_element_logic.brand_detail_tattoo_to_prisoner(adventurer_id, adventurer, tattoo, scripts)
      else
        scripts += @jail_room_script.brand_detail_tattoo_in_shortage(
          adventurer, tattoo[:name], tattoo[:cost], dungeon_master.magic_power)
        @view_element_logic.brand_detail_tattoo_in_shortage(adventurer_id, scripts)
      end

    {
      'script_list' => {
        'clear' => true,
        'elements' => elements,
      },
    }
  end

  def show_adventurer(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    script = adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def show_adventurer_only_prisoner(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    script = adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer_only_prisoner(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def show_adventurer_with_bookmark(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    script = adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer_with_bookmark(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def show_prisoner(game_master, adventurer_id, by_adventurer, back_method_name)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    script = adventurer.jail_situation_script(by_adventurer)

    elements = @view_element_logic.show_prisoner(script, adventurer, back_method_name)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # TOP > 酒場で冒険者たちの噂を聞く > 地下室に監禁されて...
  def show_adventurers_in_jails(game_master, hour_glass)
    scripts = @menu_script.show_adventurers_in_jails(game_master, hour_glass)
    back_method_name = 'hear_adventurers_in_bar'
    elements = @view_element_logic.show_adventurers_in_jails(scripts, game_master.adventurers_in_jail, back_method_name)

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  # TOP > 監禁された冒険者を見る
  # 削除

  def show_bounty_of(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    now_money = game_master.dungeon_master.money
    script = adventurer.show_bounty_script(adventurer, now_money)

    elements = @view_element_logic.show_bounty_of(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def set_prison_of(game_master, adventurer_id, jail_mode)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    adventurer.set_jail_mode(jail_mode)
    script = adventurer.set_jail_mode_script(adventurer, jail_mode)

    elements = @view_element_logic.set_prison_of(script)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def pay_bounty_of(game_master, adventurer_id, day_count)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    now_money = game_master.dungeon_master.money

    if game_master.dungeon_master.have_money?(now_money)
      adventurer.set_bounty!
      game_master.write_to_bounty_list!(adventurer.id, day_count)
      game_master.dungeon_master.decrease_money!(adventurer.bounty_price)
      script = adventurer.pay_bounty_script(adventurer, now_money)
      elements = @view_element_logic.pay_bounty_of(script)
    else
      # 資金不足で買えない
      script =
        adventurer.short_of_bounty_script(adventurer, now_money) +
          adventurer.show_bounty_script(adventurer, now_money)

      elements = @view_element_logic.show_bounty_of(script, adventurer)
    end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def release_from_jail(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    game_master.jail.delete_jail_room!(adventurer)
    adventurer.decrease_level_by_san!
    game_master.bar_chairs[adventurer.id] = adventurer
    long_rest = true
    adventurer.become_rest!(long_rest)

    # 解放したので再度ステータス表示
    adventurer = game_master.adventurer_by_id(adventurer_id)
    script =
      @jail_room_script.release_from_jail_script(adventurer) +
        adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def set_bookmark_to(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    adventurer.set_bookmarked!

    # 再度ステータス表示
    script =
      @jail_room_script.set_bookmark_to(adventurer) +
        adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end

  def reset_bookmark_to(game_master, adventurer_id)
    adventurer = game_master.adventurer_by_id(adventurer_id)
    adventurer.reset_bookmarked!

    # 再度ステータス表示
    script =
      @jail_room_script.reset_bookmark_to(adventurer) +
        adventurer.status_detail_with_header

    elements = @view_element_logic.show_adventurer(script, adventurer)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => elements,
      },
    }
  end
end
