# coding: utf-8
%w(house_view_element_logic).each do |src_name|
  require_rrk 'src/app/view_element_logic/' + src_name
end

require_rrk 'src/game_scenario/menu_scenario'

class HouseLogicController
  def initialize *args
    @house_view_element_logic = HouseViewElementLogic.new
    @menu_scenario = MenuScenario.new
  end

  def show_house_cage(house)
    {
      'menu_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.show_house_cage(house),
      },
    }
  end

  def show_blank_house_cage
    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.show_blank_house_cage
        ),
      },
    }
  end

  def show_slave_relation(house, master)
    master.confessed_log

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.show_slave_relation(house.house_cage, house.master_slave_list, master.confessed_log, master.potential_confessions)
        ),
      },
    }
  end

  def show_enhance_houce_menu(house)
    {
      'menu_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.show_enhance_houce_menu(house),
      },
    }
  end


  # 増築系
  def detail_to_enhance_house(master, house)
    cost = house.cost_of_enhance_house

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.detail_to_enhance_house(master, house, cost)
        ) +
          @house_view_element_logic.buy_to_enhance_house,
      },
    }
  end

  def detail_to_enhance_force_cage(master, house)
    cost = house.cost_of_enhance_force_cage

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.detail_to_enhance_force_cage(master, house, cost)
        ) +
          @house_view_element_logic.buy_to_enhance_force_cage,
      },
    }
  end

  def detail_to_enhance_brothel_cage(master, house)
    cost = house.cost_of_enhance_brothel_cage

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.detail_to_enhance_brothel_cage(master, house, cost)
        ) +
          @house_view_element_logic.buy_to_enhance_brothel_cage,
      },
    }
  end

  def detail_to_build_spring_house(master, house)
    cost = house.cost_of_build_spring_house

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.detail_to_build_spring_house(master, house, cost)
        ) +
          @house_view_element_logic.buy_to_build_spring_house,
      },
    }
  end

  def enhance_house(master, house)
    cost = house.cost_of_enhance_house
    if master.money < cost
      return _lack_of_money_to_enhance(master.money)
    else
      master.money -= cost
      house.enhance_house!

      scripts =
        @menu_scenario.enhance_house(master.money)
    end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          scripts
        ),
      },
    }
  end

  def enhance_force_cage(master, house)
    cost = house.cost_of_enhance_force_cage
    if master.money < cost
      return _lack_of_money_to_enhance(master.money)
    else
      master.money -= cost
      house.enhance_force_cage!

      scripts =
        @menu_scenario.enhance_force_cage(master.money)
    end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          scripts
        ),
      },
    }
  end

  def enhance_brothel_cage(master, house)
    cost = house.cost_of_enhance_brothel_cage
    if master.money < cost
      return _lack_of_money_to_enhance(master.money)
    else
      master.money -= cost
      house.enhance_brothel_cage!

      scripts =
        @menu_scenario.enhance_brothel_cage(master.money)
    end

    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          scripts
        ),
      },
    }
  end

  def build_spring_house(master, house)
    cost = house.cost_of_build_spring_house
    if master.money < cost
      return _lack_of_money_to_enhance(master.money)
    else
      master.money -= cost
      house.activate_spring_house!

      scripts =
        @menu_scenario.build_spring_house(master.money)
    end

    menu_elements = show_enhance_houce_menu(house)
    menu_elements.merge({
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          scripts
        ),
      },
    })
  end

  def _lack_of_money_to_enhance(money)
    {
      'script_list' => {
        'clear'    => true,
        'elements' => @house_view_element_logic.make_scripts_from(
          @menu_scenario.lack_of_money_to_enhance(money)
        )
      },
    }
  end
end
