# coding: utf-8

require_rrk 'src/game_define/dungeon/adventurers_list'
require_rrk 'src/game_define/dungeon/slaves_list'
require_rrk 'src/app/helper/game_define_helper'
if GameDefineHelper::FGO_MODE
  AdventurersList.adventure_list_by_ver.keys.each do |key|
    require_rrk "src/defines/script/dungeon/fgo/adventurers/adventurer_#{key}_script"
  end
  AdventurersList.append_adventure_list_by_ver.keys.each do |key|
    require_rrk "src/defines/script/dungeon/fgo/adventurers/adventurer_#{key}_script"
  end
  SlavesList.slave_list_by_mode.keys.each do |key|
    require_rrk "src/defines/script/dungeon/fgo/adventurers/adventurer_#{key}_script"
  end
elsif GameDefineHelper::ORIGINAL_MODE
  (1..5).each do |key|
    require_rrk "src/defines/script/dungeon/original/adventurer_#{key}_script"
  end
else
  AdventurersList.adventure_list_by_ver.keys.each do |key|
    require_rrk "src/defines/script/dungeon/adventurers/adventurer_#{key}_script"
  end
  SlavesList.slave_list_by_mode.keys.each do |key|
    require_rrk "src/defines/script/dungeon/adventurers/adventurer_#{key}_script"
  end
end

%w(forwardable).each do |src_name|
  require_rrk 'src/app/helper/' + src_name
end

class AdventurerScriptStatus
  extend Forwardable

  attr_reader :flags, :script,
              :daily_status_script_in_jail, :daily_resist_status_script_in_jail,
              :jail_flash_flags, :jail_summary_text

  def_delegators :@script,
                 :down_to_slave_script, :jail_training_script, :jail_training_short_script, :betrayed_servant_battle_opening_script,
                 :generate_motivation_by, :generate_virgin_text, :generate_origin_from

  def initialize(original_id, adventurer)
    @adventurer = adventurer

    @daily_status_script_in_jail = ['']
    @daily_resist_status_script_in_jail = ['']
    @jail_summary_text = ['']

    @script =
      if GameDefineHelper::ORIGINAL_MODE
        if adventurer.in_devout?
          Adventurer2Script.new
        elsif adventurer.high_hierarchy?
          Adventurer3Script.new
        elsif adventurer.flattered_personality? # お調子者
          Adventurer4Script.new
        elsif adventurer.weak_personality? # 弱気
          Adventurer5Script.new
        else # 強気
          Adventurer1Script.new
        end
      else
        eval("Adventurer#{original_id}Script.new")
      end
    @flags = {}
    @jail_flash_flags = {}
  end

  def clear_jail_flash_flags!
    @jail_flash_flags = {}
  end

  def ero_trap_effect_script
    @script.ero_trap_effect_script(@adventurer)
  end

  def ero_trap_sperm_n_orgasms_script(sperm_shot_params, orgasms_params, life_point_at_that_time=nil)
    @script.ero_trap_sperm_n_orgasms_script(@adventurer, sperm_shot_params, orgasms_params, life_point_at_that_time)
  end

  def ero_trap_sperm_shot_script(params)
    @script.ero_trap_sperm_shot_script(@adventurer, params)
  end

  def ero_trap_orgasms_by_params_script(params, life_point_at_that_time = nil)
    @script.ero_trap_orgasms_by_params_script(@adventurer, params, life_point_at_that_time)
  end

  def status_script_in_jail
    @script.status_script_in_jail(@adventurer)
  end

  def resist_status_script_in_jail
    @script.resist_status_script_in_jail(@adventurer)
  end

  def get_teleport_pass(floor)
    @script.get_teleport_pass(floor, @adventurer)
  end

  def get_teleport_pass_shorten(floor)
    @script.get_teleport_pass_shorten(floor, @adventurer)
  end

  def teleport_for_rescue_script(floor, is_lonely)
    @script.teleport_for_rescue_script(floor, @adventurer, is_lonely)
  end

  def teleport_script(floor, is_lonely)
    @script.teleport_script(floor, @adventurer, is_lonely)
  end

  def get_treasure_script(is_lonely, money_amount, master_money)
    @script.get_treasure_script(@adventurer, is_lonely, money_amount, master_money)
  end

  def battle_prelude_script(is_only, brain_washing_active_member_is_majority)
    @script.battle_prelude_script(@adventurer, is_only, brain_washing_active_member_is_majority)
  end

  def battle_prelude_only_n_washed_script
    @script.battle_prelude_only_n_washed_script(@adventurer)
  end

  def battle_prelude_all_washed_script
    @script.battle_prelude_all_washed_script(@adventurer)
  end

  def set_daily_status_script_in_jail
    @daily_status_script_in_jail = status_script_in_jail
  end

  def set_daily_resist_status_script_in_jail
    @daily_resist_status_script_in_jail = resist_status_script_in_jail
  end

  def set_jail_summary_text!(monster_name, monster_count, day_count)
    @jail_summary_text = @script.jail_summary_text(monster_name, monster_count, day_count)
  end
end
