# coding: utf-8
require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/app/model/dungeon/dungeon_master/unlocked_secretary'
require_rrk 'src/app/model/dungeon/dungeon_master/magic_factory'
require_rrk 'src/app/model/dungeon/dungeon_master/slave_servant_cage'
require_rrk 'src/app/model/dungeon/dungeon_master/auction_room'
require_rrk 'src/app/helper/forwardable'
require_rrk 'src/app/helper/game_define_helper'

class DungeonMaster
  extend Forwardable
  include TrapList
  include MonsterListByType
  include GameDefineHelper

  def_delegators :@unlocked_secretary, :unlocked_monsters, :unlocked_traps,
                 :unlocked_pool_traps,
                 :get_trap_or_alternative, :get_monster_or_alternative,
                 :unlock_summon!, :unlock_trap!, :unlock_item!, :unlock_book!, :is_unlocked_monster?,
                 :unlock_building!, :unlock_tentacle!,
                 :unlock_building_or_buy!,
                 :has_brothel_room?, :has_brothel_large_room?, :has_brothel_pole_dance_tent?, :has_brothel_show_tent?,
                 :has_book?,
                 :can_buy?, :can_buy_trap?, :can_buy_item?, :can_buy_book?, :can_buy_building?, :can_buy_tentacle?,
                 :is_unlocked_trap?,
                 :has_tentacle_lab_v?, :has_tentacle_lab_v_lv2?, :has_tentacle_lab_a?, :has_tentacle_lab_b?,
                 :has_tentacle_lab_brainwash?, :has_tentacle_lab_brainwash_lv2?, :has_tentacle_lab_hard?

  def_delegators :@magic_factory,
                 :release_slave, :set_maternity_room, :set_tentacle_lab , :set_slave_brothel

  def_delegators :@slave_servant_cage, :select_active_slave_servants, :has_betrayed_servant?,
                 :betray_servant_count, :betrayed_slaves, :rest_of_stalkers!

  attr_reader :money, :magic_power,
              :deep_floor_guards,
              :today_brothel_sales, :today_magic_power_from_jail, :today_magic_power_from_magic_factory,
              :magic_maintenance_cost,
              :sold_slaves, :hour_glass, :slaves,
              :escape_penalty, :capture_bonus, :maternity_bonus, :physical_trap_bonus ,:ero_trap_bonus

  INIT_MONEY = 6_000
  INIT_MAGIC_POWER = 4_000
  def initialize(slaves)
    @money = INIT_MONEY
    @magic_power = INIT_MAGIC_POWER
    @today_brothel_sales = 0
    @today_magic_power_from_jail = 0
    @today_magic_power_from_magic_factory = 0
    @magic_maintenance_cost = 0
    @sold_slaves = {}
    @unlocked_secretary = UnlockedSecretary.new
    @slaves = slaves
    @magic_factory = MagicFactory.new(slaves)
    @slave_servant_cage = SlaveServantCage.new
    @auction_room = AuctionRoom.new

    @monster_gain_point_by_maternity_room = 0
    @monster_equip_life_bonus_point = 0 # モンスターの生命力ボーナス

    @deep_floor_guards = {} # フロア数 => [モンスターID, ...] の構成

    @escape_penalty = 0
    @capture_bonus = 0
    @maternity_bonus = 0
    @physical_trap_bonus = 0
    @ero_trap_bonus = 0
    @tentacle_lab_bed = []
  end

  def slave_in_magic_factory
    @magic_factory.slaves
  end

  def set_hour_glass(hour_glass)
    @hour_glass = hour_glass
  end

  NEW_GAME_MONEY_BONUS = 100_000
  NEW_GAME_MAGIC_BONUS =  30_000
  def add_clear_bonus!(repeat_count)
    _add_money(NEW_GAME_MONEY_BONUS * repeat_count)
    add_magic_power!(NEW_GAME_MAGIC_BONUS * repeat_count)
  end

  def get_money_from_brothel!(inc_money)
    @today_brothel_sales = inc_money
    _add_money(inc_money)
  end

  def decrease_money!(dec_money)
    @money -= dec_money
  end

  def have_money?(dec_money)
    @money >= dec_money
  end

  def decrease_magic_power!(dec_magic_power)
    @magic_power -= dec_magic_power
  end

  def have_magic_power?(dec_magic_power)
    @magic_power >= dec_magic_power
  end

  # 最下層に到達されたので略奪された！
  def sacked_treasure!
    limit = @money * 0.2
    sacked_amount = [limit.to_i, 3_000].max
    if money < sacked_amount
      decrease_money!(money) # 全額持ってかれる
    else
      decrease_money!(sacked_amount)
    end
    sacked_amount
  end

  def add_sold_slaves(adventurer)
    buyer_type = get_buyer_type
    puts "すでに売却済み" if @sold_slaves[adventurer.id]
    @sold_slaves[adventurer.id] = adventurer
    adventurer.set_buyer_type(buyer_type)
  end

  def sell_slave!(adventurer, sell_price)
    add_sold_slaves(adventurer)
    _add_money(sell_price)
    remove_slave_from_auction(adventurer.id)
  end

  def remove_slave_from_auction(slave_key)
    @auction_room.remove_slave(slave_key)
  end

  def buy_slave_to_factory(slave_key, adventurer, buy_price)
    @magic_factory.prison_slave!(adventurer)
    remove_slave_from_auction(slave_key)
    decrease_money!(buy_price)
  end

  def buy_slave_n_convert(slave_key, adventurer, buy_price)
    @slave_servant_cage.brainwash_slave!(adventurer)
    remove_slave_from_auction(slave_key)
    decrease_money!(buy_price)
  end

  def buy_slave_n_convert_magic(slave_key, adventurer, buy_price)
    add_magic_power!(adventurer.drain_magic_power)
    remove_slave_from_auction(slave_key)
    decrease_money!(buy_price)
  end

  def get_buyer_type
    buyer_type = %w(brothel gang rich beast_man tentacle).sample
    buyer_type || get_buyer_type
  end

  #=============== 魔術工房 start ==============
  def maternity_room_slaves
    @magic_factory.maternity_room
  end

  def tentacle_lab_slaves
    @magic_factory.tentacle_lab
  end

  def slave_brothel_slaves
    @magic_factory.slave_brothel
  end

  MATERNITY_FIRST_INCREASE_POINT = 0.7
  MATERNITY_INCREASE_POINT = 0.2
  MATERNITY_BONUS_POINT_LIMIT = 150.0
  def increase_monster_by_maternity!(maternity_count)
    return if maternity_count.zero?

    @magic_factory.generate_magic_power_by_maternity!
    @magic_factory.decrease_resist_for_spring_by_maternity!

    if @monster_gain_point_by_maternity_room < MATERNITY_BONUS_POINT_LIMIT
      point_gain = MATERNITY_FIRST_INCREASE_POINT +
        ((maternity_count - 1) * (MATERNITY_INCREASE_POINT + maternity_bonus))
      if (@monster_gain_point_by_maternity_room + point_gain) < 5.0
        point_gain *= 4.0
      end
      @monster_gain_point_by_maternity_room += point_gain
    elsif @monster_gain_point_by_maternity_room > MATERNITY_BONUS_POINT_LIMIT
      @monster_gain_point_by_maternity_room = MATERNITY_BONUS_POINT_LIMIT
    end
  end

  def maternity_bonus_rate
    (100.0 + @monster_gain_point_by_maternity_room) / 100
  end

  def magic_factory_in_night
    script = []

    # 1. 奴隷娼館
    slave_brothel_count = @magic_factory.slave_brothel.count
    if slave_brothel_count.nonzero?
      sales, today_guest_count, abort_guest_count = @magic_factory.sale_spring_in_brothel!(self)
      get_money_from_brothel!(sales)
      script += @magic_factory.slave_brothel_script(sales, today_guest_count, abort_guest_count)
      script += [' ']
    else
      get_money_from_brothel!(0)
    end

    # 2. 触手強制肉体改造部屋
    tentacle_script = @magic_factory.add_tentacle_effect_if!
    unless tentacle_script.empty?
      script += tentacle_script
      script += [' ']
    end

    # 3. 強制出産部屋の数だけ魔物コストボーナス増加
    maternity_room_count = @magic_factory.maternity_room.count
    if maternity_room_count.nonzero?
      @magic_factory.maternity_room.values.each do |slave|
        slave.sex_with_monster_for_maternity!(slave)
      end
      increase_monster_by_maternity!(maternity_room_count)
      script += @magic_factory.maternity_room_script
    end

    # 4. 精神摩耗の回復
    @magic_factory.rest_mentally_exhaust_of_slaves

    @magic_factory.collection_magic_power(self)

    script += [' ', ' ']

    script
  end
  #=============== 魔術工房 end ==============
  #=============== 奴隷娼館 start ==============
  def brothel_is_full?(slave_id, spring_type)
    spring_room_type = @magic_factory.room_type_by(spring_type)

    assigned_spring_room_type = @magic_factory.spring_room_type_of(slave_id)
    now_capacity = @unlocked_secretary.now_capacity_of_brothel_room(spring_room_type)
    self_using_bonus = (assigned_spring_room_type == spring_room_type) ? 1 : 0
    now_using_size = @magic_factory.now_brothel_using_sizeof(spring_room_type)
    (now_capacity + self_using_bonus) <= now_using_size
  end
  #=============== 奴隷娼館 end ==============

  MONEY_LIMIT = 10_000_000
  def _add_money(money)
    return if @money >= MONEY_LIMIT

    @money += money
    if @money > MONEY_LIMIT
      @money = MONEY_LIMIT
    end
  end

  #============== オークション部屋 start ==========
  def add_slave_to_auction_room(slave, sell_info)
    @auction_room.add_slave(slave, sell_info)
  end

  def auction_room_slaves
    @auction_room.slaves
  end

  def has_money_to_buy(slave_id)
    buy_price = @auction_room.buy_price(slave_id)
    have_money?(buy_price)
  end
  #============== オークション部屋 end ==========

  #============== ボーナスつきアイテムの配備 start ==========
  def develop_item!(item_key)
    case item_key
      when 1
        @escape_penalty += 5
      when 2
        @escape_penalty += 7
      when 3
        @escape_penalty += 6
      when 4
        @escape_penalty += 6
      when 11
        @capture_bonus += 5
      when 12
        @capture_bonus += 7
      when 13
        @capture_bonus += 8
      when 14
        @capture_bonus += 9
      when 21
        @maternity_bonus += 0.3
      when 22
        @maternity_bonus += 0.4
      when 23
        @maternity_bonus += 0.5
      when 24
        @maternity_bonus += 0.6
      when 31
        @physical_trap_bonus += 4
      when 32
        @physical_trap_bonus += 5
      when 33
        @physical_trap_bonus += 7
      when 34
        @physical_trap_bonus += 11
      when 41
        @ero_trap_bonus += 3
      when 42
        @ero_trap_bonus += 4
      when 43
        @ero_trap_bonus += 6
      when 44
        @ero_trap_bonus += 10
      else
        puts "Invalid Item Key!!(#{item_key})"
        nil
    end
  end
  #============== ボーナスつきアイテムの配備 end ==========

  MAGIC_POWER_LIMIT = 9_999_999
  def add_magic_power!(inc_magic_power)
    return 0 if inc_magic_power.zero?
    return 0 if @magic_power >= MAGIC_POWER_LIMIT
    old_magic_power = magic_power
    @magic_power += inc_magic_power
    if @magic_power > MAGIC_POWER_LIMIT
      @magic_power = MAGIC_POWER_LIMIT
    end
    new_magic_power = magic_power
    diff = new_magic_power - old_magic_power
  end

  def add_magic_power_from_jail!(inc_magic_power)
    diff = add_magic_power!(inc_magic_power)
    @today_magic_power_from_jail = diff
    diff
  end

  def add_magic_power_from_magic_factory!(inc_magic_power)
    diff = add_magic_power!(inc_magic_power)
    @today_magic_power_from_magic_factory = diff
    diff
  end

  def pay_maintenance_magic_cost!
    @magic_maintenance_cost = @slave_servant_cage.maintenance_cost
    @magic_power -= @magic_maintenance_cost
  end

  def add_elite_guards_in_deep!
    # 35F 以下に配備される。
    floor = rand(35..40)

    @deep_floor_guards[floor] ||= []
    @deep_floor_guards[floor] << [101].sample # とりあえずウルク決め打ち
    floor
  end

  ELITE_MANY_BORDER = 30 # 6フロアに平均5ユニットいたら打ち止め
  def has_many_guards?
    guard_unit_count >= ELITE_MANY_BORDER
  end

  def guard_unit_count
    @deep_floor_guards.values.flatten.size
  end
end
