# coding: utf-8

require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/defines/dungeon/item_list'
require_rrk 'src/defines/dungeon/carpenter_list'

class UnlockedSecretary
  include TrapList
  include MonsterListByType

  attr_reader :unlocked_monsters, :unlocked_traps, :unlocked_items, :unlocked_books, :unlocked_buildings,
              :brothel_rooms_count, :brothel_large_rooms_count

  def initialize
    @unlocked_monsters = default_monsters_list
    @unlocked_traps = TrapList::DEFAULT_TRAPS
    @unlocked_items = ItemList::DEFAULT_ITEMS
    @unlocked_books = []
    @unlocked_buildings = CarpenterList::DEFAULT_BUILDINGS
    @unlocked_tentacles = TentacleList::DEFAULT_TENTACLES
    @brothel_rooms_count = 0
    @brothel_large_rooms_count = 0
  end

  def unlocked_pool_traps
    @unlocked_traps[:enchant_pool] + @unlocked_traps[:tentacle_pool]
  end

  def unlock_summon!(monster_key)
    monster_type = _get_monster_type(monster_key)
    @unlocked_monsters[monster_type] << monster_key unless is_unlocked_monster_n_type?(monster_type, monster_key)
  end

  def unlock_trap!(trap_key)
    trap_type = _get_trap_type(trap_key)
    @unlocked_traps[trap_type] << trap_key unless is_unlock_trap?(trap_type, trap_key)
  end

  def is_unlock_trap?(trap_type, trap_key)
    @unlocked_traps[trap_type].include?(trap_key)
  end

  def unlock_item!(item_key)
    item_type = _get_item_type(item_key)
    @unlocked_items[item_type] << item_key unless is_unlock_item?(item_type, item_key)
  end

  def is_unlock_item?(item_type, item_key)
    @unlocked_items[item_type].include?(item_key)
  end

  def unlock_book!(book_key)
    @unlocked_books << book_key unless has_book?(book_key)
  end

  def has_book?(book_key)
    @unlocked_books.include?(book_key)
  end

  def is_unlock_building?(building_key)
    @unlocked_buildings.include?(building_key)
  end

  def unlock_building_or_buy!(building_key)
    # 買い増し
    if building_key == 1
      @brothel_rooms_count += 1
    elsif building_key == 2
      @brothel_large_rooms_count += 1
    end

    if !is_unlock_building?(building_key)
      # 未購入
      unlock_building!(building_key)
    end
  end

  def unlock_building!(building_key)
    @unlocked_buildings << building_key unless is_unlock_building?(building_key)
  end

  def is_unlock_tentacle?(tentacle_key)
    @unlocked_tentacles.include?(tentacle_key)
  end

  def unlock_tentacle!(tentacle_key)
    @unlocked_tentacles << tentacle_key unless is_unlock_tentacle?(tentacle_key)
  end

  def is_unlocked_monster?(monster_key)
    monster_type = _get_monster_type(monster_key)
    @unlocked_monsters[monster_type].include?(monster_key)
  end

  def can_buy?(monster_key, lock_list)
    lock_list ||= []
    # 制約リストの中身、全部買ってる？
    monster_type = _get_monster_type(monster_key)
    (lock_list & @unlocked_monsters[monster_type]) == lock_list
  end

  def can_buy_trap?(trap_key, lock_list, all_lock_list)
    lock_list ||= []
    trap_type = _get_trap_type(trap_key)
    result = !is_unlock_trap?(trap_type, trap_key) &&
      # 制約リストの中身、全部買ってる？
      (lock_list & @unlocked_traps[trap_type]) == lock_list &&
      (all_lock_list & @unlocked_traps.values.flatten) == all_lock_list
    result
  end

  def can_buy_item?(item_key, lock_list)
    lock_list ||= []
    item_type = _get_item_type(item_key)
    !is_unlock_item?(item_type, item_key) &&
      # 制約リストの中身、全部買ってる？
      (lock_list & @unlocked_items[item_type]) == lock_list
  end

  def can_buy_book?(book_key)
    !@unlocked_books.include?(book_key)
  end

  def can_buy_building?(building_key, lock_list)
    lock_list ||= []
    if [1, 2].include?(building_key)
      (lock_list & @unlocked_buildings) == lock_list
    else
      !is_unlock_building?(building_key) &&
        # 制約リストの中身、全部買ってる？
        (lock_list & @unlocked_buildings) == lock_list
    end
  end

  def can_buy_tentacle?(tentacle_key, lock_list)
    lock_list ||= []
    !is_unlock_tentacle?(tentacle_key) &&
      # 制約リストの中身、全部買ってる？
      (lock_list & @unlocked_tentacles) == lock_list
  end

  def is_unlocked_trap?(trap_key)
    trap_type = _get_trap_type(trap_key)
    @unlocked_traps[trap_type].include?(trap_key)
  end

  def get_monster_or_alternative(monster_key, floor)
    # アンロックされていればそのもの
    # アンロックされていなければ、アンロック済みで最も強力なやつ。
    # タイプそのものがアンロックされていなければ「ならずもの」
    monster_type = _get_monster_type(monster_key)
    return monster_key if is_unlocked_monster_n_type?(monster_type, monster_key)
    @unlocked_monsters[monster_type].last || _get_alternative_monster(floor)
  end

  def has_brothel_room?
    @unlocked_buildings.include?(1)
  end

  def has_brothel_large_room?
    @unlocked_buildings.include?(2)
  end

  def has_brothel_pole_dance_tent?
    @unlocked_buildings.include?(3)
  end

  def has_brothel_show_tent?
    @unlocked_buildings.include?(4)
  end

  # 「とりあえずならず者」
  def _get_alternative_monster(floor)
    if floor >= 30
      4 # 黒犬兵団
    elsif floor >= 20
      3 # 傭兵団
    elsif floor >= 10
      2 # 牙の団
    else
      1
    end
  end

  DEFAULT_TRAP_TYPE = :simple_trap
  def get_trap_or_alternative(trap_type)
    # デフォルトが来たらその中からランダムで
    @unlocked_traps[DEFAULT_TRAP_TYPE].sample if trap_type == DEFAULT_TRAP_TYPE

    # 指定されたタイプのうち、アンロック済みで最も強力なやつ。
    # タイプそのものがアンロックされていなければ雑魚トラップから選出
    trap_no = get_unlocked_trap_by_type(trap_type)
    trap_no
    trap_no || @unlocked_traps[DEFAULT_TRAP_TYPE].sample
  end

  def is_unlocked_monster_n_type?(monster_type, monster_key)
    @unlocked_monsters[monster_type].include?(monster_key)
  end

  def get_unlocked_trap_by_type(trap_type)
    @unlocked_traps[trap_type].last
  end

  def _get_monster_type(monster_key)
    monster = monster_define_list[monster_key]
    puts "Monster Not Found!(#{monster_key})" unless monster
    monster[:type]
  end

  def _get_trap_type(trap_key)
    trap = TrapList::TRAP_DEFINE_LIST[trap_key]
    puts "trap_key NOT FOUND!: #{trap_key}" unless trap
    trap[:type]
  end

  def _get_item_type(item_key)
    item = ItemList::ITEM_DEFINE_LIST[item_key]
    puts "item_key NOT FOUND!: #{item_key}" unless item
    item[:type]
  end

  def now_capacity_of_brothel_room(spring_room_type)
    case spring_room_type
      when 'force_jail'
        99 # 肉便器はほぼ無限大
      when 'brothel_room'
        brothel_rooms_count
      when 'brothel_large_room'
        brothel_large_rooms_count
      when 'brothel_pole_dance_tent'
        has_brothel_pole_dance_tent? ? 1 : 0
      when 'brothel_show_tent'
        has_brothel_show_tent? ? 1 : 0
      else
        puts "INVALID spring_room_type: (#{spring_room_type})"
        0
    end
  end

  def has_tentacle_lab_v?
    @unlocked_tentacles.include?(2)
  end

  def has_tentacle_lab_v_lv2?
    @unlocked_tentacles.include?(4)
  end

  def has_tentacle_lab_a?
    @unlocked_tentacles.include?(3)
  end

  def has_tentacle_lab_b?
    @unlocked_tentacles.include?(1)
  end

  def has_tentacle_lab_brainwash?
    @unlocked_tentacles.include?(5)
  end

  def has_tentacle_lab_brainwash_lv2?
    @unlocked_tentacles.include?(7)
  end

  def has_tentacle_lab_hard?
    @unlocked_tentacles.include?(6)
  end
end
