# coding: utf-8

require_rrk 'src/app/model/dungeon/jail/jail_room.rb'
require_rrk 'src/app/model/dungeon/jail/jail_monster_party.rb'
require_rrk 'src/defines/script/dungeon/jail_script.rb'
require_rrk 'src/app/service/dungeon/jail/jail_service.rb'
require_rrk 'src/app/service/dungeon/module/dungeon_common.rb'
require_rrk 'src/app/util/debug_util'

class Jail
  include DebugUtil
  attr_reader :jail_rooms, :success_rescue_list

  def initialize
    @jail_rooms = {}
    @jail_service = JailService.new
    @success_rescue_list = {}
  end

  # 賞金首フラグのクリア
  def clear_now_revoke_bounty_flag_n_rtn
    @jail_rooms.values.map {|room| room.clear_now_revoke_bounty_flag_n_rtn }.compact
  end
  # ======= 売却処理 start ==================
  def sell_slaves_if!(dungeon_master)
    @jail_service.sell_slaves_if!(self, dungeon_master)
  end
  # ======= 売却処理 end ==================

  # ======= 監禁処理 start ==================
  def progress!(day_count, escape_penalty)
    # 監禁中
    _progress_in_jail!(day_count, escape_penalty)
  end

  def _progress_in_jail!(day_count, escape_penalty)
    jail_rooms_summaries = _make_jail_rooms_summaries

    rooms_in_resist = []
    rooms_in_training = []
    @jail_rooms.values.each do |room|
      room.add_day_count!
      if room.in_resist?
        rooms_in_resist << room
      else
        rooms_in_training << room
      end
    end

    script = []
    script += _progress_in_resist(rooms_in_resist, escape_penalty)
    script += _progress_in_training(rooms_in_training)

    # 結果が出ないとサマリが出せないのでどうしてもここで取る必要がある。
    summaries_element = _make_summaries_script(day_count, jail_rooms_summaries)

    summaries_element.merge({ progress_script: script })
  end

  def _make_jail_rooms_summaries
    jail_rooms_summaries_in_resit = {}
    jail_rooms_summaries_in_training = {}
    @jail_rooms.each do |adv_id, room|
      if room.in_resist?
        jail_rooms_summaries_in_resit[room.floor_number] ||= {}
        jail_rooms_summaries_in_resit[room.floor_number][adv_id] = {
          room: room,
          text: @jail_service.summary_base_text(room),
        }
      else
        jail_rooms_summaries_in_training[room.floor_number] ||= {}
        jail_rooms_summaries_in_training[room.floor_number][adv_id] = {
          room: room,
          text: @jail_service.summary_base_text(room),
        }
      end
    end

    # 階層ごとに抵抗中 => 陵辱中の順序にしたい
    jail_rooms_summaries = {}
    jail_rooms_summaries_in_resit.sort.each do |floor, params|
      jail_rooms_summaries[floor] ||= {}
      params.each do |adv_id, room_params|
        jail_rooms_summaries[floor][adv_id] = room_params
      end
    end
    jail_rooms_summaries_in_training.sort.each do |floor, params|
      jail_rooms_summaries[floor] ||= {}
      params.each do |adv_id, room_params|
        jail_rooms_summaries[floor][adv_id] = room_params
      end
    end
    jail_rooms_summaries
  end

  def _make_summaries_script(day_count, jail_rooms_summaries)
    border_script = @jail_service.jail_summary_border(day_count)

    body_element = []
    jail_rooms_summaries.each do |floor, floor_params|
      floor_params.each do |adv_id, room_params|
        text = @jail_service.summary_text(room_params[:room], room_params[:text])
        adv_id = room_params[:room].adventurer.id
        body_element << {
          'text' => text,
          'type' => 'button',
          'action' => "show_prisoner_in_jail_#{adv_id}",
        }
      end
    end
    unless jail_rooms_summaries.empty?
      prelude_script = @jail_service.jail_summary_prelude_text(jail_rooms_summaries.count)
    end

    {
      border_script: border_script,
      body_element: body_element,
      prelude_script: (prelude_script || []),
    }
  end

  def _progress_in_resist(rooms, escape_penalty)
    scripts = rooms.map { |room| @jail_service.decrease_resist(room, escape_penalty) }
    (scripts.inject(:+) || [])
  end

  def _progress_in_training(rooms)
    scripts = rooms.map do |room|
      @jail_service.progress_of_training(room)
    end
    scripts.inject(:+) || []
  end
  # ======= 監禁処理 end ==================

  # ======= 捕獲処理 start ==================
  def capture!(adventurer, floor)
    monster_count =
      if adventurer.captured_initial_count > 0
        adventurer.captured_initial_count
      else
        rand(1..3)
      end

    monster_party = JailMonsterParty.new(
      monster: adventurer.captured_monster,
      count: monster_count,
      betrayed_servant: adventurer.captured_betrayed_servant,
      adventurer: adventurer,
    )
    room = JailRoom.new(adventurer, monster_party, floor)
    @jail_rooms[adventurer.id] = room

    betrayed_servant = monster_party.betrayed_servant
    if betrayed_servant &&
      !(betrayed_servant.is_active? && betrayed_servant.has_betrayed_penis?)
      adventurer.leave_captured_betrayed_servant!
      monster_party.detach_betrayed_servant!
    end
  end
  # ======= 捕獲処理 end ==================

  # ======= 救出処理 start ==================
  include DungeonCommon
  JAIL_PASS_RATE = 0.9
  def engage_jail?(floor)
    floor_prisoner_ids = get_prisoner_ids_by_floor(floor)
    return false if floor_prisoner_ids.empty?

    engage_rate = 100 - ((JAIL_PASS_RATE ** floor_prisoner_ids.size) * 100).to_i
    random_roll(engage_rate, 100)
  end

  def get_jail_room_by(floor)
    adventurer_id = get_prisoner_ids_by_floor(floor).sample
    @jail_rooms[adventurer_id]
  end

  def get_prisoner_ids_by_floor(floor)
    prisoner_ids = []
    @jail_rooms.each do |jail_id, jail_room|
      prisoner_ids << jail_id if jail_room.floor == floor
    end
    prisoner_ids
  end

  def get_prisoner_ids_all
    @jail_rooms.keys
  end

  def get_floor_by_prisoner_id(prisoner_id)
    target_room = @jail_rooms[prisoner_id]
    target_room.floor_number if target_room
  end

  def get_short_name_by_prisoner_id(prisoner_id)
    target_room = @jail_rooms[prisoner_id]
    target_room ? target_room.adventurer.short_name : ''
  end
  # ======= 救出処理 end ==================

  # ======= 脱走処理 start ==================
  def escaped_adventurers
    rooms = jail_rooms.values.select {|room| room.is_escaped }
    adventurers = rooms.map do |room|
      puts "#{room.adventurer.short_name}(#{room.adventurer.id})が脱走しました" if DEBUG_MODE_FOR_ADV
      delete_jail_room!(room.adventurer)
      # 脱走時の共通処理
      @jail_service.set_jail_escape_common(room.adventurer, room.floor_number)
      room.adventurer
    end

    adventurers
  end
  # ======= 脱走処理 end ==================

  # ======= 解放処理 start ==================
  def liberate_room!(adventurer, leader, is_lonely, floor_number)
    puts "#{adventurer.short_name}(#{adventurer.id})が解放されました" if DEBUG_MODE_FOR_ADV
    delete_jail_room!(adventurer)
    # 救出台詞の実装
    @jail_service.set_jail_rescued_common(adventurer, leader, is_lonely, floor_number)
  end
  # ======= 解放処理 end ==================

  def is_empty?
    @jail_rooms.empty?
  end

  def print_jail_room
    @jail_rooms.values.each do |room|
      puts "#{room.adventurer.short_name}は#{room.monster.count}#{room.monster.unit_name}の#{room.monster.name}によって監禁されています。"
    end
  end

  def delete_jail_room!(adventurer)
    adventurer.release_from_jail! # フラグクリア
    room = jail_rooms[adventurer.id]
    room.cleaning!
    jail_rooms.delete(adventurer.id)
  end

  def prisoner_count
    jail_rooms.count
  end

  def collection_magic_power(dungeon_master)
    magic_power = 0
    jail_rooms.values.each do |room|
      magic_power += room.adventurer.drain_magic_power! if room.adventurer.have_generated_magic_power?
    end
    dungeon_master.add_magic_power_from_jail!(magic_power)
  end

  def set_success_rescue!(jail_room, party_leader)
    success_rescue_list[jail_room.adventurer.id] = party_leader
  end

  def clean_status_of_rooms!
    @success_rescue_list = {}
    jail_rooms.values.each do |room|
      room.status_clear!
    end
  end
end
