# coding: utf-8

require_rrk 'src/app/model/dungeon/monster_party.rb'

class JailMonsterParty < MonsterParty
  attr_reader :betrayed_servant

  MAX_MONSTER_COUNT = 100
  def initialize(params)
    super(params) # フロアから静寂と暗闇を指定

    @monster = params[:monster]
    @over_flow = params[:count] > MAX_MONSTER_COUNT
    @rest_enemies_count = _generate_initial_rest_enemies_count(params[:count])
    @initial_enemies_count = params[:count] # セリフ表示に必要
    @skills = @monster.skills || []
    @skill_personality = params[:skill_personality] || []
    @skill_holic = params[:skill_holic] || []
    @bad_status = @monster.bad_status || {}
    @bad_sex_status = @monster.bad_sex_status || {}

    base_betrayed_servant = params[:betrayed_servant]
    if base_betrayed_servant &&
      base_betrayed_servant.is_active? && # 体力があって
      base_betrayed_servant.has_betrayed_penis? && # ちんちんついていて
      true # !base_betrayed_servant.in_training_in_jail_as_betrayed? # 他の人を調教中ではない

      # よっしゃ配属や！
      @betrayed_servant = base_betrayed_servant
      @betrayed_servant.set_training_in_jail_as_betrayed!
      adventurer = params[:adventurer]
      @betrayed_servant.betrayed_status_text!(adventurer.short_name + 'を監禁中')
    end
  end

  def has_betrayed_servant?
    !@betrayed_servant.nil?
  end

  def cleaning!
    @betrayed_servant.unset_training_in_jail_as_betrayed! if @betrayed_servant
  end

  # 監禁中に使う
  SUMMON_RATE = 0.3
  SUMMON_RATE_SMALL = 0.7
  def summon_companions!
    high_monster_count = (MAX_MONSTER_COUNT * 0.8).to_i

    if @rest_enemies_count.between?(1, 5)
      # 少数だと話が進まないので一気に増やす。
      # ただしこのターンで脱走前提。スキルがあると脱走できないのでそれ用。
      increase = rand(4..6)
    else
      rate = (@rest_enemies_count < high_monster_count) ? SUMMON_RATE_SMALL : SUMMON_RATE
      increase = @rest_enemies_count * rate # とりあえず決め打ち
      increase *= 0.05 if @rest_enemies_count > (MAX_MONSTER_COUNT * 0.93).to_i # 一定数を超えたら伸びはゆるやかに
    end

    if MAX_MONSTER_COUNT < @rest_enemies_count
      @over_flow = true
    elsif MAX_MONSTER_COUNT < (@rest_enemies_count + increase.to_i)
      @over_flow = true
      over_flow_count = (@rest_enemies_count + increase.to_i) - MAX_MONSTER_COUNT
      add_count = over_flow_count % rand(1..9)
      @rest_enemies_count = MAX_MONSTER_COUNT + add_count
    else
      @over_flow = false
      @rest_enemies_count += increase.to_i
    end
  end

  # 限界値をガン無視して1.3倍
  def summon_companions_by_bounty!
    @rest_enemies_count = (@rest_enemies_count * 1.3).to_i
  end

  # 遭遇戦に比べてダメージ能力が劣るよ
  def _penalty_in_jail
    0.6
  end

  def over_flow?
    @over_flow
  end

  def has_arrest_skill?
    skills.include?('arrest')
  end

  def _generate_initial_rest_enemies_count(count)
    if MAX_MONSTER_COUNT < count
      over_flow_count = count - MAX_MONSTER_COUNT
      add_count = over_flow_count % rand(1..9)
      (MAX_MONSTER_COUNT + add_count).to_i
    else
      count.to_i
    end
  end
end
