# coding: utf-8

require_rrk 'src/app/helper/forwardable'
require_rrk 'src/app/model/dungeon/utils/secure_random' # ランダム文字列生成
require_rrk 'src/app/service/dungeon/module/dungeon_common'

class JailRoom
  extend Forwardable
  include DungeonSecureRandom
  include DungeonCommon

  attr_reader :identify_code, :adventurer, :monster_party, :floor_number, :is_escaped,
              :day_count, :force_sex_day_count, :force_sex_full_day_count, :status_in_this_turn, :failed_reason,
              :rescue_party_leader

  def_delegators :@adventurer,
                 :resist_point, :resist_point_before_turn,
                 :san_level, :san_level_before_turn
  def_delegators :@monster_party, :over_flow?, :summon_companions!, :summon_companions_by_bounty!, :cleaning!,
                 :betrayed_servant

  def initialize(adventurer, monster_party, floor_number)
    @identify_code = make_identify_code
    @adventurer = adventurer
    @monster_party = monster_party
    @floor_number = floor_number
    @is_escaped = false
    @day_count = 1 # 監禁何日目？
    @force_sex_day_count = 1 # 陵辱何日目？
    @force_sex_full_day_count = 0 # 陵辱＆満員になってから何日目？
    @status_in_this_turn = nil
    @now_revoke_bounty_flag = false
    @failed_reason = nil # 脱走判定に失敗した理由

    @adventurer.capture_in_jail!
    @rescue_party_leader = nil
  end

  def refresh_by_turn!
    @failed_reason = nil # 脱走判定に失敗した理由
  end

  def empty_monster?
    @monster_party.rest_enemies_count.zero?
  end

  def escape!
    @is_escaped = true
  end

  def floor
    floor_number
  end

  def in_resist?
    !@adventurer.give_up_to_resist?
  end

  def jail_day_count
    if in_resist?
      day_count
    else
      force_sex_day_count
    end
  end

  def fall_level
    if in_resist?
      @adventurer.resist_point # 抵抗値。これがゼロになると犯される。
    else
      @adventurer.san_level # 正気度。これがゼロになると奴隷宣言
    end
  end

  def monster_has_arrest_skill?
    @monster_party.has_arrest_skill?
  end

  FEW_MONSTER_COUNT = 10
  def few_monsters?
    @monster_party.rest_enemies_count < FEW_MONSTER_COUNT
  end

  BETRAYED_SERVANT_RESIST_BONUS = 10
  def decrease_resist!
    @adventurer.clear_decrease_resist!
    base_decrease_point = monster_count / 2

    if @monster_party.has_betrayed_servant?
      base_decrease_point +=
        BETRAYED_SERVANT_RESIST_BONUS +
          (@monster_party.betrayed_servant.level * 0.5).to_i
    end

    penalty_decrease_point = 0
    if @adventurer.has_skill_personality?(:androphobia)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '男性恐怖症', 0.8, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:paralysis)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '麻痺', 0.3, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:fever)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '疫熱', 0.3, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:rat)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '発情', 0.3, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:aphrodisiac_fever)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '媚薬発情', 0.5, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:hypno)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '催眠', 0.5, base_decrease_point)
    end
    if @adventurer.has_bad_status_in_dive?(:aphrodisiac)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '催淫', 0.5, base_decrease_point)
    end
    # ここから下は永続系
    if @adventurer.has_skill_holic?(:dirty_sex_holic)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '汚辱中毒', 0.1, base_decrease_point)
    end
    if @adventurer.has_skill_holic?(:gang_rape_holic)
      penalty_decrease_point += _decrease_resist_penalty(@adventurer, '輪姦被虐中毒', 0.2, base_decrease_point)
    end

    decrease_point = (base_decrease_point + penalty_decrease_point).to_i
    @adventurer.decrease_resist_in_jail!(decrease_point)
    @adventurer.down_to_submit_if!
  end

  def _decrease_resist_penalty(adventurer, status_name, rate, base_decrease_point)
    penalty = (base_decrease_point * rate).to_i
    adventurer.set_decrease_resist(status_name, penalty)
    penalty
  end

  def decrease_san_level!
    @adventurer.decrease_san_level!(
      monster_count, monster_party.is_beast, monster_party.is_tentacle, monster_party.is_dirty, force_sex_full_day_count)
  end

  def break_san_level?
    @adventurer.break_san_level?
  end

  def give_up_to_resist?
    @adventurer.give_up_to_resist?
  end

  def is_now_slave_call?
    @adventurer.is_now_slave_call?
  end

  def is_now_slave_call!
    @adventurer.is_now_slave_call!
  end

  def clear_slave_call!
    @adventurer.clear_slave_call!
  end

  def monster_count
    @monster_party.rest_enemies_count
  end

  def monster_threat
    monster_count * monster_party.power
  end

  def monster
    monster_party.monster
  end

  def monster_skill_personality
    monster.skill_personality
  end

  def monster_skill_holic
    monster.skill_holic
  end

  def monster_bad_status_in_body
    monster.bad_status_in_body
  end

  def add_day_count!
    @day_count += 1
    unless in_resist?
      @force_sex_day_count += 1
      if monster_party.over_flow? # 満員？
        @force_sex_full_day_count += 1
      end
    end
  end

  def _betray_servant_train_if!
    return unless monster_party.has_betrayed_servant?

    count = rand(3..10) # 何回セックスしたか

    c_exp = rand(count..(count * 5).to_i)
    v_exp = rand(count..(count * 4).to_i)
    a_exp = rand(count..(count * 2).to_i)
    b_exp = rand(count..(count * 3).to_i)
    oral_exp = rand(count..(count * 3).to_i)

    c_voltage = c_exp * rand(30..70)
    v_voltage = v_exp * rand(25..80)
    a_voltage = a_exp * rand(12..25)
    b_voltage = b_exp * rand(12..35)

    v_sex_exp = (v_exp * rand(0.3..0.5)).to_i
    a_sex_exp = (v_exp * rand(0.2..0.4)).to_i

    v_sperm_shot = (v_sex_exp * rand(0.2..0.8)).to_i
    a_sperm_shot = (a_sex_exp * rand(0.15..0.6)).to_i
    oral_sperm_shot = (oral_exp * rand(0.2..0.4)).to_i
    skin_sperm_shot = ((c_exp + b_exp) * rand(0.2..0.6)).to_i

    adventurer.add_lesbian_exp!(count)
    adventurer.add_force_sex_exp!(v_sex_exp + a_sex_exp + oral_exp)
    adventurer.add_c_exp!(c_exp)
    adventurer.add_v_exp!(v_exp)
    adventurer.add_a_exp!(a_exp)
    adventurer.add_b_exp!(b_exp)
    adventurer.add_c_voltage!(c_voltage)
    adventurer.add_v_voltage!(v_voltage)
    adventurer.add_a_voltage!(a_voltage)
    adventurer.add_b_voltage!(b_voltage)
    adventurer.add_oral_exp!(oral_exp)
    adventurer.add_v_sex_exp!(v_sex_exp)
    adventurer.add_a_sex_exp!(a_sex_exp)
    adventurer.add_v_sperm_shot!(v_sperm_shot)
    adventurer.add_a_sperm_shot!(a_sperm_shot)
    adventurer.add_oral_sperm_shot!(oral_sperm_shot)
    adventurer.add_skin_sperm_shot!(skin_sperm_shot)

    # 性感帯開発
    ero_develop_bonus_rate =
      [(count * rand(5..15)), 120].min +
        (adventurer.get_this_turn_orgasms_sum * 0.10) # 絶頂数ボーナス
    ero_develop_bonus_gain = [
      0.8,
      [(ero_develop_bonus_rate * 0.05), 0.2].max
    ].min
    [:c, :v, :a, :b].each do |type|
      if random_roll(ero_develop_bonus_rate, 300)
        adventurer.add_ero_develop_rate!(type, ero_develop_bonus_gain)
      end
    end
  end

  def add_train_exp!
    _betray_servant_train_if!

    count = monster_count

    c_exp = rand(count..(count * 3.5).to_i)
    v_exp = rand(count..(count * 5).to_i)
    a_exp = rand(count..(count * 4).to_i)
    b_exp = rand(count..(count * 2).to_i)
    oral_exp = rand(count..(count * 3).to_i)

    c_voltage = c_exp * rand(10..20)
    v_voltage = v_exp * rand(15..30)
    a_voltage = a_exp * rand(8..15)
    b_voltage = b_exp * rand(5..10)

    v_sex_exp = (v_exp * rand(0.6..0.8)).to_i
    a_sex_exp = (v_exp * rand(0.4..0.8)).to_i

    v_sperm_shot = (v_sex_exp * rand(0.4..1.2)).to_i
    a_sperm_shot = (a_sex_exp * rand(0.4..1.2)).to_i
    oral_sperm_shot = (oral_exp * rand(0.6..0.9)).to_i
    skin_sperm_shot = ((c_exp + b_exp) * rand(0.2..0.5)).to_i

    adventurer.add_men_exp!(count)
    adventurer.add_force_sex_exp!(v_sex_exp + a_sex_exp + oral_exp)
    adventurer.add_c_exp!(c_exp)
    adventurer.add_v_exp!(v_exp)
    adventurer.add_a_exp!(a_exp)
    adventurer.add_b_exp!(b_exp)
    adventurer.add_c_voltage!(c_voltage)
    adventurer.add_v_voltage!(v_voltage)
    adventurer.add_a_voltage!(a_voltage)
    adventurer.add_b_voltage!(b_voltage)
    adventurer.add_oral_exp!(oral_exp)
    adventurer.add_v_sex_exp!(v_sex_exp)
    adventurer.add_a_sex_exp!(a_sex_exp)
    adventurer.add_v_sperm_shot!(v_sperm_shot)
    adventurer.add_a_sperm_shot!(a_sperm_shot)
    adventurer.add_oral_sperm_shot!(oral_sperm_shot)
    adventurer.add_skin_sperm_shot!(skin_sperm_shot)

    # 性感帯開発
    ero_develop_bonus_rate =
      count +
        (adventurer.get_this_turn_orgasms_sum * 0.03) # 絶頂数ボーナス
    ero_develop_bonus_gain = [
      0.3,
      [(ero_develop_bonus_rate * 0.05), 0.1].max
    ].min
    [:c, :v, :a, :b].each do |type|
      if random_roll(ero_develop_bonus_rate, 300)
        adventurer.add_ero_develop_rate!(type, ero_develop_bonus_gain)
      end
    end
  end

  # ======= ステータス更新系 ==========
  # 返り討ち
  def status_repulse!(rescue_party_leader)
    @rescue_party_leader = rescue_party_leader
    @status_in_this_turn = 'repulse'
  end

  # 見捨てる
  def status_abort!(rescue_party_leader)
    unless status_in_this_turn == 'repulse' # 返り討ちが優先
      @rescue_party_leader = rescue_party_leader
      @status_in_this_turn = 'abort'
    end
  end

  def status_clear!
    @rescue_party_leader = nil
      @status_in_this_turn = nil
  end

  def set_failed_reason!(status)
    @failed_reason = status # 脱走判定に失敗した理由
  end

  def revoke_bounty!
    adventurer.revoke_bounty!
    @now_revoke_bounty_flag = true
  end

  def clear_now_revoke_bounty_flag_n_rtn
    if @now_revoke_bounty_flag
      @now_revoke_bounty_flag = false
      adventurer.id
    else
      nil
    end
  end

  def in_brain_washing_first_day?
    @day_count == 1 &&
      adventurer.in_brain_washing?
  end
end
