# coding: utf-8

require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/game_define/dungeon/adventurers_list'
require_rrk 'src/game_define/dungeon/slaves_list'
require_rrk 'src/app/service/dungeon/party_build_service'
require_rrk 'src/app/model/dungeon/adventurer'
require_rrk 'src/app/model/dungeon/slave'
require_rrk 'src/app/model/dungeon/jail'

require_rrk 'src/app/helper/game_define_helper'
require_rrk 'src/defines/dungeon/utils/version_helper'

require_rrk 'src/app/service/dungeon/dungeon_progress_service'
require_rrk 'src/app/service/dungeon/module/dungeon_common'

class LuidaBar
  include DungeonCommon
  include MonsterListByType
  attr_reader :party_build_service, :chairs, :adventurers,
              :party_by_adventurer_id, :slave_in_magic_factory,
              :lost_adventurer_ids,
              :bounty_list

  def initialize
    @chairs = {} # パーティー未加入の人たち
    @adventurers = {}
    @sold_adventurers = {} # 売却されてもう二度と戻らない冒険者たち
    @slave_in_magic_factory = {} # 魔術工房捕獲済み奴隷
    @party_by_adventurer_id = {}
    @bounty_list = {} # 期限付き賞金首リスト
    @party_build_service = PartyBuildService.new(@party_by_adventurer_id)
    @original_using_names = {} # オリジナル用名前被り防止リスト
    @lost_adventurer_ids = [] # 魔力転換ないし売却されて迷宮内にいなくなったリスト
  end

  def boot_from_list!
    if GameDefineHelper::ORIGINAL_MODE
      @adventurers_name_list = AdventurersList.original_name_list
      @adventurers_class_list = AdventurersList.original_class_list
    else
      @adventurers_list = AdventurersList.adventure_list_by_ver
      @slaves_list = SlavesList.slave_list_by_mode
    end
  end

  def test_build_chairs
    @chairs = {}

    return initial_build_chairs_for_original if GameDefineHelper::ORIGINAL_MODE

    # 初期作成
    @adventurers_list.each do |key, adventurer_params|
      adventurer = Adventurer.new(key, adventurer_params.merge({ original_id: key }))
      adventurer.coming_day_text = '１日目にカルデアから初期の派遣時に送り込まれた。' if GameDefineHelper::FGO_MODE
      @chairs[key] = adventurer
      @adventurers[key] = adventurer
    end
    floor = 1
    jail = Jail.new
    @slaves_list.each do |key, slave_adventurer_params|
      slave_adventurer = Adventurer.new(key, slave_adventurer_params.merge({ original_id: key }))
      slave_adventurer.coming_day_text = 'カルデアから先遣隊として送り込まれた。' if GameDefineHelper::FGO_MODE

      @adventurers[key] = slave_adventurer
      slave_adventurer.be_initial_slave!(default_monster, floor)
      jail.capture!(slave_adventurer, floor)
    end
    _add_train_days!(jail, 3)

    @chairs
  end

  def _add_train_days!(jail, day_count)
    jail.jail_rooms.each do |key, room|
      # n日間陵辱
      day_count.times { room.add_train_exp! }
      slave = Slave.new(
        id: room.adventurer.id,
        name: room.adventurer.name,
        short_name: room.adventurer.short_name,
        bad_status: room.adventurer.bad_status,
        bad_sex_status: room.adventurer.bad_sex_status,
        adventurer: room.adventurer,
      )
      @slave_in_magic_factory[key] = slave
    end
    jail.jail_rooms.keys.each do |key|
      room = jail.jail_rooms[key]
      adventurer = room.adventurer
      jail.delete_jail_room!(adventurer)
    end
  end

  def initial_build_chairs_for_original
    adv_class_keys = @adventurers_class_list.keys

    capable_names = {}
    @adventurers_name_list.each do |race_type, names|
      capable_names[race_type] = names
    end

    # ３〜５人程度捕虜が欲しい
    floor = 1
    jail = Jail.new
    prisoners_count = rand(3..5)
    (1..prisoners_count).to_a.each do |key|
      slave_adventurer = _generate_original_adventurer_n_add_chair(key, 1, adv_class_keys, capable_names, -1)
      slave_adventurer.be_initial_slave!(default_monster, floor)
      jail.capture!(slave_adventurer, floor)
    end
    _add_train_days!(jail, 5) # ５日間陵辱

    # 次に冒険者生成
    ((1 + prisoners_count)..(50 + prisoners_count)).each { |key| _generate_original_adventurer_n_add_chair(key, 1, adv_class_keys, capable_names, 1) }
    @chairs
  end

  def _generate_original_adventurer(key, standard_level, adv_class_keys, capable_names, hour_glass_day)
    type = capable_names.keys.sample
    @original_using_names[type] ||= []
    name = capable_names[type].sample
    return if name.nil? # 使える名前がないので諦める
    capable_names[type].delete(name)
    capable_names.delete(type) if capable_names[type].empty?
    @original_using_names[type] << name # 利用履歴に追加

    hierarchy = %w(noble rich middle poor).sample
    adv_class_type = adv_class_keys.sample
    adv_class_name = @adventurers_class_list[adv_class_type][hierarchy.to_sym].sample

    self_call =
      case hierarchy
        when 'noble'
          %w(私 わたし わたくし 妾).sample
        when 'rich'
          %w(私 わたし わたくし あたし ぼく).sample
        when 'middle'
          %w(私 アタシ ボク オレ).sample
        when 'poor'
          %w(私 アタシ アタイ ボク オレ わっち).sample
      end

    level = rand(standard_level..(standard_level + 5))

    talents = []
    bonus_drop_rate = (standard_level / 2) # 最大40だから、ドロップ率は最大20%plus
    battle_bonus = 0
    equip_point_bonus = 0
    if  (adv_class_type == :scout)
      if random_roll((20 + bonus_drop_rate), 100) # 一定確率で闇の英雄
        talents << :dark_hero
        battle_bonus = 20
        equip_point_bonus = 40
      end
    else
      if hierarchy == 'noble'
        if random_roll((20 + bonus_drop_rate), 100) # 一定確率で王族
          hierarchy = 'royal'
          talents << :royal
          battle_bonus = 50
          equip_point_bonus = 30
          adv_class_name = @adventurers_class_list[adv_class_type][:royal].sample # 職業上書き
        elsif random_roll((40 - bonus_drop_rate), 100) # それ以外なら騎士叙勲
          talents << :knight
          battle_bonus = 30
          equip_point_bonus = 15
        end
      end
    end

    if hierarchy == 'noble'
      talents << :noble
      battle_bonus = 20
      equip_point_bonus = 15
    end

    adv_params = {
      name: name,
      short_name: name,
      level: level,
      self_call: self_call,
      positiveness: rand(-2..2),
      hooter_level: rand(0..3),
      lawful_level: %w(lawful neutral chaotic).sample,
      good_level: %w(good neutral evil).sample,
      race_type: type,
      class_type: adv_class_type.to_s,
      class_name: adv_class_name,
      hierarchy: hierarchy,
      battle_bonus: battle_bonus,
      talents: talents,
      equip_point_bonus: equip_point_bonus,
    }

    # オリジナル
    adventurer = Adventurer.new(key, adv_params.merge({ original_id: key }))
    @adventurers[key] = adventurer

    if hour_glass_day < 0
      adventurer.coming_day_text = '帝国が初期に編成した迷宮探索隊の一員として送り込まれた。'
    elsif hour_glass_day == 1
      adventurer.coming_day_text = '１日目に帝国が大規模に編成した第一次迷宮攻略団の一員として初期の派遣時に送り込まれた。'
    else
      adventurer.coming_day_text = "#{hour_glass_day}日目に増援として送り込まれた。"
    end

    adventurer
  end

  def _generate_original_adventurer_n_add_chair(key, standard_level, adv_class_keys, capable_names, hour_glass_day)
    adventurer = _generate_original_adventurer(key, standard_level, adv_class_keys, capable_names, hour_glass_day)
    @chairs[key] = adventurer
  end

  def build_chairs_for_append_adventurers
    return nil if AdventurersList.append_adventure_list_by_ver.empty?

    AdventurersList.append_adventure_list_by_ver.each do |key, adventurer_params|
      adventurer = Adventurer.new(key, adventurer_params.merge({ original_id: key }))
      @chairs[key] = adventurer
      @adventurers[key] = adventurer
    end
  end

  ADVENTURER_ID_MAX = 999
  def build_chairs_for_append_adventurers_for_original(hour_glass)
    standard_level = [(hour_glass.day * 0.6).to_i, 40].min
    adv_class_keys = @adventurers_class_list.keys

    # 使える名前を列挙
    capable_names = {}
    @adventurers_name_list.each do |race_type, names|
      capable_names[race_type] = (names - (@original_using_names[race_type] || []))
    end
    capable_size = capable_names.values.flatten.size

    new_start_key = @adventurers.keys.max + 1
    return if new_start_key > ADVENTURER_ID_MAX
    new_end_key = new_start_key + ([10, capable_size].min - 1) # 使える名前が10なかったら縮小
    new_end_key = ADVENTURER_ID_MAX if new_end_key > ADVENTURER_ID_MAX

    (new_start_key..new_end_key).each do |key|
      _generate_original_adventurer(key, standard_level, adv_class_keys, capable_names, hour_glass.day)
    end
  end

  include VersionHelper
  APPEND_UNDER_LIMIT_FOR_FGO = 10
  APPEND_LIMIT_FOR_FGO = 2
  def build_chairs_for_append_adventurers_for_fgo(hour_glass)
    # Ver8未満だと無限追加は行われない
    return if (DUNGEON_VERSION < 8.0)

    max_key = @adventurers.keys.max + 1
    if ((max_key > ADVENTURER_ID_MAX) || # 一定数以上より多くの冒険者は作れないよ
      (lost_adventurer_ids.size < APPEND_UNDER_LIMIT_FOR_FGO)) # 喪失した人間が少ないと諦める
      return
    end

    standard_level = [(hour_glass.day * 0.6).to_i, 40].min
    target_adventurer_ids = lost_adventurer_ids.shuffle[0..APPEND_LIMIT_FOR_FGO]

    used_adventurer_ids = []
    adventurer_list = @adventurers_list
    target_adventurer_ids.each do |adventurer_id|
      next if max_key > ADVENTURER_ID_MAX # 一定数以上より多くの冒険者は作れないよ

      adventurer_params = adventurer_list[adventurer_id]
      if adventurer_params
        adventurer_params[:level] = standard_level # レベルを上書き
        adventurer = Adventurer.new(max_key, adventurer_params.merge({ original_id: adventurer_id }))
        adventurer.coming_day_text = "#{hour_glass.day}日目にカルデアからの増援として送り込まれた。"
        @chairs[max_key] = adventurer
        @adventurers[max_key] = adventurer
        used_adventurer_ids << adventurer_id
        max_key += 1
      end
    end

    # 消費したIDは喪失済リストから削除
    set_lost_adventurer_ids((lost_adventurer_ids - used_adventurer_ids))
  end

  def default_monster
    monster = monster_define_list[1]
    @_default_monster ||= Monster.new({
                  id: 1,
                  name: monster[:name],
                  power: monster[:power],
                  life_point: monster[:life_point],
                  unit_name: monster[:unit_name],
                  skills: monster[:skills],
                  bad_status: monster[:bad_status] || {},
                  bad_sex_status: monster[:bad_sex_status] || {},
                  is_beast: monster[:is_beast],
                  is_tentacle: monster[:is_tentacle],
                  is_dirty: monster[:is_dirty],
                })
  end

  def recycle_parties(parties)
    parties = party_build_service.recycle_parties(parties, @chairs)
    parties
  end

  def build_parties!(jail, new_pot)
    party_build_service.build_parties!(@chairs, jail, @adventurers, new_pot)
  end

  def party_build_service
    @party_build_service
  end

  def build_in_dungeon(adventure, floor)
    party = party_build_service.build_in_dungeon!(adventure, floor)
  end

  REVOKE_LIMIT = 15
  def write_to_bounty_list!(adventurer_id, day_count)
    limit_day = day_count + REVOKE_LIMIT
    @bounty_list[limit_day] ||= []
    @bounty_list[limit_day] << adventurer_id
  end

  def revoke_bounties_by_adv(adv_id)
    bounty_list.each do |day, list|
      if list.include?(adv_id)
        @bounty_list[day] = list - [adv_id]
      end
    end
  end

  def revoke_bounties_by_day(day_count)
    return if bounty_list[day_count].nil?
    bounty_list[day_count].each do |adv_id|
      adv = @adventurers[adv_id]
      adv.revoke_bounty!
    end

    bounty_list.delete(day_count)
  end

  def remove_using_name(sold_adventurer)
    @original_using_names[sold_adventurer.race_type].delete(sold_adventurer.name)
  end

  def add_lost_adventurer_ids(adventurer_id)
    return if !(FGO_MODE && @adventurers_list[adventurer_id])
    @lost_adventurer_ids << adventurer_id
  end

  def set_lost_adventurer_ids(lost_adventurer_ids)
    @lost_adventurer_ids = lost_adventurer_ids
  end

  # 冒険者にbackportingかけるならここ
  def back_porting!
#    @adventurers_list.each do |key, adventurer_params|
#      adv = @adventurers[key]
#    end
  end
end
