# coding: utf-8

require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/app/model/dungeon/monster'
require_rrk 'src/app/helper/game_define_helper'

require_rrk 'src/app/helper/forwardable'

class MonsterParty
  extend Forwardable

  attr_reader :rest_enemies_count, :initial_enemies_count, :monster,
              :bad_status, :bad_sex_status, :skills,
              :is_beast, :is_tentacle, :is_dirty,
              :betrayed_servant, :in_darkness, :in_silence

              include MonsterListByType
  def_delegators :@monster, :bad_status_in_dive

  include GameDefineHelper

  def initialize(params)
    @rest_enemies_count = 0
    @betrayed_servant = nil

    @in_darkness = params[:in_darkness]
    @in_silence = params[:in_silence]
  end

  ENEMY_BOOST_RATE = 1.5 # ココを増やすとお手軽にデバッグ
  def generate_enemies!(party, dungeon_master, betrayed_servant)
    if betrayed_servant
      @betrayed_servant = betrayed_servant
      @betrayed_servant.start_battle_of_betray_servant!(party.leader.short_name)
    end

    floor_key = party.floor
    enemies =
      floor_enemies_list[floor_key] || floor_enemies_list[1]
    base_enemy_id = enemies[:enemy_id]
    enemy_id = dungeon_master.get_monster_or_alternative(base_enemy_id, floor_key)
    monster = monster_define_list[enemy_id]

    _generate_monster(party, floor_key, monster, enemy_id, dungeon_master)
  end

  TRIAL_BONUS_DAY = 15
  def _generate_day_cost_bonus_rate(floor, day_count)
    rate =
      if floor <= 5
        1.0
      elsif floor <= 20
        1.0 + (day_count * 0.02)
      else
        1.0 + (day_count * 0.04)
      end

    if !TRIAL_MODE || (day_count <= TRIAL_BONUS_DAY)
      return rate
    end

    # 体験版ならば一定日数後にボーナスが付く。ただしキャップ付き
    day_bonus_count = [(day_count - TRIAL_BONUS_DAY), 7].min
    return (rate + (day_bonus_count * 0.12))
  end

  def decrease_rest_enemies_count!(count)
    @rest_enemies_count -= count
  end

  def calculate_exp_point(decrease_monsters_count)
    (@monster.exp_point * decrease_monsters_count).to_i
  end

  # 全滅？
  def all_down?
    (@rest_enemies_count <= 0)
  end

  BETRAYED_SERVANT_GUARD_BONUS_RATE = 0.7
  def calculate_decrease_monsters_by(base_fire_power, is_wizard)
    if _is_active_of_betrayed_servant
      fire_power = base_fire_power / 2
      fire_power_for_betrayed = ((base_fire_power - fire_power) * BETRAYED_SERVANT_GUARD_BONUS_RATE).to_i

      @betrayed_servant.get_damage!(fire_power_for_betrayed)
      if betrayed_servant_is_down?
        @betrayed_servant.set_battle_status_with_down!
      end
    else
      fire_power = base_fire_power
    end

    fire_power *= 0.5 if @in_darkness
    fire_power *= 0.5 if @in_silince && is_wizard

    decrease_count = (fire_power / monster.life_point)
    decrease_count.to_i
  end

  def has_active_betrayed_servant?
    @betrayed_servant && !@betrayed_servant.is_down?
  end

  def betrayed_servant_is_down?
    @betrayed_servant && @betrayed_servant.is_down?
  end

  # 地下牢とかに配属されたので抜けるよ
  def detach_betrayed_servant!
    @betrayed_servant.unset_training_in_jail_as_betrayed!
    @betrayed_servant = nil
  end

  def _is_active_of_betrayed_servant
    @betrayed_servant && !@betrayed_servant.is_down?
  end

  def monster_name
    monster.name
  end

  def unit_name
    monster.unit_name
  end

  def exp_point
    monster.exp_point
  end

  def attack_power(turn_count)
    turn_bonus =
      if (turn_count == 1)
        0.5 # ２発目までで撃破できれば軽傷
      else
        1.0 + ((turn_count -2) * 0.1)
      end

    turn_bonus = [turn_bonus, 2.0].min
    (_base_attack_power * turn_bonus * _penalty_in_jail)
  end

  def _base_attack_power
    base_power = @rest_enemies_count * power
    base_power
  end

  def betrayed_servant_attack
    return unless _is_active_of_betrayed_servant

    betrayed_servant.fire_power
  end

  # 継承して使ってね
  def _penalty_in_jail
    1.0
  end

  def capture_count(active_level)
    # 生き残ったパーティが足止めできるので残りが捕まえに行く
    @rest_enemies_count - (active_level / power)
  end

  def has_skill?(skill_name)
    @skills.include?(skill_name)
  end

  def power
    monster.power
  end

  # 最小値
  MIN_COST_10F = 120
  MIN_COST_0F  = 30

  MIN_COST_GROWTH_RATE = 1
  MIN_COST_FLOOR_BORDER_HIGH = 30
  MIN_COST_FLOOR_BORDER_LOW  = 10
  MIN_COST_FLOOR_BORDER_5F   = 5
  def monster_min_cost_by(floor)
    if floor < MIN_COST_FLOOR_BORDER_LOW
      base_cost = MIN_COST_0F + (floor * 4)
      law_floor_cost_bonus =
        if floor > MIN_COST_FLOOR_BORDER_5F
          ((floor - MIN_COST_FLOOR_BORDER_5F) * 10)
        else
          0
        end

      return (base_cost + law_floor_cost_bonus).to_i
    end

    base_cost =
      MIN_COST_10F +
        (((floor - MIN_COST_FLOOR_BORDER_LOW) ** 2) * MIN_COST_GROWTH_RATE)

    high_floor_cost_bonus =
      if floor > MIN_COST_FLOOR_BORDER_HIGH
        (floor - MIN_COST_FLOOR_BORDER_HIGH) ** 2 * 15.0
      else
        0
      end
    return (base_cost + high_floor_cost_bonus).to_i
  end

  # 最大値
  MAX_COST_10F = 240.0
  MAX_COST_0F  = 50

  MAX_COST_LOWER_GROWTH_RATE  = 6
  MAX_COST_HIGHER_GROWTH_RATE = 4.2
  MAX_COST_FLOOR_BORDER_HIGH  = 30
  MAX_COST_FLOOR_BORDER_LOW   = 10
  MAX_COST_FLOOR_BORDER_5F    = 5

  def monster_max_cost_by(floor)
    if floor < MAX_COST_FLOOR_BORDER_LOW
      base_cost = MAX_COST_0F + (floor * MAX_COST_LOWER_GROWTH_RATE)
      law_floor_cost_bonus =
        if floor > MAX_COST_FLOOR_BORDER_5F
          ((floor - MAX_COST_FLOOR_BORDER_5F) * 34)
        else
          0
        end

      return (base_cost + law_floor_cost_bonus).to_i
    end

    base_cost =
      MAX_COST_10F +
        (((floor - MAX_COST_FLOOR_BORDER_LOW) ** 2) * MAX_COST_HIGHER_GROWTH_RATE)

    high_floor_cost_bonus =
      if floor > MAX_COST_FLOOR_BORDER_HIGH
        ((floor - MAX_COST_FLOOR_BORDER_HIGH) ** 2) * 13.0
      else
        0
      end
    return (base_cost + high_floor_cost_bonus).to_i
  end

  ADJUST_RATE = 1.0
  def _calculate_cost(monster)
    power = monster[:power]
    life_point = monster[:life_point]
    total_fight_power = power * life_point
    sqrt_power = total_fight_power ** 0.5
    diff_power = total_fight_power - sqrt_power

    compensation = 0
    compensation, diff_power = _split_min_power(compensation, diff_power, 5, 0.9) #  0 ~ 5
    compensation, diff_power = _split_min_power(compensation, diff_power, 5, 0.8) #  5 ~ 10
    compensation, diff_power = _split_min_power(compensation, diff_power, 10, 0.7) # 10 ~ 20
    compensation, diff_power = _split_min_power(compensation, diff_power, 20, 0.6) # 20 ~ 50
    compensation, diff_power = _split_min_power(compensation, diff_power, 30, 0.5) # 20 ~ 50
    compensation, diff_power = _split_min_power(compensation, diff_power, 4_000, 0.4) # 実質無限

    bonus_rate = monster[:cost_bonus_rate] || 1.0 # スキルとかがあるとコストが上がる

    (sqrt_power + compensation) * bonus_rate * ADJUST_RATE
  end

  def _split_min_power(compensation, diff_power, value_point, rate)
    return compensation, diff_power if  diff_power <= 0
    compensation += ([diff_power, value_point].min * rate)
    diff_power -=  value_point
    return compensation, diff_power
  end

  def summon_guards!(party, dungeon_master)
    floor_key = party.floor
    monster_key = dungeon_master.deep_floor_guards[floor_key].pop
    monster = guard_monster_define_list[monster_key]

    _generate_monster(party, floor_key, monster, monster_key, dungeon_master)
  end

  def _generate_monster(party, floor_key, monster, enemy_id, dungeon_master)
    cost = _calculate_cost(monster)
    @monster = Monster.new({
                             id: enemy_id,
                             name: monster[:name],
                             power: monster[:power],
                             life_point: monster[:life_point],
                             unit_name: monster[:unit_name],
                             exp_point: cost,
                             cost: cost,
                             skills: monster[:skills],
                             bad_status: monster[:bad_status] || {},
                             bad_status_in_dive: (monster[:bad_status_in_dive] || {}),
                             bad_sex_status: monster[:bad_sex_status] || {},
                             skill_personality: monster[:skill_personality] || {},
                             skill_holic: monster[:skill_holic] || {},
                             is_beast: monster[:is_beast],
                             is_tentacle: monster[:is_tentacle],
                             is_dirty: monster[:is_dirty],
                           })

    if floor_key <= 2 && party && party.fresh_noob_party?
      # 全員Lv1パーティーでかつ２階までなら７匹以下のモンスターしか出ない
      enemy_number = rand(1..3)
    else
      min_cost = monster_min_cost_by(floor_key)
      max_cost = monster_max_cost_by(floor_key)
      day_cost_bonus_rate = _generate_day_cost_bonus_rate(floor_key, dungeon_master.hour_glass.day)
      maternity_bonus_cost = dungeon_master.maternity_bonus_rate
      if (min_cost > max_cost)
        middle_value = (min_cost +  max_cost) / 2
        min_cost = middle_value
        max_cost = middle_value
      end

      enemy_cost = (rand(min_cost..max_cost) * ENEMY_BOOST_RATE) * day_cost_bonus_rate * maternity_bonus_cost
      enemy_number = (enemy_cost.to_f / cost).ceil
    end

    @rest_enemies_count = enemy_number
    @initial_enemies_count = enemy_number # セリフ表示に必要
    @skills = monster[:skills] || []
    @bad_status = monster[:bad_status] || {}
    @bad_sex_status = monster[:bad_sex_status] || {}
    @is_beast = monster[:is_beast] || false
    @is_tentacle = monster[:is_tentacle] || false
    @is_dirty = monster[:is_dirty] || false

    @monster
  end
end
