# coding: utf-8
require_rrk 'src/app/model/save/save_data'

class SaveSecretary
  def list_saves_for_load
    _list_saves_common('load_from_')
  end

  def list_saves_for_save
    _list_saves_common('save_to_')
  end

  def _list_saves_common(prefix)
    button_elements =
      ('0'..'9').to_a.map do |save_number|
        save_file_path = _make_save_file_path(save_number)
        if File.exist?(save_file_path)
          # セーブファイルがあるときの処理
          access_time = File.mtime(save_file_path).strftime("%Y-%m-%d %H:%M:%S")
          save_label = "#{save_number}： セーブデータ#{save_number}  (#{access_time})"
          {
            'type' => 'button',
            'text' => save_label,
            'action' => prefix + save_number.to_s,
          }
        else
          # セーブファイルがないときの処理
          {
            'type' => 'button',
            'text' => "#{save_number}： セーブデータがありません",
            'action' => 'no_save',
          }
        end
      end

    button_elements
  end

  def load_from_(slot_number)
    file_path = _make_save_file_path(slot_number)
    save_data = nil
    File.open(file_path, 'r') do |in_f|
      save_data = YAML.load(in_f)
      save_data = _load_from(save_data)
    end

    return save_data
  end

  def save_to_(slot_number, save_data)
    save_file_path = _make_save_file_path(slot_number)
    save_file = _make_save_data(save_data)

    File.open(save_file_path, 'w:UTF-8') do |out_f|
      out_f.write save_file.to_yaml
    end

    puts "have saved in #{slot_number}"
  end

  def _make_save_file_path(save_number)
    File.join(File.absolute_path("."), 'saves', "save#{save_number}.save")
  end

  def _load_from(save_data)
    # save_dataからデータをパースする

    {
      'version' => save_data.version,
      'game_master' => save_data.game_master,
      'bard' => save_data.bard,
      'hour_glass' => save_data.hour_glass,
      'version_data' => save_data.version_data,
    }
  end

  def _make_save_data(params)
    # セーブ用にデータ(save_data)を作る
    save_data = SaveData.new(params)
    save_data.set_version!
    save_data
  end

  def write_adventure_log(bard)
    now_time = Time.now.strftime('%Y-%m-%d_%H_%M_%S')

    save_file_path = _make_adventure_log_file_path(now_time)
    save_file = _make_adventure_log(bard)

    File.open(save_file_path, 'w:UTF-8') do |out_f|
      out_f.write save_file.to_yaml
    end
  end

  def _make_adventure_log(bard)
    bard.this_turn_adventure_in_dungeon + (
      bard.bard.this_turn_adventure_element_in_jail[:jail_elements][:progress_script] +
        bard.bard.this_turn_adventure_element_in_jail[:jail_elements][:prelude_script]
      ) +
      bard.this_turn_adventure_in_midnight
  end

  def _make_adventure_log_file_path(now_time)
    File.join(File.absolute_path("."), 'logs', "adventure_log_#{now_time}.log")
  end
end
