# coding: utf-8
require_rrk 'src/game_define/rrk/module/method_prefix_helper'
require_rrk 'src/game_define/rrk/slave_defines_selector'

class MapMenuService
  include MethodPrefixDefinesHelper
  include SlaveDefineSelector

  def show_slaves_in_map_menu(house, member_list, slave_cage_history_keys, master)
    slave_keys_in_field = _make_slave_keys_in_field(member_list, slave_cage_history_keys, master)

    menu_elements = []
    slave_keys_in_field.each do |slave_key|
      slave_data = SLAVE_LIST[slave_key]

      if slave_data
        menu_elements << {
          'type'   => 'button',
          'text'   => slave_data['short_name'],
          'action' => SHOW_SLAVE_IN_MAP_PREFIX + slave_key.to_s,
        }
      end
    end

    menu_elements <<
      {
        'type'   => 'button',
        'text'   => '戻る',
        'action' => 'show_search_slave_menu',
      }
    menu_elements
  end

  include ViewSizeHelper
  def show_slave_in_map_menu_button(slave_id, slave_data)
    [
      {
      'type'   => 'button',
      'text'   => slave_data['short_name'] + 'を罠にはめる',
      'action' => CAPTURE_SLAVE_IN_MAP_PREFIX + slave_id.to_s,
      'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
    ]
  end

  def _make_slave_keys_in_field(member_list, slave_cage_history_keys, master)
    member_keys = member_list.keys - slave_cage_history_keys['history'] # すでに捕獲済みは除く

    member_keys_in_field = []
    member_keys.each do |member_key|
      member = member_list[member_key]
      if member['restrict'].nil?
        # ボタン作る
        member_keys_in_field << member_key
      else
        # 以下の条件全て満たす場合はボタン作る
        if _have_slaves_history?(member['restrict']['have_slaves_history'], slave_cage_history_keys['history'])
          # ボタン作る
          member_keys_in_field << member_key
        elsif member['restrict']['have_confessed_slaves'] && _slave_is_confessed?(member_key, member['restrict']['have_confessed_slaves'], master)
          # ボタン作る
          member_keys_in_field << member_key
        elsif member['restrict']['have_confessed_slaves_any'] && _slave_is_confessed_any?(member_key, member['restrict']['have_confessed_slaves_any'], master)
          # ボタン作る
          member_keys_in_field << member_key
        end
      end
    end
    member_keys_in_field
  end

  def _have_slaves_history?(have_slaves_history, slave_history_keys)
    return false if have_slaves_history.nil?

    if have_slaves_history == (have_slaves_history && slave_history_keys)
      true # 指定の奴隷が全て在籍していた
    else
      false
    end
  end

  def _slave_is_confessed?(member_key, restricts, master)
    return false if restricts.nil?
    return false unless master.confessed_slave_ids
    master.confessed_slave_ids.include?(member_key)
  end

  def _slave_is_confessed_any?(member_key, restricts, master)
    return false if restricts.nil?
    return false unless master.confessed_slave_ids
    master.confessed_slave_ids.include?(member_key)
  end
end
