# coding: utf-8
%w(train_defines).each do |src_name|
  require_rrk 'src/game_define/rrk/' + src_name
end
# train-groupの定義
require_rrk 'src/app/chain_engine_helper/define/train_menu_helper'
# prefixの定義
require_rrk 'src/game_define/rrk/module/method_prefix_helper'

# turn > round > step という構造
class TrainMenuService
  attr_accessor :round_available_group_menu
  include TrainMenu # train-groupの定義
  include MethodPrefixDefinesHelper

  def initialize *args
    @train_definitions = TrainDefines::TRAIN_DEFINITIONS
    @round_available_train =
      {
        'group_menu' => {},
        'commands'   => [],
      }
    @round_available_group_menu = []
  end

  # 最初にラウンドを始めた時専用
  def in_start_round(slave, house)
    in_finish_step(slave, house)
  end

  def in_finish_step(slave, house)
    @round_available_train = _make_available_commands(slave, house)
    @round_available_group_menu = _round_available_group_menus
  end

  def show_train_report
    [
      {
        'type' => 'button',
        'text' => '調教を終える',
        'action' => 'close_train_report',
      },
    ]
  end

  def command_group_menu
    @round_available_group_menu +
      [
        {
         'type' => 'button',
         'text' => '調教を終える',
         'action' => 'finish_train',
       },
      ]
  end

  def _round_available_group_menus
    @round_available_train['group_menu'].map do |group_menu_key|
      train_menu = TrainGroupList[group_menu_key]
      train_menu.merge!(
        {
          'action' => SELECT_TRAINING_GROUP_PREFIX + group_menu_key,
          'type'   => 'button',
        }
      )
      train_menu
    end
  end

  def round_available_commands(train_group)
    @round_available_train['commands'][train_group].map do |train_key, train_value|
      {
        'text'   => train_value['name'],
        'action' => DO_TRAIN_PREFIX + train_key.to_s,
        'type'   => 'button',
      }
    end
  end

  # 前回コマンド終了時に実行
  # コマンド選択時に選択可能なコマンドを列挙
  def _make_available_commands(slave, house)
    available_commands = {}
    available_categories = []

    @train_definitions.each do |train_no, train|
      train_restrict = train['restrict']
      if (
      _accept_train_restrict_lesser?(slave, train_restrict['lesser']) &&
        _accept_train_restrict_greater?(slave, train_restrict['greater']) &&
        _accept_train_restrict_have_item?(house, train_restrict['have_item']) &&
        _accept_train_restrict_of_equip?(slave, train_restrict['on_equip']) &&
        _accept_train_restrict_on_situation?(slave, train_restrict['on_situation']) &&
        _accept_train_restrict_on_status?(slave, train_restrict['on_status'])
      )
        # 制約条件を突破
        available_commands[train['category']] ||= {}
        available_commands[train['category']][train_no] = train

        unless available_categories.include?(train['category'])
          available_categories << train['category']
        end
      end
    end

    {
      'group_menu' => available_categories,
      'commands'   => available_commands,
    }
  end

  def _accept_train_restrict_lesser?(slave, restrict_lesser)
    return true if restrict_lesser.nil? # 制約なしなら素通し

    restrict_lesser.all? do |param_key, value|
      slave.send(param_key) < value
    end
  end

  def _accept_train_restrict_greater?(slave, restrict_greater)
    return true if restrict_greater.nil? # 制約なしなら素通し

    restrict_greater.all? do |param_key, value|
      slave.send(param_key) >= value
    end
  end

  def _accept_train_restrict_have_item?(house, restrict_have_item)
    return true if restrict_have_item.nil? # 制約なしなら素通し

    (restrict_have_item == (restrict_have_item & house.train_items))
  end

  def _accept_train_restrict_of_equip?(slave, restrict_of_equip)
    return true if restrict_of_equip.nil? # 制約なしなら素通し

    _accept_train_restrict_of_item_with?(slave, restrict_of_equip['with']) &&
      _accept_train_restrict_of_item_without?(slave, restrict_of_equip['without'])
  end

  def _accept_train_restrict_of_item_with?(slave, restrict_on_equip_with)
    return true if restrict_on_equip_with.nil? # 制約なしなら素通し

    slave.equip_item?(restrict_on_equip_with)
  end

  def _accept_train_restrict_of_item_without?(slave, restrict_on_equip_without)
    return true if restrict_on_equip_without.nil? # 制約なしなら素通し

    !slave.equip_item?(restrict_on_equip_without)
  end

  def _accept_train_restrict_on_situation?(slave, restrict_on_situation)
    return true if restrict_on_situation.nil? # 制約なしなら素通し

    _accept_train_restrict_on_situation_with?(slave, restrict_on_situation['on']) &&
      _accept_train_restrict_on_situation_without?(slave, restrict_on_situation['out'])
  end

  def _accept_train_restrict_on_situation_with?(slave, restrict_on_situation)
    return true if restrict_on_situation.nil? # 制約なしなら素通し

    if restrict_on_situation == 'on_outdoor'
      slave.in_outdoor
    elsif restrict_on_situation == 'on_recording'
      slave.in_recording
    else
      false
    end
  end


  def _accept_train_restrict_on_situation_without?(slave, restrict_on_situation)
    return true if restrict_on_situation.nil? # 制約なしなら素通し

    if restrict_on_situation == 'on_outdoor'
      !(slave.in_outdoor)
    elsif restrict_on_situation == 'on_recording'
      !(slave.in_recording)
    else
      false
    end
  end

  def _accept_train_restrict_on_status?(slave, restrict_on_status)
    return true if restrict_on_status.nil? # 制約なしなら素通し

    _accept_train_restrict_on_status_with?(slave, restrict_on_status['with']) &&
      _accept_train_restrict_on_status_without?(slave, restrict_on_status['without'])
  end

  def _accept_train_restrict_on_status_with?(slave, restrict_on_status)
    return true if restrict_on_status.nil? # 制約なしなら素通し

    if restrict_on_status == 'hypnosis'
      slave.in_hypnosis?
    else
      false
    end
  end

  def _accept_train_restrict_on_status_without?(slave, restrict_on_status)
    return true if restrict_on_status.nil? # 制約なしなら素通し

    if restrict_on_status == 'on_outdoor'
      !(slave.in_hypnosis?)
    else
      false
    end
  end
end
