# coding: utf-8
require_rrk 'src/app/helper/view_helper'
require_rrk 'src/game_define/rrk/module/method_prefix_helper'

class SlaveSellService
  include ViewHelper
  include MethodPrefixDefinesHelper

  def slaves_on_sell(house, is_day)
    slave_on_sale_elements = []
    slave_ids = house.free_slave_ids(is_day)
    slave_ids.each do |slave_id|
      slave = house.house_cage[slave_id]
      if _on_sell?(slave)
        sell_price = assess_sell_price(slave)
        slave_on_sale_elements <<
            {
                'type' => 'button',
                'text' => "#{slave.name} (#{sell_price})",
                'action' => SELL_DETAIL_SLAVE_PREFIX + slave_id.to_s,
            }
      end
    end
    slave_on_sale_elements
  end

  def slave_detail(slave)
    assess = _assess_slave_selling(slave)

    return assess['description']
  end

  def assess_sell_price(slave)
    assess = _assess_slave_selling(slave)
    assess['price']
  end

  def _assess_slave_selling(slave)
    vanilla_sell_price = _calculate_vanilla_sell_price(slave)
    sell_bonus = _calculate_sell_bonus(slave, vanilla_sell_price)
    sell_bonus_by_level = _calculate_sell_bonus_by_level(slave)
    final_sell_price = (vanilla_sell_price + sell_bonus_by_level + sell_bonus.inject(0) {|sum, n| sum + n['price'] }).to_i

    sell_bonus_by_level_description =
      if sell_bonus_by_level.nonzero?
        [
          "冒険者としての技量評価： #{_add_comma(sell_bonus_by_level)}",
        ]
      else
        []
      end
    description =
      _slave_status_description(slave) +
        [
            "基本売却価格： #{_add_comma(vanilla_sell_price.to_i)}",
        ] +
        sell_bonus_by_level_description +
        sell_bonus.map { |bonus| "#{bonus['description']}" } +
        [
            "最終売却価格： #{_add_comma(final_sell_price)}",
        ]
    {
        'price' => final_sell_price,
        'description' => description,
    }
  end

  def _slave_status_description(slave)
    scripts = [
      '「売るのはこいつかい？」',
      '奴隷商人は' + slave.short_name + 'を値踏みするようにジロジロと見ている',
      ' ',
    ]

    evaluation_comment = []
    evaluation_comment +=
      if slave.lust >= 600
        ['「ヒヒヒ、男なしじゃいられない、って顔をしているぜ」']
      elsif slave.lust >= 500
        ['「ヒヒヒ、肉の疼きが抑えられない、って顔をしているぜ」']
      elsif slave.lust >= 400
        ['「ヒヒヒ、いい仕上がりじゃねえか」']
      else
        []
      end

    evaluation_comment +=
      if slave.obedience >= 600
        ['「男には絶対服従って顔だな」']
      elsif slave.obedience >= 500
        ['「オラッ、もっと顔をよく見せろよ。  ………この媚びた顔、いい感じだぜ。」']
      elsif slave.obedience >= 400
        ['「よく躾けられているじゃねぇか」']
      else
        []
      end

    unless evaluation_comment.empty?
      scripts += (evaluation_comment + [' '])
    end

    slave_param_sum = slave.lust + slave.obedience + slave.submit
    scripts +=
      if slave_param_sum > 1800
        [
          '「この男に媚びた顔と絶品の肉の仕上がり。ヒヒヒ、まさに肉奴隷って顔だな。高く買わせてもらうぜ。」',
          '奴隷商はニヤニヤと笑いながら' + slave.short_name + 'を目でねぶっている。' +
            '商人としての理性で外見には出さぬよう表情を押さえつけているようだが、股間は焚き付けられたように猛っていた。' +
            '売り払われるまでの間に味見をしたとしても、高く売ってくれるなら自分には関係のないことだ。あなたはそう思った。',
        ]
      elsif slave_param_sum > 1500
        [
          '「ヒヒヒ、いいねえ、この仕上がり。まさに身の程をわきまえさせられた奴隷ってやつだね。」',
          '奴隷商が肉体をまさぐるたびに甘い声で鳴く' + slave.short_name + 'に、相手もあてられたようだ',
          '「ああ任せてくれよ、いい値で買い取るぜ。買い手がつくのが楽しみだ」',
          '奴隷商の好色な視線が' + slave.short_name + 'に注がれていた。買い手がつく前に何度「味見」をされるだろうか。' +
            slave.short_name + 'にとってのこれからの人生を思えば、地獄の始まりに過ぎないわけだが。',
        ]
      elsif slave_param_sum > 1200
        [
          '「へへへ、いい仕上がりだぜ。その腕、頼りにしてるぜ？」',
          '奴隷商は' + slave.short_name + 'をなで回しながら頭の中でいくらで売れるか考えているようだ',
          '「なにしろ俺たちは持ちつ持たれつなんだからよ」',
        ]
      else
        [
          '「うーん、調教がイマイチかな。再調教の手間賃は引いた額になるぜ？」',
          '奴隷商は渋い顔をしながらそろばんを弾いている ',
        ]
      end

    scripts
  end

  def _calculate_vanilla_sell_price(slave)
    lust_point = (slave.lust - 300) - [((slave.lust - 600).to_f * 2.0 / 3.0).to_i, 0].max
    obedience_point = (slave.obedience - 400) - [((slave.obedience - 800).to_f * 1.5 / 3.0).to_i].max
    submit_point = (slave.submit - 350) - [((slave.submit - 600).to_f * 2.5 / 3.0).to_i].max

    slave.sell_price_bonus.to_f *
    ((lust_point * 0.65) +
        (obedience_point * 0.9) +
        (submit_point * 0.75))
  end

  def _calculate_sell_bonus(slave, vanilla_sell_price)
    bonus = []

    if slave.has_talent?('hooters')
      bonus_price = (vanilla_sell_price.to_f * 0.15).to_i
      bonus << {
        'price' => bonus_price,
        'description' => "巨乳 (＋ #{_add_comma(bonus_price)})",
      }
    end

    return bonus

    if slave.virgin?
      bonus_price = (vanilla_sell_price.to_f * 0.7).to_i
      bonus << {
          'price' => bonus_price,
          'description' => "処女 (＋ #{_add_comma(bonus_price)})",
      }
    end
    bonus
  end

  def _calculate_sell_bonus_by_level(slave)
    return 0 if slave.max_level <= 10

    bonus_max_level = slave.max_level - 10
    bonus_level = slave.level - 10

    ((bonus_max_level * bonus_max_level * 50) + (bonus_level * bonus_level * 30)).to_i
  end

  def _on_sell?(slave)
    slave.lust > 350 &&
        slave.obedience > 250 &&
        slave.submit > 250 &&
      !(slave.forbid_sell_flag)
  end
end
