# coding: utf-8

require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/service/dungeon/progress/battle_service'
require_rrk 'src/defines/script/dungeon/progress_script'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/app/service/dungeon/progress/trap_service'
require_rrk 'src/app/util/debug_util'
require_rrk 'src/defines/dungeon/utils/version_helper'

class DungeonProgressService
  include DungeonCommon
  include DebugUtil
  include VersionHelper

  def initialize(bar, dungeon_master)
    @battle_service = DungeonProgressBattleService.new(bar, dungeon_master)
    @trap_service = TrapService.new
    @progress_script = ProgressScript.new
  end

  def _sort_by_floor(parties)
    party_by_floor = {}
    parties.values.each do|party|
      party_by_floor[party.floor] ||= []
      party_by_floor[party.floor] << party
    end
    sorted_parties = {}
    party_by_floor.values.flatten.each do |party|
      sorted_parties[party.identify_code] = party
    end
    sorted_parties
  end

  def shuffle_for_traps_n_betrays(dungeon_master, parties)
    # 最初に物理トラップ
    shuffle_box_for_trap = []
    # 次にエロトラップ
    shuffle_box_for_ero_trap = []
    # 次にプールトラップ
    shuffle_box_for_pool_trap = []
    # 最後に洗脳サーヴァント
    shuffle_box_for_betray_servant = []
    # ついでにエロ回復スライム
    shuffle_box_for_recover_slime = []

    parties.each do |_, party|
      next unless party.in_dungeon? # 地上なら無視

      if party.in_distress?
        # 物理：遭難中ならさっさとお帰りいただく
        shuffle_box_for_trap << party.identify_code
        # エロ：
        shuffle_box_for_ero_trap += ([party.identify_code] * 1)
        # プール：遭難中だとそれなりに
        shuffle_box_for_pool_trap += ([party.identify_code] * 2)
        # エロ回復スライム：遭難中は重点
        shuffle_box_for_recover_slime += ([party.identify_code] * 3)

        unless party.only_n_brain_washing? # 洗脳オンリーだと見逃す
          # 洗脳済みサーヴァント：遭難中は比較的見逃しがち
          shuffle_box_for_betray_servant += ([party.identify_code] * 1)
        end
      elsif party.lonely?
        # 物理：孤立中も比較的はよ帰って欲しい
        shuffle_box_for_trap << party.identify_code
        # エロ：
        shuffle_box_for_ero_trap += ([party.identify_code] * 1)
        # プール：孤立中もそれなりに
        shuffle_box_for_pool_trap += ([party.identify_code] * 2)
        # エロ回復スライム：孤立中は重点
        shuffle_box_for_recover_slime += ([party.identify_code] * 3)
        unless party.only_n_brain_washing? # 洗脳オンリーだと見逃す
          # 洗脳済みサーヴァント：遭難中は比較的見逃しがち
          shuffle_box_for_betray_servant += ([party.identify_code] * 1)
        end
      else
        if party.scout_is_active?
          # 物理：スカウト健在なら比較的甘く
          shuffle_box_for_trap += ([party.identify_code] * 2)
          # エロ：
          shuffle_box_for_ero_trap += ([party.identify_code] * 1)
          # プール：スカウト健在なら比較的甘く
          shuffle_box_for_pool_trap += ([party.identify_code] * 2)
        else
          # 物理：そうでないなら集中的に
          shuffle_box_for_trap += ([party.identify_code] * 4)
          # エロ：スカウト不在だと厳しい
          shuffle_box_for_ero_trap += ([party.identify_code] * 2)
          # プール：スカウト不在だと厳しい
          shuffle_box_for_pool_trap += ([party.identify_code] * 4)
        end

        # エロ回復スライム：健在な人が多いと割と見逃しがち
        shuffle_box_for_recover_slime += ([party.identify_code] * 1)

        if party.all_brain_washing?
          # 洗脳された人間のみだと見逃す
        elsif party.is_all_down?
          # 洗脳済みサーヴァント：全滅済みは見逃しがち
          shuffle_box_for_betray_servant += ([party.identify_code] * 2)
        else
          # 洗脳済みサーヴァント：集中してねらう
          shuffle_box_for_betray_servant += ([party.identify_code] * 3)
        end
      end
    end

    [shuffle_box_for_trap, shuffle_box_for_ero_trap, shuffle_box_for_pool_trap, shuffle_box_for_betray_servant, shuffle_box_for_recover_slime]
  end

  def adventure_progress_by(party, jail, game_master, party_id, result, trap_result)
    dungeon_master = game_master.dungeon_master

    events = []
    if party.inactive?
      result[:inactive_parties] << party_id
      rest_in_bar = {
        party: party,
      }
      event = ['rest_in_bar', rest_in_bar]
      events << event
      return events
    end

    adventurers_size_in_start = party.adventurers.size

    result_by_party = {
      break_party: false,
      engage_jail: false,
      hit_trap: false,
      battle_enemy: false,
      no_action: false,
    }

    party.floor_in_turn_start!
    floor_in_start = party.floor
    turn_start = {
      leader: party.leader,
      is_lonely: party.is_only?,
      turn_count: party.turn_count,
      positiveness: party.leader.positiveness,
      floor: floor_in_start,
      in_dive: party.in_dive?,
      in_ground: party.in_ground?,
    }
    events << ['turn_start_text', turn_start]

    if party.in_dungeon?
      # 探索前にエロ回復スライム判定
      # 終了時だとだいたい全滅している
      ero_damage_from_recover_slime_result = @trap_service.ero_damage_from_recover_slime_result_if(party)
      events << ['ero_recover_slime_drain', ero_damage_from_recover_slime_result] unless ero_damage_from_recover_slime_result.empty?

      # トラップ判定
      trap_event = _get_trap(party, trap_result, game_master)
      events << trap_event unless trap_event.nil?

      # 遭難中なら探索前に淫魔の抜け穴
      # 終了時だとだいたい全滅している
      # 遭難パーティーの場合、確定ペナルティのかわりに淫魔の抜け穴を使って進撃できる。
      if (DUNGEON_VERSION >= 8.0) &&
        !party.in_ground? &&
        party.in_distress? &&
        lonely_progress_by_tunnel?(party)

        # 1.ペナルティ
        ero_tunnel_params = @battle_service.ero_tunnel_params(party)
        events << ['ero_tunnel', ero_tunnel_params]
        # 2.念のため撤退行動へ
        party.decide_to_return! if party.in_dive?
        # 3.進撃
        party.progress_floor!
      end

      # 戦闘判定
      events += @battle_service.event_with_message(party, jail, dungeon_master, result_by_party)
    end

    events += progress_floor_recursive(party, jail, dungeon_master, result_by_party)

    # 探索が終わるとエロ装備判定
    ero_damage_from_equip_result = @trap_service.ero_damage_from_equip_if(party)
    events << ['ero_equip_damage', ero_damage_from_equip_result] unless ero_damage_from_equip_result.empty?

    floor_in_end = party.floor

    progress_in_turn_end_event = @battle_service.progress_message_in_turn_end(party, floor_in_start, floor_in_end)
    if progress_in_turn_end_event.nil?
      events << ['party_is_all_down_maybe']
    else
      events << progress_in_turn_end_event
    end
    party.floor_in_turn_end!

    adventurers_size_in_end = party.adventurers.size

    if adventurers_size_in_start != adventurers_size_in_end
      result[:break_party] << party_id
    else
      if result_by_party[:engage_jails]
        result[:engage_jails_journey] << party_id
      elsif result_by_party[:hit_trap]
        result[:hit_trap_journey] << party_id
      elsif result_by_party[:battle_enemy]
        result[:battle_journey] << party_id
      else
        result[:simple_journey] << party_id
      end
    end

    events
  end

  def _get_trap(party, trap_result, game_master)
    trap_no = trap_result[party.identify_code]
    return nil unless trap_no

    event = @trap_service.hit_traps_by_party_if(party, trap_no, game_master)
    return event
  end

  def progress_floor_recursive(party, jail, dungeon_master, result_by_party)
    return [] if party.inactive?
    return [] if party.is_blank_members?

    events = []

    if party.in_ground? && !party.in_dive?
      # 撤退成功。地上に着きました。
      get_ground_params = {
        party: party,
        is_only: party.is_only?,
      }
      get_ground_event = ['get_ground', get_ground_params]
      events << get_ground_event
      if party.bad_status_or_equip_in_dive?
        target_adventurers = {}
        target_adventurers[:ero_equip]  = party.ero_equip_adventurers
        target_adventurers[:bad_status] = party.bad_status_adventurers

        church_event = ['church_purification', target_adventurers]
        events << church_event
      end
      party.each_clear_status_in_dive!
      # 地上脱出時の諸々クリア処理
      party.each_clear_in_exit

      status_up_params = party.each_level_up!

      party.inactivate!
      status_up_event = ['party_status_up', status_up_params]
      events << status_up_event
      return events
    end

    # そろそろ引き返す？の判断
    if party.in_dive? && party.decide_to_return?
      party.decide_to_return!
      party.flash_return_in_this_turn! # 本来はstatusを返すが・・・
      return_decide_params = {
        leader: party.leader,
        is_only: party.is_only?,
      }
      event = ['return_decide', return_decide_params]
      events << event
      if party.turn_count == 1
        # 日帰り防止のためここで打ち止め
        return events
      end
    end

    if _use_teleport_pass_for_rescue?(party, jail)
      target_floor = jail.get_floor_by_prisoner_id(party.rescue_target)
      teleport_target_floor = target_floor - (target_floor % 10)
      # 救援中なので最も近い階層へ
      floors_by_pass = party.adventurers.values.map {|adv|
        pass = adv.last_teleport_pass
        pass ? _get_floor_by_pass(pass) : nil
      }
      max_floor = floors_by_pass.compact.max # 何かは入るはず

      if teleport_target_floor > max_floor
        teleport_target_floor = max_floor # 希望のフロアまでたどり着けないので、その手前の行けるところまで行く
      end
      party.teleport_to_floor!(teleport_target_floor)

      # テレポート
      teleport_pass_params = {
        leader: party.leader,
        is_lonely: party.is_only?,
        floor: teleport_target_floor,
      }

      puts party.leader.short_name + "隊は救援のため#{teleport_target_floor}階にテレポートした"  if DEBUG_SCRIPT_MODE

      teleport_event = ['use_teleport_pass_for_rescue', teleport_pass_params]
      events << teleport_event
      events += @battle_service.event_with_message(party, jail, dungeon_master, result_by_party)
      events += progress_floor_recursive(party, jail, dungeon_master, result_by_party)
      events
    elsif _use_teleport_pass?(party)
      pass = party.leader.last_teleport_pass
      floor = _get_floor_by_pass(pass)
      party.teleport_to_floor!(floor)

      # テレポート
      teleport_pass_params = {
        leader: party.leader,
        is_lonely: party.is_only?,
        floor: floor,
      }
      teleport_event = ['use_teleport_pass', teleport_pass_params]
      events << teleport_event
      events += @battle_service.event_with_message(party, jail, dungeon_master, result_by_party)
      events += progress_floor_recursive(party, jail, dungeon_master, result_by_party)
      events
    elsif party_is_progress?(party, jail)
      party.progress_floor!

      if party.in_max_floor? # 最下層だとお宝ゲットチャンス
        sacked_is_success =
          if party.scout_is_active?
            random_roll(33, 100)
          else
            random_roll(10, 100)
          end
        if sacked_is_success
          sacked_money = dungeon_master.sacked_treasure!
          party.get_treasure!(sacked_money)
          puts "#{party.leader.short_name}隊に#{sacked_money}略奪された！" if DEBUG_SCRIPT_MODE

        # お宝ゲット
          get_treasure_params = {
            leader: party.leader,
            is_lonely: party.is_only?,
            money_amount: sacked_money.to_i,
            master_money: dungeon_master.money,
          }
          get_treasure_event = ['get_treasure', get_treasure_params]
          events << get_treasure_event
        end
      end

      pass = _get_teleport_pass(party)
      if pass # 持っていなければワープパスを渡す
        party.leader.add_teleport_pass_inventory!(pass)
        get_teleport_pass_params = {
          leader: party.leader,
          floor: party.floor,
        }
        get_teleport_pass_event = ['get_teleport_pass', get_teleport_pass_params]
        events << get_teleport_pass_event
      end

      # 戦闘判定
      events += @battle_service.event_with_message(party, jail, dungeon_master, result_by_party)
      events += progress_floor_recursive(party, jail, dungeon_master, result_by_party)
      events
    end

    # 進むも戻るもできず、フロアにとどまる

    events
  end

  def _use_teleport_pass_for_rescue?(party, jail)
    if party.in_ground? && party.in_dive? &&
      party.in_rescue? &&
      party.adventurers.values.any? {|adv| adv.has_any_teleport_pass? }
      target_floor = jail.get_floor_by_prisoner_id(party.rescue_target)

      return (target_floor && (target_floor >= 10))
    else
      false
    end
  end

  def _use_teleport_pass?(party)
    party.in_ground? && party.in_dive? && party.leader.has_any_teleport_pass?
  end

  TELEPORT_PASS_PREFIX = 'teleport_pass'
  def _get_teleport_pass(party)
    return nil unless (party.in_dive? && party.scout)
    return nil unless (party.floor % 10 == 0) # 10Fごとに発行
    return nil if (party.floor > 30) # パスが出るのは30Fまで
    pass = "#{TELEPORT_PASS_PREFIX}#{party.floor}"
    return nil if party.leader.has_item?(pass)
    pass
  end

  def _get_floor_by_pass(pass_item)
    floor_str = pass_item.sub(TELEPORT_PASS_PREFIX, '')
    floor_str.to_i
  end

  def lonely_progress_by_tunnel?(party)
    return false if party.floor <= 1
    return false if party.in_ground?
    return false unless party.in_distress?

    random_roll(30, 100)
  end

  def party_is_progress?(party, jail)
    return true if party.in_ground?
    return true if party.floor == 1 && party.in_dive?

    if party.in_rescue_ready? # 探索待機モード
      floor = jail.get_floor_by_prisoner_id(party.rescue_target)
      short_name = jail.get_short_name_by_prisoner_id(party.rescue_target)
      if floor # 再度捕縛された。救出モードへ
        party.set_rescue_mode(party.rescue_target, short_name)
      else
        if party.has_rescue_ready_turn?
          # 通常探索モードへ
          party.clear_rescue_mode!
        else
          party.decrease_rescue_ready_turn!
        end
      end
    end

    if party.in_rescue?
      floor = jail.get_floor_by_prisoner_id((party.rescue_target))
      if floor.nil?
        # 解放された。救出待機モードへ
        party.set_rescue_ready_mode!
      elsif party.floor == floor
        # とどまり捜索を続ける
        return false
      elsif (party.floor > floor) && party.in_dive?
        # 反転モードへ
        party.decide_to_return_for_rescue!
      elsif (party.floor < floor) && !party.in_dive?
        # 潜行モードへ
        party.decide_to_return_for_rescue!
      end
    end

    max_progress_limit =
      if (party.lonely? || party.is_all_down?)
        30
      elsif party.in_rescue?
        10
      elsif party.in_dive?
        5
      else
        7
      end
    # 日帰りでダンジョンを往復されると困る
    if (party.this_turn_floor_progress >= max_progress_limit)
      return false
    end

    # スキルによる進む力
    progress_point =
      (party.scout_level + party.total_level_by_active) * 2 *
        party.progress_in_up_bonus *
        party.lonely_withdraw_bonus # 孤立退却中だと高速化
    if party.in_rescue?
      progress_point *= 1.8
    end
    # 連続進行によるペナルティ
    progress_penalty =
      if party.in_distress? && !party.in_dive?
        0
      elsif party.floor > party.floor_in_turn
        (party.floor - party.floor_in_turn) * 3
      else
        0
      end

    # リーダーの性格で進行成功率が変化
    progress_point *= (1.0 + (party.leader.positiveness * 0.1))
    # 浅めの階層かつ単独行だとちょっとボーナスつけて早めにお帰りいただく
    progress_point *= 4.0 if (party.in_distress? && !party.in_dive? && party.floor < 5)

    # フロアの難易度
    floor_difficult = party.floor * 10
    # 帰りは速い
    if !party.in_dive?
      progress_point *=
        if (party.lonely? || party.is_all_down?)
          2.5 # 停滞防止
        else
          1.5
        end
    end

    return random_roll((progress_point - progress_penalty), floor_difficult)
  end
end
