# coding: utf-8
require_rrk 'src/app/service/dungeon/module/dungeon_common.rb'
require_rrk 'src/defines/script/dungeon/capture_script.rb'

class CaptureService
  include DungeonCommon

  attr_reader :bar

  def initialize(bar, dungeon_master)
    @script_service = CaptureScript.new
    @dungeon_master = dungeon_master
    @jails = {}
    @bar = bar
  end

  VICTIM_CAPTURE_RATE = 0.15 # すでに倒れているものとPassengerに割り振られる
  def get_capture_count(adventurer, monster_party, party, now_down_count, party_victims_count, is_concentrate_target, first_concentrate_target)
    capture_count = monster_party.capture_count(party.active_member_levels)

    # 戦力が少数の場合全力を一人に投入する
    if is_concentrate_target
      if first_concentrate_target
        return capture_count
      else
        return 0
      end
    end

    if adventurer.has_talent?(:self_sacrifice)
      capture_count *= 1.2
    end

    if party_victims_count > 0
      capture_count *= (1.0 - VICTIM_CAPTURE_RATE)
    end

    capture_count_by_adv = capture_count / now_down_count
    capture_count_by_adv *= 2 if adventurer.is_bounty_target?
    capture_count_by_adv.to_i
  end

  # すでに倒された者と同行者も捕獲される。とりあえず１５％分をわりふる
  def get_downs_n_passenger_capture_count(passenger, monster_party, party_victims_count, is_concentrate_target)
    return 0 if is_concentrate_target

    count = monster_party.rest_enemies_count * VICTIM_CAPTURE_RATE / party_victims_count.to_f
    [count.to_i, 1].max
  end

  ESCAPE_RATE = 4
  LONELY_PENALTY_RATE = 0.3
  BETRAYED_SERVANT_PENALTY_RATE = 0.5
  CAPTURE_RATE = 0.8
  # 1.捕まった
  # 1-1. 捕まったがすぐに脱出に成功した
  # 1-2. 捕まった。脱出判定も失敗した。
  # 2.逃げ出した
  # 2-1. パーティとの合流に成功した
  # 2-2. パーティとの合流に失敗した
  def capture_adventurer_if(adventurer, party, monster_party, capture_count, already_down = false)
    level_point = adventurer.level * ESCAPE_RATE
    level_point *= 1.2 if adventurer.scout?

    # 体力のある洗脳サーヴァントじゃないと働かないよ
    if monster_party.has_active_betrayed_servant?
      betrayed_servant = monster_party.betrayed_servant
    end

    is_captured =
      if capture_count.zero?
        false
      elsif monster_party.has_skill?('indifferent')
        false
      else
        lonely_penalty_n_deep = party.is_only? && party.floor > 5
        escape_point =
          if lonely_penalty_n_deep
            (level_point * LONELY_PENALTY_RATE) # 単独行動だとつかまりやすい
          else
            level_point
          end

        escape_point -= _capture_bonus
        if betrayed_servant # 有力な洗脳サーヴァントがいれば捕獲されやすい
          escape_point *= BETRAYED_SERVANT_PENALTY_RATE
        end

        if escape_point >= capture_count # そのままだと100%逃走可能
          random_roll(8, 100) # 10%くらいは可能性残しておくか
        else
          !(random_roll((escape_point * CAPTURE_RATE).to_i, capture_count))
        end
      end
    is_merged = _is_merged?(level_point, party.active_member_levels, monster_party.rest_enemies_count)

    if is_captured
      betrayed_servant.add_betrayed_capture_status!(adventurer) if betrayed_servant
      # つかまった
      @jails[adventurer.id] = adventurer
      adventurer.status_text = "#{party.leader.short_name}隊で冒険中、#{party.floor}Fで倒された。捕獲された"
      captured = true
      party.withdraw(adventurer.id, captured)
      bar.party_by_adventurer_id.delete(adventurer.id)
      party.unset_betrayed_servant_target!
      adventurer.set_captured!(party.floor, capture_count, monster_party.monster, monster_party.betrayed_servant)

      betrayed_servant = monster_party.betrayed_servant
      if betrayed_servant
        if !(betrayed_servant.is_active? && betrayed_servant.has_betrayed_penis?)
          monster_party.detach_betrayed_servant!
        end
      end

      event_by_adv = already_down ? 'down_adv_is_captured' : 'captured'
    else
      if is_merged
        # 逃げられた、合流できた
        # パーティーが全滅していると無人パーティーに合流することになる。セリフがおかしくなるのでいったん離脱させる。
        captured = false
        adventurer.status_text = "#{party.leader.short_name}隊で冒険中、#{party.floor}Fで倒された。逃走成功、合流成功"
        party.withdraw(adventurer.id, captured)
        bar.party_by_adventurer_id.delete(adventurer.id)
        adventurer.set_to_merge!(party.identify_code, party.floor)
        event_by_adv = already_down ? 'down_adv_is_escape_and_merge' : 'escape_and_merge'
      else
        # 逃げられた、合流できない
        captured = false
        adventurer.status_text = "#{party.leader.short_name}隊で冒険中、#{party.floor}Fで倒された。逃走成功、合流失敗"
        party.withdraw(adventurer.id, captured)
        bar.party_by_adventurer_id.delete(adventurer.id)
        adventurer.set_stray_adventure!(party.floor)
        event_by_adv = already_down ? 'down_adv_is_escape_and_not_merge' : 'escape_and_not_merge'
      end
    end

    adventurer.recover_now_down!
    event_by_adv
  end

  # 1.捕まった
  # 1-1. 捕まったがすぐに脱出に成功した
  # 1-2. 捕まった。脱出判定も失敗した。
  # 2.逃げ出した
  # 2-1. パーティとの合流に成功した => これはない
  # 2-2. パーティとの合流に失敗した
  PASSENGER_DEEP_PENALTY_RATE = 0.7
  def capture_passenger_if(passenger, party, monster_party, capture_count)
    level_point = passenger.level * 7 # passengerなので若干不利目に
    level_point *= 1.2 if passenger.scout?

    # 体力のある洗脳サーヴァントじゃないと働かないよ
    if monster_party.has_active_betrayed_servant?
      betrayed_servant = monster_party.betrayed_servant
    end

    is_captured =
      if capture_count.zero?
        false
      elsif monster_party.has_skill?('indifferent')
        false
      else
        penalty_of_deep = party.floor > 10
        escape_point =
          if penalty_of_deep
            (level_point * PASSENGER_DEEP_PENALTY_RATE) # 同行者が深層にいるとつかまりやすい
          else
            level_point
          end

        escape_point -= _capture_bonus
        if betrayed_servant # 有力な洗脳サーヴァントがいれば捕獲されやすい
          escape_point *= BETRAYED_SERVANT_PENALTY_RATE
        end
        # まだ体力があれば逃げやすい
        escape_point *= 2.0 unless passenger.is_down?

        if escape_point >= capture_count # そのままだと100%逃走可能
          random_roll(8, 100) # 10%くらいは可能性残しておくか
        else
          !(random_roll(escape_point, capture_count))
        end
      end
    if is_captured
      # つかまった
      betrayed_servant.add_betrayed_capture_status!(adventurer) if betrayed_servant
      # つかまった
      @jails[passenger.id] = passenger
      passenger.status_text = "#{party.leader.short_name}隊で冒険中、#{party.floor}Fで倒された。捕獲された"
      party.withdraw_passenger(passenger)
      bar.party_by_adventurer_id.delete(passenger.id)
      passenger.set_captured!(party.floor, capture_count, monster_party.monster, monster_party.betrayed_servant)
      party.unset_betrayed_servant_target!
      event_by_adv = 'passenger_is_captured'
    else
      # 逃げられた、合流できない。passengerは合流できない
      passenger.status_text = "#{party.leader.short_name}隊に随伴中、#{party.floor}Fで倒された。逃走成功"
      party.withdraw_passenger(passenger)
      bar.party_by_adventurer_id.delete(passenger.id)
      passenger.set_stray_adventure!(party.floor)
      event_by_adv = 'passenger_is_escaped_and_not_merge'
    end

    passenger.recover_now_down!
    event_by_adv
  end

  def _is_merged?(level_point, party_active_member_levels, enemies_count)
    random_roll((level_point + (party_active_member_levels * 10)), enemies_count)
  end

  def _capture_bonus
    @dungeon_master.capture_bonus
  end
end
