# coding: utf-8

require_rrk 'src/app/model/dungeon/party'
require_rrk 'src/app/util/debug_util'

class PartyBuildService
  def initialize(bar_party_by_adventurer_id)
    @bar_party_by_adventurer_id = bar_party_by_adventurer_id
  end

  # ダンジョン内ではぐれた冒険者が結成するパーティー
  def build_in_dungeon!(adventurer, floor)
    in_bar = false
    adventurer.initialize_in_join_party! # いろいろクリア
    party = Party.new(adventurer, in_bar, floor) # まずリーダーを配置
    @bar_party_by_adventurer_id[adventurer.id] = party
    party
  end

  def build_parties!(chairs, jail, adventurer_list, new_pot = false)
    parties = {}
    prisoner_ids = jail.get_prisoner_ids_all
    temp_party = nil

    # a.先に救出隊を編成する
    # a.1 救出隊のリーダーを救出対象ごとに選任
    rescue_leaders = {}
    prisoner_ids.each do |prisoner_id|
      leader = chairs.values.find {|adv| adv.is_best_friendly_adventurer?(prisoner_id) }
      rescue_leaders[prisoner_id] = leader if leader
    end

    # a.2 救出隊のリーダーを救出対象ごとに選任
    rescue_leaders.each do |rescue_target_id, leader|
      next if chairs.count < Party::LIMIT_SIZE
      next unless chairs[leader.id] # もうリーダーが配置済み

      rescue_target_adv = adventurer_list[rescue_target_id]

      in_bar = true
      floor = 0
      leader.initialize_in_start_adventure! # 地上編成用。いろいろクリア
      temp_party = Party.new(leader, in_bar, floor) # まずリーダーを配置
      leader.status_text = "#{rescue_target_adv.short_name}救出のために地上でパーティーを結成"

      # ソート済が前提
      chairs.each do |adventurer_id, adventurer|
        break unless _join_party_for_rescue_if_possible(temp_party, chairs, adventurer_id, adventurer, rescue_target_adv)
      end

      if temp_party.full_member?
        puts "#{rescue_target_adv.short_name}救出のために地上でパーティーを編成" if DebugUtil::DEVELOP_PRINT_MODE
        temp_party.adventurers.values.each do |adv|
          chairs.delete(adv.id) # 参加者は椅子から消えてもらう
          @bar_party_by_adventurer_id[adv.id] = temp_party
        end
        parties[temp_party.identify_code] = temp_party
        temp_party.set_rescue_mode(rescue_target_adv.id, rescue_target_adv.short_name)
      end
    end

    # 次に通常パーティー編成
    adventurers_by_leader_aptitude = sort_by_adventurer_leader_aptitude(chairs)
    adventurers_by_leader_aptitude.values.each do |adventurers|
      adventurers.each do |leader|
        next if chairs.count < Party::LIMIT_SIZE
        next unless chairs[leader.id] # もうリーダーが配置済み

        in_bar = true
        floor = 0
        leader.initialize_in_start_adventure! # 地上編成用。いろいろクリア
        temp_party = Party.new(leader, in_bar, floor) # まずリーダーを配置
        leader.status_text = "地上でパーティーを編成"

        leader.friendly_list.each do |friendly_adventurer_id| # 友好キャラがいたら配置
          adventurer = chairs[friendly_adventurer_id]
          _join_party_if_possible(temp_party, chairs, friendly_adventurer_id, adventurer) if (adventurer && !adventurer.in_rest?)
        end
        if temp_party.full_member? # いっぱいならそれで終わり
          parties[temp_party.identify_code] = temp_party
          next
        end

        # ソート済が前提
        chairs.each do |adventurer_id, adventurer|
          break unless _join_party_if_possible(temp_party, chairs, adventurer_id, adventurer)
        end

        if temp_party.full_member?
          temp_party.adventurers.values.each do |adv|
            chairs.delete(adv.id) # 参加者は椅子から消えてもらう
            @bar_party_by_adventurer_id[adv.id] = temp_party
          end
          parties[temp_party.identify_code] = temp_party
        end
      end
    end

    chairs.values.each do |adventurer|
      adventurer.decrease_rest_turns!
    end
    parties
  end

  def _join_party_for_rescue_if_possible(party, chairs, adventurer_id, adventurer, rescue_target_adv)
    return false if party.full_member?
    return false if adventurer.in_rest_for_rescue?(rescue_target_adv.id)
    if adventurer && party.has_empty_position(adventurer.class_type)
      adventurer.initialize_in_start_adventure! # 地上編成用。いろいろクリア
      party.add(chairs[adventurer_id])
      adventurer.status_text = "#{rescue_target_adv.short_name}救援のため#{party.leader.short_name}隊に参加"
    end
    true
  end

  def _join_party_if_possible(party, chairs, adventurer_id, adventurer)
    return false if party.full_member?
    return false if adventurer.in_rest?
    if adventurer && party.has_empty_position(adventurer.class_type)
      adventurer.initialize_in_start_adventure! # 地上編成用。いろいろクリア
      party.add(chairs[adventurer_id])
      adventurer.status_text = "#{party.leader.short_name}隊に参加"
    end
    true
  end

  def sort_by_adventurer_leader_aptitude(chairs)
    adventurers_by_leader_aptitude = {}
    # リーダー適性表作り
    chairs.each do |_key, adventurer|
      next if adventurer.in_rest?
      adventurer_leader_aptitude = adventurer.leader_aptitude
      adventurers_by_leader_aptitude[adventurer_leader_aptitude] ||= []
      adventurers_by_leader_aptitude[adventurer_leader_aptitude] << adventurer
    end

    keys = adventurers_by_leader_aptitude.keys.sort {|value| -1.0 * value } # 降順に並べる
    sorted_adventurers_by_leader_aptitude = {}
    keys.each do |key|
      sorted_adventurers_by_leader_aptitude[key] = adventurers_by_leader_aptitude[key]
    end
    sorted_adventurers_by_leader_aptitude
  end

  def recycle_parties(parties, chairs)
    delete_party_keys = []
    parties.each do |id, party|
      if party.inactive?
        members = party.adventurers
        members.values.each do |adventurer|
          # keyはfighterとかだから使わない
          chairs[adventurer.id] = adventurer
          adventurer.become_rest!
        end
        party.passengers.each do |passenger|
          # keyはfighterとかだから使わない
          chairs[passenger.id] = passenger
          passenger.become_rest!
        end
        delete_party_keys << id
      end
    end

    delete_party_keys.each { |key| parties.delete(key) }
    parties
  end
end
