# coding: utf-8

require_rrk 'src/app/model/dungeon/adventurer'
require_rrk 'src/app/model/dungeon/monster'
require_rrk 'src/app/model/dungeon/monster_party'
require_rrk 'src/app/service/dungeon/module/dungeon_common'
require_rrk 'src/app/service/dungeon/jail/capture_service'
require_rrk 'src/app/service/dungeon/progress/trap_service'
require_rrk 'src/defines/dungeon/skill_list'
require_rrk 'src/defines/script/dungeon/battle_script'
require_rrk 'src/defines/dungeon/skill_list'

class DungeonProgressBattleService
  include DungeonCommon
  include DungeonSkillList

  def initialize(bar, dungeon_master)
    @battle_script = BattleScript.new
    @capture_service = CaptureService.new(bar, dungeon_master)
    @trap_service = TrapService.new
    @adventurers_list = bar.adventurers
  end

  # ターン終了時のメッセージ
  def progress_message_in_turn_end(party, floor_in_start, floor_in_end)
    if party.in_ground? # 地上に着いたら
      return_to_ground_params = {
        leader: party.leader,
        is_only: party.is_only?,
        floor_in_start: floor_in_start,
      }
      return_to_ground_event = ['return_to_ground', return_to_ground_params]
      return return_to_ground_event
    end

    # 全滅した場合空パーティがここに来る
    return nil if party.adventurers.size == 0

    progress_floor_params = {
      leader: party.leader,
      is_only: party.is_only?,
      start_floor: floor_in_start,
      end_floor: floor_in_end,
      floor_max: party.floor_max,
      is_return_in_this_turn: party.return_in_this_turn?,
      is_in_dive: party.in_dive?,
    }
    progress_floor_event = ['progress_floor', progress_floor_params]
    return progress_floor_event
  end

  def _floor_event_about_battle(party, dungeon_master, result_by_party)
    event = nil
    return event if party.inactive?

    progress_result = _progress_event(party)
    @battle_script.progress_script(progress_result, party)

    event = {
      floor: party.floor,
      progress_result: progress_result,
    }
    if progress_result[:fired]
      result_by_party[:merge_enemy] = true
      fire_result = _progress_fire_event(party, dungeon_master, result_by_party)
      event[:fire_result] = fire_result
    elsif (party.in_deep_floor? &&
      dungeon_master.deep_floor_guards[party.floor] &&
      !dungeon_master.deep_floor_guards[party.floor].empty?)
      # 深層階にいてかつ精鋭モンスターが配備されている
      base_engage_rate = 40
      engage_rate = base_engage_rate + (dungeon_master.deep_floor_guards[party.floor].size * 10)
      if random_roll(engage_rate, 100)
        progress_result[:fired] = true
        result_by_party[:merge_enemy] = true
        is_elite = true
        fire_result = _progress_fire_event(party, dungeon_master, result_by_party, is_elite)
        fire_result[:guard_monster] = true
        event[:fire_result] = fire_result
      end
    end
    event
  end

  def _floor_event_about_battle_for_rescue(party, jail, jail_room, result_by_party)
    events = {}
    return events if party.inactive?

    situation_params = {
      wizard_is_active: party.wizard_is_active?,
      in_darkness: false, # 地下牢だと暗闇はない
      in_silence: false # 地下牢だと静寂はない
    }

    header_text_info = {
      situation_params: situation_params,
      party: party,
      leader: party.leader,
      is_lonely: party.is_only?,
      jail_room: {
        floor: jail_room.floor,
        adventurer: jail_room.adventurer,
        monster_party: jail_room.monster_party,
      },
    }
    # 戦闘開始の定型句
    events[:header_text_info] = header_text_info
    # 戦闘結果
    events[:events_in_battle] = _progress_fire_event_in_rescue(party, jail, jail_room, result_by_party)
    events
  end

  def ero_tunnel_params(party)
    target = party.leader
    bonus_rate = target.has_tattoo?(:reward_of_infection) ? 2 : 1

    if random_roll((30 * bonus_rate), 100) # エロステータス獲得
      bad_status_key = [:semen_holic, :v_semen_holic, :drug_sex_holic, :masochism_ecstasy_holic, :onany_holic].sample
      target.add_skill_holic!(bad_status_key)
      bad_status_name = target.get_skill_holic_name(bad_status_key)
    elsif random_roll((25 * bonus_rate), 100) # エロ装備獲得
      ero_equip_key = ERO_EQUIPS_LIST.keys.sample
      target.get_ero_equip!(ero_equip_key)
      ero_equip_name = target.get_ero_equip_name(ero_equip_key)
    else # 最低でも一時バステは確定
      bad_status_key = BAD_STATUS_IN_DIVE_NAMES.keys.sample
      target.add_bad_status_in_dive!(bad_status_key)
      bad_status_name = target.get_bad_status_in_dive_name(bad_status_key)
    end

    if !target.hooter_level_max? && random_roll((25 * bonus_rate), 100)
      hooter_level_name = target.hooter_level_name
      target.boom_hooter_level!
    end

    ero_type = %w(sex lesbian_sex).sample
    if ero_type == 'sex'
      target.get_ero_tunnel_sex!
    elsif ero_type == 'lesbian_sex'
      target.get_ero_tunnel_lesbian_sex!
    end

    # 副作用として体力の回復
    target.drain_level_n_recover_by_ero_tunnel!

    result = {
      leader: party.leader,
      floor: party.floor,
      ero_type: ero_type,
      hooter_level_name: hooter_level_name,
      bad_status_name: bad_status_name,
      ero_equip_name: ero_equip_name,
      life_point: party.leader.life_point,
    }
    result
  end

  # フロア単位のイベント
  def event_with_message(party, jail, dungeon_master, result_by_party)
    if party.in_ground? # 地上に着いていたら必要ない
      result_by_party[:no_action] = true
      return []
    end

    # 全滅した場合空パーティがここに来る
    if party.adventurers.size == 0
      result_by_party[:no_action] = true
      return []
    end

    events = []

    # トラップ
=begin
    trap_event = @trap_service.hit_trap_if(party, party.floor, dungeon_master)
    unless trap_event.empty?
      result_by_party[:hit_trap] = true
      events << ['trap_event', trap_event]
    end
=end
    # 監禁中に遭遇 + 戦闘
    if jail.engage_jail?(party.floor)
      result_by_party[:engage_jail] = true
      jail_room = jail.get_jail_room_by(party.floor)
      if party.no_down_members?
        # 戦闘可能なら
        if party.rescue_prisoner?(jail_room.monster_threat)
          # 戦闘加入
          event = {}
          event[:header_text] = @battle_script.rush_to_jail_room(party, jail_room)
          event[:battle_event] = _floor_event_about_battle_for_rescue(party, jail, jail_room, result_by_party)
          params = {
            leader: party.leader,
            is_lonely: party.is_only?,
            jail_room: jail_room,
            event: event,
          }
          events << ['join_to_battle_for_rescue', params]
        else
          # 向こうから気づかれて強制戦闘
          # 勝てないと見て撤退
          script = @battle_script.escape_from_jail_room(party, jail_room)
          event = {
            leader: party.leader,
            is_lonely: party.is_only?,
            jail_room: jail_room,
            script: script,
          }
          events << ['escape_from_jail_room', event]
        end
      else
        # 向こうから気づかれて強制戦闘
        # 戦闘不能なら
        script = @battle_script.escape_from_jail_room_by_down(party, jail_room)
        event = {
          leader: party.leader,
          is_lonely: party.is_only?,
          jail_room: jail_room,
          script: script,
        }
        events << ['escape_from_jail_room_by_down', event]
      end
    else
      # 戦闘
      event = _floor_event_about_battle(party, dungeon_master, result_by_party)
      if event.nil?
        # 戦闘なし
      else
        if event[:fire_result]
          # 戦闘があった
          result = {
            leader: party.leader,
            is_lonely: party.is_only?,
          }
          battle_event = ['battle_to_enemy', result, event[:fire_result]]
        else
          progress_result = event[:progress_result]
          if progress_result[:rout]
            # 出会ったが全力で逃走に成功
            result = {
              leader: party.leader,
              is_lonely: party.is_only?,
              floor: party.floor,
            }
            battle_event = ['engage_and_rout', result]
          elsif progress_result[:engaged]
            # 出会ったがやり過ごした
            result = {
              leader: party.leader,
              is_lonely: party.is_only?,
              floor: party.floor,
            }
            battle_event = ['engage_and_through', result]
          else
            # そもそも接敵しなかった
            result = {
              leader: party.leader,
              is_lonely: party.is_only?,
              floor: party.floor,
            }
            battle_event = ['no_engage_enemy', result]
          end
        end
        events << ['progress_floor_and_engage_enemy', battle_event, event]
        result_by_party[:battle_enemy] = true
      end
    end

    events
  end

  def _progress_event(party)
    result = {
      engaged: false,
      fired:   false,
      rout: false,
    }

    # 敵と接触した？
    if _not_engaged?(party)
      return result
    else
      result[:engaged] = true
    end

    # やりすごした？
    if _not_merged?(party)
      return result
    end

    # 全員戦闘不能の場合、全力逃走
    if party.only_down_members? && _success_rout?(party)
      result[:rout] = true
      return result
    end

    # だめだ戦闘だ
    result[:fired] = true
    result
  end

  def _progress_fire_event(party, dungeon_master, result_by_party, is_guard=false)
    in_darkness = (party.floor % 15).zero? && !party.wizard_is_active? # 暗闇フロアか？ かつ 魔法使いが戦闘不能か？
    in_silence = [12, 22, 32].include?(party.floor) # 静寂フロアか？

    # 敵の生成
    monster_party = MonsterParty.new(in_darkness: in_darkness, in_silence: in_silence)
    if is_guard
      # 深層限定、精鋭モンスターパターン
      monster_party.summon_guards!(party, dungeon_master)
    else
      # 通常パターン
      betrayed_servant = @adventurers_list[party.betrayed_servant_id_stalked]
      # 調教中でなければ参戦
      if betrayed_servant && !betrayed_servant.in_training_in_jail_as_betrayed?
        betrayed_servant.set_betrayed_status(:fired)
        betrayed_servant.set_betrayed_fire_status!(party.leader.short_name)
      end

      monster_party.generate_enemies!(party, dungeon_master, betrayed_servant)
    end

    situation_params = {
      wizard_is_active: party.wizard_is_active?,
      in_darkness: in_darkness,
      in_silence: in_silence
    }

    events = {}
    monster_party_info = {
      monster_name: monster_party.monster_name,
      unit_name: monster_party.unit_name,
      count: monster_party.initial_enemies_count,
      betrayed_servant: monster_party.betrayed_servant,
    }
    events[:header_text_info] = {
      situation_params: situation_params,
      party: party,
      leader: party.leader,
      is_lonely: party.is_only?,
      floor: party.floor,
      monster_party_info: monster_party_info,
    }

    if party.is_all_down?
      events[:event_by_advs] = _battle_to_monsters_in_all_down(party, monster_party, result_by_party)
    else
      events[:event_by_advs] = _battle_to_monsters(party, monster_party, result_by_party)
    end
    events
  end

  def _progress_fire_event_in_rescue(party, jail, jail_room, result_by_party)
    events = {}
    # 敵の生成
    monster_party = jail_room.monster_party

    if party.is_all_down?
      events[:event_by_advs] = _battle_to_monsters_in_all_down(party, monster_party, result_by_party)
      jail_room.status_abort!(party.leader)
    else
      adventurer = jail_room.adventurer
      party.in_rescue_battle!(adventurer.id)
      events_in_battle = _battle_to_monsters(party, monster_party, result_by_party)
      if events_in_battle[:is_victory]
        events_in_battle['liberate_room'] = [adventurer, party]
        jail.liberate_room!(adventurer, party.leader, party.is_only?, jail_room.floor)
        if party.in_rescue?
          if party.rescue_dive_target?
            party.clear_rescue_mode_success!
            party.decide_to_return_with_cargo!(adventurer) # 同行させた状態で撤退
          else
            party.continue_with_cargo!(adventurer) # 同行させた状態で続行
          end
        else
          party.decide_to_return_with_cargo!(adventurer) # 同行させた状態で撤退
        end
        jail.set_success_rescue!(jail_room, party.leader)
      else
        # TODO: 敗北時の処理も記述すること
        # hoge.fuga
        jail_room.status_repulse!(party.leader)
      end
      party.finish_in_rescue_battle_if
      events[:event_by_advs] = events_in_battle
    end
    events
  end

  LOW_CAPTURE_COUNT = 10
  # 敵と遭遇したが戦闘可能な者がいない
  def _battle_to_monsters_in_all_down(party, monster_party, result_by_party)
    monster_party.betrayed_servant.set_battle_with_all_down_status! if monster_party.betrayed_servant
    result_by_party[:break_party] = true

    events_by_advs = {
      all_party_down: {
        leader: party.leader,
        is_lonely: party.is_only?,
      }
    }
    down_members = party.down_members
    victim_member_count = party.passengers.count + down_members.count
    capture_events = {
      downs_captured_if: [],
    }

    # 捕獲に回せる数が少ないなら、一人に集中させてしまう。
    capture_count = monster_party.capture_count(party.active_member_levels)
    is_concentrate_target = (capture_count < LOW_CAPTURE_COUNT)
    first_concentrate_target = is_concentrate_target # 最初のメンバーに捕獲を集中するフラグ

    down_members.each do |member|
      capture_count = @capture_service.get_capture_count(member, monster_party, party, down_members.count, victim_member_count, is_concentrate_target, first_concentrate_target)
      first_concentrate_target = false if first_concentrate_target # 一人に集中フラグの解除

      already_down = false
      capture_result = @capture_service.capture_adventurer_if(member, party, monster_party, capture_count, already_down)
      capture_events[:downs_captured_if] << {
        adv_id: member.id,
        is_lonely: party.is_only?,
        adventurer: member,
        capture_count: capture_count,
        already_down: false,
        capture_result: capture_result,
        monster_name: monster_party.monster_name,
      }
    end

    # 防衛線の一角が破られたことで他に戦闘不能者がいればこの者たちも捕獲される
    party.passengers.each do |passenger|
      capture_count = @capture_service.get_downs_n_passenger_capture_count(passenger, monster_party, victim_member_count, is_concentrate_target)
      capture_result = @capture_service.capture_passenger_if(passenger, party, monster_party, capture_count)

      capture_events[:passengers_captured_if] ||= []
      capture_events[:passengers_captured_if] << {
        adv_id: passenger.id,
        adventurer: passenger,
        capture_count: capture_count,
        capture_result: capture_result,
        monster_name: monster_party.monster_name,
      }
    end
    events_by_advs[:capture_events] = capture_events
    events_by_advs
  end

  def _battle_to_monsters(party, monster_party, result_by_party)
    is_victory = true
    monster_total_size = monster_party.rest_enemies_count

    party_info = {
      party: party,
      leader: party.leader,
      is_lonely: party.is_only?,
      fighter: party.adventurers['fighter'],
      wizard: party.adventurers['wizard'],
      scout: party.adventurers['scout'],
      is_no_companions: party.no_companions?,
      is_only_down_members: party.only_down_members?,
    }
    monster_party_info = {
      monster_name: monster_party.monster_name,
      unit_name: monster_party.unit_name,
      number: monster_party.rest_enemies_count,
      is_indifferent: monster_party.has_skill?('indifferent'),
    }

    if monster_party.betrayed_servant
      monster_party_info[:betrayed_servant] = monster_party.betrayed_servant
    end

    events_in_battle = {
      party_info: party_info,
      monster_party_info: monster_party_info,
    }
    # 開幕攻撃
    events_in_battle[:opening_fire] = _wizard_opening_fire(party, monster_party)

    if monster_party.all_down?
      # ここ外せる
      events_in_battle[:all_down_by_opening_fire] = true
    else
      brain_washing_active_member = party.find_brain_washing_active_member
      is_brain_washing_active_member_is_majority = party.brain_washing_active_member_is_majority?

      party.start_battle!

      # 洗脳済み冒険者の味方撃ち結果その１
      if is_brain_washing_active_member_is_majority
        friendly_fire_result = {}
        party.adventurers.values.each do |adv|
          if adv.in_brain_washing?
            friendly_fire_result[adv] =
              if adv.is_down?
                {
                  is_down: true,
                }
              else
                {}
              end
          else
            # 味方撃ちの対象になる非洗脳サイドはなにもしない
          end
        end
      end

      kill_score = {}
      turn_count = 0
      bad_status_event_texts = []
      bad_status_event_texts = _battle_by_turn(party, monster_party, kill_score, turn_count, bad_status_event_texts)

      # 洗脳済み冒険者の味方撃ち結果その２
      if is_brain_washing_active_member_is_majority
        party.adventurers.values.each do |adv|
          if adv.in_brain_washing?
            # 洗脳済側はその１で記入済み
          else
            friendly_fire_result[adv] = {
              damage_point: adv.friendly_damage_point_in_turn
            }
          end
        end
        events_in_battle[:party_info][:friendly_fire_result] = friendly_fire_result
      end

      if party.no_now_down?
        # パーティー勝利
        events_in_battle[:kill_score] = kill_score
        events_in_battle[:bad_status_event_texts] = bad_status_event_texts
        is_victory = true
      else
        # パーティー敗北
        result_by_party[:break_party] = true

        is_victory = false

        # 捕獲系セリフ
        now_down_adventurers = party.now_down_adventurers
        victim_member_count = party.passengers.count + party.already_down_adventurers.count
        now_down_adventurer_ids = []

        # 捕獲に回せる数が少ないなら、一人に集中させてしまう。
        capture_count = monster_party.capture_count(party.active_member_levels)
        is_concentrate_target = (capture_count < LOW_CAPTURE_COUNT)
        first_concentrate_target = is_concentrate_target # 最初のメンバーに捕獲を集中するフラグ
        # 賞金首がいたら分配を見直す。賞金首は割当２倍
        target_member_size =
          now_down_adventurers.count +
            (now_down_adventurers.select {|adv| adv.is_bounty_target? }).count
        now_down_adventurers.each do |member|
          now_down_adventurer_ids << member.id
          capture_count = @capture_service.get_capture_count(member, monster_party, party, target_member_size, victim_member_count, is_concentrate_target, first_concentrate_target)
          first_concentrate_target = false # 一人に集中フラグの解除

          # この時点で合流可能か決めてしまう
          capture_result = @capture_service.capture_adventurer_if(member, party, monster_party, capture_count, false)

          # 洗脳組が多数派かつ捕獲されたのが非洗脳組
          if (is_brain_washing_active_member_is_majority &&
            brain_washing_active_member &&
            (brain_washing_active_member.id != member.id))
            events_in_battle[:now_downs_by_brain_wash_captured_if] ||= []
            events_in_battle[:now_downs_by_brain_wash_captured_if] << {
              adv_id: member.id,
              is_lonely: party.is_only?,
              adventurer: member,
              capture_count: capture_count,
              already_down: false,
              capture_result: capture_result,
              monster_name: monster_party.monster_name,
              brain_washing_active_member: brain_washing_active_member,
            }
          else
            events_in_battle[:now_downs_captured_if] ||= []
            events_in_battle[:now_downs_captured_if] << {
              adv_id: member.id,
              is_lonely: party.is_only?,
              adventurer: member,
              capture_count: capture_count,
              already_down: false,
              capture_result: capture_result,
              monster_name: monster_party.monster_name,
            }
          end
        end

        party.already_down_adventurers.each do |member|
          next if now_down_adventurer_ids.include?(member.id)
          capture_count = @capture_service.get_downs_n_passenger_capture_count(member, monster_party, victim_member_count, is_concentrate_target)

          # この時点で合流可能か決めてしまう
          capture_result = @capture_service.capture_adventurer_if(member, party, monster_party, capture_count, true)

          events_in_battle[:already_downs_captured_if] ||= []
          events_in_battle[:already_downs_captured_if] << {
            adv_id: member.id,
            is_lonely: party.is_only?,
            adventurer: member,
            capture_count: capture_count,
            already_down: true,
            capture_result: capture_result,
            monster_name: monster_party.monster_name,
          }
        end

        # 防衛線の一角が破られたことで他に戦闘不能者がいればこの者たちも捕獲される
        party.passengers.each do |passenger|
          capture_count = @capture_service.get_downs_n_passenger_capture_count(passenger, monster_party, victim_member_count, is_concentrate_target)
          capture_result = @capture_service.capture_passenger_if(passenger, party, monster_party, capture_count)

          events_in_battle[:passengers_captured_if] ||= []
          events_in_battle[:passengers_captured_if] << {
            adv_id: passenger.id,
            adventurer: passenger,
            capture_count: capture_count,
            capture_result: capture_result,
            monster_name: monster_party.monster_name,
          }
        end
      end
      events_in_battle
    end

    # 殺害数に応じたボーナス
    bonus_exp_point = (monster_party.calculate_exp_point(monster_total_size) * 0.4).to_i
    party.adventurers.values.each do |adventurer|
      adventurer.add_experience!(bonus_exp_point)
    end

    if monster_party.betrayed_servant
      betrayed_servant = monster_party.betrayed_servant
      events_in_battle[:betrayed_servant_result_text] =
        if monster_party.betrayed_servant_is_down?
          # 洗脳サーヴァントはダメージを与えたが撃破された
          betrayed_servant.result_down_text(
            betrayed_servant, betrayed_servant.this_battle_damage_point_sum)
        else
          # ダメージを与えた報告
          betrayed_servant.set_battle_status!
          if is_victory
            # 洗脳サーヴァントは逃走した
            betrayed_servant.result_escape_text(
              betrayed_servant, betrayed_servant.this_battle_damage_point_sum, monster_party.monster.name)
          else
            # 洗脳サーヴァントは元・仲間を撃破した
            betrayed_servant.result_victory_text(
              betrayed_servant, betrayed_servant.this_battle_damage_point_sum, monster_party.monster.name)
          end
        end
    end

    events_in_battle[:is_victory] = is_victory
    events_in_battle[:active_member_count] = party.party_active_member_size # 倒された時に使用
    events_in_battle
  end

  def _battle_by_turn(party, monster_party, kill_score, turn_count, bad_status_event_texts)
    # 全滅？
    if party.is_all_down?
      return nil
    end

    turn_count += 1 # ターン経過で敵の攻撃力が上がる
    # 振り分け
    splitted_enemies = _splitted_enemies(party, monster_party.rest_enemies_count)

    over_kill_powers = {}
    # 損害？
    splitted_enemies.each do |adventurer_id, enemies_size|
      adventurer = party.get_adventurer_by_id(adventurer_id)
      # ここでいうdecrease_monsters_countとは火力由来で「殺せるはずだった」モンスター数。実際に削れているわけではない
      decrease_monsters_count = monster_party.calculate_decrease_monsters_by(adventurer.fire_power, adventurer.wizard?)
      kill_score[adventurer_id] ||= 0
      new_enemies_size = enemies_size - decrease_monsters_count
      if new_enemies_size >= 0
        _kill_monsters(adventurer, kill_score, decrease_monsters_count, monster_party)
      else
        _kill_monsters(adventurer, kill_score, enemies_size, monster_party)
        # 余剰火力を周囲の味方のフォローに分配する
        over_kill_powers[adventurer_id] ||= 0
        over_kill_powers[adventurer_id] += (-1 * new_enemies_size * 0.5).to_i
      end
    end

    # モンスターパーティーに随伴した洗脳サーヴァントが撃破されたらそれを反映する
    if party.betrayed_servant_id_stalked && monster_party.betrayed_servant_is_down?
      party.unset_betrayed_servant_target!
    end

    unless monster_party.all_down?
      # 全滅してなかったらオーバーキルパワーでさらに減らす。
      over_kill_powers.each do |adventurer_id, kill_count|
        if kill_count > 0 && monster_party.rest_enemies_count > 0
          adventurer = party.get_adventurer_by_id(adventurer_id)
          if monster_party.rest_enemies_count > kill_count
            _kill_monsters(adventurer, kill_score, kill_count, monster_party)
          else
            rest_enemies_count = monster_party.rest_enemies_count
            _kill_monsters(adventurer, kill_score, rest_enemies_count, monster_party)
          end
        end
      end

      # バッドステータス付与
      result = _bad_status_by_monster(party, monster_party, turn_count)
      unless result.nil?
        if result[:bad_status_in_dive] # エロバステ付与
          bad_status_in_dive = BadStatusInDiveList[result[:bad_status_in_dive]]
          bad_status_name = bad_status_in_dive.nil? ? '' : bad_status_in_dive[:name]
          bad_status_event_texts += @battle_script.get_bad_status(result[:adventurer], monster_party.monster_name, bad_status_name)
        end
      end

      # ダメージを受ける
      _damage_by_monster(party, monster_party, turn_count)

      # 洗脳済みサーヴァントがいたらそちらからもダメージを受ける
      _damage_by_betrayed_servant(party, monster_party, turn_count)

      # 洗脳済みの味方がいたらそちらからもダメージを受ける
      _damage_by_friendly_fire(party)

      # ダウンしていたらそこで中止
      return nil unless party.no_now_down?

      # 敵は全滅せず、味方も全員生存していたら続行
      bad_status_event_texts = _battle_by_turn(party, monster_party, kill_score, turn_count, bad_status_event_texts)
    end

    bad_status_event_texts
  end

  FIGHTER_GUARD_EXTRA_RATE = 1.2
  def _gravity(adv)
    return 0 if adv.is_down?
    (adv.enemy_gravity_in_attack * (adv.fighter? ? FIGHTER_EXTRA_RATE : 1.0))
  end

  def _gravity_in_defense(adv)
    return 0 if adv.is_down?
    vanilla_point = (adv.enemy_gravity_in_defense * (adv.fighter? ? FIGHTER_GUARD_EXTRA_RATE : 1.0))
    vanilla_point *= 2 if adv.is_bounty_target?
    vanilla_point
  end

  def _bad_status_by_monster(party, monster_party, turn_count)
    success_bad_status_in_dive = []
    (monster_party.bad_status_in_dive || {}).each do |skill, rate|
      if random_roll(rate, 100)
        success_bad_status_in_dive << skill
      end
    end

    unless success_bad_status_in_dive.empty?
      bad_status_in_dive = success_bad_status_in_dive.sample
      target_adventurer = party.adventurers.values.sample
      target_adventurer.get_bad_status_in_dive!(bad_status_in_dive)
      bad_status_skill_result = {
        adventurer: target_adventurer,
        bad_status_in_dive: bad_status_in_dive,
      }
      return bad_status_skill_result
    end

    nil
  end

  def _damage_by_monster(party, monster_party, turn_count)
    attack_power = monster_party.attack_power(turn_count)
    gravity_sum = party.active_adventurers.inject(0) { |diff, adv| diff + _gravity_in_defense(adv) }
    party.active_adventurers.each do |adv|
      damage = ((_gravity_in_defense(adv).to_f / gravity_sum) * attack_power)
      adv.get_damage!(damage)
    end
  end

  def _damage_by_betrayed_servant(party, monster_party, turn_count)
    attack_point = monster_party.betrayed_servant_attack
    return unless attack_point # 洗脳サーヴァントが不在orダウンなら何もなし

    target = party.active_adventurers.sample
    if target
      damage_point = target.get_damage!(attack_point)
      monster_party.betrayed_servant.gain_betrayed_damage_point!(damage_point)
    end
  end

  def _damage_by_friendly_fire(party)
    if party.brain_washing_active_member_is_majority?
      target = party.active_adventurers_reject_brain_washing.sample
      return unless target

      damage = (target.fire_power * rand(0.5..1.0)).ceil
      actual_damage_point = target.get_friendly_damage!(damage)
      actual_damage_point
    end
  end

  def _kill_monsters(adventurer, kill_score, kill_count, monster_party)
    kill_score[adventurer.id] += kill_count
    adventurer.add_kill_score!(kill_count)
    monster_party.decrease_rest_enemies_count!(kill_count)
    exp_point = monster_party.calculate_exp_point(kill_count)
    adventurer.add_experience!(exp_point)
  end

  def _exp_point_by(kill_score, party, exp_point)
    party.fighter.add_exp(kill_score[party.fighter.id] * exp_point) if party.fighter_is_active?
    party.wizard.add_exp(kill_score[party.wizard.id] * exp_point) if party.wizard_is_active?
    party.scout.add_exp(kill_score[party.scout.id] * exp_point) if party.scout_is_active?
  end

  def adventurer_vs_monsters(adventurer, monsters)
    monsters
    adventurer
  end

  def _not_engaged?(party)
    engage_point =
      if party.in_deep_floor?
        (party.floor * 8.0)
      elsif party.floor > 30
        (party.floor * 6.0)
      else
        (party.floor * 4.0)
      end
    engage_point *= 2.0 if party.is_betrayed_servant_target?
    random_roll(party.scout_level, engage_point)
  end

  MERGE_RATE = 6.6
  def _not_merged?(party)
    # 全員レベル1かつHP全開かつ戦闘未経験の場合、必ず交戦する
    return false if (party.all_battle_virgin? && party.fresh_noob_party?)
    # 洗脳された人間が孤立していると、モンスターは回避できる
    return false if party.only_n_brain_washing? && random_roll(70, 100)
    # 洗脳された人間のみのパーティーなら、やはり回避できる
    return false if party.all_brain_washing? && random_roll(60, 100)

    merge_point = (party.floor * _merge_rate(party)) * MERGE_RATE
    merge_point *= 2.0 if party.is_betrayed_servant_target?

    betrayed_servant = @adventurers_list[party.betrayed_servant_id_stalked]
    if betrayed_servant && betrayed_servant.has_skill_for_betrayed?(:good_smell)
      merge_point *= 2.0
    end

    res = random_roll(party.scout_level.to_f, merge_point)
    res
  end

  DEFAULT_ROUT_RATE = 40
  # 戦えないから全力敗走
  def _success_rout?(party)
    floor = party.floor
    rout_rate = DEFAULT_ROUT_RATE - (floor / 2)

    betrayed_servant = @adventurers_list[party.betrayed_servant_id_stalked]
    if betrayed_servant && betrayed_servant.has_skill_for_betrayed?(:high_speed)
      rout_rate = (rout_rate * 0.6).to_i
    end

    if floor > 5 && party.lonely? # 5階以上だとつかまりやすく
      rout_rate = (rout_rate * 0.6).to_i
    end
    random_roll(rout_rate, 100)
  end

  def _merge_rate(party)
    rate =
      if party.in_deep_floor?
        6.0
      elsif party.floor > 30
        4.5
      elsif party.floor > 18
        3.6
      else
        3.0
      end

    rate *= 2.0 if party.leader_is_funatic?
    rate
  end

  # 開幕呪文
  def _wizard_opening_fire(party, monster_party)
    adventurer = party.wizard
    return [] unless adventurer # 魔法使いがいると開幕呪文
    return [] if adventurer.is_down?
    return [] if [12, 22, 32].include?(party.floor) # 静寂フロアは魔法攻撃なし

    is_paralysis_failed = adventurer.has_bad_status_in_dive?(:paralysis) && random_roll(25, 100)
    is_fever_failed = adventurer.has_bad_status_in_dive?(:fever) && random_roll(25, 100)
    is_hypno_failed = adventurer.has_bad_status_in_dive?(:hypno) && random_roll(25, 100)

    opening_fire_down_count =
      if is_paralysis_failed || is_fever_failed || is_hypno_failed
        0
      else
        is_wizard = true
        monster_party.calculate_decrease_monsters_by(party.opening_fire_power, is_wizard)
      end
    unless opening_fire_down_count.zero?
      adventurer.add_kill_score!(opening_fire_down_count)
      exp_point = monster_party.calculate_exp_point(opening_fire_down_count)
      adventurer.add_experience!(exp_point)
    end

    monster_party.decrease_rest_enemies_count!(opening_fire_down_count)
    is_full_clear = monster_party.all_down?
    # こっち残す
    event = {
      is_full_clear: is_full_clear,
      opening_fire_down_count: opening_fire_down_count,
      is_paralysis_failed: is_paralysis_failed,
      is_fever_failed: is_fever_failed,
      is_hypno_failed: is_hypno_failed,
    }
    event
  end

  FIGHTER_EXTRA_RATE = 1.0
  def _splitted_enemies(party, enemies_size)
    # 一人当たりの分担数

    active_members_size = party.active_adventurers.size
    base_part = enemies_size.to_f / active_members_size
    number_by_adventurer = {}
    if party.fighter_is_active?
      part_count = base_part.ceil
      number_by_adventurer[party.fighter.id] = part_count
      enemies_size -= part_count
    end
    if party.wizard_is_active?
      if enemies_size > 0
        part_count = base_part.ceil
        number_by_adventurer[party.wizard.id] = part_count
        enemies_size -= part_count
      else
        number_by_adventurer[party.wizard.id] = 0
      end
    end
    if party.scout_is_active?
      if enemies_size > 0
        part_count = base_part.ceil
        number_by_adventurer[party.scout.id] = part_count
        enemies_size -= part_count
      else
        number_by_adventurer[party.scout.id] = 0
      end
    end

    if enemies_size > 0
      # 残りはリーダー
      if party.leader.is_down?
        # 倒れていたら別の人。
        adv = party.active_adventurers.sample
        number_by_adventurer[adv.id] += enemies_size.to_i
      else
        if number_by_adventurer[party.leader.id]
          number_by_adventurer[party.leader.id] += enemies_size.to_i
        elsif number_by_adventurer[party.fighter.id]
          number_by_adventurer[party.fighter.id] += enemies_size.to_i
        elsif number_by_adventurer[party.wizard.id]
          number_by_adventurer[party.wizard.id] += enemies_size.to_i
        elsif number_by_adventurer[party.scout.id]
          number_by_adventurer[party.scout.id] += enemies_size.to_i
        end
      end
    end
    number_by_adventurer
  end

  # 分配がおかしいのでちょっと修正。ダミー化
=begin
  LEADER_EXTRA_RATE  = 1.1
  def _splitted_enemies_dummy(party, enemies_size)
    # 一人当たりの分担数
    gravity_sum = party.adventurers.values.inject(0) { |diff, adv| diff + _gravity(adv) }
    gravity_sum *= LEADER_EXTRA_RATE
    number_by_adventurer = {}
    if party.scout_is_active?
      base_enemy_count_by_adventurer = ((party.scout.enemy_gravity_in_atack.to_f / gravity_sum) * enemies_size)
      enemy_count_by_adventurer = _get_enemy_count_by_adventurer(party.scout, base_enemy_count_by_adventurer)
      number_by_adventurer[party.scout.id] = enemy_count_by_adventurer
      enemies_size -= enemy_count_by_adventurer
    end
    if party.wizard_is_active?
      base_enemy_count_by_adventurer = ((party.wizard.enemy_gravity_in_atack.to_f / gravity_sum) * enemies_size)
      enemy_count_by_adventurer = _get_enemy_count_by_adventurer(party.wizard, base_enemy_count_by_adventurer)
      number_by_adventurer[party.wizard.id] = enemy_count_by_adventurer
      enemies_size -= enemy_count_by_adventurer
    end
    if party.fighter_is_active? # 戦士は割当二倍
      base_enemy_count_by_adventurer = ((party.fighter.enemy_gravity_in_atack.to_f / gravity_sum) * enemies_size)
      enemy_count_by_adventurer = _get_enemy_count_by_adventurer(party.fighter, base_enemy_count_by_adventurer)
      part_target = (enemy_count_by_adventurer * FIGHTER_EXTRA_RATE).to_i
      number_by_adventurer[party.fighter.id] = part_target
      enemies_size -= part_target
    end
    # 残りはリーダー
    number_by_adventurer[party.leader.id] += enemies_size.to_i if number_by_adventurer[party.leader.id]
    number_by_adventurer
  end
=end

  def _get_enemy_count_by_adventurer(adventurer, base_enemy_count_by_adventurer)
    if base_enemy_count_by_adventurer >= 5
      enemy_count_by_adventurer = (base_enemy_count_by_adventurer * adventurer.monster_adjust_by_skill(base_enemy_count_by_adventurer))
    else
      enemy_count_by_adventurer = base_enemy_count_by_adventurer
    end
    enemy_count_by_adventurer.to_i
  end
end
