# coding: utf-8

require_rrk 'src/app/model/dungeon/party'
require_rrk 'src/app/util/debug_util'
require_rrk 'src/defines/script/dungeon/adventurer_script'
require_rrk 'src/defines/script/dungeon/battle_script'
require_rrk 'src/defines/script/dungeon/capture_script'
require_rrk 'src/defines/script/dungeon/trap_script'
require_rrk 'src/defines/script/dungeon/shorten/battle_script'
require_rrk 'src/defines/script/dungeon/shorten/capture_script'

class ScriptService
  include DebugUtil

  def initialize(bar_adventurers)
    @adventurer_script = AdventurerScript.new
    @battle_script = BattleScript.new
    @capture_script = CaptureScript.new
    @trap_script = TrapScript.new

    @shorten_capture_script = ShortenCaptureScript.new
    @shorten_battle_script = ShortenBattleScript.new

    @adventurers = bar_adventurers
  end

  def make_script_from(events)
    scripts = []
    events.each do |event|
      case event[0]
        when 'rest_in_bar'
          # バーに還元待ちなので何もしないよ
        when 'turn_start_text'
          progress_params = event[1]
          script = @battle_script.progress_message_in_turn_start(
            progress_params[:leader], progress_params[:is_lonely], progress_params[:turn_count],
            progress_params[:floor], progress_params[:in_dive], progress_params[:in_ground], progress_params[:positiveness])
          scripts += script
        when 'progress_floor'
          params = event[1]
          script = @battle_script.progress_message_in_turn_end(
            params[:leader], params[:is_lonely], params[:start_floor], params[:end_floor],
            params[:floor_max], params[:is_return_in_this_turn], params[:is_in_dive])
          scripts += script
        when 'get_teleport_pass'
          params = event[1]
          leader = params[:leader]
          script = leader.get_teleport_pass(params[:floor])
          scripts += script
        when 'use_teleport_pass_for_rescue'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          script = leader.teleport_for_rescue_script(params[:floor], is_lonely)
          scripts += script
        when 'use_teleport_pass'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          script = leader.teleport_script(params[:floor], is_lonely)
          scripts += script
        when 'get_treasure'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          money_amount = params[:money_amount]
          master_money = params[:master_money]
          script = leader.get_treasure_script(is_lonely, money_amount, master_money)
          scripts += script
        when 'return_to_ground'
          params = event[1]
          script = @battle_script.return_to_ground_message(params[:leader], params[:is_lonely], params[:floor_in_start])
          scripts += script
        when 'get_ground'
          params = event[1]
          script = @battle_script.event_in_ground_with_message(params[:party], params[:is_lonely])
          scripts += script
        when 'return_decide'
          params = event[1]
          script = @battle_script.return_decide_message(params[:leader], params[:is_lonely])
          scripts += script
        when 'church_purification'
          params = event[1]
          script = @battle_script.church_purification_message(params[:bad_status], params[:ero_equip])
          scripts += script
        when 'party_status_up'
          params = event[1]
          # 各メンバースキルアップ
          script = []
          level_up_params = params[:level_up]
          params[:status_up_by_adv].each do |key, up_params|
            lv_up_params = level_up_params[key]
            # レベルアップ
            script += @adventurer_script.level_up(lv_up_params[:adv], lv_up_params[:level_up_diff]) if lv_up_params

            # 名誉アップ
            honor_up = up_params[:honor_up]
            script += @adventurer_script.honor_up(honor_up[:adventurer], honor_up[:now_honor], honor_up[:now_honor]) if honor_up
            # 装備アップ
            equip_up = up_params[:equip_up]
            script += @adventurer_script.equip_up(equip_up[:adventurer], equip_up[:equip_point_diff]) if equip_up
          end
          # リーダー名誉ボーナス
          honor_params = params[:leader_honor_up]
          if honor_params
            script += @adventurer_script.honor_plus_by_leader(honor_params[:leader], honor_params[:honor_plus_sum])
          end

          scripts += script
        when 'ero_tunnel'
          params = event[1]
          scripts += @battle_script.ero_tunnel_script(
            params[:leader], params[:floor], params[:ero_type], params[:hooter_level_name], params[:bad_status_name], params[:ero_equip_name], params[:life_point])
        when 'hit_ero_trap'
          # エロトラップ
          trap_params = event[1]
          script = _ero_trap_scripts(trap_params)
          scripts += script
        when 'ero_recover_slime_drain'
          drain_params = event[1]
          script = _ero_recover_slime_drain_scripts(drain_params)
          scripts += script
        when 'ero_equip_damage'
          # エロ装備によるダメージ
          trap_params = event[1]
          script = _ero_equip_scripts(trap_params)
          scripts += script
        when 'progress_floor_and_engage_enemy' # 戦闘時の最初の台詞
          event_params = event[1]
          if event_params[0] == 'battle_to_enemy'
            # 戦闘
            script = _make_progress_floor_and_engage_enemy(event[2])
          elsif event_params[0] == 'engage_and_through'
            # 接敵したがやりすごした
            params = event_params[1]
            script = @battle_script.progress_and_engaged_script(
              params[:leader], params[:is_lonely], params[:floor])
          elsif event_params[0] == 'engage_and_rout'
            # 接敵＆全力逃走
            params = event_params[1]
            script = @battle_script.progress_and_rout_script(
              params[:leader], params[:is_lonely], params[:floor])
            scripts += script
          elsif event_params[0] == 'no_engage_enemy'
            # 接敵はなかった
            params = event_params[1]
            script = @battle_script.progress_and_no_engaged_script(
              params[:leader], params[:is_lonely], params[:floor])
          else
            puts "なぜここにきたし？：#{event_params[0]}" if DEVELOP_PRINT_MODE
            script = []
          end
          scripts += script
        when 'engage_and_rout'
          puts "~~~~~~~~~~engage_and_rout" if DEVELOP_PRINT_MODE
          event_params = event[1]
          script = @battle_script.progress_and_rout_script(
            event_params[:leader], event_params[:is_lonely], event_params[:floor])
          scripts += script
        when 'engage_and_through'
          puts "~~~~~~~~~~engage_and_through" if DEVELOP_PRINT_MODE
          event_params = event[1]
          script = @battle_script.progress_and_engaged_script(
            event_params[:leader], event_params[:is_lonely], event_params[:floor])
          scripts += script
        when 'battle_to_enemy'
          puts "~~~~~~~~~~battle_to_enemy" if DEVELOP_PRINT_MODE
          event_params = event[1]
          script = @battle_script.progress_and_fired_script(
            event_params[:leader], event_params[:is_lonely], event_params[:floor])
          scripts += script
        when 'join_to_battle_for_rescue'
          script = _make_progress_floor_and_engage_jails_and_battle(event[1])
          scripts += script
        when 'escape_from_jail_room'
          # 勝てないと見て地下牢を回避
          params = event[1]
          script = params[:script]
          scripts += script
        when 'escape_from_jail_room_by_down'
          params = event[1]
          script = params[:script]
          scripts += script
        when 'party_is_all_down_maybe'
          puts "party_is_all_down_maybe なので無視" if DEVELOP_PRINT_MODE && DEVELOP_DETAIL_PRINT_MODE
        else
          puts "~~~~~~~~~~event is NOT FOUND!!: #{event[0]}"
      end
    end
    scripts
  end

  def make_shorten_script_from(events)
    scripts = []
    events.each do |event|
      case event[0]
        when 'rest_in_bar'
          # バーに還元待ちなので何もしないよ
        when 'turn_start_text'
          progress_params = event[1]
          script = @shorten_battle_script.progress_message_in_turn_start(
            progress_params[:leader], progress_params[:is_lonely], progress_params[:turn_count],
            progress_params[:floor], progress_params[:in_dive], progress_params[:in_ground], progress_params[:positiveness])
          scripts += script
        when 'progress_floor'
          params = event[1]
          script = @shorten_battle_script.progress_message_in_turn_end(
            params[:leader], params[:is_lonely], params[:start_floor], params[:end_floor],
            params[:floor_max], params[:is_return_in_this_turn], params[:is_in_dive])
          scripts += script
        when 'get_teleport_pass'
          params = event[1]
          leader = params[:leader]
          script = leader.get_teleport_pass_shorten(params[:floor])
          scripts += script
        when 'use_teleport_pass_for_rescue'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          script = leader.teleport_for_rescue_script(params[:floor], is_lonely)
          scripts += script
        when 'use_teleport_pass'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          script = leader.teleport_script(params[:floor], is_lonely)
          scripts += script
        when 'get_treasure'
          params = event[1]
          leader = params[:leader]
          is_lonely = params[:is_lonely]
          money_amount = params[:money_amount]
          master_money = params[:master_money]
          script = leader.get_treasure_script(is_lonely, money_amount, master_money)
          scripts += script
        when 'return_to_ground'
          params = event[1]
          script = @battle_script.return_to_ground_message(params[:leader], params[:is_lonely], params[:floor_in_start])
          scripts += script
        when 'get_ground'
          params = event[1]
          script = @battle_script.event_in_ground_with_message(params[:party], params[:is_lonely])
          scripts += script
        when 'return_decide'
          params = event[1]
          script = @battle_script.return_decide_message(params[:leader], params[:is_lonely])
          scripts += script
        when 'church_purification'
          params = event[1]
          script = @battle_script.church_purification_message(params[:bad_status], params[:ero_equip])
          scripts += script
        when 'party_status_up'
          params = event[1]
          # 各メンバースキルアップ
          script = []
          level_up_params = params[:level_up]
          params[:status_up_by_adv].each do |key, up_params|
            lv_up_params = level_up_params[key]
            # レベルアップ
            script += @adventurer_script.level_up(lv_up_params[:adv], lv_up_params[:level_up_diff]) if lv_up_params

            # 名誉アップ
            honor_up = up_params[:honor_up]
            script += @adventurer_script.honor_up(honor_up[:adventurer], honor_up[:now_honor], honor_up[:now_honor]) if honor_up
            # 装備アップ
            equip_up = up_params[:equip_up]
            script += @adventurer_script.equip_up(equip_up[:adventurer], equip_up[:equip_point_diff]) if equip_up
          end
          # リーダー名誉ボーナス
          honor_params = params[:leader_honor_up]
          if honor_params
            script += @adventurer_script.honor_plus_by_leader(honor_params[:leader], honor_params[:honor_plus_sum])
          end

          scripts += script
        when 'ero_tunnel'
          params = event[1]
          scripts += @battle_script.ero_tunnel_script(
            params[:leader], params[:floor], params[:ero_type], params[:hooter_level_name], params[:bad_status_name], params[:ero_equip_name], params[:life_point])
        when 'hit_ero_trap'
          # エロトラップ
          trap_params = event[1]
          script = _ero_trap_scripts(trap_params)
          scripts += script
        when 'ero_recover_slime_drain'
          drain_params = event[1]
          script = _ero_recover_slime_drain_scripts(drain_params)
          scripts += script
        when 'ero_equip_damage'
          # エロ装備によるダメージ
          trap_params = event[1]
          script = _ero_equip_scripts(trap_params)
          scripts += script
        when 'progress_floor_and_engage_enemy' # 戦闘時の最初の台詞
          event_params = event[1]
          case event_params[0]
            when 'battle_to_enemy'
              # 戦闘
              script = _make_progress_floor_and_engage_enemy(event[2])
            when 'engage_and_through'
              # 接敵したがやりすごした
              params = event_params[1]
              script = @shorten_battle_script.progress_and_engaged_script(
                params[:leader], params[:is_lonely], params[:floor])
            when 'engage_and_rout'
              # 接敵＆全力逃走
              params = event_params[1]
              script = @shorten_battle_script.progress_and_engage_and_rout_script(
                params[:leader], params[:is_lonely], params[:floor])
            when 'no_engage_enemy'
              # 接敵はなかった
              params = event_params[1]
              script = @shorten_battle_script.progress_and_no_engaged_script(
                params[:leader], params[:is_lonely], params[:floor])
            else
              puts "なぜここにきたし！：#{event_params[0]}" if DEVELOP_PRINT_MODE
              script = []
          end
          scripts += script
        when 'engage_and_through'
          puts "~~~~~~~~~~engage_and_through" if DEVELOP_PRINT_MODE
          event_params = event[1]
          script = @shorten_battle_script.progress_and_engaged_script(
            event_params[:leader], event_params[:is_lonely], event_params[:floor])
          scripts += script
        when 'battle_to_enemy'
          puts "~~~~~~~~~~battle_to_enemy" if DEVELOP_PRINT_MODE
          event_params = event[1]
          script = @battle_script.progress_and_fired_script(
            event_params[:leader], event_params[:is_lonely], event_params[:floor])
          scripts += script
        when 'join_to_battle_for_rescue'
          script = _make_progress_floor_and_engage_jails_and_battle(event[1])
          scripts += script
        when 'escape_from_jail_room'
          # そのうちショートscriptに置き換え
=begin
          # 勝てないと見て地下牢を回避
          params = event[1]
          script = params[:script]
          scripts += script
=end
        when 'escape_from_jail_room_by_down'
          # そのうちショートscriptに置き換え
=begin
          params = event[1]
          script = params[:script]
          scripts += script
=end
        when 'party_is_all_down_maybe'
          puts "shorten, party_is_all_down_maybe なので無視"  if DEVELOP_PRINT_MODE
        else
          puts "shorten, ~~~~~~~~~~event is NOT FOUND!!: #{event[0]}"
      end
    end
    scripts
  end

  def _make_progress_floor_and_engage_jails_and_battle(params)
    event = params[:event]
    script = []
    script += event[:header_text] # 助けは天に通じた
    battle_event = event[:battle_event]
    is_rescue = true
    guard_monster = false
    script += _make_header_script_from(
      battle_event[:header_text_info], # パーティーがなだれ込んだ
      battle_event[:events_in_battle][:event_by_advs],
      guard_monster,
      is_rescue,
    )
    script
  end

  def _make_progress_floor_and_engage_enemy(event)
    script = []
    progress_result = event[:progress_result]
    if progress_result[:fired] # 交戦した
      fire_result = event[:fire_result]
      is_rescue = false
      script += _make_header_script_from(
        fire_result[:header_text_info],
        fire_result[:event_by_advs],
        fire_result[:guard_monster],
        is_rescue
      )
    elsif progress_result[:engaged] # 接敵したがやり過ごした
      puts "本当にここに来るのか？：接敵したがやり過ごした" if DEVELOP_PRINT_MODE
      script
    else # そもそも接敵しなかった
      puts "本当にここに来るのか？：そもそも接敵しなかった" if DEVELOP_PRINT_MODE
      script
    end
    script
  end

  def _make_header_script_from(header_text_info, event_by_advs, guard_monster=false, is_rescue=false)
    script = []

    script +=
      if is_rescue
        @battle_script.rescue_battle_script(
          header_text_info[:leader],
          header_text_info[:is_lonely],
          header_text_info[:jail_room])
      elsif guard_monster
        @battle_script.generate_guard_enemies_scripts(
          header_text_info[:leader],
          header_text_info[:is_lonely],
          header_text_info[:floor],
          header_text_info[:monster_party_info])
      else
        @battle_script.generate_enemies_scripts(
          header_text_info[:party],
          header_text_info[:leader],
          header_text_info[:is_lonely],
          header_text_info[:floor],
          header_text_info[:monster_party_info],
          header_text_info[:situation_params]
        )
      end

    puts "header_text_info:#{header_text_info}" if event_by_advs.nil? && DEVELOP_PRINT_MODE
    if event_by_advs && event_by_advs[:all_party_down]
      script += _all_party_down_script(event_by_advs[:all_party_down])
      script += _make_all_down_capture_script(event_by_advs[:capture_events], header_text_info[:is_lonely])
      return script
    end

    party = event_by_advs[:party_info][:party]
    script += party.active_adv_battle_prelude_script

    unless (event_by_advs[:opening_fire].nil?|| event_by_advs[:opening_fire].empty?)
      opening_fire = event_by_advs[:opening_fire]
      # 開幕呪文
      script +=
        @battle_script.wizard_opening_fire_script(event_by_advs[:party_info][:wizard], event_by_advs[:monster_party_info][:unit_name], opening_fire[:opening_fire_down_count], opening_fire[:is_full_clear], opening_fire)
      if opening_fire[:is_full_clear]
        script += @battle_script.blank_kill_score_script(event_by_advs[:party_info], event_by_advs[:monster_party_info])
      end
    end

    if event_by_advs[:is_victory]
      if DEVELOP_PRINT_MODE &&
        event_by_advs[:all_down_by_opening_fire].nil? &&
        event_by_advs[:kill_score].nil?
        puts "event_by_advs[:kill_score] is nil！！！"
        puts "event_by_advs.keys: #{event_by_advs.keys}"
        puts "event_by_advs: #{event_by_advs}"
      end

      kill_score_script =
        if event_by_advs[:kill_score].nil?
          []
        else
          event_by_advs[:kill_score].map do |adv_id, score|
            @battle_script.kill_score_by_adv_script(
              @adventurers[adv_id], score, event_by_advs[:monster_party_info])
          end
        end

      friendly_fire_result = event_by_advs[:party_info][:friendly_fire_result]
      if friendly_fire_result && !friendly_fire_result.empty?
        script += @battle_script.get_friendly_fire_script(friendly_fire_result)
      end
      script += @battle_script.defeat_enemy_script(kill_score_script.join('、'), event_by_advs[:party_info][:leader], event_by_advs[:party_info][:is_lonely], event_by_advs[:monster_party_info][:monster_name])
      # 洗脳サーヴァントがいたら、その結果も付記する
      if event_by_advs[:betrayed_servant_result_text]
        script += event_by_advs[:betrayed_servant_result_text]
      end
      script += event_by_advs[:bad_status_event_texts] if event_by_advs[:bad_status_event_texts]
      script += [' ']
    else # 敗北した
      script += event_by_advs[:bad_status_event_texts] if event_by_advs[:bad_status_event_texts]
      # 洗脳サーヴァントがいたら、その結果も付記する
      if event_by_advs[:betrayed_servant_result_text]
        script += event_by_advs[:betrayed_servant_result_text]
      end

      friendly_fire_result = event_by_advs[:party_info][:friendly_fire_result]
      if friendly_fire_result && !friendly_fire_result.empty?
        script += @battle_script.get_friendly_fire_script(friendly_fire_result)
      end

      # 倒れたscript
      (event_by_advs[:now_downs_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += @battle_script.down_script(adventurer, params[:monster_name])
      end
      (event_by_advs[:now_downs_by_brain_wash_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += @battle_script.down_by_brain_wash_script(
          adventurer, params[:monster_name], params[:brain_washing_active_member])
      end
      (event_by_advs[:already_downs_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += @capture_script.already_down_capture_message_common(adventurer, params[:is_lonely])
        script += @battle_script.already_down_script(adventurer, params[:monster_name])
      end

      # 健在な冒険者
      if event_by_advs[:is_lonely]
        # もともとパーティが一人なら短め。そもそも分断されるはずがない。
        script += @battle_script.defeated_script(event_by_advs[:party_info][:leader], event_by_advs[:is_lonely])
      elsif event_by_advs[:active_member_count] == 0
        # 誰も何もできないはずなのでほっとく
        # party = event_by_advs[:party_info][:party]
        # script += @battle_script.all_member_down_script(event_by_advs[:party_info][:leader], event_by_advs[:is_lonely], party.active_adv_names)
      else
        script += @battle_script.remain_member_retreat_script(event_by_advs[:party_info][:leader], event_by_advs[:is_lonely])
      end

      # 倒された冒険者がそのあとどうなったか
      (event_by_advs[:now_downs_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += _script_from_capture_event(adventurer, params[:capture_result], event_by_advs[:party_info], event_by_advs[:monster_party_info], params[:capture_count])
      end
      # 同行者はどうなった？
      (event_by_advs[:passengers_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += _script_from_capture_event(adventurer, params[:capture_result], event_by_advs[:party_info], event_by_advs[:monster_party_info], params[:capture_count])
      end
=begin
      (event_by_advs[:already_downs_captured_if] || []).each do |params|
        adventurer = params[:adventurer]
        script += @capture_script.already_down_capture_message_common(adventurer, params[:is_lonely])
        script += @battle_script.already_down_script(adventurer, params[:monster_name])
      end
=end
    end
    script
  end

  def _all_party_down_script(party_info)
    @battle_script.no_battle_members_scripts(party_info[:leader], party_info[:is_lonely])
  end

  def _make_all_down_capture_script(capture_events, is_lonely)
    script = []

    if capture_events[:downs_captured_if]
      capture_events[:downs_captured_if].each do |params|
        adventurer = params[:adventurer]
        script += @capture_script.already_down_capture_message_common(adventurer, is_lonely)
        script += @battle_script.already_down_script(adventurer, params[:monster_name])
      end
    end
    if capture_events[:passengers_captured_if]
      capture_events[:passengers_captured_if].each do |params|
        capture_result = params[:capture_result]
        adventurer = params[:adventurer]

        script += @capture_script.already_down_capture_message_common(adventurer, is_lonely)
        script += @battle_script.already_down_script(adventurer, params[:monster_name])
      end
    end
    script
  end

  def _script_from_capture_event(adventurer, event, party_info, monster_party_info, capture_count)
    case event
      when 'captured' # 捕獲
        @capture_script.capture_event(adventurer, party_info, monster_party_info, capture_count)
      when 'escape_and_merge' # 逃走成功＆合流成功
        is_merged = true
        @capture_script.capture_escape_event(adventurer, party_info, monster_party_info, capture_count, is_merged)
      when 'escape_and_not_merge' # 合流失敗 & 遭難
        is_merged = false
        @capture_script.capture_escape_event(adventurer, party_info, monster_party_info, capture_count, is_merged)
      when 'down_adv_is_captured' # ダウン済み & 捕獲
        @capture_script.capture_event_of_passenger(adventurer, party_info, monster_party_info, capture_count)
      when 'down_adv_is_escape_and_not_merge' # ダウン済み & 合流失敗 & 遭難
        @capture_script.capture_escape_event_of_passenger(adventurer, monster_party_info, capture_count, party_info[:is_lonely])
      when 'passenger_is_captured' # 同行者が捕獲
        @capture_script.capture_event_of_passenger(adventurer, party_info, monster_party_info, capture_count)
      when 'passenger_is_escaped_and_not_merge' # 同行者が逃走成功。合流はもともと考えない
        @capture_script.capture_escape_event_of_passenger(adventurer, monster_party_info, capture_count, party_info[:is_lonely])
      else
        puts "ここにくるのはおかしい：_script_from_capture_event:#{event}" if DEVELOP_PRINT_MODE
        []
    end

  end

  def _ero_trap_scripts(trap_params)
    if trap_params[:event] == 'trap_event_failed'
      # @trap_script.trap_1_failed_script(trap_params[:leader], trap_params[:is_lonely])

      return [
        'トラップ失敗しました',
      ]
    end

    unless trap_params[:event] == 'trap_event'
      return [
        "定義されていないトラップイベントです：#{trap_params[:event]}",
      ]
    end

    script =
      case trap_params[:trap_no]
        when 1
          @trap_script.trap_1_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 2
          @trap_script.trap_2_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 3
          @trap_script.trap_3_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 4
          @trap_script.trap_4_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 5
          @trap_script.trap_5_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 6
          @trap_script.trap_6_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 7
          @trap_script.trap_7_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 8
          @trap_script.trap_8_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:target], trap_params[:effect][:drop_count], trap_params[:effect][:new_floor])
        when 11
          @trap_script.trap_11_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target])
        when 12
          @trap_script.trap_12_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target])
        when 13
          @trap_script.trap_13_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target])
        when 21
          @trap_script.trap_21_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect][:duplicate])
        when 22
          @trap_script.trap_22_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:adventures])
        when 23
          @trap_script.trap_23_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:adventures], trap_params[:parties])
        when 24
          @trap_script.trap_24_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:adventures], trap_params[:parties])
        when 31
          @trap_script.trap_31_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor])
        when 41
          @trap_script.trap_41_script(
            trap_params[:leader], trap_params[:is_lonely], trap_params[:floor],
            trap_params[:target], trap_params[:effect],
            trap_params[:boom_hooter_success], trap_params[:is_hooter_level_max])
        when 42
          @trap_script.trap_42_script(
            trap_params[:leader], trap_params[:is_lonely], trap_params[:floor],
            trap_params[:target], trap_params[:effect],
            trap_params[:boom_hooter_success], trap_params[:is_hooter_level_max])
        when 43
          @trap_script.trap_43_script(
            trap_params[:leader], trap_params[:is_lonely], trap_params[:floor],
            trap_params[:target], trap_params[:effect],
            trap_params[:boom_hooter_success], trap_params[:is_hooter_level_max])
        when 51
          @trap_script.trap_51_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 52
          @trap_script.trap_52_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 53
          @trap_script.trap_53_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 54
          @trap_script.trap_54_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 61
          @trap_script.trap_61_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 62
          @trap_script.trap_62_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 63
          @trap_script.trap_63_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 64
          @trap_script.trap_64_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 71
          @trap_script.trap_71_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 72
          @trap_script.trap_72_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 73
          @trap_script.trap_73_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:life_point])
        when 74
          @trap_script.trap_74_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 75
          @trap_script.trap_75_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 76
          @trap_script.trap_76_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 81
          @trap_script.trap_81_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 82
          @trap_script.trap_82_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 91
          @trap_script.trap_91_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 92
          @trap_script.trap_92_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 93
          @trap_script.trap_93_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target_and_result], trap_params[:effect])
        when 94
          @trap_script.trap_94_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target_and_result], trap_params[:effect])
        when 95
          @trap_script.trap_95_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 96
          @trap_script.trap_96_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 101
          @trap_script.trap_101_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots])
        when 102
          @trap_script.trap_102_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots])
        when 103
          @trap_script.trap_103_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 121
          @trap_script.trap_121_script(trap_params[:leader], trap_params[:is_only], trap_params[:floor], trap_params[:adventurer], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots],
                                       trap_params[:party], trap_params[:boom_hooter_success], trap_params[:without_rescue], trap_params[:companion_is_brainwashed], trap_params[:is_hooter_level_max])
        when 122
          @trap_script.trap_122_script(trap_params[:leader], trap_params[:is_only], trap_params[:floor], trap_params[:adventurer], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots],
                                       trap_params[:party], trap_params[:boom_sensitive_success], trap_params[:boom_sensitive_position], trap_params[:without_rescue], trap_params[:companion_is_brainwashed])
        when 131
          @trap_script.trap_131_script(trap_params[:leader], trap_params[:is_only], trap_params[:floor], trap_params[:adventurer], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots],
                                       trap_params[:party], trap_params[:companion_is_brainwashed], trap_params[:tentacle_holic_success])
        when 132
          @trap_script.trap_132_script(trap_params[:leader], trap_params[:is_only], trap_params[:floor], trap_params[:adventurer], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots],
                                       trap_params[:party], trap_params[:companion_is_brainwashed], trap_params[:tentacle_holic_success])
        when 141
          @trap_script.trap_141_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms])
        when 142
          @trap_script.trap_142_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms])
        when 143
          @trap_script.trap_143_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms])
        when 151
          @trap_script.trap_151_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 152
          @trap_script.trap_152_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 161
          @trap_script.trap_161_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 162
          @trap_script.trap_162_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        when 163
          @trap_script.trap_163_script(trap_params[:leader], trap_params[:is_lonely], trap_params[:floor], trap_params[:target], trap_params[:effect], trap_params[:this_turn_orgasms], trap_params[:this_turn_sperm_shots], trap_params[:life_point])
        else
          puts "unknown traps_no: #{trap_params[:trap_no]}"
          []
      end
    script
  end

  def _ero_recover_slime_drain_scripts(drain_params)
    @trap_script.ero_recover_slime_drain_script(drain_params[:target], drain_params[:target_life_point], drain_params[:is_success], drain_params[:is_lonely], drain_params[:this_turn_orgasms])
  end

  def _ero_equip_scripts(trap_params)
    script = []
    trap_params.each do |adventurer_id, params|
      script +=
        case params[:equip_no]
          when 141
            @trap_script.trap_equip_141_script(params[:target], params[:this_turn_orgasms])
          when 142
            @trap_script.trap_equip_142_script(params[:target], params[:this_turn_orgasms])
          when 143
            @trap_script.trap_equip_143_script(params[:target], params[:this_turn_orgasms])
          else
            puts "unknown er-equip-traps_no: #{params[:equip_no]}"
            []
        end
    end
    script
  end

  def get_party_summaries(parties_list_by_floor)
    @capture_script.get_party_summaries(parties_list_by_floor)
  end

  def get_jail_summaries(jails_list_by_floor)
    @capture_script.get_jail_summaries(jails_list_by_floor)
  end

  def party_progress_status(parties, identify_code, status_text, party_text)
    @capture_script.party_progress_status(parties, identify_code, status_text, party_text)
  end

  def jail_room_status(jail, identify_code, adv_id, room_text)
    @capture_script.jail_room_status(jail, identify_code, adv_id, room_text)
  end

  def capture_summaries_header(day_count)
    @capture_script.capture_summaries_header(day_count)
  end

  def capture_summaries_footer(day_count)
    @capture_script.capture_summaries_footer(day_count)
  end

  def party_summaries_header(day_count)
    @capture_script.party_summaries_header(day_count)
  end

  def party_summaries_footer(day_count)
    @capture_script.party_summaries_footer(day_count)
  end
end
