# coding: utf-8

require_rrk 'src/app/view_element_logic/dungeon/base_view_element_logic'
require_rrk 'src/app/chain_engine_cover/dungeon/view_element_converter'
require_rrk 'src/app/service/chain_engine/menu/main_menu_service'
require_rrk 'src/defines/dungeon/menu_helper/dungeon_menu_list_helper'
require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/defines/dungeon/item_list'
require_rrk 'src/defines/dungeon/carpenter_list'
require_rrk 'src/defines/dungeon/tentacle_list'
require_rrk 'src/app/helper/constants/dungeon/dungeon_constants'
require_rrk 'src/defines/dungeon/utils/version_helper'

class DungeonViewElementLogic < BaseViewElementLogic
  include DungeonMenuHelper
  include DungeonConstants
  include MonsterListByType
  include VersionHelper

  def initialize(mode_options)
    super
  end

  def show_dungeon_top_menu(dungeon_master)
    menus =
      if DUNGEON_VERSION >= 5.0
        if product_mode
          DUNGEON_MASTER_MENU_LIST + DUNGEON_MASTER_PRODUCTION_MENU_LIST
        else
          DUNGEON_MASTER_MENU_LIST
        end
      else
        DUNGEON_MASTER_MENU_LIST_UNDER_V4 + DUNGEON_MASTER_MENU_LIST
      end

    if (DUNGEON_VERSION >= 5.0) && dungeon_master.has_book?(:summon_high_monster)
      menus += DUNGEON_MASTER_HIGH_SUMMON_MENU_LIST
    end

    @view_element_converter.make_menu_items_to(menus + back_to_top)
  end

  def show_summons_power_up(dungeon_master)
    monster_list = dungeon_master_summons_list.reject do |key, value|
      dungeon_master.is_unlocked_monster?(key) ||
        !dungeon_master.can_buy?(key, value[:lock])
    end
    elements = monster_list.map do |key, monster|
      {
        'text' => monster[:name] + '(' + _add_comma(monster[:price]) + ')',
        'action' => SHOW_SUMMON_DETAIL_PREFIX + key.to_s,
      }
    end

    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_traps_power_up(dungeon_master)
    trap_list = TrapList::DUNGEON_MASTER_ROCKED_TRAPS_LIST.select do |key, params|
      dungeon_master.can_buy_trap?(key, params[:lock], params[:all_lock])
    end
    elements = trap_list.map do |key, trap|
      {
        'type' => 'button',
        'text' => trap[:name] + '(' + _add_comma(trap[:price]) + ')',
        'action' => SHOW_TRAP_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_items_power_up(dungeon_master)
    item_list = ItemList::DUNGEON_MASTER_ROCKED_ITEMS_LIST.select do |key, params|
      dungeon_master.can_buy_item?(key, params[:lock])
    end
    elements = item_list.map do |key, item|
      {
        'type' => 'button',
        'text' => item[:name] + '(' + _add_comma(item[:price]) + ')',
        'action' => SHOW_ITEM_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_books_power_up(dungeon_master)
    book_list = ItemList::DUNGEON_MASTER_ROCKED_BOOKS_LIST.select do |key, params|
      dungeon_master.can_buy_book?(params[:key])
    end
    elements = book_list.map do |key, item|
      {
        'type' => 'button',
        'text' => item[:name] + '(' + _add_comma(item[:price]) + ')',
        'action' => SHOW_BOOK_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_merchant_shop(dungeon_master)
    item_list = ItemList::DUNGEON_MASTER_ROCKED_ITEMS_LIST.select do |key, params|
      dungeon_master.can_buy_item?(key, params[:lock])
    end
    elements = item_list.map do |key, item|
      {
        'type' => 'button',
        'text' => item[:name] + '(' + _add_comma(item[:price]) + ')',
        'action' => SHOW_ITEM_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_book_shop(dungeon_master)
    book_list = ItemList::DUNGEON_MASTER_ROCKED_BOOKS_LIST.select do |key, params|
      dungeon_master.can_buy_book?(params[:key])
    end
    elements = book_list.map do |key, item|
      {
        'type' => 'button',
        'text' => item[:name] + '(' + _add_comma(item[:price]) + ')',
        'action' => SHOW_BOOK_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_carpenter_shop(dungeon_master)
    building_list = CarpenterList::BROTHEL_BUILDINGS_LIST.select do |key, params|
      dungeon_master.can_buy_building?(key, params[:lock])
    end
    elements = building_list.map do |key, building|
      {
        'type' => 'button',
        'text' => building[:name] + '(' + _add_comma(building[:price]) + ')',
        'action' => SHOW_BUILDING_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  def show_tentacle_shop(dungeon_master)
    tentacle_list = TentacleList::TENTACLE_LABS_LIST.select do |key, params|
      dungeon_master.can_buy_tentacle?(key, params[:lock])
    end
    elements = tentacle_list.map do |key, building|
      {
        'type' => 'button',
        'text' => building[:name] + '(' + _add_comma(building[:price]) + ')',
        'action' => SHOW_TENTACLE_DETAIL_PREFIX + key.to_s,
      }
    end
    elements += back_to_('show_dungeon_top_menu')
    @view_element_converter.make_menu_items_to(elements)
  end

  # 購入検討中の召喚の詳細を見る
  def show_summon_detail_of(script_elements, monster_key, monster_name)
    script_elements + [
      {
        'type'   => 'button',
        'text'   => monster_name + 'を召喚する',
        'action' => SHOW_SUMMON_UNLOCK_PREFIX + monster_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_trap_detail_of(description_elements, trap_key, trap_name)
    description_elements + [
      {
        'type'   => 'button',
        'text'   => trap_name + 'を開発する',
        'action' => SHOW_TRAP_UNLOCK_PREFIX + trap_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_item_detail_of(description_elements, item_key, item_name)
    description_elements + [
      {
        'type'   => 'button',
        'text'   => item_name + 'を購入する',
        'action' => SHOW_ITEM_UNLOCK_PREFIX + item_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_book_detail_of(description_elements, item_key, item_name)
    description_elements + [
      {
        'type'   => 'button',
        'text'   => item_name + 'を購入する',
        'action' => SHOW_BOOK_UNLOCK_PREFIX + item_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_building_detail_of(description_elements, building_key, building_name)
    description_elements + [
      {
        'type'   => 'button',
        'text'   => building_name + 'を購入する',
        'action' => SHOW_BUILDING_UNLOCK_PREFIX + building_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_tentacle_detail_of(description_elements, tentacle_key, tentacle_name)
    description_elements + [
      {
        'type'   => 'button',
        'text'   => tentacle_name + 'を開発する',
        'action' => SHOW_TENTACLE_UNLOCK_PREFIX + tentacle_key.to_s,
        'width'  => BUTTON_WIDTH_IN_SCRIPT_LIST,
      },
      {
        'type'   => 'label',
        'text'   => ' ',
      },
    ]
  end

  def show_summon_magic_room(scripts)
    list = [
      {
        'type' => 'button',
        'text' => '魔力(800)を消費し、ウルクたちを魔法陣から召喚する',
        'action' => 'summon_elite_monster_101',
      },
    ]
    @view_element_converter.make_script_menu_items_to(
      list
    ) +
      make_scripts_from(scripts)
  end

  def summon_elite_monster(header_script, scripts)
    make_scripts_from(header_script) +
      show_summon_magic_room(scripts)
  end

  def shortage_of_summon_elite_monster(header_script, scripts)
    make_scripts_from(header_script) +
      show_summon_magic_room(scripts)
  end
end
