# coding: utf-8
# LogicControllerは複数のlistを扱いつつ処理を共通化するためにある

require_rrk 'src/app/view_element_logic/dungeon/base_view_element_logic'
require_rrk 'src/app/service/chain_engine/menu/main_menu_service'
require_rrk 'src/app/model/save/save_secretary'
%w(view_element_converter).each do |src_name|
  require_rrk 'src/app/chain_engine_cover/dungeon/' + src_name
end
require_rrk 'src/defines/dungeon/menu_helper/script_menu_list'
require_rrk 'src/defines/script/dungeon/menu_sub_script'
require_rrk 'src/defines/dungeon/main_menu/jail_script_helper'
require_rrk 'src/app/helper/game_define_helper'
require_rrk 'src/defines/dungeon/utils/version_helper'

class MenuViewElementLogic < BaseViewElementLogic
  include Dungeon::MainMenu::JailScriptHelper
  include GameDefineHelper
  include VersionHelper

  def initialize(mode_options)
    @main_menu_service = Dungeon::MainMenuService.new
    # セーブ用
    @save_secretary = SaveSecretary.new
    @menu_sub_script = MenuSubScript.new
    super
  end

  def top_menu(scripts)
    make_scripts_from(scripts)
  end

  def update_in_opening_menu(dungeon_master, auction_room_slaves)
    @main_menu_service.update_in_opening_menu(dungeon_master, auction_room_slaves)
  end

  def show_initial_menu
    @view_element_converter.make_menu_items_to(
      @main_menu_service.show_save_load
    )
  end

  # トップメニューを見る
  def show_main_menu
    @view_element_converter.make_menu_items_to(
      @main_menu_service.turn_available_group_menu.values)
  end

  def go_to_new_game
    @view_element_converter.make_menu_items_to(
      @main_menu_service.go_to_new_game
    )
  end

  # ターンごとの更新
  def update_by_turn(game_master, is_night, is_new_dawn)
    @main_menu_service.update(game_master, is_night, is_new_dawn)
  end

  #============ セーブ & ロード start
  # セーブとロード画面を見る
  def show_save_load
    @view_element_converter.make_menu_items_to(
      @main_menu_service.show_save_load + back_to_top
    )
  end

  # セーブ一覧を見る
  def show_save_slots(scripts)
    result = @view_element_converter.make_script_menu_items_to(
      @save_secretary.list_saves_for_save
    ) +
      make_scripts_from(scripts)
    result
  end

  # ロードするセーブデータ一覧を見る
  def show_load_slots(scripts)
    @view_element_converter.make_script_menu_items_to(
      @save_secretary.list_saves_for_load
    ) +
      make_scripts_from(scripts)
  end

  def show_load_slots_in_initial(scripts)
    @view_element_converter.make_script_menu_items_to(
      @save_secretary.list_saves_for_load +
        [
          {
            'type' => 'button',
            'text' => 'はじめから',
            'action' => 'opening_menu_from_load',
          }
        ]
    ) +
      make_scripts_from(scripts)
  end

  def save_to_(slot_number, save_data)
    @save_secretary.save_to_(slot_number, save_data)
  end

  def save_to_view_elements(scripts)
    @view_element_converter.make_script_menu_items_to(
      @save_secretary.list_saves_for_save
    ) +
      make_scripts_from(scripts)
  end

  def load_from_(slot_number)
    @save_secretary.load_from_(slot_number)
  end

  def load_from_view_elements(scripts)
    @view_element_converter.make_script_menu_items_to(
      @save_secretary.list_saves_for_load
    ) +
      make_scripts_from(scripts)
  end

  def write_adventure_log(scripts, bard)
    @save_secretary.write_adventure_log(bard)
    make_scripts_from(scripts)
  end
  #============ セーブ & ロード end


  # 冒険者の現状を見るのメニュー画面
  include ScriptMenuHelper
  def hear_adventurers_in_bar(scripts)
    make_scripts_from(scripts) +
      @view_element_converter.make_script_menu_items_to(
        SHOW_ADVENTURERS_SCRIPT_MENU_LIST
      )

  end

  # 冒険者の様子を見る
  def show_adventurers(list, text_list, bar_party_by_adventurer_id)
    _show_adventurers(list, text_list, bar_party_by_adventurer_id, 'default')
  end

  # 冒険者の様子を見る
  def show_adventurers_show_jails(list, text_list, bar_party_by_adventurer_id)
    _show_adventurers(list, text_list, bar_party_by_adventurer_id, 'show_jails')
  end

  # 冒険者の様子を見る
  def show_adventurers_only_prisoner(list, text_list, bar_party_by_adventurer_id)
    _show_adventurers(list, text_list, bar_party_by_adventurer_id, 'only_prisoner')
  end

  # 冒険者の様子を見る
  def show_adventurers_with_bookmark(list, text_list, bar_party_by_adventurer_id)
    _show_adventurers(list, text_list, bar_party_by_adventurer_id, 'with_bookmark')
  end

  # 冒険者の様子を見る
  def _show_adventurers(list, text_list, bar_party_by_adventurer_id, mode)
    show_only_jail_mode = %w(only_prisoner show_jails).include?(mode)
    show_adventurer_prefix =
      if mode == 'with_bookmark'
        'show_adventurer_with_bookmark'
      elsif mode == 'only_prisoner'
        'show_adventurer_only_prisoner'
      elsif mode == 'show_jails'
        'show_adventurer_in_jail'
      else
        'show_adventurer'
      end

    elements_in_bar = []
    elements_in_dungeon = []
    elements_in_jail = []
    elements_in_magic_factory = []
    elements_in_slave = []

    unless show_only_jail_mode
      list[:adventurers_in_bar].each do |adventurer_id, adventurer|
        elements_in_bar << {
          'type' => 'button',
          'text' => @menu_sub_script.adventurer_in_bar_text(adventurer),
          'action' => "#{show_adventurer_prefix}_#{adventurer_id}"
        }
      end
      list[:adventurers_in_dungeon].each do |adventurer_id, params|
        party = bar_party_by_adventurer_id[adventurer_id]
        elements_in_dungeon << {
          'type' => 'button',
          'text' => @menu_sub_script.adventurer_in_dungeon_text(params, party),
          'action' => "#{show_adventurer_prefix}_#{adventurer_id}"
        }
      end
      list[:adventurers_in_magic_factory].each do |adventurer_id, params|
        elements_in_magic_factory << {
          'type' => 'button',
          'text' => @menu_sub_script.adventurer_in_magic_factory_text(params),
          'action' => "#{show_adventurer_prefix}_#{adventurer_id}"
        }
      end
      list[:adventurers_in_slave].each do |adventurer_id, params|
        elements_in_slave << {
          'type' => 'button',
          'text' => @menu_sub_script.adventurer_in_slave_text(params),
          'action' => "#{show_adventurer_prefix}_#{adventurer_id}"
        }
      end
    end

    show_brands_mode = (mode == 'show_jails')
    list[:adventurers_in_jail].each do |adventurer_id, params|
      elements_in_jail << {
        'type' => 'button',
        'text' => @menu_sub_script.adventurer_in_jail_text(params, show_brands_mode),
        'action' => "#{show_adventurer_prefix}_#{adventurer_id}"
      }
    end

    elements = make_scripts_from(text_list[:header_script])

    if (text_list[:all_empty] && !text_list[:all_empty].empty?)
      elements += make_scripts_from(text_list[:all_empty])
    end

    unless elements_in_bar.empty?
      elements +=
        make_scripts_from(text_list[:bar_header_text]) +
          @view_element_converter.make_script_menu_items_to(
            elements_in_bar
          )
    end

    unless elements_in_dungeon.empty?
      elements +=
        make_scripts_from(text_list[:dungeon_header_text]) +
          @view_element_converter.make_script_menu_items_to(
            elements_in_dungeon
          )
    end

    unless elements_in_jail.empty?
      elements +=
        make_scripts_from(text_list[:jail_header_text]) +
          @view_element_converter.make_script_menu_items_to(
            elements_in_jail
          )
    end

    unless elements_in_magic_factory.empty?
      elements +=
        make_scripts_from(text_list[:magic_factory_header_text]) +
          @view_element_converter.make_script_menu_items_to(
            elements_in_magic_factory
          )
    end

    unless elements_in_slave.empty?
      elements +=
        make_scripts_from(text_list[:slave_header_text]) +
          @view_element_converter.make_script_menu_items_to(
            elements_in_slave
          )
    end
    elements += make_scripts_from(text_list[:footer_script])
    elements
  end

  def show_adventurers_in_dungeon(elements_by_floor, scripts)
    result = []
    elements_by_floor.each do |floor, elements|
      result += make_scripts_from([elements[:header_text]])
      result += @view_element_converter.make_script_menu_items_to(elements[:parties])
      result += make_scripts_from([' ',])
    end
    result += make_scripts_from(scripts)

    result
  end

  def show_jails_for_custom(text, adventurers_in_jail)
    elements = []

    footer_text = [' ', ' ']
    show_brands_mode = true
    elements_in_jail = []
    adventurers_in_jail.each do |adventurer_id, params|
      elements_in_jail << {
        'type' => 'button',
        'text' => @menu_sub_script.adventurer_in_jail_text(params, show_brands_mode),
        'action' => "show_jail_details_for_custom_#{adventurer_id}"
      }
    end

    elements +=
      make_scripts_from(text) +
        @view_element_converter.make_script_menu_items_to(
          elements_in_jail
        ) +
        make_scripts_from(footer_text)
    elements
  end

  def show_betray_servant_cage(script, manage_penis_text, betrayed_slaves)
    elements_in_betray_servant_cage = []
    betrayed_slaves.each do |slave_id, slave|
      elements_in_betray_servant_cage << {
        'type' => 'button',
        'text' => @menu_sub_script.betray_servant_text(slave),
        'action' => "show_betray_servant_detail_#{slave_id}"
      }
    end

    elements =
      make_scripts_from(script) +
        @view_element_converter.make_script_menu_items_to(
          elements_in_betray_servant_cage
        )

    unless betrayed_slaves.empty?
      elements_to_manage_betray_servant_penis = [
        {
          'type' => 'button',
          'text' => manage_penis_text,
          'action' => 'manage_betray_servant_penis'
        }
      ]
      elements +=
        make_scripts_from([' ']) + # 配置位置の調整
          @view_element_converter.make_script_menu_items_to(
            elements_to_manage_betray_servant_penis
          )
    end

    elements
  end

  def manage_betray_servant_penis(script, betrayed_slaves)
    elements_for_betray_servants =
      betrayed_slaves.map do |slave_id, slave|
        {
          'type' => 'button',
          'text' => @menu_sub_script.betray_servant_penis_text(slave),
          'action' => "toggle_betray_servant_penis_#{slave_id}"
        }
      end
    elements =
      make_scripts_from(script) +
        @view_element_converter.make_script_menu_items_to(
          elements_for_betray_servants
        )

    elements
  end

  def show_adventurers_in_jails(text, adventurers_in_jail, back_method_name)
    elements = []

    show_brands_mode = true
    elements_in_jail = []

    unless back_method_name == 'none'
      elements_in_jail << {
        'type' => 'button',
        'text' => '戻る',
        'action' => back_method_name
      }
    end

    adventurers_in_jail.each do |adventurer_id, params|
      elements_in_jail << {
        'type' => 'button',
        'text' => @menu_sub_script.adventurer_in_jail_text(params, show_brands_mode),
        'action' => "show_adventurer_in_jail_#{adventurer_id}"
      }
    end

    elements +=
      make_scripts_from(text) +
        @view_element_converter.make_script_menu_items_to(
          elements_in_jail
        )
    elements
  end

  # ダンジョンに入るパーティー単体を見る
  def show_party_detail(scripts)
    @view_element_converter.make_script_menu_items_to(
      back_to_('show_parties_in_dungeon')
    ) +
      make_scripts_from(scripts)
  end

  def show_jail_room(scripts)
    @view_element_converter.make_script_menu_items_to(
      back_to_('show_adventurers_in_jails')
    ) +
      make_scripts_from(scripts)
  end

  # 冒険者単体を見る
  def show_adventurer(scripts, adventurer)
    _show_adventurer_common(scripts, adventurer, 'show_adventurers')
  end

  def show_adventurer_only_prisoner(scripts, adventurer)
    _show_adventurer_common(scripts, adventurer, 'show_adventurers_only_prisoner')
  end

  def show_adventurer_with_bookmark(scripts, adventurer)
    _show_adventurer_common(scripts, adventurer, 'show_adventurers_with_bookmark')
  end

  def _show_adventurer_common(scripts, adventurer, back_method_name)
    elements = []
    elements += @view_element_converter.make_script_menu_items_to(
      back_to_(back_method_name)
    )

    unless adventurer.in_sold?
      buttons = [
        {
          'type' => 'button',
          'text' => "#{adventurer.name}に賞金をかける（#{adventurer.bounty_price}）",
          'action' => "show_bounty_of_#{adventurer.id}",
        },
      ]

      if DUNGEON_VERSION >= 8.0
        buttons +=
          if adventurer.release_jail_mode?
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しても手出ししないよう魔物たちに命令を出す",
                'action' => "left_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}を他の冒険者同様捕獲するよう魔物たちに命令を出す",
                'action' => "default_prison_of_#{adventurer.id}",
              },
            ]
          elsif adventurer.left_jail_mode?
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しないよう魔物たちに命令を出す",
                'action' => "release_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}を他の冒険者同様捕獲するよう魔物たちに命令を出す",
                'action' => "default_prison_of_#{adventurer.id}",
              },
            ]
          else
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しないよう魔物たちに命令を出す",
                'action' => "release_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しても手出ししないよう魔物たちに命令を出す",
                'action' => "left_prison_of_#{adventurer.id}",
              },
            ]
          end
      end

      elements += @view_element_converter.make_script_menu_items_to(buttons)
    end

    if adventurer.in_jail_room?
      buttons = [
        {
          'type' => 'button',
          'text' => "#{adventurer.name}を地上に解放する",
          'action' => "release_from_jail_#{adventurer.id}",
        },
      ]

      elements += @view_element_converter.make_script_menu_items_to(buttons)
    end

    bookmark_button = []
    bookmark_button <<
      if adventurer.bookmarked?
        {
          'type' => 'button',
          'text' => "#{adventurer.name}を監視リストから外す",
          'action' => "reset_bookmark_to_#{adventurer.id}",
        }
      else
        {
          'type' => 'button',
          'text' => "#{adventurer.name}を監視リストに入れる",
          'action' => "set_bookmark_to_#{adventurer.id}",
        }
      end
    elements += @view_element_converter.make_script_menu_items_to(bookmark_button)

    elements += make_scripts_from(scripts)
    elements
  end

  def show_prisoner(status_scripts, adventurer, back_method_name)
    scripts = status_scripts

    elements = []
    elements += @view_element_converter.make_script_menu_items_to(
      back_to_(back_method_name) # 昨日の地下牢の様子
    )
    unless adventurer.in_sold?
      buttons = [
        {
          'type' => 'button',
          'text' => "#{adventurer.name}に賞金をかける（#{adventurer.bounty_price}）",
          'action' => "show_bounty_of_#{adventurer.id}",
        }
      ]
      elements += @view_element_converter.make_script_menu_items_to(buttons)
    end

    if adventurer.in_jail_room?
      tattoo_custom_text =
        if FGO_MODE
          '霊基改竄'
        else
          '魔術刻印による改造'
        end

      buttons = []
      if adventurer.in_jail_room?
        buttons += [
          {
            'type'   => 'button',
            'text'   => "#{adventurer.name}を地上に解放する",
            'action' => "release_from_jail_#{adventurer.id}",
          },
        ]
      end

      if DUNGEON_VERSION >= 8.0
        buttons +=
          if adventurer.release_jail_mode?
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しても手出ししないよう魔物たちに命令を出す",
                'action' => "left_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}を他の冒険者同様捕獲するよう魔物たちに命令を出す",
                'action' => "default_prison_of_#{adventurer.id}",
              },
            ]
          elsif adventurer.left_jail_mode?
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しないよう魔物たちに命令を出す",
                'action' => "release_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}を他の冒険者同様捕獲するよう魔物たちに命令を出す",
                'action' => "default_prison_of_#{adventurer.id}",
              },
            ]
          else
            [
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しないよう魔物たちに命令を出す",
                'action' => "release_prison_of_#{adventurer.id}",
              },
              {
                'type' => 'button',
                'text' => "#{adventurer.name}は監禁しても手出ししないよう魔物たちに命令を出す",
                'action' => "left_prison_of_#{adventurer.id}",
              },
            ]
          end
      end
      buttons += [
        {
          'type'   => 'button',
          'text'   => tattoo_custom_text,
          'action' => "show_jail_details_for_custom_#{adventurer.id}",
        },
      ]
      elements += @view_element_converter.make_script_menu_items_to(buttons)
    end

    elements += make_scripts_from(scripts)
    elements
  end


  def show_bounty_of(scripts, adventurer)
    elements = []
    elements += @view_element_converter.make_script_menu_items_to(
      back_to_('show_adventurers')
    )
    buttons = [
      {
        'type' => 'button',
        'text' => "#{adventurer.name}にかける賞金として#{adventurer.bounty_price}を支払う",
        'action' => "pay_bounty_of_#{adventurer.id}",
      }
    ]
    elements += @view_element_converter.make_script_menu_items_to(buttons)
    elements += make_scripts_from(scripts)
    elements
  end

  def pay_bounty_of(scripts)
    elements = []
    elements += @view_element_converter.make_script_menu_items_to(
      back_to_('show_adventurers')
    )
    elements += make_scripts_from(scripts)
    elements
  end

  def set_prison_of(scripts)
    elements = []
    elements += @view_element_converter.make_script_menu_items_to(
      back_to_('show_adventurers')
    )
    elements += make_scripts_from(scripts)
    elements
  end

  def show_jail_details_for_custom(slave_id, slave, dungeon_master, scripts)
    scripts += [' ', ' ',]
    footer_scripts = [
      ' ',
      ' ',
      "========= #{slave.short_name} の様子 ================",
      ' ',
    ]
    make_scripts_from(scripts) +
      @view_element_converter.make_script_menu_items_to(
        show_prisoner_custom_list(slave_id, slave, dungeon_master)
      ) +
      make_scripts_from(footer_scripts) +
      # ここからadventurerのステータス表示
      make_scripts_from(slave.status_script)
  end

  def show_betray_servant_detail(slave_id, slave, dungeon_master, scripts)
    scripts += [' ', ]
    footer_scripts = [' ', ' ',]
    make_scripts_from(scripts) +
      @view_element_converter.make_script_menu_items_to(
        show_betray_servant_custom_list(slave_id, slave, dungeon_master)
      ) +
      make_scripts_from(footer_scripts)
  end

  def brand_detail_tattoo_to_prisoner(slave_id, slave, tattoo, scripts)
    scripts += [' ', ' ',]
    footer_scripts = [' ', ' ',]

    make_scripts_from(scripts) +
      @view_element_converter.make_script_menu_items_to(
        brand_detail_tattoo_buttons(slave_id, slave, tattoo)
      ) +
      make_scripts_from(footer_scripts)
  end

  def brand_detail_tattoo_in_shortage(slave_id, scripts)
    scripts += [' ', ' ',]
    footer_scripts = [' ', ' ',]

    make_scripts_from(scripts) +
      @view_element_converter.make_script_menu_items_to(
        brand_detail_tattoo_buttons_in_shortage(slave_id)
      ) +
      make_scripts_from(footer_scripts)
  end
end
