# coding: utf-8
# ゲームに使うロジックはこちら
%w(view_element_converter).each do |src_name|
  require_rrk 'src/app/chain_engine_cover/' + src_name
end

%w(chain_engine/menu/train_menu_service).each do |src_name|
  require_rrk 'src/app/service/' + src_name
end

require_rrk 'src/app/view_element_logic/base_view_element_logic'
require_rrk 'src/app/helper/view_helper'

class TrainViewElementLogic < BaseViewElementLogic
  include ViewHelper

  def initialize *args
    @view_element_converter = ViewElementConverter.new
    @train_menu_service = TrainMenuService.new
  end

  def in_start_round(slave, house)
    @train_menu_service.in_start_round(slave, house)
  end

  def show_train_group_menu
    # 実行可能なコマンドがあるグループだけ chain_view_converter に渡す
    @view_element_converter.make_group_menu_items_to(
      @train_menu_service.command_group_menu)
  end

  def show_train_commands(train_group)
    # 指定されたグループの実行可能なコマンドだけ chain_view_converter に渡す
    @view_element_converter.make_menu_items_to(
      @train_menu_service.round_available_commands(train_group))
  end

  def in_finish_step(slave, house)
    @train_menu_service.in_finish_step(slave, house)
  end

  def show_train_report
    @view_element_converter.make_group_menu_items_to(
      @train_menu_service.show_train_report)
  end

  def show_slave_profile(slave)
    edited_slave_profile = _edit_slave_profile(slave)
    left_profile  = @view_element_converter.make_profile_from(
      edited_slave_profile['left_profile'])
    right_profile = @view_element_converter.make_profile_from(
      edited_slave_profile['right_profile'])
    return {
      'left_profile'  => left_profile,
      'right_profile' => right_profile,
    }
  end

  def _edit_slave_profile(slave)
    {
      'left_profile' => [
        {
          'text' => '名前:' + slave.short_name,
        },
        {
          'text' => '体力:' + _add_comma(slave.strength),
        },
        {
          'text' => '反抗:' + _add_comma(slave.rebel),
        },
        {
          'text' => '興奮:' + _add_comma(slave.voltage),
        },
        {
          'text' => 'Ｃ快:' + _add_comma(slave.c_ecstasy),
        },
        {
          'text' => 'Ｖ快:' + _add_comma(slave.v_ecstasy),
        },
        {
          'text' => 'Ａ快:' + _add_comma(slave.a_ecstasy),
        },
        {
          'text' => 'Ｂ快:' + _add_comma(slave.b_ecstasy),
        },
        ],
      'right_profile' => [
        {
          'text' => '肉欲:' + _add_comma(slave.lust),
        },
        {
          'text' => '従順:' + _add_comma(slave.obedience),
        },
        {
          'text' => '屈服:' + _add_comma(slave.submit),
        },
        {
          'text' => '性技:' + _add_comma(slave.technique),
        },
        {
          'text' => '好意:' + _add_comma(slave.affection),
        },
      ],
    }
  end
end
