# coding: utf-8
module Dungeon
  module MenuHelper
    require_rrk 'src/app/helper/game_define_helper'
    require_rrk 'src/app/util/debug_util'

    include GameDefineHelper

    def auction_room_list(auction_room_slaves)
      slave_size = auction_room_slaves.size
      auction_room_text =
        if slave_size > 0
          "奴隷商人の店（#{slave_size}人）"
        else
          '奴隷商人の店'
        end

      {
        9 => {
          'text'   => auction_room_text,
          'action' => 'show_auction_room',
        },
      }
    end

    def top_group_menu(dungeon_master)
      common_extra_list =
        if mode_to_show_save_load?
          GROUP_MENU_DEFINITIONS_FOR_PRODUCTION
        else
          {}
        end

      if DebugUtil::LIVE_CAST_MODE
        common_extra_list = common_extra_list.merge(GROUP_MENU_DEFINITIONS_FOR_LIVE_CAST)
      end

      night_extra_list = common_extra_list.dup
      if product_mode && dungeon_master.has_book?(:brainwash)
        night_extra_list.merge!(betrayed_servant_menu)
      end

      modes = {
        new_dawn: GROUP_MENU_DEFINITIONS_IN_NEW_DAWN,
        day: GROUP_MENU_DEFINITIONS_IN_DAY.merge(common_extra_list).sort,
        night: GROUP_MENU_DEFINITIONS_IN_NIGHT.merge(night_extra_list).sort,
      }
      modes
    end

    def betrayed_servant_menu
      if FGO_MODE
        GROUP_MENU_BETRAYED_EXTRA_DEFINITIONS_IN_NIGHT_FOR_FGO
      else
        GROUP_MENU_BETRAYED_EXTRA_DEFINITIONS_IN_NIGHT
      end
    end

    GROUP_MENU_DEFINITIONS = {
    }.freeze

    GROUP_MENU_DEFINITIONS_IN_NEW_DAWN = {
      0 => {
        'text'   => '次へ',
        'action' => 'progress_turn',
      },
    }

    GROUP_MENU_DEFINITIONS_IN_DAY = {
      0 => {
        'text'   => 'ターン終了（昼）',
        'action' => 'progress_turn',
      },
      1 => {
        'text'   => '昨日のダンジョンの様子',
        'action' => 'show_dungeon',
      },
      2 => {
        'text'   => '昨日の地下牢の様子',
        'action' => 'show_jail',
      },
      3 => {
        'text'   => '昨日の魔術工房の様子',
        'action' => 'show_magic_factory_night',
      },
      4 => {
        'text'   => '酒場で冒険者たちの噂を聞く',
        'action' => 'hear_adventurers_in_bar',
      },
      5 => {
        'text'   => 'ダンジョンの強化',
        'action' => 'show_dungeon_top_menu',
      },
    }

    GROUP_MENU_DEFINITIONS_IN_NIGHT = {
      0 => {
        'text'   => 'ターン終了（夜）',
        'action' => 'progress_turn',
      },
      1 => {
        'text'   => 'ダンジョンの様子',
        'action' => 'show_dungeon',
      },
      2 => {
        'text'   => '酒場で冒険者たちの噂を聞く',
        'action' => 'hear_adventurers_in_bar',
      },
      3 => {
        'text'   => '監禁された冒険者を見る',
        'action' => 'show_adventurers_show_jails',
      },
      5 => {
        'text'   => '魔術工房に赴く',
        'action' => 'show_magic_factory',
      },
    }

    GROUP_MENU_BETRAYED_EXTRA_DEFINITIONS_IN_NIGHT = {
      4 => {
        'text'   => '洗脳済み従属奴隷の様子を見る',
        'action' => 'show_betray_servant_cage',
      },
    }

    GROUP_MENU_BETRAYED_EXTRA_DEFINITIONS_IN_NIGHT_FOR_FGO = {
      4 => {
        'text'   => '洗脳サーヴァントの様子を見る',
        'action' => 'show_betray_servant_cage',
      },
    }

    GROUP_MENU_DEFINITIONS_FOR_PRODUCTION = {
      101 => {
        'text'   => 'セーブ / ロード',
        'action' => 'show_save_load',
      },
    }.freeze

    GROUP_MENU_DEFINITIONS_FOR_LIVE_CAST = {
      102 => {
        'text'   => '今日の冒険ログを記録する',
        'action' => 'write_adventure_log',
      },
    }.freeze
  end
end
