# coding: utf-8
require_rrk 'src/defines/script/dungeon/script_utils'
require_rrk 'src/app/model/dungeon/adventurer'

class JailScript
  include ScriptUtils

  def summary_base_text(room)
    adventurer = room.adventurer
    adventurer_short_status_bar = adventurer.short_status_bar

    adventurer_short_status_bar += '　' if adventurer_short_status_bar.length > 0
    "#{half_number_to_full(room.floor_number)}階　#{adventurer.short_name}　#{adventurer_short_status_bar}"
  end

  def _jail_resist_status(room)
    if room.in_resist?
      if room.resist_point_before_turn == room.resist_point
        "（抵抗：#{half_number_to_full(room.resist_point)}）"
      else
        "（抵抗：#{half_number_to_full(room.resist_point_before_turn)}→#{half_number_to_full(room.resist_point)}）"
      end
    else
      if room.san_level_before_turn == room.san_level
        "（正気：陵辱開始→#{half_number_to_full(room.san_level)}）"
      else
        "（正気：#{half_number_to_full(room.san_level_before_turn)}→#{half_number_to_full(room.san_level)}）"
      end
    end
  end

  def summary_text(room, base_text)
    after_status_text =
      if room.is_escaped # 脱走した？
        '［脱走した！］'
      elsif room.in_resist? && !room.failed_reason.nil? # バステで脱走失敗？
        '［脱走に失敗］'
      elsif room.adventurer.is_now_slave_call? # 奴隷宣言した？
        '［奴隷宣言］'
      else
        ''
      end

    base_text + _capture_status_name(room) + _jail_resist_status(room) + after_status_text
  end

  MAX_JAIL_BORDER_LENGTH = 30
  def jail_summary_border(day_count)
    day_text = "#{half_number_to_full(day_count)}日目（夜）の地下牢"

    [
      ('＝' * 7) + day_text + ('＝' * (MAX_JAIL_BORDER_LENGTH - (day_text.length * 2)))
    ]
  end

  def jail_summary_prelude_text(jail_count)
    [
      ' ',
      '迷宮に無数に存在する地下牢。それらは部屋として使える迷宮の地下空間を女を閉じ込めるための地下室として改造したものだ。',
      '地下室に一つだけもうけられている出入り口には、鉄格子がはめられており、施錠できるようになっている。',
      ' ',
      "地下牢には#{half_number_to_full(jail_count)}組の冒険者が捕らえられている。いずれの地下牢も全ては異様な熱気に包まれている。",
      'まだ冒険者が抵抗する間は女の悲鳴と囃し立てる魔物たちの声が響き、' +
        '抵抗する術がつき輪姦陵辱される地下室からは女のくぐもった悲鳴や甘い嬌声と魔物たちの嘲る声が響く。',
      '背徳と獣欲でギラギラと輝く狂宴は、洞窟の薄暗がりの中で今日も催されていた……',
      ' ',
      ' ',
    ]
  end

  def _capture_status_name(room)
    text = "監禁#{half_number_to_full(room.day_count)}日目"
    text += "（陵辱調教#{half_number_to_full(room.force_sex_day_count)}日目）" unless room.in_resist?
    text
  end

  def ready_to_escape(room)
    [
      room.adventurer.short_name + 'は監視の目を盗み、逃げ出す機会を伺っている……',
    ]
  end

  # ただし監禁中に限る、陵辱中は不可
  def ready_to_escape_n_success(room)
    [
      '（脱走判定成功： ' + room.adventurer.short_name + 'は監視が緩んだ隙を逃さず、地下牢から逃げ出した！）',
      ' ',
    ]
  end

  # ただし監禁中に限る、陵辱中は不可
  def ready_to_escape_n_failed(room)
    [
      '（脱走判定失敗： ' + room.adventurer.short_name + 'はチャンスを伺ったが、逃げ出す機会は訪れなかった）',
    ]
  end

  # ただし監禁中に限る、陵辱中は不可
  def ready_to_escape_n_failed_by_bad_status(room)
    status_text =
      if room.failed_reason == 'rape_desire_tattoo'
        '[陵辱願望刻印による効果]'
      elsif room.failed_reason == 'hypno'
        '[催眠による効果]'
      elsif room.failed_reason == 'rat'
        '[発情による効果]'
      elsif room.failed_reason == 'aphrodisiac'
        '[催淫による効果]'
      elsif room.failed_reason == 'aphrodisiac_fever'
        '[催淫による効果]'
      elsif room.failed_reason == 'paralysis'
        '[麻痺による効果]'
      elsif room.failed_reason == 'fever'
        '[疫熱による効果]'
      else
        '[ステータスによる効果]'
      end

    [
      '（脱走判定成功？： ' + room.adventurer.short_name + 'は脱走する好機を見出したが、' +
        status_text + 'によって逃げ出すことができなかった……）',
    ]
  end

  # ただし監禁中に限る、陵辱中は不可
  def ready_to_escape_success(room)
    [
      '（脱走判定成功）',
      room.adventurer.short_name + 'は監視の目が緩んだチャンスを目ざとく捉えた！',
      '居眠りをしている見張りの' + room.monster_party.monster.name + 'の目を盗み、' + room.adventurer.short_name + 'は地下牢の錠を外し、逃げ出すことに成功した！',
      ' ',
    ]
  end

  # 意図的に逃された
  def release_n_escape_success_situation(room)
    [
      '（脱走判定成功……）',
      room.adventurer.short_name + 'は、監禁されている地下牢の鍵がかかっていないことに気づいた！',
      'どうやら近くに見張りもいないようだ。今が好機だと考えた' +
        room.adventurer.short_name + 'は音もなく地下牢の扉を開け、逃げ出すことに成功した！',
      'この好機が仕組まれたものとも知らない' + room.adventurer.short_name + 'は、迷宮の中へと再び身を躍らせた……',
      ' ',
    ]
  end

  # 意図的に逃された
  def release_n_escape_success(room)
    [
      room.adventurer.short_name + 'は、監禁されている地下牢の鍵がかかっていないことに気づいた！',
      'どうやら近くに見張りもいないようだ。今が好機だと考えた' +
        room.adventurer.short_name + 'は音もなく地下牢の扉を開け、逃げ出すことに成功した！',
      'この好機が仕組まれたものとも知らない' + room.adventurer.short_name + 'は、迷宮の中へと再び身を躍らせた……',
      ' ',
    ]
  end

  # 脱走判定には成功したが、発情や催眠によって失敗
  def abort_to_escape_by_bad_status(room)
    adventurer = room.adventurer
    header = [
      '（脱走判定成功……？）',
      adventurer.short_name + 'は監視の目が緩んだチャンスを目ざとく捉えた！',
      '居眠りをしている見張りの' + room.monster_party.monster.name + 'の目を盗み、' + adventurer.short_name + 'は今なら地下牢の錠を外し、逃げ出せると考えた。',
      'しかし……',
    ]
    body =
      if room.failed_reason == 'rape_desire_tattoo'
        [
          '[陵辱願望刻印による効果]',
          adventurer.short_name + 'は強く後ろ髪を引かれる思いを感じた。',
          'ここにいれば、力強いオスの、バッキバキにいきりたった肉棒でたっぷりと犯してもらえる。' +
            'セックスという女にとって至高の悦びを、どうして自分から手放さなければならないのだろう？',
          adventurer.short_name + 'の頭の中で悪魔が囁く。どうして自分はそんなことを考えてしまうのだろう？',
          adventurer.short_name + 'は頭を振り、おぞましい考えを振り払おうとするが、陵辱を望む考えが、まるで誰かの手で脳に流し込まれるようにして大きくなり、逆らえなくなっていく。',
          'ついに、燃え盛る肉欲の炎に屈した' +
            adventurer.short_name + 'はついにその場で自慰行為を始めてしまった。セックスにはおよそ足らない中途半端な快楽。それでも、己を慰めなければ狂ってしまいそうなのだ。',
          '「ああ……　ここから、逃げなければいけないのにぃ……」',
          '脱出のチャンスは、無情にも肉欲に絡め取られた' + adventurer.short_name + 'の前を通り過ぎて行った。',
        ]
      elsif room.failed_reason == 'hypno'
        [
          '[催眠による効果]',
          adventurer.short_name + 'はなぜだかここにいなければいけない気がした。',
          'いま地下牢の外に出ることはひどい吹雪の中に裸で飛び出すようなものだ。それに比べて地下牢は暖炉で暖められた避難小屋のようなものに思えた。',
          'どうして自分は外に出なければいけないなどと考えたのだろう。',
          adventurer.short_name + 'はそう考え、地下牢の中に自分の意思で戻った。その結論に、彼女は何の疑問も感じなかった。',
        ]
      elsif room.failed_reason == 'rat'
        [
          '[発情による効果]',
          adventurer.short_name + 'は強く後ろ髪を引かれる思いを感じた。',
          'ここにいれば、力強いオスの、バッキバキにいきりたった肉棒でたっぷりと犯してもらえる。' +
            'セックスという女にとって至高の悦びを、どうして自分から手放さなければならないのだろう？',
          adventurer.short_name + 'の頭の中で悪魔が囁く。逡巡すればするほど、肉欲の炎が自分の中で燃え盛っていく。',
          adventurer.short_name + 'はついにその場で自慰行為を始めてしまった。セックスにはおよそ足らない中途半端な快楽。それでも、己を慰めなければ狂ってしまいそうなのだ。',
          '「ああ……　ここから、逃げなければいけないのにぃ……」',
          '脱出のチャンスは、無情にも肉欲に絡め取られた' + adventurer.short_name + 'の前を通り過ぎて行った。',
        ]
      elsif room.failed_reason == 'aphrodisiac'
        [
          '[催淫による効果]',
          adventurer.short_name + 'は強く後ろ髪を引かれる思いを感じた。',
          'ここにいれば、力強いオスの、バッキバキにいきりたった肉棒でたっぷりと犯してもらえる。' +
            'セックスという女にとって至高の悦びを、どうして自分から手放さなければならないのだろう？',
          adventurer.short_name + 'の頭の中で悪魔が囁く。逡巡すればするほど、肉欲の炎が自分の中で燃え盛っていく。',
          adventurer.short_name + 'はついにその場で自慰行為を始めてしまった。セックスにはおよそ足らない中途半端な快楽。それでも、己を慰めなければ狂ってしまいそうなのだ。',
          '「ああ……　ここから、逃げなければいけないのにぃ……」',
          '脱出のチャンスは、無情にも肉欲に絡め取られた' + adventurer.short_name + 'の前を通り過ぎて行った。',
        ]
      elsif room.failed_reason == 'aphrodisiac_fever'
        [
          '[催淫による効果]',
          adventurer.short_name + 'は強く後ろ髪を引かれる思いを感じた。',
          'ここにいれば、力強いオスの、バッキバキにいきりたった肉棒でたっぷりと犯してもらえる。' +
            'セックスという女にとって至高の悦びを、どうして自分から手放さなければならないのだろう？',
          adventurer.short_name + 'の頭の中で悪魔が囁く。逡巡すればするほど、肉欲の炎が自分の中で燃え盛っていく。',
          adventurer.short_name + 'はついにその場で自慰行為を始めてしまった。セックスにはおよそ足らない中途半端な快楽。それでも、己を慰めなければ狂ってしまいそうなのだ。',
          '「欲しい、おちんぽ欲しいのぉぉ……」',
          '耐え難い肉欲の渇きに耐えかねて、ついに' + adventurer.short_name + 'は自分の意思で、地下牢からの脱出のチャンスを手放してしまった。',
        ]
      elsif room.failed_reason == 'paralysis'
        [
          '[麻痺による効果]',
          adventurer.short_name + 'が地下牢の錠に指をかけた瞬間、麻痺の発作が彼女を襲った。',
          '視界が歪み、指が震え力が入らない。精密で強い力が必要になる錠開けを、この状態で行うなど到底不可能な話だった。',
          '脱出のチャンスにもかかわらず' + adventurer.short_name + 'の肉体は、彼女を裏切ったのだ……',
          adventurer.short_name + 'は脱出への希望の足音が遠ざかっていく音を痺れの残る頭の中で聞いていた。',
        ]
      elsif room.failed_reason == 'fever'
        [
          '[疫熱による効果]',
          adventurer.short_name + 'が地下牢の錠に指をかけた瞬間、疫病の発作が彼女を襲った。',
          '視界が暗く染まり、耐え難い悪寒で背筋が震える。強烈な吐き気で胃の中身をぶちまけそうになる。',
          '精密で強い力が必要になる錠開けを、この状態で行うなど到底不可能な話だった。',
          '脱出のチャンスにもかかわらず' + adventurer.short_name + 'の肉体は、彼女を裏切ったのだ……',
          adventurer.short_name + 'は脱出への希望の足音が遠ざかっていく音を遠のく意識の中で聞いていた。',
        ]
      else
        []
      end

      header + body + [' ']
  end

  def ready_to_escape_failed(room)
    [
      'だが逃げ出す好機は訪れなかった……',
      ' ',
    ]
  end

  def escape_in_brain_washing(room)
    [
      ' ',
      '（脱走判定成功）',
      'たっぷりとセックスを楽しんだ' + room.adventurer.short_name + 'は、体にこびりついた大量の白濁液をどこからか手に入れたボロ布とぬるま湯でゆっくりと清めていく。',
      '完全に、とは言えないが、なんとか見られるレベルには身なりを整えることができた。',
      'すでに鍵を使って錠を開けた地下牢から堂々と歩み去る' + room.adventurer.short_name + 'は、先ほどまで己を思う存分輪姦し尽くしてきた' + room.monster_party.monster.name + 'たちが床に倒れ伏している姿を振り返りもしない。',
      '肉の歓びを味わう一時の安らぎは終わり。これからまた、他の女冒険者たちに「女としての最高の幸せ」を教え込むための崇高な義務へと戻るのだ。',
      '地下牢の床には、精液を最後の一滴まで徹底的に搾り取られた' + room.monster_party.monster.name + 'たちがげっそりと瘦せこけ、倒れ伏していた……。',
      ' ',
      ' ',
    ]
  end

  def room_status_text(room)
    "#{room.floor}Fで監禁中（#{room.day_count}日目）"
  end

  def room_status_text_in_train(room)
    "#{room.floor}Fで監禁陵辱中（監禁#{room.day_count}日目、陵辱#{room.force_sex_day_count}日目）"
  end

  def situation_in_resist_shortly(room, old_resist_in_jail, new_resist_in_jail)
    monster_party = room.monster_party
    monster = monster_party.monster
    adventurer = room.adventurer
    resist_diff = old_resist_in_jail - new_resist_in_jail
    decrease_penalty_script = room.adventurer.decrease_resist.map {|status, value| "（ステータス「#{status}」によりさらに抵抗値減[#{value}]）" }
    script = []

    script += [
      adventurer.short_name + 'を' + monster_name_and_unit_name(monster, monster_party.rest_enemies_count) + 'たちが取りかこみ、' +
        '服を剥ぎ取り嬲りものにしようと襲いかかっていく。',
    ]

    script += [
      [
        adventurer.short_name + 'は必死に抵抗するが、徐々に体力を削り取られていく……',
      ],
      [
        '襲いかかる' + monster.name + 'たちは' + adventurer.short_name + 'の実力の前に、そうやすやすとは襲いかかることはできないようだ……',
        'だが、終わりのない攻撃は徐々に' + adventurer.short_name + 'の体力をヤスリのように削り取っていく……',
      ],
      [
        '素手といえども' + adventurer.short_name + 'は' + monster.name + 'を寄せ付けはしない。' +
          'だがジリジリと体力を削りおとされることで、破滅は近づきつつある。' +
          'その先に何があるかを、' + adventurer.short_name + 'はわからぬほど愚かではなかった……',
      ],
      [
        '「近づくなって、このぉっ！」',
        adventurer.short_name + 'は' + monster.name + 'を押し返すが次から次へと魔物たちは現れる。全くキリがなかった。',
      ],
      [
        monster.name + 'の指先の鋭い爪が防具にひっかかり、服の端をビリビリと引き裂いていく。',
        '「いい加減にしてっ！」',
        '白い肌をあらわにした' + adventurer.short_name + 'は' + monster.name + 'を拳でうちのめす。しかし、次から次へと魔物たちは現れていく。' +
          'じりじりと' + adventurer.short_name + 'の体力が削られていった。',
      ],
      [
        '次々と襲いかかる' + monster.name + 'たちの攻めは、生きたまま組み伏せて媚肉を貪ることを目的としているために、ダメージこそ与えることはできないがそれでも' +
          adventurer.short_name + 'の体力と抵抗は容赦なく削り取られていった',
      ],
      [
        '襲いかかる' + monster.name + 'を打ち倒しながら' + adventurer.short_name + 'は考える。',
        '今の自分に残された体力は、破滅までを数える砂時計の砂の残り量に等しい。可能な限り温存しなければならない。しかし間断なく襲いかかる' +
          monster.name + 'をあしらい続けることができなければまたそれも破滅だ。体力の温存と抵抗、危うい綱渡りのような行為を続けながら' +
          adventurer.short_name + 'はじわじわと体力を失っていった……',
      ],
    ].sample

    script += decrease_penalty_script
    script += [
      "（抵抗（#{room.adventurer.short_name}）：#{half_number_to_full(old_resist_in_jail)} → #{half_number_to_full(new_resist_in_jail)}（−#{half_number_to_full(resist_diff)}））",
    ]
    script
  end

  def situation_in_resist_n_left_shortly(room)
    monster_party = room.monster_party
    monster = monster_party.monster
    adventurer = room.adventurer
    script = [
      monster.name + 'たちは、主人であるあなたの命令に従い、' + adventurer.short_name + 'を監禁するだけで手を出そうとはしない……',
      adventurer.short_name + 'は手を出してこない' + monster.name + 'たちの様子を訝しがっているようだ……',
    ]
    script
  end

  def situation_in_training_n_left_shortly(room)
    monster_party = room.monster_party
    monster = monster_party.monster
    adventurer = room.adventurer
    script = [
      monster.name + 'たちは、主人であるあなたの命令に従い、' + adventurer.short_name + 'を監禁するだけで手を出そうとはしない……',
      'ある時を境に手出しをすることを控え始めた' + monster.name + 'たちの行動を' +
        adventurer.short_name + 'は訝しがっているようだ……',
      ' ',
    ]
    script
  end

  def situation_in_resist(room, old_resist_in_jail, new_resist_in_jail)
    monster_party = room.monster_party
    monster = monster_party.monster
    adventurer = room.adventurer
    resist_diff = old_resist_in_jail - new_resist_in_jail
    decrease_penalty_script = room.adventurer.decrease_resist.map {|status, value| "（ステータス「#{status}」によりさらに抵抗値減[#{value}]）" }
    script = []

    if room.over_flow?
      script += [
        "（薄暗い地下室の中には#{monster_name_and_unit_name(monster_party.monster, monster_party.rest_enemies_count)}がいる。" +
          '数が多すぎて入りきれず、幾人かは他の獲物を求めて部屋を離れたようだ）',
      ]
    end

    script += [
      [
        '薄暗い地下室の闇の中に女の悲鳴と男たちの怒声や囃し立てるような声が響く。',
        'ここは地下のダンジョンに無数に存在する地下室の一つ。' + adventurer.short_name + 'は' + monster_name_and_unit_name(monster_party.monster, monster_party.rest_enemies_count) + 'によって今まさに身ぐるみを剥がされ、嬲られようとしていた。',
      ],
      [
        'ここは地下のダンジョンに無数に存在する地下室の一つ。薄暗い闇の中に女の悲鳴と男たちの怒声や囃し立てるような声が響く。',
        adventurer.short_name + 'を' + monster_name_and_unit_name(monster, monster_party.rest_enemies_count) + 'たちが取りかこみ、' +
          '服を剥ぎ取ろうと禽獣のように襲いかかっていく。もちろんその目当ては' + adventurer.short_name + 'の媚肉を辱めることだ',
      ],
    ].sample
    if room.adventurer.has_bad_status_in_dive?(:androphobia)
      script += [
        '（バッドステータス：男性恐怖症）',
        '下卑た笑いを浮かべ、媚肉をむさぼるために襲いかかる不埒者に対し、' + room.adventurer.short_name + 'の動きは鈍い。',
        'もし自分が抵抗しなければ、どれほど惨めなことになるか、それを' + room.adventurer.short_name + 'はその身で理解していた。',
        '理解していたからこそ抵抗ができないのだ。かつて犯され嬲り尽くされたトラウマが' + room.adventurer.short_name + 'の手足をすくませ、精神を萎縮させている。',
        "それをいいことに#{monster.name}たちは格好の獲物である目の前の動きの鈍い肉贄に、肉欲にギラついた視線を絡みつかせながらなけなしの抵抗を削り落としていった。",
        '服や鎧が一つ一つ、乱暴に引きちぎられ、白い肌が次第にあらわになっていく。まだ肉体を犯されてはいないが、それも時間の問題だろう。',
        room.adventurer.short_name + 'の破滅のときは容赦なく近づいてくる……',
      ]
    elsif room.adventurer.has_bad_status_in_dive?(:hypno)
      script += [
        '（バッドステータス：催眠）',
        '下卑た笑いを浮かべ、媚肉をむさぼるために襲いかかる不埒者に対し、' + room.adventurer.short_name + 'の動きは鈍い。',
        '抵抗しなければいけないことは分かっている。だが' + room.adventurer.short_name + 'の頭の中で、自分と同じ声が聞こえる。',
        '抵抗しなければ犯してもらえる。セックスしていただける。それは女として素晴らしいことなのだ。なぜ抵抗する必要がある？',
        'その言葉がおかしいことはわかっていた。だがなぜか強く否定できない。',
        "#{room.adventurer.short_name}の様子がおかしいことをいいことに、#{monster.name}たちは格好の獲物である目の前の動きの鈍い肉贄に、肉欲にギラついた視線を絡みつかせながらなけなしの抵抗を削り落としていった。",
        '服や鎧が一つ一つ、乱暴に引きちぎられ、白い肌が次第にあらわになっていく。まだ肉体を犯されてはいないが、それも時間の問題だろう。',
        room.adventurer.short_name + 'の破滅のときは容赦なく近づいてくる……',
      ]
    else
      script += [
        'だがそれでも冒険者として鍛え上げた' + room.adventurer.short_name + 'は、素手であるにもかかわらず' + monster_party.monster.name + 'に必死の抵抗を示し寄せつけようとはしない。',
        'だが服や鎧が一つ一つ、乱暴に引きちぎられ、白い肌が次第にあらわになっていく。まだ肉体を犯されてはいないが、それも時間の問題だろう。',
        'それは' + room.adventurer.short_name + 'にも理解していた。だが助けがくるかもしれない。まだ希望を捨ててはいない' + room.adventurer.short_name + 'は必死に抵抗を繰り返し、破滅の時を先延ばしにしていた……',
      ]
    end

    script += decrease_penalty_script
    script += [
      "（抵抗（#{room.adventurer.short_name}）：#{half_number_to_full(old_resist_in_jail)} → #{half_number_to_full(new_resist_in_jail)}（−#{half_number_to_full(resist_diff)}））",
    ]
    script
  end

  def give_up_to_resist_shortly(room)
    adventurer = room.adventurer
    monster = room.monster_party.monster
    script = [
      'ついに' + adventurer.short_name + 'の抵抗する体力が尽きたようだ。ぜぇぜぇと呼吸を見出し、フラつく' +
        adventurer.short_name + 'の白い肌を貪るべく、' + monster.name + 'たちが襲いかかっていく',
      "（#{adventurer.short_name}の陵辱調教が始まりました）",
    ]
    script
  end

  def give_up_to_resist(room)
    adventurer = room.adventurer
    monster_party = room.monster_party
    script = [
      'ここはダンジョンに無数に存在する地下室の一つ。地下室に閉じ込められながらも、壁を背にしながら' + adventurer.short_name + 'は多数の' + monster_party.monster.name + 'たちを徒手空拳で打ち払い、' +
        '貞操の危機を紙一重でかわしてきた。しかし今までの抵抗により服はボロ布同然で、華美な意匠を施されていたはずの防具もクズ鉄同然に部屋の床に転がり男たちに踏みつけられている。',
      '積み重なった疲労と数の暴力が、ついに' + adventurer.short_name + 'に隙を生じさせてしまう。',
      monster_party.monster.name + 'は' + adventurer.short_name + 'を引きずり倒し組み伏せ、その美肉を今まさに貪ろうとしていた。' + adventurer.short_name + 'ものしかかった' +
        monster_party.monster.name + 'を投げ飛ばそうとするが、疲労によって手に力が入らない。ついに破局が訪れた。',
    ]

    if adventurer.has_any_equip_belt_in_dive?
      adventurer.clear_equip_belt_in_dive! # 凌辱するからエロ貞操帯は外すね
      script += [
        monster_party.monster.name + 'は煮えたぎる獣欲を叩きつけるべく、' + adventurer.short_name + 'の秘所を守っていた貞操帯に手をかける。',
        'あれほど外れなかった頑丈な貞操帯はあっさりと外れてしまった。' +
          '邪悪な迷宮の主人によってかけられた貞操帯の魔術強化が、邪悪な迷宮の主人の下僕である魔物の前には役に立つはずがなかった。',
        '頑丈な貞操帯という' + adventurer.short_name + 'がすがろうとしていたわずかな希望が目の前で打ち砕かれたことでいよいよ凌辱は避けられないものとなった。',
        monster_party.monster.name + 'は己の肉杭を、' + adventurer.short_name + 'の最も神聖な秘裂へと当てがう。' +
          adventurer.short_name + 'の短く押し殺した悲鳴が響いた。',
      ]
    end

    script += [
      adventurer.short_name + 'の秘められたる穴に肉槍が強引に侵入したのだ。',
    ]
    script +=
    if adventurer.virgin?
      adventurer.lost_virgin!('monster_force_sex', monster_party.monster.name)
      [
        '男を知らぬ処女穴に、興奮した男の野太い肉槍が何の気遣いもなく侵入する。',
        adventurer.short_name + 'は初めて味わう激痛に目を見開き大粒の涙をこぼしながら声にならない悲鳴をあげる。',
        monster_party.monster.name + 'もそのただならぬ様子から、' + adventurer.short_name + 'が処女だと気づいたようだが、むしろ初物を貪ることができる嗜虐的な歓びに' +
          '顔をいやらしく歪ませると、さらに腰を激しく振り始めた。',
        '部屋にはほんの少し前まで男を知らなかった体を好き勝手にいたぶられる' + adventurer.short_name + 'の悲痛な悲鳴と許しを乞う声が響いている。もちろん' +
          monster_party.monster.name + 'たちが許すはずがなかった。今までの鬱憤を晴らすように痛めつけるような陵辱を、今にも壊れそうな' + adventurer.short_name + 'の体に刻みつけている……',
      ]
    else
      [
        '男の太い肉杭が圧倒的な存在感をもって、' + adventurer.short_name + 'を内側から押し拡げる。',
        'かつての陵辱の記憶をいやでも思い出させようとするその圧倒的で暴力的なストロークが、' + adventurer.short_name + 'の心の古傷を容赦なく引っ掻いていく。',
        'あの時は助かったが、今回はもうダメかもしれない。' + adventurer.short_name  + 'の心の中で絶望のどす黒いシミが再び広がり始めていた……',
      ]
    end
    script += [
      "（#{adventurer.short_name}の陵辱調教が始まりました）",
    ]
    script
  end

  def situation_in_training(room, old_level, new_level)
    monster_party = room.monster_party
    monster = monster_party.monster
    betrayed_servant = monster_party.betrayed_servant
    adventurer = room.adventurer
    level_diff = old_level - new_level

    jail_detail_script = adventurer.jail_training_script(adventurer, monster_party)
    main_script = adventurer.jail_training_short_script(adventurer, monster_party)

    betrayed_servant_script =
      if betrayed_servant
        [
          [
            adventurer.short_name + 'を責め苛む者は魔物である' + monster.name + 'たちだけではない。',
            monster.name + 'たちに混じり、悪意に満ちた笑いを浮かべた' +
              betrayed_servant.short_name + 'も' + adventurer.short_name + 'の陵辱に加わっていた。',
            '女ゆえに、いや女だからこそ知り尽くした女の弱点を容赦なく徹底的に責め潰していく。',
            'セックスのテクニックとその耐性という攻撃と防御の関係において、' + betrayed_servant.short_name + 'と' +
              adventurer.short_name + 'の実力差は大人と子供の腕力の違いのように歴然だった。',
            '女から生えた肉槍が' + adventurer.short_name + 'の膣肉を引きずるように限界まで引き抜かれると、今度は一気に最奥まで押し込まれた。',
            'その衝撃に耐え切れず、低く唸るような、獣のような声をあげて無様に絶頂する。',
            '魔物たちの前でかつての仲間に許しを乞いながら何度も絶頂する' + adventurer.short_name + 'の姿を、' + monster.name + 'たちは嘲り笑っていた……',
          ],
          [
            adventurer.short_name + 'を責め苛む者は魔物である' + monster.name + 'たちだけではない。',
            monster.name + 'たちに混じり、' +
              betrayed_servant.short_name + 'も' + adventurer.short_name + 'の陵辱に加わっていた。',
            '叶わぬと知りながら必死に抵抗する' + adventurer.short_name + 'を、悪意に満ちた笑いを浮かべ' +
              betrayed_servant.short_name + 'は蹂躙していく。',
            '魔物たちに指示を出し、決して一時も休ませず、わずかな無駄もなく、容赦なく' + adventurer.short_name + 'の理性と尊厳をヤスリのように削り落としていく。',
            adventurer.short_name + 'が意識を失いかければ倦怠剤を、体力が限界を超えたとみれば最低限の回復薬を投与し、容赦なく桁違いの快楽を' +
              adventurer.short_name + 'の快楽神経の中に流し込んでいく。',
            adventurer.short_name + 'の四肢がばね仕掛けの人形のようにビクビクと跳ね回る様子を、' + betrayed_servant.short_name +
              'はさもおかしそうに、嗜虐的な笑いで眺めていた……',
          ],
          [
            monster.name + 'たちに混じり、' +
              betrayed_servant.short_name + 'も' + adventurer.short_name + 'の陵辱に加わっていた。',
            '女体のすべてを知り尽くした女であるがゆえに、どこまでが快楽の限界でどこからが苦痛なのかを完璧に理解していた' +
              betrayed_servant.short_name + 'は魔物たちに指示を出しながら、時折自らもその輪の中に加わった。',
            '徹底的に嬲り抜かれた' + adventurer.short_name + 'は容赦のない快楽によって何度も意識を失いながら、' +
              monster.name + 'たちの嗜虐心を大いに満足させた。',
          ],
        ].sample
      else
        []
      end

    sub_script =
      if adventurer.in_brain_washing?
        [
          adventurer.short_name + 'は自分の中で' + monster.name + 'の肉槍がゴリゴリと内側を削るように動くたび、ゾクゾクと背筋に走る甘い快楽に身を震わせている。',
        ]
      else
        [
          adventurer.short_name + 'は自分の中で' + monster.name + 'の肉槍がゴリゴリと内側を削るように動くたび、自分の最も大切な何かが削られていく感覚を感じていた。',
        ]
      end

    unless adventurer.decrease_san.empty?
      sub_script += ["（基本値：-#{half_number_to_full(adventurer.base_san_decrease_point)}）"]
      sub_script += adventurer.decrease_san.map.map {|status, value| "（ステータス「#{status}」による消耗：#{half_number_to_full(value * -1)}）" }
    end

    sub_script += [
      "（正気度（#{adventurer.short_name}）：#{half_number_to_full(old_level)} => #{half_number_to_full(new_level)}（−#{half_number_to_full(level_diff)}））",
    ]

    adventurer.set_jail_detail_script(
      jail_detail_script + sub_script
    )
    main_script + betrayed_servant_script + sub_script
  end

  def situation_in_escape(adventurer, floor_number)
    script = [
      '（脱走判定成功）',
    ]

    script += [
      [
        "#{adventurer.short_name}は地下#{half_number_to_full(floor_number)}階の地下牢で監禁されていたが、運良く脱走に成功した",
      ],
      [
        "地下#{half_number_to_full(floor_number)}階の地下牢で監禁されていた" +
          adventurer.short_name + 'は監視の目が緩んだチャンスを目ざとく捉えた！',
        '居眠りをしている見張りが身につけていた錠前を盗むことに成功した！',
        adventurer.short_name + 'は地下牢の錠を外し、逃げ出すことに成功した！',
      ],
      [
        "地下#{half_number_to_full(floor_number)}階の地下牢で監禁されていた" +
          adventurer.short_name + 'は監視の目が緩んだチャンスを目ざとく捉えた！',
        '居眠りをしている見張りの目を盗み、' + adventurer.short_name + 'は地下牢の錠を外し、逃げ出すことに成功した！',
      ],
    ].sample

    script + [' ',]
  end

  def situation_in_rescued(adventurer, leader, is_lonely, floor_number)
    [
      '（救出成功）',
      "#{adventurer.short_name}は地下#{half_number_to_full(floor_number)}階の地下牢で監禁されていたが、" +
        "#{_party_leader_short_names_by(leader.short_name, is_lonely)}によって救出された！",
    ]
  end

  def down_to_slave(room)
    monster_party = room.monster_party
    adventurer = room.adventurer

    script = adventurer.down_to_slave_script(adventurer, monster_party)

    script += [
      ' ',
      '度重なる限界を超えた陵辱調教。休みなく執拗に徹底的に叩き込まれる肉のリンチの前に、ついに' + 
      adventurer.short_name + 'は折れてしまった。',
      monster_party.monster.name + 'たちは' + adventurer.short_name + 'を体を丸めてようやく入るほどの狭い檻に入れると、すでに訪れていた奴隷商人に引き渡した。',
      '今まで冒険者として高めてきた彼女の名声と、持ち前の美貌が彼女自身の奴隷としての値段を吊り上げることになるのだ。買い手が誰になるかは分からないが、いずれにせよ' + adventurer.short_name + 'にとって幸福な結末になりはしないだろう。',
      '最後に' + adventurer.short_name + 'が入れられた檻が閉められ頑丈な錠がかけられる。檻が閉まるときにガシャンと大きな音を立てた。' +
        'それはまるで彼女の女としての人生の終わりを示すような冷酷な響きを含んでいた。',
      '（' + adventurer.short_name + 'はついに男たちの前で奴隷宣言しました。肉奴隷として売却されます）',
      ' ',
      adventurer.short_name + 'は、心が折れ、奴隷としての宣言をした時に、自分の中で何かが砕けた音を聞いた気がした。それはとても大切なもので、なにか' +
      adventurer.short_name + 'にとっては取り返しのつかないものが、永久に失われたことのように思えた。',
      'もっとも、もはや彼女にそれがなにかを知る術はない',
      '（' + adventurer.short_name + 'の精神が折れたことで、' + magic_power_print(adventurer.generated_magic_power.to_i) + 'の魔力が賢者の大水晶へと回収されました）',
      ' ',
    ]
    script
  end

  def summon_monster_shortly(room, diff_monster_count, new_monster_count)
    return [] if diff_monster_count.zero?
    monster_party = room.monster_party
    [
      '女の声を聞きつけて、どこからか新しい' + monster_party.monster.name + 'が現れたようだ。' +
      "（#{monster_party.monster.name}はさらに#{half_number_to_full(diff_monster_count)}#{monster_party.unit_name}増えて、#{half_number_to_full(new_monster_count)}#{monster_party.unit_name}になった）",
      ' ',
    ]
  end

  def over_flow_monster(room, new_monster_count)
    monster_party = room.monster_party
    [
      '女の声を聞きつけて、どこからか新しい' + monster_party.monster.name + 'が現れたようだ。女に飢えたオスが、この絶好の機会を見逃すはずがないのだ',
      "しかし地下牢の中は#{monster_party.monster.name}だらけでもはや入る隙間もない。#{monster_party.monster.name}たちは諦めて立ち去っていった",
      "（#{monster_party.monster.name}は#{half_number_to_full(new_monster_count)}#{monster_party.unit_name}になった）",
      ' ',
    ]
  end

  def summon_monster(room, diff_monster_count, new_monster_count)
    return [] if diff_monster_count.zero?

    monster_party = room.monster_party
    [
      '女の声を聞きつけて、どこからか新しい' + monster_party.monster.name + 'が現れたようだ。女に飢えたオスが、この絶好の機会を見逃すはずがないのだ',
      "（#{monster_party.monster.name}はさらに#{half_number_to_full(diff_monster_count)}#{monster_party.unit_name}増えて、#{half_number_to_full(new_monster_count)}#{monster_party.unit_name}になった）",
      ' ',
    ]
  end

  def summon_by_bounty(room, old_monster_count, new_monster_count)
    diff_monster_count = new_monster_count - old_monster_count
    return [] if diff_monster_count.zero?

    adventurer = room.adventurer
    monster_party = room.monster_party
    [
      '賞金をかけられた' + _adventurer_type + 'である' + adventurer.short_name + 'を捕まえたという知らせを聞きつけて' +
        'どこからか新しい' + monster_party.monster.name + 'が現れたようだ。',
      adventurer.short_name + 'を奴隷に堕とせば、もらえるはずの賞金のおこぼれに預かれるかもしれないという下心もある。何より飢えたオスたちにとって、' +
        '媚肉を貪ることができる機会など見逃すはずがないからだ。',
      "（#{monster_party.monster.name}はさらに#{half_number_to_full(diff_monster_count)}#{monster_party.unit_name}増えて、#{half_number_to_full(new_monster_count)}#{monster_party.unit_name}になった）",
      ' ',
    ]
  end

  def get_skill_personality(adventurer, skill_personality)
    skill_personality_name = adventurer.get_skill_personality_name(skill_personality)
    puts "skill_personality_name IS NOT FOUND:##{skill_personality}" unless skill_personality_name
    [
      '異常な状況と容赦のない暴力的な陵辱が' + adventurer.short_name +
        'の精神を歪め、壊していく。ひと時の休みもない陵辱の前に脳が焼ききれそうになる。',
      'それは不可逆的で取り返しのつかない刻印を、' + adventurer.short_name + 'の精神の最も深くに大きくはっきりと刻み込んでいった',
      adventurer.short_name + "は #{skill_personality_name} を取得しました",
    ]
  end

  def get_bad_status_in_body(adventurer, bad_status_in_body)
    bad_status_in_body_name = adventurer.get_bad_status_in_body_name(bad_status_in_body)
    puts "bad_status_in_body_name IS NOT FOUND:##{bad_status_in_body}" unless bad_status_in_body_name
    [
      '異常な状況と容赦のない暴力的な陵辱が' + adventurer.short_name +
        'の肉体を苛み壊し作り変えていく。許容限度を超えた快楽と法外な肉悦の前に全身の快楽神経が悲鳴をあげる。',
      '恐ろしい効果を持つ取り返しのつかない刻印を、' + adventurer.short_name + 'の肉体の最も本質的な部分にはっきりと刻み込んでいった',
      adventurer.short_name + "は #{bad_status_in_body_name} を取得しました",
    ]
  end

  def get_skill_holic(adventurer, skill_holic)
    skill_holic_name = adventurer.get_skill_holic_name(skill_holic)
    puts "skill_holic_name IS NOT FOUND:##{skill_holic}" unless skill_holic_name
    [
      '異常な状況と容赦のない暴力的な陵辱が' + adventurer.short_name +
        'の精神を歪め、壊していく。ひと時の休みもない陵辱の前に脳が焼ききれそうになる。',
      'それは不可逆的で取り返しのつかない刻印を、' + adventurer.short_name + 'の精神の最も深くに大きくはっきりと刻み込んでいった',
      adventurer.short_name + "は #{skill_holic_name} を取得しました",
    ]
  end
end
