# coding: utf-8
require 'src/defines/script/dungeon/script_utils.rb'

class PartyScript
  include ScriptUtils

  def party_status(party)
    fighter_status = party.fighter.status_short_script
    wizard_status  = party.wizard.status_short_script
    scout_status   = party.scout.status_short_script

    script = party.leader.status_short_script + 'は、'
    script +=
      case party.leader.class_type
        when 'fighter'
          wizard_status + 'と' + scout_status
        when 'wizard'
          fighter_status + 'と' + scout_status
        when 'scout'
          fighter_status + 'と' + wizard_status
      end
    script +=
      if party.in_rescue?
        'とともに' + party.leader.status_text + 'した'
      else
        'とともにパーティーを結成した'
      end

    script
  end

  def print_adventurers_summary_status(party)
    header = [
      party.leader.short_name + '隊にはメンバーが' + party.adventurers.size.to_s + '人います',
    ]
    if party.in_rescue?
      [
        '地下室に監禁されている' + party.rescue_target_short_name + 'の救援のために行動中',
      ]
    elsif party.in_rescue_ready?
      [
        '地下室に監禁されていた' + party.rescue_target_short_name + 'の行方を追って行動中',
      ]
    else
      []
    end
    status_script = party.members_sorted.map { |adv| adv_status(adv) }.inject(:+)
    header + status_script + [' ', ]
  end

  def print_passengers_summary_status(party)
    return [' ',] if party.passengers.empty?

    header = [
      party.leader.short_name + '隊には同行者が' + party.passengers.size.to_s + '人います',
    ]
    status_script = party.passengers.map { |adv| adv_status(adv) }.inject(:+)
    header + status_script + [' ', ' ',]
  end

  def adv_status(adventurer)
    [
      "#{adventurer.short_name}（#{adventurer.class_name}、ＬＶ：#{adventurer.level}）",
      "ＨＰ：#{adventurer.life_point}／#{adventurer.life_point_limit}、名声：#{adventurer.honor}、装備：#{adventurer.equip_point}、総撃破数：#{adventurer.kill_score}",
    ]
  end
end
