# coding: utf-8
require 'jrubyfx'
require 'jrubyfx-fxmlloader'
%w(AnchorPane HBox VBox).each do |source|
  java_import "javafx.scene.layout.#{source}"
end
%w(TextArea Button Hyperlink Label ListView TextField TextArea Pagination).each do |source|
  java_import "javafx.scene.control.#{source}"
end
%w(Font).each do |source|
  java_import "javafx.scene.text.#{source}"
end

PRODUCTION_MODE = true

if PRODUCTION_MODE
  require 'src/app/helper/loader.rb'
else
  require './src/app/helper/loader.rb'
end
# こいつは絶対最初
include RrkLoaderHelper

=begin
# Helper
(
%w(make_view_helper util_var_helper) +
  %w(global/util_var_define) +
  %w(chain_config/mode) +
  %w(chain_engine/view_helper chain_engine/list_edit_helper)
).each do |src_name|
  require_rrk 'src/helper_unit/' + src_name
end
require_rrk 'src/app/command/script/scripts_of_private'
=end

(
%w(chain_engine/save_data_service)
).each do |src_name|
  require_rrk 'src/app/service/' + src_name
end

# Controller, Model
# LogicController
%w(menu progress dungeon factory auction).each do |src_name|
  require_rrk 'src/app/logic_controller/dungeon/' + "#{src_name}_logic_controller"
end

%w(hour_glass game_master bard).each do |src_name|
  require_rrk 'src/app/model/dungeon/' + src_name
end

# ヘルパー
%w(chain_view).each do |src_name|
  require_rrk 'src/app/chain_engine/' + src_name
end

%w(view_element_converter).each do |src_name|
  require_rrk 'src/app/chain_engine_cover/dungeon/' + src_name
end

require_rrk 'src/app/chain_engine_helper/method_def/rrk/dungeon/dungeon_master_helper'

#%w(menu/house train/train train/train_menu menu/save)
%w().each do |src_name|
  require_rrk 'src/app/chain_engine_helper/method_def/rrk/' + src_name
end

#%w(menu/rrk_menu_shop_helper)
%w().each do |src_name|
  require_rrk 'src/app/chain_engine_helper/method_def/rrk/' + src_name
end

require_rrk 'src/app/chain_engine_helper/fx_parts/chain_parts_generator'

#%w(train_menu_helper).each do |src_name|
%w().each do |src_name|
  require_rrk 'src/app/rrk_define/action/' + src_name
end

require_rrk 'src/defines/script/dungeon/menu_script'

require_rrk 'src/app/helper/action/dungeon/action_henlper'
require_rrk 'src/app/helper/game_define_helper'
require_rrk 'src/app/helper/method_def/dungeon/menu/save_slots_helper'

# method_def系
#%w(train/train train/train_menu menu/map)
%w().each do |src_name|
  require_rrk 'src/app/chain_engine_helper/method_def/rrk/' + src_name
end

#%w(house_method menu_method_helper save_method_helper spring_method_helper train_method)
%w().each do |src_name|
  require_rrk 'src/app/chain_engine_helper/method_normal/' + src_name
end

# バージョン管理用
require_rrk 'src/app/model/version/version_secretary'
# セーブ用
require_rrk 'src/app/chain_engine_helper/method_def/rrk/menu/save'
# ステータス表示用
require_rrk 'src/app/chain_engine_helper/method_def/rrk/dungeon/show_adventurer_helper'
# 霊基改竄用
require_rrk 'src/app/chain_engine_helper/method_def/rrk/dungeon/brand_prisoner_helper'


require_rrk 'src/defines/dungeon/trap_list'
require_rrk 'src/defines/dungeon/monster_list_by_type'
require_rrk 'src/defines/dungeon/main_menu/bar_helper'
require_rrk 'src/defines/dungeon/main_menu/magic_factory_helper'
require_rrk 'src/app/helper/method_def/dungeon/menu/show_slave_helper'
require_rrk 'src/game_define/dungeon/constants'

resource_root :saves, File.join(File.dirname(__FILE__), 'saves'), 'src/saves'
resource_root :images, File.join(File.dirname(__FILE__), 'assets/image'), 'src/assets/image'
fxml_root File.join(File.dirname(__FILE__)), 'src'

WINDOW_HEIGHT = 640
WINDOW_WIDTH = 1080
GAME_TITLE = 'Pleasure of Slavery'

class DungeonApp < JRubyFX::Application
  def start(stage)
    with(stage, :title => GAME_TITLE, :width => WINDOW_WIDTH, :height => WINDOW_HEIGHT) do
      fxml StartController
    end.show
  end
end

class StartController
  include JRubyFX::Controller
  fxml 'start.fxml'

  on :start_game do
    @stage.fxml = DungeonMainController
    @stage.show
  end
end

class DungeonMainController
  include JRubyFX::Controller
  # 難易度とか開発モードとか調整系
  include GameDefineHelper

  include FxParts::GenerateHelper

  include MethodDef::Dungeon::Menu::SaveSlotsHelper
  include Action::Dungeon::ActionHelper

  include MonsterListByType
  include TrapList

  include ChainMenuSaveHelper
  include ShowAdventurerHelper
  include DungeonMasterHelper

  # 酒場メニュー
  include Dungeon::MainMenu::BarHelper

  # 魔術工房系メニュー
  include Dungeon::MainMenu::MagicFactoryHelper
  include MethodDef::Dungeon::Menu::ShowSlaveHelper
  # 霊基改竄系
  include BrandPrisonerHelper

  fxml 'ero_trap.fxml'

  on :opening_menu do
    # はじめから
    # TODO: はじめから
    _boot_game_common

    elements = @menu_logic_controller.opening_menu(@game_master.dungeon_master, @game_master.auction_room_slaves)
    assign_to_lists(elements)
  end

  on :restart_new_game do
    bonus = RestartBonus.new(@master)

    # はじめから
    _boot_game_common

    bonus.load_data(@master)

    elements = @menu_logic_controller.opening_menu(@game_master.dungeon_master, @game_master.auction_room_slaves)
    assign_to_lists(elements)
  end

  def opening_menu_from_load
    _boot_game_common
    _start_game_common
  end

  # ボーナス付きニューゲーム
  def opening_menu_after_clear
    _boot_game_common

    # 長期間モードに転換
    @game_master.add_clear_bonus!

    _start_game_common
  end

  on :start_mode do
    # はじめから
    # TODO: はじめから
    _boot_game_common
    _start_game_common
  end

  on :load_mode_in_initial do
    _boot_game_common
    elements = @menu_logic_controller.show_load_slots_in_initial
    assign_to_lists(elements)
  end

  on :train_group_menu do
    train_group_menu_elements = @train_logic_controller.show_train_group_menu
    assign_elements_to(@group_menu_list, train_group_menu_elements)
  end

  def _start_game_common
    elements = @menu_logic_controller.opening_menu(@game_master.dungeon_master, @game_master.auction_room_slaves)
    assign_to_lists(elements)
  end

  # ゲーム起動共通時
  def _boot_game_common
#    @menu_service = MenuService.new

    @game_master = GameMaster.new
    @bard = Bard.new
    @version_secretary = VersionSecretary.new
    @hour_glass = HourGlass.new
    @game_master.set_hour_glass(@hour_glass)

    mode_options = {
      'trial_mode' => TRIAL_MODE,
    }
    @menu_logic_controller = MenuLogicController.new(mode_options)
    @factory_logic_controller = FactoryLogicController.new
    @progress_logic_controller = ProgressLogicController.new
    @dungeon_logic_controller = DungeonLogicController.new(mode_options)
    @auction_logic_controller = AuctionLogicController.new

=begin
    @menu_logic_controller    = MenuLogicController.new(mode_options)
    @train_logic_controller   = TrainLogicController.new(mode_options)
    @spring_logic_controller = SpringLogicController.new(mode_options)
    @map_logic_controller     = MapLogicController.new
    @scripts_logic_controller = ScriptsLogicController.new
    @house_logic_controller   = HouseLogicController.new

    @training_service = TrainingService.new
    @slave_sell_service = SlaveSellService.new

    @version_secretary = VersionSecretary.new
    @version_secretary.boot!
=end
    _set_data_in_boot

    _clear_script_list
  end

  def _set_data_in_boot
    @game_master.boot_from_list!
  end

  def _limit_day
    if ORIGINAL_MODE
      return Dungeon::Constants::DAY_LIMIT_FOR_TRIAL_ORIGINAL if TRIAL_MODE
      return Dungeon::Constants::DAY_LIMIT_ORIGINAL
    end

    return Dungeon::Constants::DAY_LIMIT_FOR_TRIAL if TRIAL_MODE
    return Dungeon::Constants::DAY_LIMIT_LONG if @game_master.long_day_mode
    Dungeon::Constants::DAY_LIMIT
  end

  def progress_turn
    if @hour_glass.last_night?(_limit_day)
      return show_ending
    end

    @game_master.build_game! if @game_master.before_build?
    @game_master.build_chairs_for_append_adventurers if @game_master.append_adventurers_time?(@hour_glass)
    @game_master.build_chairs_for_append_adventurers_for_original(@hour_glass) if @game_master.append_adventurers_time_for_original?(@hour_glass)
    if @game_master.append_adventurers_time_for_fgo?(@hour_glass)
      @game_master.build_chairs_for_append_adventurers_for_fgo(@hour_glass)
    end

    @game_master.progress_by_turn(@game_master.dungeon_master, @bard, @hour_glass)

    script_elements = @progress_logic_controller.progress_turn(@bard, @hour_glass)

    @hour_glass.ahead!
    _elapse_turn_common!

    menu_elements = @menu_logic_controller.reload_top_menu
    assign_to_lists(script_elements.merge(menu_elements))

    _scroll_to_top
  end

  def _scroll_to_top
    @script_list.scrollTo(0)
  end

  def show_ending
    elements = @menu_logic_controller.show_ending(TRIAL_MODE)
    assign_to_lists(elements)
  end

  def show_dungeon
    elements = @progress_logic_controller.show_dungeon(@bard)
    assign_to_lists(elements)

    _scroll_to_top
  end

  # 監禁された冒険者を見る
  # 削除

  # 洗脳サーヴァント一覧
  def show_betray_servant_cage
    elements = @menu_logic_controller.show_betray_servant_cage(@game_master, @hour_glass)
    assign_to_lists(elements)
  end

  # 洗脳従属奴隷のふたなり管理
  def manage_betray_servant_penis
    elements = @menu_logic_controller.manage_betray_servant_penis(@game_master, @hour_glass)
    assign_to_lists(elements)
  end

  # 話を聞くだけ
  # 昨日の地下牢の様子
  def show_jail
    elements = @progress_logic_controller.show_jail(@bard)
    assign_to_lists(elements)

    _scroll_to_top
  end

  def show_magic_factory_night
    elements = @progress_logic_controller.show_magic_factory_night(@bard)
    assign_to_lists(elements)

    _scroll_to_top
  end

  # 冒険者を見るの一覧
  def hear_adventurers_in_bar
    elements = @menu_logic_controller.hear_adventurers_in_bar(@game_master, @hour_glass)
    assign_to_lists(elements)

    _scroll_to_top
  end

  def _elapse_turn_common!
    @menu_logic_controller.update_by_turn(@game_master, @hour_glass.night?, @hour_glass.new_dawn?)
  end

  def _show_top_menu
    elements = @menu_logic_controller.top_menu(@hour_glass)
    assign_to_lists(elements)
  end

  # メニューで「戻る」って行ったら基本これ
  def show_game_top_menu
    _show_top_menu
  end

  def show_dungeon_top_menu
    elements = @dungeon_logic_controller.show_dungeon_top_menu(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_summons_power_up
    elements = @dungeon_logic_controller.show_summons_power_up(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_traps_power_up
    elements = @dungeon_logic_controller.show_traps_power_up(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_merchant_shop
    elements = @dungeon_logic_controller.show_merchant_shop(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_book_shop
    elements = @dungeon_logic_controller.show_book_shop(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_summon_magic_room
    elements = @dungeon_logic_controller.show_summon_magic_room(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_carpenter_shop
    elements = @dungeon_logic_controller.show_carpenter_shop(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_tentacle_shop
    elements = @dungeon_logic_controller.show_tentacle_shop(@game_master.dungeon_master)
    assign_to_lists(elements)
  end

  def show_auction_room
    elements = @auction_logic_controller.show_auction_room(@game_master)
    assign_to_lists(elements)
  end
end

DungeonApp.launch

