# coding: utf-8
module PreDefined
  module List
    module Merchant
      module BuildingList
        def all_building_list
          all_buildings =
            building_list.merge(building_1_extension_list)

          all_buildings
        end

        def building_list
          {
            'extension_building_100' => {
              name: '調教小屋の増築',
              price: _calculate_barn_extension_price,
              description: %w(
                調教小屋を修理し、収容できる奴隷を増やします。
                定員が１名増えます
              ),
              target: 'slave_barn',
              size: 1,
            },
            'building_1' => {
              name: '売春小屋',
              price: 30000,
              description: %w(
                購入すれば奴隷に売春をさせることができます。
                廃屋ですが、地下牢があります。
                牢には多数の拘束具が残されています。
                ………過去になにがあったのでしょうか。
                地下と、地下へと通じる部分は手入れをすれば地下室の一部は使えるでしょう
                地下室そのものは広々としていますが、大半の部分は修理が必要です。
                最初の定員は２人です。あとから増築可能です。
              )
            },
          }
        end

        def building_1_extension_list
          {
            'extension_building_102' => {
              name: '娼婦部屋の増築',
              price: _calculate_brothel_extension_price,
              description: %w(
                売春小屋の地上部分を修理し、使える場所を広げます。
                ベッドを備えた部屋が１つ使えるようになります。
                娼婦売春に使える定員が１名増えます。
                娼婦売春を選ぶと調教済の奴隷に客への濃厚奉仕をさせることができます。
              ),
              target: 'spring_brothel',
              size: 1,
            },
            'extension_building_101' => {
              name: '地下牢の増築',
              price: _calculate_dungeon_extension_price,
              description: %w(
                売春小屋の地下牢を修理し、使える場所を広げます。
                拘束器具を備えた地下牢が１つ使えるようになります。
                便器売春に使える定員が１名増えます
                便器売春を選ぶと調教済の奴隷を身動きとれぬよう拘束し、客に肉便器として奉仕させることができます。
              ),
              target: 'spring_dungeon',
              size: 1,
            },
            'extension_building_103' => {
              name: '見世物小屋の増築',
              price: _calculate_show_house_extension_price,
              description: %w(
                売春小屋のとなりに見世物小屋として使える建物を増設します。
                奴隷に特殊な行為をさせる小屋が１つ使えるようになります。
                見世物売春に使える定員が１名増えます
                見世物売春を選ぶと調教済の奴隷を大勢の観客の前で見世物として、特殊な売春をさせることができます。
              ),
              target: 'spring_show_house',
              size: 1,
            },
          }
        end

        EXPAND_PENALTY_BASE_LINE = 2 # このラインを超えると購入価格が上がる
        PENALTY_BASE_RATE = 1.4 # この値の累乗がペナルティ
        DEFAULT_BARN_SIZE = 2
        DEFAULT_BARN_PRICE = 3_000
        EXPAND_BARN_PRICE = 7_000

        def _spring_expand_rate
          if @game_type == 'trainer'
            0.7
          else
            1.0
          end
        end

        def _barn_expand_rate
          if @game_type == 'trainer'
            0.5 # 娼婦小屋よりは安くしてよい
          else
            1.0
          end
        end

        def _show_house_expand_rate
          1.0
        end

        def _calculate_barn_extension_price
          base_rate = _barn_expand_rate

          expand_count = (@slave_barn[:size] - DEFAULT_BARN_SIZE)
          too_many_penalty_rate = _calculate_too_many_penalty_rate(expand_count)
          price =
            _calculate_building_price(
              expand_count, too_many_penalty_rate,
              DEFAULT_BARN_PRICE, EXPAND_BARN_PRICE,
              base_rate
            )

          price.to_i
        end

        DEFAULT_DUNGEON_SIZE = 1
        DEFAULT_DUNGEON_PRICE = 5_000
        EXPAND_DUNGEON_PRICE = 10_000
        def _calculate_dungeon_extension_price
          base_rate = _spring_expand_rate
          expand_count = (@spring_house.dungeon_max_size - DEFAULT_DUNGEON_SIZE)
          too_many_penalty_rate = _calculate_too_many_penalty_rate(expand_count)
          price = _calculate_building_price(
            expand_count, too_many_penalty_rate,
            DEFAULT_DUNGEON_PRICE, EXPAND_DUNGEON_PRICE,
            base_rate
          )
          price.to_i
        end

        DEFAULT_BROTHEL_SIZE = 0
        DEFAULT_BROTHEL_PRICE = 10_000
        EXPAND_BROTHEL_PRICE = 20_000
        def _calculate_brothel_extension_price
          base_rate = _spring_expand_rate

          expand_count = (@spring_house.brothel_max_size - DEFAULT_BROTHEL_SIZE)
          too_many_penalty_rate = _calculate_too_many_penalty_rate(expand_count)
          price = _calculate_building_price(
            expand_count, too_many_penalty_rate,
            DEFAULT_BROTHEL_PRICE, EXPAND_BROTHEL_PRICE,
            base_rate
          )
          price.to_i
        end

        DEFAULT_SHOW_HOUSE_SIZE = 0
        DEFAULT_SHOW_HOUSE_PRICE = 300_000
        EXPAND_SHOW_HOUSE_PRICE = 150_000
        def _calculate_show_house_extension_price
          base_rate = _show_house_expand_rate
          expand_count = (@spring_house.show_house_max_size - DEFAULT_SHOW_HOUSE_SIZE)
          too_many_penalty_rate = _calculate_too_many_penalty_rate(expand_count)
          price = _calculate_building_price(
            expand_count, too_many_penalty_rate,
            DEFAULT_SHOW_HOUSE_PRICE, EXPAND_SHOW_HOUSE_PRICE,
            base_rate
          )
          price.to_i
        end

        # 建物のお値段計算処理
        def _calculate_building_price(expand_count, too_many_penalty_rate, default_price, expand_price, by_mode_rate)
          price = default_price + (expand_count * too_many_penalty_rate * expand_price * by_mode_rate).round(-3).to_i
          price
        end

        # 増築回数が多すぎる場合はペナルティーで価格が上がる
        def _calculate_too_many_penalty_rate(expand_count)
          too_many_penalty_rate = 1
          extra_expand_count = (expand_count - EXPAND_PENALTY_BASE_LINE)
          if extra_expand_count > 0
            too_many_penalty_rate =  PENALTY_BASE_RATE ** extra_expand_count
          else
            ; # なにもしない。１のまま
          end
          too_many_penalty_rate
        end
      end
    end
  end
end
