# coding: utf-8
module Save
  module Common
    def packaged?
      File.exist?(save_dir_path_in_packaged)
    end

    def save_dir_path_in_packaged
      (File.join(File.dirname(__FILE__), '../', '../../../../saves')).to_s
    end

    def save_dir_path
      if packaged?
        save_dir_path_in_packaged
      else
        (File.join(File.dirname(__FILE__), '../../saves')).to_s
      end
    end

    def make_save_file_path(save_number)
      File.join(File.absolute_path("."), 'saves', "save#{save_number}.save")
    end
  end

  module Load
    def load_data
      clear_script_list
      list_saves_for_load
    end

    def load_from(save_unit)
      save_unit.for_backward_compatibility

      @global_save = save_unit.global_save
      @day = save_unit.day
      @master = save_unit.master
      @game_mode = save_unit.game_mode
      @game_type = save_unit.game_type
      @slave_barn = save_unit.slave_barn
      @house = save_unit.house
      @slaves_status = save_unit.slaves_status
      @spring_house = save_unit.spring_house
      @items = save_unit.items
      @money = save_unit.money
      @buildings = save_unit.buildings
      @coin_box = save_unit.coin_box
      @spring_service = save_unit.spring_service
      @master = save_unit.master
      @items_on_sale = save_unit.items_on_sale || {}
      @exploit_time_flag = save_unit.exploit_time_flag # 集金フラグ
      @daily_spring_news_flag = save_unit.daily_spring_news_flag # 売春報告フラグ
      @hourglass = save_unit.hourglass
      # MEMO: about SAVE, セーブファイルに入れたい項目を増やす時はここをいじる。(4/4)

      load_for_initialize
    end

    def list_saves_for_load
      button_set = []
      ('0'..'9').to_a.each do |save_number|
        save_file_path = make_save_file_path(save_number)
        if File.exist?(save_file_path)
          access_time = File.mtime(save_file_path).strftime("%Y-%m-%d %H:%M:%S")
          save_label = "#{save_number}： セーブデータ#{save_number}  (#{access_time})"
          button_set << {text: save_label, action: "load_from_#{save_number}"}
          # セーブファイルがあるときの処理
        else
          # セーブファイルがないときの処理
          save_label = "#{save_number}： セーブデータがありません"
          button_set << {text: save_label, action: "no_save"}
        end
      end

      script_list_view_of_buttons(button_set)
    end

    ('0'..'9').to_a.each do |save_number|
      define_method('load_from_' + save_number) do
        File.open(make_save_file_path(save_number), 'r') do |in_f|
          save_unit = YAML.load(in_f)
          load_from(save_unit)
        end

        show_money_in_notice_text

        clear_upper_texts
        scripts = ['ゲームをロードしました']
        refresh_script_list_from(scripts)
      end
    end
  end

  module Write
    def make_save_data
      save_file = SaveUnit.new
      save_file.set_save_params(
        global_save: @global_save || GlobalSave.new,
        day: @day,
        master: @master,
        slave_barn: @slave_barn,
        house: @house,
        slaves_status: @slaves_status,
        spring_house: @spring_house,
        items: @items,
        money: @money,
        buildings: @buildings,
        coin_box: @coin_box,
        spring_service: @spring_service,
        master: @master,
        items_on_sale: @items_on_sale,
        game_mode: @game_mode,
        game_type: @game_type,
        exploit_time_flag: @exploit_time_flag,
        daily_spring_news_flag: @daily_spring_news_flag,
        hourglass: @hourglass,
      ) # MEMO: about SAVE, セーブファイルに入れたい項目を増やす時はここをいじる。(3/4)
      save_file
    end

    def save_data
      clear_script_list
      list_saves_for_save
    end

    def list_saves_for_save
      button_set = SaveDataService::make_save_list
      script_list_view_of_buttons(button_set)
    end

    def no_save
#      puts 'セーブデータがないよ'
    end

    ('0'..'9').to_a.each do |save_number|
      define_method('save_as_' + save_number) do
        save_file_path = make_save_file_path(save_number)
        save_file = make_save_data

        File.open(save_file_path, 'w:UTF-8') do |out_f|
          out_f.write save_file.to_yaml
        end

        puts "have saved in #{save_number}"

        scripts = ['ゲームを保存しました']
        refresh_script_list_from(scripts)

        list_saves_for_save
      end
    end
  end
end
